/* attack.c */
/* Last modified: April 1, 2007 */


static BB RankAttack[64][256], RankAttack90[64][256];
static BB RankAttack45l[64][256], RankAttack45r[64][256]; 
static ui column45l[64] = {0,  0,  0,  0,  0,  0,  0,  0,
		           1,  1,  1,  1,  1,  1,  1,  1,
		           2,  2,  2,  2,  2,  2,  2,  2,
		           3,  3,  3,  3,  3,  3,  3,  3,
		           4,  4,  4,  4,  4,  4,  4,  4,
		           5,  5,  5,  5,  5,  5,  5,  5,
		           6,  6,  6,  6,  6,  6,  6,  6,
		           7,  7,  7,  7,  7,  7,  7,  7};
static ui column45r[64] = {0,  0,  0,  0,  0,  0,  0,  0,
		           1,  1,  1,  1,  1,  1,  1,  1,
		           2,  2,  2,  2,  2,  2,  2,  2,
		           3,  3,  3,  3,  3,  3,  3,  3,
		           4,  4,  4,  4,  4,  4,  4,  4,
		           5,  5,  5,  5,  5,  5,  5,  5,
		           6,  6,  6,  6,  6,  6,  6,  6,
		           7,  7,  7,  7,  7,  7,  7,  7};
static ui inverse45l[64] = {0, 15, 22, 29, 36, 43, 50, 57,
			    1,  8, 23, 30, 37, 44, 51, 58,
			    2,  9, 16, 31, 38, 45, 52, 59,
			    3, 10, 17, 24, 39, 46, 53, 60,
			    4, 11, 18, 25, 32, 47, 54, 61,
			    5, 12, 19, 26, 33, 40, 55, 62,
			    6, 13, 20, 27, 34, 41, 48, 63,
			    7, 14, 21, 28, 35, 42, 49, 56};
static ui inverse45r[64] = {7,  8, 17, 26, 35, 44, 53, 62,
			    6, 15, 16, 25, 34, 43, 52, 61,
			    5, 14, 23, 24, 33, 42, 51, 60,
			    4, 13, 22, 31, 32, 41, 50, 59,
			    3, 12, 21, 30, 39, 40, 49, 58,
			    2, 11, 20, 29, 38, 47, 48, 57,
			    1, 10, 19, 28, 37, 46, 55, 56,
			    0,  9, 18, 27, 36, 45, 54, 63};


/*
 * Find the attacks from <sq> on a 8-bit vector supplied as <bitRank>.
 * <bitRank> represents a rank, but it can be rotated to represent a 
 * file or diagonal.  From crafty.
 */
static ui attackedBitRank(ui sq, ui bitRank)
{
  ui start;
  ui attacked = 0;

  ASSERT(sq >= A1 && sq <= H8);
  ASSERT(bitRank >= 0);

  /* find attacks to right of <sq> */
  if (sq < 7) 
    {
      ASSERT(sq+1 >= 0);
      start = 1 << (sq+1);
      while (start < 256) 
	{
	  attacked = attacked | start;
	  if (bitRank & start) break;
	  ASSERT(start >= 0);
	  start = start << 1;
	}
    }

  /* find attacks to left of <sq> */
  if (sq > 0) 
    {
      ASSERT(sq-1 >= 0);
      start = 1 << (sq-1);
      while (start > 0) 
	{
	  attacked = attacked | start;
	  if (bitRank & start) break;
	  ASSERT(start >= 0);
	  start = start >> 1;
	}
    }

  return (attacked & 255);
}


void initRankAttacks()
{
  ui firstOne8bit[256];
  ui i, j, sq, t, t45;
  ui attacked;

  /* Index of first bit in an 8-bit value */
  firstOne8bit[0]=8;
  for (i=0; i<256; i++)
    {
      for (j=0; j<8; j++)
	{
	  if (i & (1<<j))
	    {
	      firstOne8bit[i] = j;
	      break;
	    }
	}
    }

  /* Precalculate all possible attacks from <sq> on a 8-bit vector */
  for (sq=0; sq<64; sq++)
    {
      for (i=0; i<256; i++) 
	{
	  attacked = attackedBitRank(column(sq), i);
	  RankAttack[sq][i] = 0;
	  /* mobility[sq][i] = 0; */
	  while (attacked) 
	    {
	      t = firstOne8bit[attacked];
	      /* This is a neat trick.  (sq&56) "slides" sq to first row. */
	      RankAttack[sq][i] |= OneMask[(sq&56) + t];
	      ASSERT(t >= 0);
	      attacked &= ~(1<<t);
	    }
	}
      /* mobility[sq][i] = PopCnt(RankAttack[sq][i]); */
    }

  /* Precalculate all possible attacks from <sq> on a 8-bit vector, rotated
     90 degrees. */
  for (sq=0; sq<64; sq++) 
    {
      for (i=0; i<256; i++) 
	{
	  attacked = attackedBitRank(row(sq), i);
	  RankAttack90[sq][i] = 0;
	  /* mobility90[sq][i] = 0; */
	  while (attacked) 
	    {
	      t = firstOne8bit[attacked];
	      RankAttack90[sq][i] |= OneMask[rotate90[((sq&7)<<3) + t]];
	      ASSERT(t >= 0);
	      attacked &= ~(1<<t);
	    }
	}
      /* mobility90[sq][i]=PopCnt(RankAttack90[sq][i]); */
    }

  /* Precalculate all possible attacks from <sq> on a 8-bit vector, rotated
     45 degrees left. */
  for (sq=0; sq<64; sq++) 
    {
      for (i=0; i<256; i++) 
	{
	  attacked = attackedBitRank(column45l[sq], i);
	  RankAttack45l[sq][i] = 0;
	  /* mobility45l[sq][i] = 0; */
	  while (attacked) 
	    {
	      t = firstOne8bit[attacked];
	      t45 = inverse45l[((rotate45l[sq])&56) + t];
	      if ((row(sq)+column(sq)) == (row(t45)+column(t45)))
		RankAttack45l[sq][i] |= OneMask[t45];
	      ASSERT(t >= 0);
	      attacked &= ~(1<<t);
	    }
	}
      /* mobility45l[sq][i]=PopCnt(RankAttack45l[sq][i]); */
    }

  /* Precalculate all possible attacks from <sq> on a 8-bit vector, rotated
     45 degrees right. */
  for (sq=0; sq<64; sq++) 
    {
      for (i=0; i<256; i++) 
	{
	  attacked = attackedBitRank(column45r[sq], i);
	  RankAttack45r[sq][i] = 0;
	  /* mobility45r[sq][i] = 0; */
	  while (attacked) 
	    {
	      t = firstOne8bit[attacked];
	      t45 = inverse45r[((rotate45r[sq])&56) + t];
	      if ((row(sq)-column(sq)) == (row(t45)-column(t45)))
		RankAttack45r[sq][i] |= OneMask[t45];
	      ASSERT(t >= 0);
	      attacked &= ~(1<<t);
	    }
      /* mobility45r[sq][i]=PopCnt(RankAttack45r[sq][i]); */
	}
    }
}


/* Is sq attacked by White Pawn? */
ui isSqAttackedByWP(ui sq, sPos *pos, sOthers *others)
{
  ASSERT(sq >= A1 && sq <= H8);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  if (others->BPCaptures[sq] & pos->WPBoard) return 1;
  return 0;
}


/* Is sq attacked by White Knight? */
ui isSqAttackedByWN(ui sq, sPos *pos, sOthers *others)
{
  ASSERT(sq >= A1 && sq <= H8);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  if (others->NMoves[sq] & pos->WNBoard) return 2;
  return 0;
}


/* Is sq attacked by White Bishop? */
static ui isSqAttackedByWB(ui sq, sPos *pos)
{
  ASSERT(sq >= A1 && sq <= H8);
  ASSERT(pos != NULL);

  if (bishopAttacks(sq, pos) & pos->WBBoard) return 2;
  return 0;
}


/* Is sq attacked by White Rook? */
static ui isSqAttackedByWR(ui sq, sPos *pos)
{
  ASSERT(sq >= A1 && sq <= H8);
  ASSERT(pos != NULL);

  if (rookAttacks(sq, pos) & pos->WRBoard) return 4;
  return 0;
}


/* Is sq attacked by White Queen? */
static ui isSqAttackedByWQ(ui sq, sPos *pos)
{
  ASSERT(sq >= A1 && sq <= H8);
  ASSERT(pos != NULL);

  if (rookAttacks(sq, pos) & pos->WQBoard) return 8;
  if (bishopAttacks(sq, pos) & pos->WQBoard) return 8;
  return 0;
}


/* Is sq attacked by White King? */
ui isSqAttackedByWK(ui sq, sPos *pos, sOthers *others)
{
  ASSERT(sq >= A1 && sq <= H8);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  if (others->KMoves[sq] & OneMask[pos->wKSq]) return 16;
  return 0;
}


/* Form a bitboard of White pieces that are checking Black king at sq. */
ui isInCheckByW(ui sq, BB *Attackers, sPos *pos, sOthers *others)
{
  ui checkers = 0;
  BB Temp = 0;

  ASSERT(sq >= A1 && sq <= H8);
  ASSERT(Attackers != NULL);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  /* pawn and knight can't check at the same time */
  if ((Temp = (others->BPCaptures[sq] & pos->WPBoard)))
    {
      checkers++;
      *Attackers |= Temp;
    }
  else if ((Temp = (others->NMoves[sq] & pos->WNBoard)))
    {
      checkers++;
      *Attackers |= Temp;
    }

  /* rook direction first, faster than bishop direction */
  if ((Temp = rookAttacks(sq, pos) & (pos->WRBoard | pos->WQBoard)))
    {
      checkers++;
      *Attackers |= Temp;
    }
  
  /* can't have more than 2 checkers */
  if (checkers < 2 && 
      ((Temp = bishopAttacks(sq, pos) & 
	(pos->WBBoard | pos->WQBoard))))
    {
      checkers++;
      *Attackers |= Temp;
    }

  return checkers;
}


/* Can White make a move to attack sq? */
/* IMPORTANT: this function does not take into account attack from king */
ui isSqAttackableByW(ui sq, sPos *pos, sOthers *others)
{
  BB Temp;
  ui i;

  ASSERT(sq >= A1 && sq <= H8);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  /* knight */
  Temp = others->NMoves[sq] & ~pos->WBoard;
  while (Temp)
    {
      i = firstOne(Temp);
      if (pos->WNBoard & others->NMoves[i]) return 1;
      Temp &= NilMask[i];
    }

  /* bishop */
  Temp = bishopAttacks(sq, pos) & ~pos->WBoard;
  while (Temp)
    {
      i = firstOne(Temp);
      if (isSqAttackedByWB(i, pos)) return 1;
      Temp &= NilMask[i];
    }

  /* rook */
  Temp = rookAttacks(sq, pos) & ~pos->WBoard;
  while (Temp)
   {
     i = firstOne(Temp);
     if (isSqAttackedByWR(i, pos)) return 1;
     Temp &= NilMask[i];
    }


  /* queen */
  Temp = (bishopAttacks(sq, pos) |
	  rookAttacks(sq, pos)) 
    & ~pos->WBoard;
  while (Temp)
    {
      i = firstOne(Temp);
      if (isSqAttackedByWQ(i, pos)) return 1;
      Temp &= NilMask[i];
    }

  /* pawn */
  Temp = others->BPCaptures[sq] & ~pos->WBoard;
  while (Temp)
    {
      i = firstOne(Temp);
      if (!(OneMask[i] & pos->BBoard))
	{
	  if ((row(i) > 2) && (OneMask[i-8] & pos->WPBoard)) return 1;
	  if ((row(i) == 4) &&
	      !(OneMask[i-8] & (pos->WBoard | pos->BBoard)) &&
	      (OneMask[i-16] & pos->WPBoard))
	    return 1;
	}
      if (pos->WPBoard & others->BPCaptures[i]) return 1;
      Temp &= NilMask[i];
    }

  return 0;
}


/* Is sq attacked by Black Pawn? */
ui isSqAttackedByBP(ui sq, sPos *pos, sOthers *others)
{
  ASSERT(sq >= A1 && sq <= H8);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  if (others->WPCaptures[sq] & pos->BPBoard) return 1;
  return 0;
}


/* Is sq attacked by Black Knight? */
ui isSqAttackedByBN(ui sq, sPos *pos, sOthers *others)
{
  ASSERT(sq >= A1 && sq <= H8);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  if (others->NMoves[sq] & pos->BNBoard) return 2;
  return 0;
}


/* Is sq attacked by Black Bishop? */
static ui isSqAttackedByBB(ui sq, sPos *pos)
{
  ASSERT(sq >= A1 && sq <= H8);
  ASSERT(pos != NULL);

  if (bishopAttacks(sq, pos) & pos->BBBoard) return 2;
  return 0;
}


/* Is sq attacked by Black Rook? */
static ui isSqAttackedByBR(ui sq, sPos *pos)
{
  ASSERT(sq >= A1 && sq <= H8);
  ASSERT(pos != NULL);

  if (rookAttacks(sq, pos) & pos->BRBoard) return 4;
  return 0;
}


/* Is sq attacked by Black Queen? */
static ui isSqAttackedByBQ(ui sq, sPos *pos)
{
  ASSERT(sq >= A1 && sq <= H8);
  ASSERT(pos != NULL);

  if (rookAttacks(sq, pos) & pos->BQBoard) return 8;
  if (bishopAttacks(sq, pos) & pos->BQBoard) return 8;
  return 0;
}


/* Is sq attacked by Black King? */
ui isSqAttackedByBK(ui sq, sPos *pos, sOthers *others)
{
  ASSERT(sq >= A1 && sq <= H8);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  if (others->KMoves[sq] & OneMask[pos->bKSq]) return 16;
  return 0;
}


/* Form a bitboard of Black pieces that are checking White king at sq. */
ui isInCheckByB(ui sq, BB *Attackers, sPos *pos, sOthers *others)
{
  ui checkers = 0;
  BB Temp = 0;

  ASSERT(sq >= A1 && sq <= H8);
  ASSERT(Attackers != NULL);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  /* pawn and knight can't check at the same time */
  if ((Temp = (others->WPCaptures[sq] & pos->BPBoard)))
    {
      checkers++;
      *Attackers |= Temp;
    }
  else if ((Temp = (others->NMoves[sq] & pos->BNBoard)))
    {
      checkers++;
      *Attackers |= Temp;
    }

  /* rook direction first, faster than bishop direction */
  if ((Temp = rookAttacks(sq, pos) & 
      (pos->BRBoard | pos->BQBoard)))
    {
      checkers++;
      *Attackers |= Temp;
    }
  
  /* can't have more than 2 checkers */
  if (checkers < 2 && 
      ((Temp = bishopAttacks(sq, pos) & 
       (pos->BBBoard | pos->BQBoard))))
    {
      checkers++;
      *Attackers |= Temp;
    }

  return checkers;
}


/* Can Black make a move to attack sq? */
/* IMPORTANT: this function does not take into account attack from king */
ui isSqAttackableByB(ui sq, sPos *pos, sOthers *others)
{
  BB Temp;
  ui i;

  ASSERT(sq >= A1 && sq <= H8);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  /* knight */
  Temp = others->NMoves[sq] & ~pos->BBoard;
  while (Temp)
    {
      i = firstOne(Temp);
      if (pos->BNBoard & others->NMoves[i]) return 1;
      Temp &= NilMask[i];
    }

  /* bishop */
  Temp = bishopAttacks(sq, pos) & ~pos->BBoard;
  while (Temp)
    {
      i = firstOne(Temp);
      if (isSqAttackedByBB(i, pos)) return 1;
      Temp &= NilMask[i];
    }

  /* rook */
  Temp = rookAttacks(sq, pos) & ~pos->BBoard;
  while (Temp)
    {
      i = firstOne(Temp);
      if (isSqAttackedByBR(i, pos)) return 1;
      Temp &= NilMask[i];
    }

  /* queen */
  Temp = (bishopAttacks(sq, pos) |
	  rookAttacks(sq, pos))
	  & ~pos->BBoard;
  while (Temp)
    {
      i = firstOne(Temp);
      if (isSqAttackedByBQ(i, pos)) return 1;
      Temp &= NilMask[i];
    }


  /* pawn */
  Temp = others->WPCaptures[sq] & ~pos->BBoard;
  while (Temp)
    {
      i = firstOne(Temp);
      if (!(OneMask[i] & pos->WBoard))
	{
	  if ((row(i) < 5) && (OneMask[i+8] & pos->BPBoard)) return 1;
	  if ((row(i) == 3) &&
	      !(OneMask[i+8] & (pos->WBoard | pos->BBoard)) &&
	      (OneMask[i+16] & pos->BPBoard))
	    return 1;
	}
      if (pos->BPBoard & others->WPCaptures[i]) return 1;
      Temp &= NilMask[i];
    }

  return 0;
}


/* Some sort of caching for wAttacker[sq] */
ui getWAttacker(ui sq, sPos *pos, sPlyData *thisPlyData, sOthers *others)
{
  ASSERT(sq >= A1 && sq <= H8);
  ASSERT(thisPlyData != NULL);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  /* sttacker array already exists */
  if (thisPlyData->wAttacker[sq] & 1) return (ui) thisPlyData->wAttacker[sq];
  /* attacker array doesn't exists yet */
  else
    {
      thisPlyData->wAttacker[sq] = 1;    /* now it exists but still empty */
      if (isSqAttackedByWP(sq, pos, others)) 
	thisPlyData->wAttacker[sq] |= 8;
      if (isSqAttackedByWN(sq, pos, others)) 
	thisPlyData->wAttacker[sq] |= 16;
      if (isSqAttackedByWB(sq, pos)) 
	thisPlyData->wAttacker[sq] |= 16;
      if (isSqAttackedByWR(sq, pos)) 
	thisPlyData->wAttacker[sq] |= 32;
      if (isSqAttackedByWQ(sq, pos)) 
	thisPlyData->wAttacker[sq] |= 64;
      if (isSqAttackedByWK(sq, pos, others)) 
	thisPlyData->wAttacker[sq] |= 128;
      if (thisPlyData->wAttacker[sq] > 1)
	thisPlyData->wAttacker[sq] |= 2;   /* now it exists and has entry */
      return (ui) thisPlyData->wAttacker[sq];
    }
}


/* Some sort of caching for bAttacker[sq] */
ui getBAttacker(ui sq, sPos *pos, sPlyData *thisPlyData, sOthers *others)
{
  ASSERT(sq >= A1 && sq <= H8);
  ASSERT(thisPlyData != NULL);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  /* sttacker array already exists */
  if (thisPlyData->bAttacker[sq] & 1) return (ui) thisPlyData->bAttacker[sq];
  /* attacker array doesn't exists yet */
  else
    {
      thisPlyData->bAttacker[sq] = 1;    /* now it exists but still empty */
      if (isSqAttackedByBP(sq, pos, others)) 
	thisPlyData->bAttacker[sq] |= 8;
      if (isSqAttackedByBN(sq, pos, others)) 
	thisPlyData->bAttacker[sq] |= 16;
      if (isSqAttackedByBB(sq, pos)) 
	thisPlyData->bAttacker[sq] |= 16;
      if (isSqAttackedByBR(sq, pos)) 
	thisPlyData->bAttacker[sq] |= 32;
      if (isSqAttackedByBQ(sq, pos)) 
	thisPlyData->bAttacker[sq] |= 64;
      if (isSqAttackedByBK(sq, pos, others)) 
	thisPlyData->bAttacker[sq] |= 128;
      if (thisPlyData->bAttacker[sq] > 1) 
	thisPlyData->bAttacker[sq] |= 2;   /* now it exists and has entry */
      return (ui) thisPlyData->bAttacker[sq];
    }
}


/* Find what piece (pinner) is pinning pinnee to pinKing. */
ui pinner(ui pinneeColor, ui pinnee, ui pinKing, sPos *pos, sOthers *others)
{
  BB AttackBoard, Line;
  ui r, c;

  ASSERT(pinneeColor == white || pinneeColor == black);
  ASSERT(pinnee >= A1 && pinnee <= H8);
  ASSERT(pinKing >= A1 && pinKing <= H8);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  r = row(pinKing);
  c = column(pinKing);

  switch (others->direction[pinKing][pinnee])
    {
    case 0: return none;
    case 1:
    case -1:
      AttackBoard = hRookAttacks(pinnee, pos);
      /* pinnee and pinKing have to be connected though */
      if (AttackBoard & OneMask[pinKing])
	{
	  Line = AttackBoard & others->Row[r];
	  if (pinneeColor == white)
	    {
	      if (Line & pos->BRBoard) return bR;
	      else if (Line & pos->BQBoard) return bQ;
	    }
	  else
	    {
	      if (Line & pos->WRBoard) return wR;
	      else if (Line & pos->WQBoard) return wQ;
	    }
	}
      break;
    case 8:
    case -8:
      AttackBoard = vRookAttacks(pinnee, pos);
      /* pinnee and pinKing have to be connected though */
      if (AttackBoard & OneMask[pinKing])
	{
	  Line = AttackBoard & others->Column[c];
	  if (pinneeColor == white)
	    {
	      if (Line & pos->BRBoard) return bR;
	      else if (Line & pos->BQBoard) return bQ;
	    }
	  else
	    {
	      if (Line & pos->WRBoard) return wR;
	      else if (Line & pos->WQBoard) return wQ;
	    }
	}
      break;
    case 7:
      AttackBoard = lBishopAttacks(pinnee, pos);
      /* pinnee and pinKing have to be connected though */
      if (AttackBoard & OneMask[pinKing])
	{
	  Line = others->Up7Of[pinnee] & AttackBoard;
	  if (pinneeColor == white)
	    {
	      if (Line & pos->BBBoard) return bB;
	      else if (Line & pos->BQBoard) return bQ;
	    }
	  else
	    {
	      if (Line & pos->WBBoard) return wB;
	      else if (Line & pos->WQBoard) return wQ;
	    }
	}
      break;
    case -7:
      AttackBoard = lBishopAttacks(pinnee, pos);
      /* pinnee and pinKing have to be connected though */
      if (AttackBoard & OneMask[pinKing])
	{
	  Line = others->Down7Of[pinnee] & AttackBoard;
	  if (pinneeColor == white)
	    {
	      if (Line & pos->BBBoard) return bB;
	      else if (Line & pos->BQBoard) return bQ;
	    }
	  else
	    {
	      if (Line & pos->WBBoard) return wB;
	      else if (Line & pos->WQBoard) return wQ;
	    }
	}
      break;
    case 9:
      AttackBoard = rBishopAttacks(pinnee, pos);
      /* pinnee and pinKing have to be connected though */
      if (AttackBoard & OneMask[pinKing])
	{
	  Line = others->Up9Of[pinnee] & AttackBoard;
	  if (pinneeColor == white)
	    {
	      if (Line & pos->BBBoard) return bB;
	      else if (Line & pos->BQBoard) return bQ;
	    }
	  else
	    {
	      if (Line & pos->WBBoard) return wB;
	      else if (Line & pos->WQBoard) return wQ;
	    }
	}
      break;
    case -9:
      AttackBoard = rBishopAttacks(pinnee, pos);
      /* pinnee and pinKing have to be connected though */
      if (AttackBoard & OneMask[pinKing])
	{
	  Line = others->Down9Of[pinnee] & AttackBoard;
	  if (pinneeColor == white)
	    {
	      if (Line & pos->BBBoard) return bB;
	      else if (Line & pos->BQBoard) return bQ;
	    }
	  else
	    {
	      if (Line & pos->WBBoard) return wB;
	      else if (Line & pos->WQBoard) return wQ;
	    }
	}
      break;
    }

  return none;
}


/* Return the bitboard of a sliding pieces in a direction from sq */
BB ambusher(ui sq, int direction, sPos *pos, sOthers *others)
{
  BB Attacker;

  ASSERT(sq >= A1 && sq <= H8);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  switch (direction)
    {
    case 1:
      Attacker = pos->WRBoard | pos->WQBoard | pos->BRBoard | pos->BQBoard;
      return Attacker & hRookAttacks(sq, pos) & others->Up1Of[sq];
    case -1:
      Attacker = pos->WRBoard | pos->WQBoard | pos->BRBoard | pos->BQBoard;
      return Attacker & hRookAttacks(sq, pos) & others->Down1Of[sq];
    case 8:
      Attacker = pos->WRBoard | pos->WQBoard | pos->BRBoard | pos->BQBoard;
      return Attacker & vRookAttacks(sq, pos) & others->Up8Of[sq];
    case -8:
      Attacker = pos->WRBoard | pos->WQBoard | pos->BRBoard | pos->BQBoard;
      return Attacker & vRookAttacks(sq, pos) & others->Down8Of[sq];
    case 7:
      Attacker = pos->WBBoard | pos->WQBoard | pos->BBBoard | pos->BQBoard;
      return Attacker & lBishopAttacks(sq, pos) & others->Up7Of[sq];
    case -7:
      Attacker = pos->WBBoard | pos->WQBoard | pos->BBBoard | pos->BQBoard;
      return Attacker & lBishopAttacks(sq, pos) & others->Down7Of[sq];
    case 9:
      Attacker = pos->WBBoard | pos->WQBoard | pos->BBBoard | pos->BQBoard;
      return Attacker & rBishopAttacks(sq, pos) & others->Up9Of[sq];
    case -9:
      Attacker = pos->WBBoard | pos->WQBoard | pos->BBBoard | pos->BQBoard;
      return Attacker & rBishopAttacks(sq, pos) & others->Down9Of[sq];
    default:
      break;
    }

  return (BB) 0;
}


/* Return the value of a piece at sq */
int valueAtSq(ui sq, sPos *pos)
{
  BB Sq;

  ASSERT(sq >= A1 && sq <= H8);
  ASSERT(pos != NULL);

  Sq = OneMask[sq];
  if (Sq & pos->WPBoard) return pValue;
  if (Sq & pos->BPBoard) return pValue;
  if (Sq & pos->WNBoard) return nValue;
  if (Sq & pos->BNBoard) return nValue;
  if (Sq & pos->WBBoard) return bValue;
  if (Sq & pos->BBBoard) return bValue;
  if (Sq & pos->WRBoard) return rValue;
  if (Sq & pos->BRBoard) return rValue;
  if (Sq & pos->WQBoard) return qValue;
  if (Sq & pos->BQBoard) return qValue;

  if (sq == pos->wKSq) return kValue;
  if (sq == pos->bKSq) return kValue;

  return none;
}


/* Static Exchange Evaluation, championed by Bob Hyatt */
/* Probably works better if the capturer is not a king */
/* IMPORTANT: see() returns score for the side to move */
int see(ui wToMove, ui sq, ui t, sPos *pos, sOthers *others)
{
  int scoreArray[16];
  BB Attacker, Temp;
  ui index = 0;
  int lastValue;
  int direction;
  ui i;

  ASSERT(sq >= A1 && sq <= H8);
  ASSERT(t >= A1 && t <= H8);
  ASSERT(sq != t);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  /* collect all attackers, including ambusher behind sq */
  Attacker = 
    (others->WPCaptures[t] & pos->BPBoard) |
    (others->BPCaptures[t] & pos->WPBoard) |
    (others->NMoves[t] & (pos->WNBoard | pos->BNBoard)) |
    (bishopAttacks(t, pos) & 
     (pos->WBBoard | pos->BBBoard | pos->WQBoard | pos->BQBoard)) |
    (rookAttacks(t, pos) & 
     (pos->WRBoard | pos->BRBoard | pos->WQBoard | pos->BQBoard)) |
    (others->KMoves[t] & (OneMask[pos->wKSq] | OneMask[pos->bKSq]));

  /* first capture */
  scoreArray[index] = valueAtSq(t, pos);
  ASSERT(scoreArray[index] > 0 && scoreArray[index] <= qValue);
  lastValue = valueAtSq(sq, pos);
  Attacker &= NilMask[sq];
  direction = others->direction[t][sq];
  if (direction) Attacker |= ambusher(sq, direction, pos, others);
  wToMove ^= 1;

  /* go through all attackers to fill scoreArray[], smaller ones first */
  while (Attacker)
    {
      Temp = (BB) 0;

      if (wToMove)
	{
	  if ((Temp = (pos->WPBoard & Attacker))) sq = firstOne(Temp);
	  else if ((Temp = (pos->WNBoard & Attacker))) sq = firstOne(Temp);
	  else if ((Temp = (pos->WBBoard & Attacker))) sq = firstOne(Temp);
	  else if ((Temp = (pos->WRBoard & Attacker))) sq = firstOne(Temp);
	  else if ((Temp = (pos->WQBoard & Attacker))) sq = firstOne(Temp);
	  else if (OneMask[pos->wKSq] & Attacker) 
	    {
	      if (Attacker & pos->BBoard) break;
	      sq = pos->wKSq;
	    }
	  else break;        /* otherwise canibalism */
	}
      else
	{
	  if ((Temp = (pos->BPBoard & Attacker))) sq = firstOne(Temp);
	  else if ((Temp = (pos->BNBoard & Attacker))) sq = firstOne(Temp);
	  else if ((Temp = (pos->BBBoard & Attacker))) sq = firstOne(Temp);
	  else if ((Temp = (pos->BRBoard & Attacker))) sq = firstOne(Temp);
	  else if ((Temp = (pos->BQBoard & Attacker))) sq = firstOne(Temp);
	  else if (OneMask[pos->bKSq] & Attacker) 
	    {
	      if (Attacker & pos->WBoard) break;
	      sq = pos->bKSq;
	    }
	  else break;        /* otherwise canibalism */
	}

      index++;
      ASSERT(index < 16);
      scoreArray[index] = lastValue - scoreArray[index-1];
      lastValue = valueAtSq(sq, pos);
      Attacker &= NilMask[sq];
      direction = others->direction[t][sq];
      if (direction) Attacker |= ambusher(sq, direction, pos, others);
      wToMove ^= 1;
    }

  /* Now the tricky part to "minimax" the values */
  for (i = index; i > 0; i--)
    if (scoreArray[i] > -scoreArray[i-1]) scoreArray[i-1] = - scoreArray[i];

  return scoreArray[0];
}
