/* main.c */
/* Last modified: July 24, 2007 */


/* parrot - Johanes Suhardjo (johanes@nd.edu) */
/* http://www.nd.edu/~johanes/parrot.html */
/* This program is a freeware, but if you use part of my code 
 * in your own program, do me a favor, acknowledge it. */


#if !defined WINDOWS
#include <unistd.h>
#endif
#include <stdio.h>
#include <ctype.h>
#include <signal.h>
#include <string.h>
#include <time.h>

#if !defined WINDOWS
#include <stdlib.h>
#include <strings.h>
#else
#define strcasecmp stricmp
#endif
#ifdef _REENTRANT
#include <pthread.h>
#include <semaphore.h>
#endif

#include "parrot.h"

#ifdef BONUSES_PENALTIES
int PBonus, NBonus, BBonus, RBonus, QBonus, KBonus;
int PPenalty, NPenalty, BPenalty, RPenalty, QPenalty, KPenalty;
int PEndBonus, NEndBonus, BEndBonus, REndBonus, QEndBonus, KEndBonus;
int PEndPenalty, NEndPenalty, BEndPenalty, REndPenalty, QEndPenalty, KEndPenalty;
#endif

/* for getCommand thread */
#ifdef _REENTRANT
pthread_mutex_t goMutex;
#endif
ui go;
char word[WORDS][LETTERS];


/* iterative deepening search */
static int iterate(ui wToMove, sPos *pos, sPlyData *plyData,
		   sPV *pv, sHash **hash, sOthers *others, sTC *tc, 
		   BB* allMoves, ui nMoves, int origCE, double *nPMicroS, 
		   double timeReserved, ui movesLeft, BB theMove[maxPly], 
		   BB *ponderMove, BB *ponderReply)
{
  char s[8];
  char pvString[64][18];
  sPos pvPos;
  sPlyData pvPlyData[maxPly];

  ui mNum;
  ui wTM;
  ui wToMoveJ;
#ifdef TOURNAMENT
  ui i;
#endif
  ui j;
  int score = 0;
  int lastScore = 0;
  int alpha, beta;
  ui highBF = true;    /* high overal branching factor, or ply <= minPly */

  int rootScore;
  int wPieceScore, bPieceScore, maxPieceScore;
  int doubleBonus = 0;
#ifdef WINDOWS
  double clock1;
#else
  struct timeval tv;
  struct timezone tz;
  BB clock1;
#endif
  BB lastNodes;
  BB lastPonderMove = (BB) 0;
  BB lastPonderReply = (BB) 0;
  double bf = 3.0;             /* instantenous branching factor */
  double bFactor = 3.0;        /* overall branching factor */
  ui extFactor = 5;
  sPV lastPV;
  sPV thePV;


  ASSERT(wToMove == false || wToMove == true);
  ASSERT(pos != NULL);
  ASSERT(plyData != NULL);
  ASSERT(*hash != NULL);
  ASSERT(others != NULL);
  ASSERT(tc != NULL);
  ASSERT(allMoves != NULL);
  ASSERT(nMoves >= 1);
  ASSERT(!(pos->WBoard & pos->BBoard));
  ASSERT(nPMicroS != NULL);
  ASSERT(movesLeft > 0);


  lastNodes = pos->sNodes + pos->qNodes;
#ifdef WINDOWS
  clock1 = 1000000.0/CLOCKS_PER_SEC*clock();
#else
  gettimeofday(&tv, &tz);
  clock1 = (BB) tv.tv_sec*1000000 + (BB) tv.tv_usec;
#endif

  alpha = -mateScore;
  beta = mateScore;
  lastPV = pv[0];
  theMove[1] = pv[0].move[0];
  
  /* score at the root */
  rootScore = 
    evalMaterial(pos, others, &wPieceScore, &bPieceScore, &doubleBonus);
  ASSERT(rootScore >= -mateScore && rootScore <= mateScore);
  if ((wPieceScore + bPieceScore) >= 0) maxPieceScore = wPieceScore;
  else maxPieceScore = -bPieceScore;
  ASSERT(maxPieceScore >= 0);
  if (maxPieceScore >= endGameValue && pos->rootMaxPieceScore < endGameValue)
    clearHash(hash, pos);
  if (maxPieceScore < endGameValue && pos->rootMaxPieceScore >= endGameValue)
    clearHash(hash, pos);
  pos->rootMaxPieceScore = maxPieceScore;

  if (rootScore != drawScore)
    {
      rootScore = eval(wToMove, 1, pos, plyData, others, rootScore,
		       wPieceScore, bPieceScore, -mateScore, doubleBonus);
      ASSERT(rootScore >= -mateScore && rootScore <= mateScore);
    }
  else rootScore = 0;
  if (!wToMove) rootScore = -rootScore;


  pos->Flags &= NilMask[4];    /* search neither terminated */
  pos->Flags &= NilMask[7];    /*     nor almost terminated */
  pos->iteration = 0;
  while (!(pos->Flags & OneMask[4]) && 
	 pos->iteration < pos->searchDepth &&
	 (double) ((clock1-tc->clock0) * bf) < tc->timeBudget)
    {
      pos->iteration++;
#ifdef TREEDUMP
      printf("============================================\n");
      printf("             Iteration %u\n", pos->iteration);
      printf("============================================\n");
#endif
      tc->nodeInc = (BB)(*nPMicroS * tc->timeBudget / 10);
      if (pos->Flags & OneMask[7]) tc->nodeInc /= 10;
      
      ASSERT(pos->iteration <= maxPly);
      thePV = pv[0];
      pos->rootAlpha = alpha;
      pos->rootBeta = beta;
      score = rsearch(wToMove, pos->rootAlpha, pos->rootBeta, pos->iteration,
		      pos, plyData, pv, hash, others, tc, allMoves, nMoves, 
		      origCE, highBF, &thePV);
      ASSERT(score >= -mateScore && score <= mateScore);
      
      /* if out of bound, re-search with a proper bound */
      if (!(pos->Flags & OneMask[4]) && (score <= alpha))
	{
	  printf("score too low: %d<%d<%d\n", 
		 alpha, score, beta);
	  /* tend to make weak moves here, add time to think */
	  if (pos->iteration > minPly && timeReserved > 0)
	    tc->timeBudget += timeReserved;
	  ASSERT(pos->iteration < maxPly);
	  /* re-search the lower part of the score space */
	  pos->rootAlpha = -mateScore;
	  pos->rootBeta = alpha+1;
	  score = rsearch(wToMove, pos->rootAlpha, pos->rootBeta, 
			  pos->iteration, pos, plyData, pv, hash, others, tc, 
			  allMoves, nMoves, origCE, highBF, &thePV);
	  ASSERT(score >= -mateScore && score <= mateScore);
	}
      else if (!(pos->Flags & OneMask[4]) && (score >= beta))
	{
	  printf("score too high: %d<%d<%d\n", 
		 alpha, score, beta);
	  /* Save the last values in case we fail low after the re-search. */
	  lastPV = pv[0];
	  lastScore = score;
	  ASSERT(pos->iteration < maxPly);
	  /* re-search the upper part of the score space */
	  pos->rootAlpha = beta-1;
	  pos->rootBeta = mateScore;
	  score = rsearch(wToMove, pos->rootAlpha, pos->rootBeta, 
			  pos->iteration, pos, plyData, pv, hash, others, tc, 
			  allMoves, nMoves, origCE, highBF, &thePV);
	  ASSERT(score >= -mateScore && score <= mateScore);
	  /* If fail low here, we won't have a reliable move, so let's use the
	     saved move and score. */
	  if (score <= pos->rootAlpha)
	    {
	      printf("fail low after fail high at root, use last move\n");
	      pv[0].move[0] = lastPV.move[0];
	      pv[0].length = 1;
	      score = lastScore;
	    }
	}

#ifdef _REENTRANT
      /* Do not ponder in obvious positions, it only causes problems */
      if ((go & goPonder) && 
	  ((bitCount(pos->RotatedBoard90) <= pos->EGTBlimit) ||
	   (abs(score) > almostScore)))
	{
	  printf("turn off pondering\n");
	  go = goRest;
	  return score;
	}
#endif

      /* the best move and  score so far */
      if (pos->Flags & OneMask[4]) 
	{
	  theMove[pos->iteration] = theMove[pos->iteration-1];
	  score = lastScore;
	}
      else
	{
	  theMove[pos->iteration] = pv[0].move[0]; 
	  lastScore = score;
	}

      /* narrow down the search space */
      if (score > -mateScore+halfWindow)
	alpha = score - halfWindow;
      else alpha = -mateScore;
      if (score < mateScore-halfWindow)
	beta = score + halfWindow;
      else beta = mateScore;
      
      if (alpha < -mateScore) alpha = -mateScore;
      if (beta > mateScore) beta = mateScore;
      
#ifdef WINDOWS
      clock1 = 1000000.0/CLOCKS_PER_SEC*clock();
#else
      gettimeofday(&tv, &tz);
      clock1 = (BB) tv.tv_sec*1000000 + (BB) tv.tv_usec;
#endif
      
      /* if (pos->iteration >= minPly) */
	{
	  /* Michael Yee corrected the following lines to 
	     enable xboard's "show thinking" option. */ 
	  printf("%u", pos->iteration);
	  printf(" %d", score);
	  /* in centisecond */
	  printf(" %u", (ui) ((clock1-tc->clock0)/10000));
#ifdef _REENTRANT
	  if (tc->timeBudget > maxSearchBudget)
	    printf(" %.0f P", (double)(pos->sNodes + pos->qNodes));
	  else
#endif
	    printf(" %.0f S", (double)(pos->sNodes + pos->qNodes));
	  mNum = pos->moveNumber+1;
	  wTM = wToMoveJ = wToMove;
	  
	  /* print the PV */
	  pvPos = *pos;
	  for (j = 0; j < maxPly; j++) pvPlyData[j] = plyData[j];
	  if (pos->Flags & OneMask[4]) pv[0].length = 1;
	  for (j = 0; j < pv[0].length; j++)
	    {
	      if (wTM) printf("%u. ", mNum);
	      else if (j == 0) printf("%u. ... ", mNum);
	      san(wToMoveJ, j, pv[0].move[j], s, &pvPos, pvPlyData, others);
	      printf("%s ", s);
	      if (wTM) sprintf (pvString[j], "%u. %s", mNum, s);
	      else if (j == 0) sprintf (pvString[j], "%u. ... %s", mNum, s);
	      else sprintf (pvString[j], "%s", s);
	      wTM ^= 1;
	      if (wTM) mNum++;
	      makeMove(wToMoveJ, j+1, pv[0].move[j], &pvPos, 
		       pvPlyData, others);
	      wToMoveJ ^= 1;
	    }
	  if (pos->Flags & OneMask[4]) 
	    {
	      pos->iteration--;
	      if (go & goSearch) *ponderMove = lastPonderMove;
	      if (go & goPonder) *ponderReply = lastPonderReply;
	      pv[0] = lastPV;
	      printf(" CUT");
	    }
	  else 
	    {
	      if (go & goSearch)
		{
		  *ponderMove = pv[0].move[1];
		  lastPonderMove = *ponderMove;
		}
	      else if (go & goPonder)
		{
		  *ponderReply = pv[0].move[1];
		  lastPonderReply = *ponderReply;
		}
	      /* if (from(pv[0].move[0]) != 0 || to(pv[0].move[0]) != 0) */
	      lastPV = pv[0];
	    }
	  printf("\n");
	  sprintf (pvString[j], "\0");
	  fflush(stdout);
	}
      
      ASSERT(pos->moveNumber < maxMove);
#ifdef WINDOWS
      clock1 = 1000000.0/CLOCKS_PER_SEC*clock();
#else
      gettimeofday(&tv, &tz);
      clock1 = (BB) tv.tv_sec*1000000 + (BB) tv.tv_usec;
#endif
      *nPMicroS = (double) (pos->sNodes + pos->qNodes) /
	(clock1-tc->clock0);
      if (*nPMicroS > 2*NPMS) *nPMicroS = 2*NPMS;

      /* overal branching factor */
      if (pos->iteration > 1)
	bFactor = log(pos->sNodes + pos->qNodes) / log(pos->iteration);
      else bFactor = (double)(pos->sNodes + pos->qNodes) / pos->iteration;

      /* feedback controlled search based on bFactor 
	 collected between January and May 2007 */
      if (pos->iteration < minPly+1)
	{
	  pos->pvExt = pvExt0;
	  pos->checkExt = checkExt0;
	  pos->onlyReplyExt = onlyReplyExt0;
	  pos->mateThreatExt = mateThreatExt0;
	  pos->parrotExt = parrotExt0;
	  pos->pawn7thExt = pawn7thExt0;
	  highBF = false;
	}
      else if (pos->iteration == minPly+1)
	{
	  /* positional position */
	  if (pos->fastIteration > 0 &&       /* zero is dangerous situation */
	      maxPieceScore > endGameValue && /* not in the endgame */
	      (go & goSearch))                /* not pondering */
	    {
	      /* A brief test showed that with the 0.5 factor, 
		 expected iteration did not overestimate 
		 the actual iteration 94% of the time.         */
	      pos->fastIteration = 
		(ui)(exp(log(*nPMicroS * tc->timeBudget) / bFactor) * 0.5);
	      /* try to save time when pos->iteration <= pos->fastIteration */
	      pos->fastIteration /= 2;
	      if (pos->searchDepth < maxPly &&   /* if set by "sd" command */
		  pos->searchDepth/2 < pos->fastIteration)
		pos->fastIteration = pos->searchDepth / 2;
	    }
	  else pos->fastIteration = 0;
	  if (pos->fastIteration > minPly)
	    printf("Fast iteration <= %d plies\n", pos->fastIteration);

	  /* Adjust extensions based on branching factor */
	  if (bFactor < 5.0) extFactor = 0;
	  else if (bFactor >= 9.0) extFactor = 5;
	  else extFactor = (int)bFactor - 4;
	  pos->pvExt = pvExt0 - extFactor*(pvExt0/5);
	  pos->checkExt = checkExt0 - extFactor*(checkExt0/5);
	  pos->onlyReplyExt = onlyReplyExt0 - extFactor*(onlyReplyExt0/5);
	  pos->mateThreatExt = mateThreatExt0 - extFactor*(mateThreatExt0/5);
	  pos->parrotExt = parrotExt0 - extFactor*(parrotExt0/5);
	  pos->pawn7thExt = pawn7thExt0 - extFactor*(pawn7thExt0/5);
	  ASSERT(pos->pvExt <= 25);
	  ASSERT(pos->checkExt <= 100);
	  ASSERT(pos->onlyReplyExt <= 75);
	  ASSERT(pos->mateThreatExt <= 75);
	  ASSERT(pos->parrotExt <= 75);
	  ASSERT(pos->pawn7thExt <= 75);
	  if (bFactor > averageBFactor) highBF = true;
	  else highBF = false;
	}

      /* instantenous branching factor to estimate how big the next ply is */
      bf = (double) (pos->sNodes + pos->qNodes) / 
	lastNodes;
      printf("branching factors = %.2f, %.2f\n", bFactor, bf);
      ASSERT(bf > 1.0);
      if (bf < 3.0 && movesLeft <= 7) 
	bf = 3.0;
      if (bf > 6.0) bf = 6.0;

      lastNodes = pos->sNodes + pos->qNodes;
      if (lastNodes == 0) lastNodes = 1;

      /* found a mate, enough already */
      if (score > almostScore) break;
    }

#ifdef TOURNAMENT
  /* kibitzing in xboard mode */
  if (
#ifdef _REENTRANT
      tc->timeBudget < ponderingBudget &&
#endif
      pos->iteration >= minPly &&
      pos->Flags & OneMask[0])
    {
      if (score > 0) printf("tellics kibitz +%d, ", score);
      else printf("tellics kibitz %d, ", score);
      printf("%d plies %.0f nodes in %ds \(%d Knps\)\n", pos->iteration, 
	     (double)(pos->sNodes + pos->qNodes),
	     (ui)((clock1-tc->clock0)/1000000),
	     (ui)(1000*(pos->sNodes + pos->qNodes)/(clock1-tc->clock0)));
      printf("tellics kibitz PV: ");
      for (i = 0; i < j; i++) printf("%s ", pvString[i]);
      printf("\n");
    }
#endif

  return score;
}


int main(int argc, char **argv)
{
  /* getCommand() is a thread */
#ifdef _REENTRANT
  pthread_t aThread;
  int res;
#endif

  ui i, j;
  ui nMoves, noOfNonCaptures, noOfCaptures;
  ui wToMove = true, ponderWToMove;
  int score = 0;
  FILE *fd;
  sHash *hash[2];
  struct TimeCheck tc;
  sPos pos, ponderPos;
  sOthers others;
  BB theMove[maxPly], allMoves[128], nonCapture[128], winningCapture[32];
  sPV pv[maxPly];
  sPlyData plyData[maxPly], ponderPlyData[maxPly];

  ui movesLeft;
  ui checkers, checkable, canCheck;
  int origCE;
  ui drawCount = 0, acceptDraw = false, resignCount = 0;
  int topScore, secondScore;
  int ok = false;
  ui dtm;
  ui gameEnd;
  int timebookExtra = 100;
  ui timebookSpan = 11;
  ui lastOpeningNo = 0;
  ui dangerFactory;       /* true -> increase search time when in danger */
  int wKScore, bKScore, kScore;
  char s[8], s1[8];
  BB Checkers, ponderMove, ponderReply;
  double timeReserved, timePondered;
  double dangerFactor;
  double nPMicroS = NPMS;
#ifdef WINDOWS
  double clock1;
#else
  struct timeval tv;
  struct timezone tz;
  BB clock1;
#endif

  ui count;
  ui wordNo;
  int lastMoveScore;
  char lastC, c;
  char line[LETTERS];
  FILE *fdin;
  /* use Nalimov's endgame tablebase */
  void  *EGTB_cache;
  char logDate[7];
  char fname[256];
  time_t when;
  struct tm *timeStr;
  char usedWNo[10000], usedBNo[10000];    /* as many as # of openingNo's */


  printf("parrot version %s\n", version);


  /* today's date */
  time(&when);
  timeStr = localtime(&when);
  strftime(logDate, 7, "%m%d%y", timeStr);
  sprintf (fname, "list.%s", logDate);
  fd = fopen(fname, "a+");

  /* set variables to zero */
  memset(&pos, 0, sizeof(pos));
  memset((void *) plyData, 0, maxPly*sizeof(plyData[0]));
  memset((void *) pos.repKey, 0, sizeof(pos.repKey));

  /* reading configuration file */
  if ((fdin = fopen("parrot.cfg", "r")) == NULL)
    {
      printf("Can't read parrot.cfg.  Quiting.");
      exit(0);
    }
  while (fgets(line, 80, fdin) != NULL)
    {
      if (line[0] != '#' && line[0] != '\n' && line[0] != '\r')
	{
	  wordNo = -1;
	  count = 0;
	  lastC = ' ';
	  /* Better not assume that word[][] is always zero */
	  for (i = 0; i < WORDS; i++)
	    word[i][0] = '\0';

	  /* trap EOF which gives segmentation fault if not trapped */
	  i = 0;
	  c = line[i++];
	  do
	    {
	      if (isspace((int) lastC) && isspace((int) c)) continue;
	      else if (isspace((int) lastC) && !(isspace((int) c)))
		{
		  wordNo++;
		  count = 0;
		  word[wordNo][count++] = c;
		  lastC = c;
		  continue;
		}
	      else if (!(isspace((int) lastC)) && (isspace((int) c)))
		{
		  word[wordNo][count] = '\0';
		  /* Limit WORDS tokens per customer */
		  if (wordNo == WORDS) break; 
		  lastC = ' ';
		  continue;
		}
	      else
		{
		  word[wordNo][count++] = c;
		  lastC = c;
		}
	    }
	  while ((c = line[i++]) != '\0');
	  
	  if (!processCommand(word[0], word[1], word[2], word[3], word[4], 
			      word[5], word[6], word[7], word[8], word[9], 
			      word[10], word[11], &wToMove, &pos,
			      plyData, hash, &others, acceptDraw, fd))
	    printf("%s in parrot.cfg: no such command\n", word[0]);
	}
    }
  fclose(fdin);

  if(!(hash[white] = (sHash *) malloc(sizeof(sHash) * (pos.hashSize+1))))
    {
      printf("Can't malloc hash");
      exit(0);
    }  

  if(!(hash[black] = (sHash *) malloc(sizeof(sHash) * (pos.hashSize+1))))
    {
      printf("Can't malloc hash");
      exit(0);
    }  

  /**************************************************************/
 

  /* use Nalimov's endgame tablebase */
  i = 2*sizeof(sHash)*(pos.hashSize+1)/1024/1024;
  printf("hash table size: %d MBytes.\n", i);
  EGTB_cache = (void *) 0;
  EGTB_cache=malloc((size_t) pos.cacheSize);
  if (EGTB_cache)
    {
      pos.EGTBlimit = (ui) IInitializeTb(pos.egtbDir);
      ok = FTbSetCacheSize(EGTB_cache, (int)pos.cacheSize);
    }
  if (ok) 
    {
      printf("EGTB cache memory: %uM bytes.\n", pos.cacheSize/1024/1024);
      printf("EGTB directory: %s\n", pos.egtbDir); 
    }
  else printf("ERROR in initializing EGTB\n");

  /* initialization */
  init(&wToMove, &pos, plyData, hash, &others);

  /* bitboard used in clearing score from a move */
  readBinaryBooks();
  /* read used white openingNo's from usedWOpenings file */
  for (i = 0; i < 10000; i++) usedWNo[i] = false;
#if !defined TOURNAMENT
  fdin = fopen("usedWOpenings", "r");
  while (fscanf(fdin, "%d\n", &i) != EOF) usedWNo[i] = 128;   /* 8th bit */
  fclose(fdin);
#endif
  /* read used black openingNo's from usedBOpenings file */
  for (i = 0; i < 10000; i++) usedBNo[i] = false;
#if !defined TOURNAMENT
  fdin = fopen("usedBOpenings", "r");
  while (fscanf(fdin, "%d\n", &i) != EOF) usedBNo[i] = 128;   /* 8th bit */
  fclose(fdin);
#endif


#ifdef BONUSES_PENALTIES
  PBonus = NBonus = BBonus = RBonus = QBonus = KBonus = -mateScore;
  PPenalty = NPenalty = BPenalty = RPenalty = QPenalty = KPenalty = mateScore;
  PEndBonus = NEndBonus = BEndBonus = REndBonus = QEndBonus = KEndBonus = -mateScore;
  PEndPenalty = NEndPenalty = BEndPenalty = REndPenalty = QEndPenalty = KEndPenalty = mateScore;
#endif

  display(&pos);
  pos.rootMaxPieceScore = nonEndgame;


  /* call the thread now */
  go = goRest;
#ifdef _REENTRANT
  res = pthread_mutex_init(&goMutex, NULL);
  if (res != 0)
    {
      perror("Mutex initialization failed");
      exit(EXIT_FAILURE);
    }
  res = pthread_create(&aThread, NULL, getCommand, NULL);
  if (res != 0)
    {
      perror("Thread creation failed");
      exit(EXIT_FAILURE);
    }
#endif

  /* Initialize theMove[] and ponderMove before the game */
  for (i = 0; i < maxPly; i++) theMove[i] = (BB) 0;
  ponderMove = ponderReply = (BB) 0;
  while (true)
    {
      ASSERT(go > 0 && go <= goPonder);

      /* process the command */
#ifdef _REENTRANT
      usleep(1);
      if (go & goProcess)
#else
      go = goProcess;
      while ((go & goProcess) || (go & goRest))
#endif
	{
	  printf("I'm resting\n");
#if !defined _REENTRANT
	  word[0][0] = word[1][0] = word[2][0] = word[3][0] =
	    word[4][0] = word[5][0] = word[6][0] = word[7][0] =
	    word[8][0] = word[9][0] = word[10][0] = word[11][0] = '\0';
	  getCommand();
#endif

	  if (go & goProcess)
	    {
	      printf("I'm processing\n");
	      ok = processCommand(word[0], word[1], word[2], word[3], word[4], 
				  word[5], word[6], word[7], word[8], word[9], 
				  word[10], word[11], &wToMove, &pos, plyData,
				  hash, &others, acceptDraw, fd);
#ifdef _REENTRANT
	      pthread_mutex_lock(&goMutex);
#endif
	      go = ok;
#ifdef _REENTRANT
		pthread_mutex_unlock(&goMutex);
#endif
	    }
	}

      /* searching */
      if (go & goSearch)    
	{
	  score = 0;
	  origCE = 0;
	  
	  if (pos.Flags & OneMask[5]) break;
	  
	  if (!(pos.Flags & OneMask[1])) /* not forced */ 
	    {
	      printf("I'm searching\n");

	      gameEnd = false;
	      
	      /* time control */
#ifdef WINDOWS
	      tc.clock0 = 1000000.0/CLOCKS_PER_SEC*clock();
#else
	      gettimeofday(&tv, &tz);
	      tc.clock0 = (BB) tv.tv_sec*1000000 + (BB) tv.tv_usec;
#endif
	      /* try to budget time close to opponent */
	      if (pos.movesPerTC == 0) movesLeft = 15;
	      else
		{
		  if ((pos.moveNumber+1) % pos.movesPerTC == 0)
		    movesLeft = 1;
		  else movesLeft = (pos.movesPerTC+1) - 
			 ((pos.moveNumber+1) % pos.movesPerTC);
		}
#ifdef NUNN
	      pos.Flags &= NilMask[3];    /* don't randomize */
	      pos.Flags |= OneMask[6];    /* don't use opening book */
	      pos.rOpening = 0;           /* don't evaluate base on a 
						 random opening */
	      timebookExtra = 0;
#endif
	      dangerFactory = false;
	      if (pos.Flags & OneMask[10]) /* timeReserved mode */
		{
		  /* the normal timeLeft/movesLeft plus ......       */
		  tc.timeBudget = 10000*pos.timeLeft/movesLeft/2;
		  /* if there is any surplus time, use half of it */
		  if (pos.timeLeft > pos.oTimeLeft)
		    tc.timeBudget += 10000*(pos.timeLeft - pos.oTimeLeft)/2;
		  else
		    tc.timeBudget += 
		      10000*(pos.timeLeft - pos.oTimeLeft)/movesLeft;
		  timeReserved = 
		    ((10000*pos.timeLeft/movesLeft) - tc.timeBudget)/10;
		  if (timeReserved < 0) timeReserved = 0;
		  else dangerFactory = true;
		}
	      else
		{
		  tc.timeBudget = (10000*pos.timeLeft/movesLeft);
		  timeReserved = 0;
		  timebookExtra = 0;
		}
	      if (tc.timeBudget <= 0) 
		tc.timeBudget = 10000*pos.timeLeft/movesLeft;
	      /* crunch time: <= 1 minute left AND <= 7 moves left */
	      if (pos.timeLeft <= 6000 && movesLeft <= 7)
		{
		  pos.Flags |= OneMask[7];
		  /* try not to be behind in time */
		  if (pos.timeLeft <= pos.oTimeLeft)
		    tc.timeBudget = 100000;        /* 0.1 seconds */
		  else if (pos.movesPerTC > 1) tc.timeBudget = 
				   5000*(pos.timeLeft - pos.oTimeLeft);
		  else
		    {
		      tc.timeBudget = 
                                  10000*(pos.timeLeft - pos.oTimeLeft);
		      /* if possible, substract 1 second for safety */
		      if (tc.timeBudget > 1000000) tc.timeBudget -= 1000000;
		    }
		  timeReserved = 0;
		}

#ifdef _REENTRANT
	      /* If opponent's move was pondered, reduced timeBudget by time
	         used in pondering, with minimum timeBudget of 1 second.     */
	      if (pos.Flags & OneMask[11])
		{
		  algebraic(ponderMove, s);
		  if (strcmp(s, word[0]) == 0) 
		    {
		      printf("pondered move: %s\n", s);

		      if (timePondered > tc.timeBudget/2 &&
			  ponderPos.iteration > minPly &&
			  theMove[ponderPos.iteration] == 
			  theMove[ponderPos.iteration-1] &&
			  theMove[ponderPos.iteration] == 
			  theMove[ponderPos.iteration-2])
			{
			  printf("time saved by pondering = %.0f\n", 
				 timePondered/10000);
			  tc.timeBudget -= timePondered;
			  if (tc.timeBudget <= 0) tc.timeBudget = 1000000;
			  /* copy killer moves from pondering */
			  for (j = 0; j < maxPly-1; j++)
			    {
			      pos.killerFrom[j] = ponderPos.killerFrom[j];
			      pos.killerTo[j] = ponderPos.killerTo[j];
			    }
			}
		    }
		  else
		    {
		      /* Not a pondered move, reset PV */
		      pv[0].move[0] = (BB) 0;
		      pv[0].length = 0;
		    }
		}
#else
	      if (pv[0].length > 1)
		{
		  algebraic(pv[0].move[1], s);
		  if (strcmp(s, word[0]) == 0)
		    {
		      for (i = 0; i < pv[0].length-2; i++)
			pv[0].move[i] = pv[0].move[i+2];
		      pv[0].length -= 2;
		    }
		  else
		    {
		      pv[0].move[0] = (BB) 0;
		      pv[0].length = 0;
		    }
		}
	      else
		{
		  pv[0].move[0] = (BB) 0;
		  pv[0].length = 0;
		}
#endif

	      printf("Time budget = %.0f with %.0f reserved\n", 
		     tc.timeBudget/10000, timeReserved/10000);

	      /* shoot to check 100 times per move */
	      if (nPMicroS > 4*NPMS) nPMicroS = 4*NPMS;  /* avoid extremes */
	      tc.nodeCheckPoint = (BB)(nPMicroS * tc.timeBudget / 10);
	      
	      /* Is this position a draw by rule? */
	      switch(isDrawByRule(0, &pos, &plyData[0]))
		{
		case 0:
		  break;
		case 3:
		  if (pos.Flags & OneMask[0])
		    printf("1/2-1/2 {repetition}\n");
		  else
		    {
		      closing(fd, &pos, "1/2-1/2", "repetition");
		      exiting(fd, hash);
		    }
		  break;
		case 13:
		  if (pos.Flags & OneMask[0])
		    printf("1/2-1/2 {not enough material}\n");
		  else
		    {
		      closing(fd, &pos, "1/2-1/2", "not enough material");
		      exiting(fd, hash);
		    }
		  break;
		case 50:
		  if (pos.Flags & OneMask[0]) 
		    printf("1/2-1/2 {50 move rule}\n");
		  else
		    {
		      closing(fd, &pos, "1/2-1/2", "50 move rule");
		      exiting(fd, hash);
		    }
		  break;
		default:
		  break;
		}
	      
	      /* start the search */
	      pos.iteration = 0;                 /* this is really needed */
	      pos.fastIteration = maxPly;        /* anything non zero */
	      /* Try opening book.  Three strikes and you're out. */
	      theMove[0] = 0;
	      if (pos.failBook < 3 && !(pos.Flags & OneMask[6]))
		theMove[0] = openingBook(wToMove, usedWNo, usedBNo,
					 &plyData[0], &pos);
	      
	      if (theMove[0])
		{
		  lastOpeningNo = pos.openingNo;
		  if (pos.Flags & OneMask[0])
		    printf("tellics kibitz book move\n");
		  ponderMove = (BB) 0;
		}
	      else
		{
		  pos.failBook++;
		  pos.openingNo = lastOpeningNo;

		  /* timebook idea from crafty */
		  if (pos.Flags & OneMask[10] &&    /* time reserved mode */
		      !(pos.Flags & OneMask[7]) &&   /* search is still hot */
		      pos.failBook < timebookSpan)
		    {
		      tc.timeBudget += (tc.timeBudget*(timebookExtra/100.0) *
					(timebookSpan-pos.failBook)/10.0);
		      printf("After adding extra time out of book: %.0f\n", 
			     tc.timeBudget/10000);
		    }
		  
		  /* Move generation */
		  Checkers = (BB) 0;
		  if (wToMove)
		    checkers = isInCheckByB(pos.wKSq, &Checkers, &pos, &others);
		  else 
		    checkers = isInCheckByW(pos.bKSq, &Checkers, &pos, &others);
		  /* check extension */
		  if (checkers) origCE += pos.checkExt;

		  /* Is this a dangerous position? */
		  if (checkers) dangerFactor = 2.0;
		  else
		    {
		      wKScore = wKPenalty(1, &pos, plyData, &others);
		      bKScore = bKPenalty(1, &pos, plyData, &others);
		      if (-wKScore > bKScore) kScore = -wKScore;
		      else kScore = bKScore;
		      if (kScore >= 2*pValue) dangerFactor = 2.0;
		      else if (kScore >= pValue) 
			dangerFactor = (double)kScore/pValue;
		      else dangerFactor = 1.0;
		    }
		  ASSERT(dangerFactor >= 1.0 && dangerFactor <= 2.0);

		  /* Yes, this is a dangerous position. */
		  if (dangerFactor > 1.0)
		    {
		      /* zero is dangerous situation */
		      pos.fastIteration = 0;

		      /* increase the search time */
		      if (dangerFactory)
			{
			  tc.timeBudget = tc.timeBudget*dangerFactor;
			  timeReserved = timeReserved*dangerFactor;
			  printf("Danger factor = %.2f.\n", dangerFactor); 
			  printf("Time budget = %.0f with %.0f reserved\n", 
				 tc.timeBudget/10000, timeReserved/10000);
			}
		    }

		  nMoves = 0;
		  if (checkers)
		    {
		      if (wToMove) 
			nMoves = outOfBCheck(0, pos.wKSq, Checkers, allMoves, 
					     &pos, plyData, &others);
		      else
			nMoves = outOfWCheck(0, pos.bKSq, Checkers, allMoves, 
					     &pos, plyData, &others);
		      /* only one reply out of check extension */
		      if (nMoves == 1) origCE += pos.onlyReplyExt;
#ifdef VERIFY
		      noOfCaptures = captures(wToMove, 0, winningCapture, 
					      &pos, plyData, &others, 0);
		      noOfNonCaptures = nonCaptures(wToMove, 0, nonCapture, 
						    &pos, plyData, &others, 0);
		      if (noOfCaptures+noOfNonCaptures != nMoves)
			{
			  display(&pos);
			  for (j = 0; j < noOfCaptures; j++) 
			printf("ERROR in outOf?Check() in main() C%d. %d-%d\n",
			       j+1, from(winningCapture[j]), 
			       to(winningCapture[j]));
			  for (j = 0; j < noOfNonCaptures; j++) 
			printf("ERROR in outOf?Check() in main() N%d. %d-%d\n",
			       j+1, from(nonCapture[j]), to(nonCapture[j]));
			  for (j = 0; j < nMoves; j++) 
			printf("ERROR in outOf?Check() in main() A%d. %d-%d\n",
			       j+1, from(allMoves[j]), to(allMoves[j]));
			  exit(EXIT_FAILURE);
			}
#endif
		    }
		  else
		    {
		      /* try captures */
		      noOfCaptures = captures(wToMove, 0, winningCapture, 
					      &pos, plyData, &others, 0);
		      /* try non-captures */
		      noOfNonCaptures = nonCaptures(wToMove, 0, nonCapture, 
						    &pos, plyData, &others, 0);
		      
		      /* Copying all moves to allMoves[]. */
		      if (noOfCaptures > 0)
			for (i = 0; i < noOfCaptures; i++)
			  allMoves[nMoves++] = winningCapture[i];
		      if (noOfNonCaptures > 0)
			for (i = 0; i < noOfNonCaptures; i++)
			  allMoves[nMoves++] = nonCapture[i];
		    }
		  
		  if (nMoves == 1)
		    {
		      theMove[0] = allMoves[0];
		      ponderMove = (BB) 0;
		    }
		  else if (nMoves > 0)
		    {
		      /* search loop */
		      pos.sNodes = 1;
		      pos.qNodes = 0;
		      
		      /* Spent some time to use full window qsearch and find 
			 the top two scores to determine if theMove is an easy 
			 move. */
		      if (wToMove)
			{ 
			  checkable = 
			    isSqAttackableByB(pos.wKSq, &pos, &others);
			  canCheck = isSqAttackableByW(pos.bKSq, &pos, &others);
			}
		      else 
			{
			  checkable = 
			    isSqAttackableByW(pos.bKSq, &pos, &others);
			  canCheck = isSqAttackableByB(pos.wKSq, &pos, &others);
			}
		      if (!checkable && !canCheck)
			{
			  topScore = secondScore = -mateScore;
			  
			  for (i = 0; i < nMoves; i++)
			    {
			      makeMove(wToMove, 1, allMoves[i], &pos, plyData,
				       &others);
			      score = -qsearch(wToMove^1, 1, 0, -mateScore, 
					       mateScore, &pos, plyData, pv,
					       &others, &tc, 0, PVNode);
			      ASSERT(score >= -mateScore && 
				     score <= mateScore);
			      if (score > topScore)
				{
				  secondScore = topScore;
				  topScore = score;
				}
			      else if (score > secondScore) 
				secondScore = score;
			      unmakeMove(wToMove, 1, allMoves[i], &pos, 
					 plyData);
			    }
			  
			  if (topScore > secondScore + easyScore && 
			      tc.timeBudget > 1000000)
			    {
			 printf("Easy move, tc.timeBudget is reduced from %u ",
				(ui)(tc.timeBudget/1000000));
			 tc.timeBudget /= easyFactor;
			 printf("to %u seconds.\n", 
				(ui)(tc.timeBudget/1000000));
			    }
			}

		      score = iterate(wToMove, &pos, plyData, pv, hash, 
				      &others, &tc, allMoves, nMoves, origCE,
				      &nPMicroS, timeReserved, movesLeft, 
				      theMove, &ponderMove, &ponderReply);
		    }
		  else
		    {
		      if (pos.iAm == white)
			{
			  if (getBAttacker(pos.wKSq, &pos, &plyData[0], &others)
			      & 2)
			    {
			      if (pos.Flags & OneMask[0])
				printf("0-1 {Black mates}\n");
			      else
				{
				  closing(fd, &pos, "0-1", "Black mates");
				  exiting(fd, hash);
				}
			    }
			  else
			    {
			      if (pos.Flags & OneMask[0])
				printf("1/2-1/2 {stalemate}\n");
			      else
				{
				  closing(fd, &pos, 
					  "1/2-1/2", "stalemate");
				  exiting(fd, hash);
				}
			    }
			}
		      else
			{
			  if (getWAttacker(pos.bKSq, &pos, &plyData[0], &others)
			      & 2)
			    {
			      if (pos.Flags & OneMask[0])
				printf("1-0 {White mates}\n");
			      else
				{
				  closing(fd, &pos, "1-0", "White mates");
				  exiting(fd, hash);
				}
			    }
			  else
			    {
			      if (pos.Flags & OneMask[0])
				printf("1/2-1/2 {stalemate}\n");
			      else
				{
				  closing(fd, &pos, 
					  "1/2-1/2", "stalemate");
				  exiting(fd, hash);
				}
			    }
			}
		      
		      gameEnd = true;
		    }	      
		}
	      
	      if (!gameEnd)
		{
		  /* write to the game listing */
		  san(wToMove, 0, theMove[pos.iteration], s, &pos, plyData, 
		      &others);
		  if (wToMove) 
		    {
		      if (pos.moveNumber == 0)
			{
			  /* listing headers */
			  fprintf(fd, "[Event \"%s\"]\n", pos.fullDate);
			  if (pos.iAm == white) 
			    fprintf(fd, "[White \"parrot\"] ");
			  else fprintf(fd, "[Black \"parrot\"] ");
			  fprintf(fd, "        score");
			  fprintf(fd, " opening");
			  fprintf(fd, " depth");
			  fprintf(fd, "\n");
			}
		      fprintf(fd, "white");
		    }
		  else fprintf(fd, "black");
		  fprintf(fd, "%4u.", (pos.moveNumber+1));
		  fprintf(fd, " %8s", s);
		  if (pos.iteration > 0)
		    {
		      if (score < lastMoveScore - 40)
			{
			  /* put '?' in list.mmddyy file for learning */
			  if (pos.iAm == white) fprintf(fd, "\t%5d? w", score);
			  else fprintf(fd, "\t%5d? b", score);
			}
		      else
			{
			  if (pos.iAm == white) fprintf(fd, "\t%6d w", score);
			  else fprintf(fd, "\t%6d b", score);
			}
		      lastMoveScore = score;
		      fprintf(fd, " %5u", pos.openingNo);
		      fprintf(fd, " %5u", pos.iteration);
		      if (strlen(pos.solution) != 0) 
			fprintf(fd, "   %s", pos.solution);
		      else fprintf(fd, "   0x%016llX", plyData[0].key);
		    } 
		  fprintf(fd, "\n");
		  fflush(fd);
		  
		  /* it's time to make the move now */
		  algebraic(theMove[pos.iteration], s);
		  if (go & goSearch) printf("\nmove %s\n", s);
		  
#ifdef BONUSES_PENALTIES
		  printf("PBonus = %d\n", PBonus);
		  printf("NBonus = %d\n", NBonus);
		  printf("BBonus = %d\n", BBonus);
		  printf("RBonus = %d\n", RBonus);
		  printf("QBonus = %d\n", QBonus);
		  printf("KBonus = %d\n", KBonus);
		  
		  printf("PPenalty = %d\n", PPenalty);
		  printf("NPenalty = %d\n", NPenalty);
		  printf("BPenalty = %d\n", BPenalty);
		  printf("RPenalty = %d\n", RPenalty);
		  printf("QPenalty = %d\n", QPenalty);
		  printf("KPenalty = %d\n", KPenalty);
		  
		  printf("PEndBonus = %d\n", PEndBonus);
		  printf("NEndBonus = %d\n", NEndBonus);
		  printf("BEndBonus = %d\n", BEndBonus);
		  printf("REndBonus = %d\n", REndBonus);
		  printf("QEndBonus = %d\n", QEndBonus);
		  printf("KEndBonus = %d\n", KEndBonus);

		  printf("PEndPenalty = %d\n", PEndPenalty);
		  printf("NEndPenalty = %d\n", NEndPenalty);
		  printf("BEndPenalty = %d\n", BEndPenalty);
		  printf("REndPenalty = %d\n", REndPenalty);
		  printf("QEndPenalty = %d\n", QEndPenalty);
		  printf("KEndPenalty = %d\n", KEndPenalty);
#endif
		  
		  sprintf(pos.gameID, "%s_%04u", pos.fullDate, 
			  pos.openingNo);
	printf("gameID = %s, search depth = %u, %.0f/ %.0f nodes, score: %d\n",
		pos.gameID, pos.iteration, 
	       (double) pos.qNodes, (double) (pos.sNodes + pos.qNodes), score);
	 if (strlen(pos.solution)) sprintf(pos.myMove, "%s", s);
	 
	 /* it's time to make the move now */
	 makeMove(wToMove, 1, theMove[pos.iteration], &pos, plyData, &others);
	 if (!(pos.Flags & OneMask[0])) display(&pos);
#ifdef WINDOWS
	 clock1 = 1000000.0/CLOCKS_PER_SEC*clock();
#else
	 gettimeofday(&tv, &tz);
	 clock1 = (BB) tv.tv_sec*1000000 + (BB) tv.tv_usec;
#endif
	 pos.timeLeft -= (clock1 - tc.clock0)/10000;
	 pos.timeLeft += pos.timeInc;
	 pos.oTimeLeft = pos.timeLeft - tc.timeBudget/10000;
	 
	 if (pos.moveNumber < 30)
	   {
	     drawCount = 0;
	     if (score <= -resignValue) acceptDraw = true;
	     else acceptDraw = false;
	     resignCount = 0;
	   }
	 else
	   {
	     if (abs(score) <= drawMargin)
	       {
		 drawCount++;
		 /* keep the status quo of acceptDraw */
		 resignCount = 0;
	       }
	     else if (score <= -resignValue)
	       {
		 drawCount = 0;
		 acceptDraw = true; /* Opponent offers draw?  Why not? */
		 resignCount++;
	       }
	     else
	       {
		 drawCount = 0;
		 acceptDraw = false;
		 resignCount = 0;
	       }
	   }
	 
	 /* After making the move, is this position a draw by rule? */
	 switch(draw(theMove[pos.iteration]))
	   {
	   case flagDraw3:
	     if (pos.Flags & OneMask[0]) 
	       if (go & goSearch) printf("1/2-1/2 {repetition}\n");
	     else
	       {
		 closing(fd, &pos, "1/2-1/2", "repetition");
		 exiting(fd, hash);
	       }
	     break;
	   case flagDraw50:
	     if (pos.Flags & OneMask[0]) 
	       if (go & goSearch) printf("1/2-1/2 {50 move rule}\n");
	     else
	       {
		 closing(fd, &pos, "1/2-1/2", "50 move rule");
		 exiting(fd, hash);
	       }
	     break;
	   default:
	     break;
	   }
	 
	 /* After making the move, can I checkmate? */
	 if (score == mateScore) 
	   {
	     if (pos.iAm == white)
	       {
		 if (pos.Flags & OneMask[0]) 
		   if (go & goSearch) printf("1-0 {White mates}\n");
		 else
		   {
		     closing(fd, &pos, "1-0", "White mates");
		     exiting(fd, hash);
		   }
	       }
	     else
	       {
		 if (pos.Flags & OneMask[0]) 
		   if (go & goSearch) printf("0-1 {Black mates}\n");
		 else
		   {
		     closing(fd, &pos, "0-1", "Black mates");
		     exiting(fd, hash);
		   }
	       }
	   }
	 
	 /* After making the move, should we offer a draw? */
	 if (abs(score) <= drawMargin &&
	     pos.moveNumber >= 30 &&             /* not a short game */
	     movesLeft > 4 &&         /* opponent doesn't need to rush */
	     !(pos.WQBoard) && !(pos.BQBoard) && /* no more queen on board */
	     pos.oTimeLeft/movesLeft >= 100)
	   {
	     drawCount++;
	     if (drawCount >= drawCountLimit && 
		 plyData[0].count50Moves >= 2*drawOfferRepeat)
	       {
		 if (pos.Flags & OneMask[0]) 
#ifdef FICS
		   if (go & goSearch) printf("tellics draw\n");
#else
		 if (go & goSearch) 
		   {
#if !defined TOURNAMENT
		     printf("1/2-1/2 {offer draw}\n");
#endif
		   }
#endif
		 else
		   {
		     closing(fd, &pos, "1/2-1/2", "offer draw");
		     exiting(fd, hash);
		   }
		 /* if draw offer is refused, try to be not obnoxious */
		 drawCount -= drawOfferRepeat;
		 /* it is okay to accept draw if opponent offers it */
		 acceptDraw = true;
	       }
	   }
	 
	 /* After making the move, should I resign? */
	 /* we're in a bad shape */
	 if (score <= -resignValue &&
	     pos.moveNumber >= 30 &&           /* not a short game */
	     movesLeft > 4 &&         /* opponent doesn't need to rush */
	     pos.oTimeLeft/movesLeft >= 100)
	   {
	     resignCount++;
	     if (resignCount >= resignCountLimit)
	       {
		 if (pos.iAm == white)
		   {
		     if (pos.Flags & OneMask[0]) 
		       if (go & goSearch) printf("0-1 {resign}\n");
		     else
		       {
			 closing(fd, &pos, "0-1", "resign");
			 exiting(fd, hash);
		       }
		   }
		 else
		   {
		     if (pos.Flags & OneMask[0]) 
		       if (go & goSearch) printf("1-0 {resign}\n");
		     else
		       {
			 closing(fd, &pos, "1-0", "resign");
			 exiting(fd, hash);
		       }
		   }
	       }
	   }
	 
	 if (score > almostScore)
	   {
	     dtm = (ui) (mateScore-score)/2;
	     if (dtm > 0) 
	       if (go & goSearch) 
                 printf("Announcing checkmate in %u moves\n", dtm);
	   }
	 
	 /* Reset */
#ifdef UNDO_REMOVE
	 if (wToMove)
	   {
	     gameListing[pos.moveNumber].wPlyData = plyData[0];
	     gameListing[pos.moveNumber].wFailBook = pos.failBook;
	   }
	 else
	   {
	     gameListing[pos.moveNumber].bPlyData = plyData[0];
	     gameListing[pos.moveNumber].bFailBook = pos.failBook;
	   }
#endif
	 plyData[0] = plyData[1];
	 /* shift the killer moves */
	 for (j = 0; j < maxPly-1; j++)
	   {
	     pos.killerFrom[j] = pos.killerFrom[j+1];
	     pos.killerTo[j] = pos.killerTo[j+1];
	   }
	 wToMove ^= 1;
	 if (wToMove) pos.moveNumber++;
	 ASSERT(pos.moveNumber < maxMove);
	 if (movesLeft == 1 && pos.movesPerTC > 0) 
	   {
	     pos.timeLeft += pos.timeLeft0;
	     pos.oTimeLeft += pos.timeLeft0;
	   }
	 if (plyData[0].count50Moves == 0) pos.repHead = 0;
	 else pos.repHead++;
		}
	    }
	  
	  else /* forced */
	    {
	      pos.iAm ^= 1;
	    }
#ifdef _REENTRANT
	  pthread_mutex_lock(&goMutex);
#endif
	  if (ponderMove != 0 &&
	      abs(score) < almostScore) 
	    go = goPonder;
	  else 
	    {
	      ponderMove = (BB) 0;
	      go = goRest;
	    }
#ifdef _REENTRANT
	  pthread_mutex_unlock(&goMutex);
#endif
	}
      
#ifdef _REENTRANT
      /* pondering */
      if (go & goPonder)
	{
	  ASSERT(ponderMove != 0);
	  ASSERT(abs(score) < almostScore);
	  ASSERT(others.BPCaptures[62] != 0xFFFFFFFFFFFFFFFF);

	  score = 0;

	  /* ponder off */
	  if (!(pos.Flags & OneMask[11])) 
	    {
	      pthread_mutex_lock(&goMutex);
	      ponderMove = (BB) 0;
	      go = goRest;
	      pthread_mutex_unlock(&goMutex);
	    }
	  /* ponder on */
	  else
	    {
	      usleep(100000);
	      printf("I'm pondering\n");
	      
	      ponderPos = pos;
	      memcpy(ponderPlyData, plyData, maxPly*sizeof(sPlyData));
	      /* The first move to be pondered. */
	      if (pv[0].length > 1)
		{
		  for (i = 0; i < pv[0].length-2; i++)
		    pv[0].move[i] = pv[0].move[i+2];
		  pv[0].length -= 2;
		}
	      else
		{
		  pv[0].move[0] = (BB) 0;
		  pv[0].length = 0;
		}
	      ponderWToMove = wToMove;

	      tc.timeBudget = ponderingBudget; /* to signify pondering */
	      origCE = 0;

#ifdef WINDOWS
	      tc.clock0 = 1000000.0/CLOCKS_PER_SEC*clock();
#else
	      gettimeofday(&tv, &tz);
	      tc.clock0 = (BB) tv.tv_sec*1000000 + (BB) tv.tv_usec;
#endif
	  
	      /* start the search */
	      ponderPos.iteration = 0;        /* this is really needed */

	      /* Move generation */
	      Checkers = (BB) 0;
	      if (ponderWToMove)
		checkers = isInCheckByB(ponderPos.wKSq, &Checkers, 
					&ponderPos, &others);
	      else 
		checkers = isInCheckByW(ponderPos.bKSq, &Checkers, 
					&ponderPos, &others);
	      
	      nMoves = 0;
	      if (checkers)
		{
		  if (ponderWToMove) 
		    nMoves = outOfBCheck(0, ponderPos.wKSq, Checkers, allMoves,
					 &ponderPos, ponderPlyData, 
					 &others);
		  else
		    nMoves = outOfWCheck(0, ponderPos.bKSq, Checkers, allMoves,
					 &ponderPos, ponderPlyData, 
					 &others);
#ifdef VERIFY
		  noOfCaptures = captures(ponderWToMove, 0, winningCapture, 
					  &ponderPos, ponderPlyData, &others, 
					  0);
		  noOfNonCaptures = nonCaptures(ponderWToMove, 0, nonCapture, 
						&ponderPos, ponderPlyData, 
						&others, 0);
		  if (noOfCaptures+noOfNonCaptures != nMoves)
		    {
		      display(&ponderPos);
		      for (j = 0; j < noOfCaptures; j++)
			printf("ERROR in outOf?Check() in main() C%d. %d-%d\n",
			       j+1, from(winningCapture[j]), 
			       to(winningCapture[j]));
		      for (j = 0; j < noOfNonCaptures; j++) 
			printf("ERROR in outOf?Check() in main() N%d. %d-%d\n",
			       j+1, from(nonCapture[j]), to(nonCapture[j]));
		      for (j = 0; j < nMoves; j++) 
			printf("ERROR in outOf?Check() in main() A%d. %d-%d\n",
			       j+1, from(allMoves[j]), to(allMoves[j]));
		      exit(EXIT_FAILURE);
		    }
#endif
		}
	      else
		{
		  /* try captures */
		  noOfCaptures = captures(ponderWToMove, 0, winningCapture, 
					  &ponderPos, ponderPlyData, &others, 
					  0);
		  /* try non-captures */
		  noOfNonCaptures = nonCaptures(ponderWToMove, 0, nonCapture, 
						&ponderPos, ponderPlyData, 
						&others, 0);
		  
		  /* Copying all moves to allMoves[]. */
		  if (noOfCaptures > 0)
		    for (i = 0; i < noOfCaptures; i++)
		      allMoves[nMoves++] = winningCapture[i];
		  if (noOfNonCaptures > 0)
		    for (i = 0; i < noOfNonCaptures; i++)
		      allMoves[nMoves++] = nonCapture[i];
		}
	      
	      if (nMoves > 0)
		{
		  /* check if ponderMove is legal */
		  algebraic(ponderMove, s);
		  for (i = 0; i < nMoves; i++)
		    {
		      algebraic(allMoves[i], s1);
		      if (strcmp(s, s1) == 0) break;
		    }

		  if (i < nMoves)
		    {
		      san(ponderWToMove, 0, allMoves[i], s, &ponderPos, 
			  ponderPlyData, &others);
		      if (ponderWToMove)
			printf("Pondering %u. %s\n", 
			       ponderPos.moveNumber+1, s);
		      else
			printf("Pondering %u. ... %s\n", 
			       ponderPos.moveNumber+1, s);
		      makeMove(ponderWToMove, 1, allMoves[i], &ponderPos,
			       ponderPlyData, &others);
		      ponderPlyData[0] = ponderPlyData[1];
		      /* shift the killer moves */
		      /* This is like doing it in command.c after opponent 
			 moves */
		      for (j = 0; j < maxPly-1; j++)
			{
			  ponderPos.killerFrom[j] = ponderPos.killerFrom[j+1];
			  ponderPos.killerTo[j] = ponderPos.killerTo[j+1];
			}
		      ponderWToMove ^= 1;
		      if (ponderWToMove) ponderPos.moveNumber++;
		      ASSERT(ponderPos.moveNumber < maxMove);

		      /* Move generation */
		      Checkers = (BB) 0;
		      if (ponderWToMove)
			checkers = isInCheckByB(ponderPos.wKSq, &Checkers, 
						&ponderPos, &others);
		      else 
			checkers = isInCheckByW(ponderPos.bKSq, &Checkers, 
						&ponderPos, &others);
		      if (checkers) origCE += pos.checkExt;
	      
		      nMoves = 0;
		      if (checkers)
			{
			  if (ponderWToMove) 
			    nMoves = outOfBCheck(0, ponderPos.wKSq, Checkers, 
						 allMoves, &ponderPos, 
						 ponderPlyData, &others);
			  else
			    nMoves = outOfWCheck(0, ponderPos.bKSq, Checkers, 
						 allMoves, &ponderPos, 
						 ponderPlyData, &others);
			  /* only one reply out of check extension */
			  if (nMoves == 1) origCE += pos.onlyReplyExt;
#ifdef VERIFY
			  noOfCaptures = captures(ponderWToMove, 0, 
						  winningCapture, 
						  &ponderPos, ponderPlyData, 
						  &others, 0);
			  noOfNonCaptures = nonCaptures(ponderWToMove, 0, nonCapture,
							&ponderPos, 
							ponderPlyData, 
							&others, 0);
			  if (noOfCaptures+noOfNonCaptures != nMoves)
			    {
			      display(&ponderPos);
			      for (j = 0; j < noOfCaptures; j++)
				printf("ERROR in outOf?Check() in main() C%d. %d-%d\n",
				       j+1, from(winningCapture[j]), 
				       to(winningCapture[j]));
			      for (j = 0; j < noOfNonCaptures; j++) 
				printf("ERROR in outOf?Check() in main() N%d. %d-%d\n",
				       j+1, from(nonCapture[j]), 
				       to(nonCapture[j]));
			      for (j = 0; j < nMoves; j++) 
				printf("ERROR in outOf?Check() in main() A%d. %d-%d\n",
				       j+1, from(allMoves[j]), 
				       to(allMoves[j]));
			      exit(EXIT_FAILURE);
			    }
#endif
			}
		      else
			{
			  /* try captures */
			  noOfCaptures = captures(ponderWToMove, 0, 
						  winningCapture, 
						  &ponderPos, ponderPlyData, 
						  &others, 0);
			  /* try non-captures */
			  noOfNonCaptures = nonCaptures(ponderWToMove, 0,
							nonCapture, 
							&ponderPos, 
							ponderPlyData, 
							&others, 0);
		  
			  /* Copying all moves to allMoves[]. */
			  if (noOfCaptures > 0)
			    for (i = 0; i < noOfCaptures; i++)
			      allMoves[nMoves++] = winningCapture[i];
			  if (noOfNonCaptures > 0)
			    for (i = 0; i < noOfNonCaptures; i++)
			      allMoves[nMoves++] = nonCapture[i];
			}
	      
		      if (nMoves > 0)
			{
			  /* ponder loop */
			  ponderPos.sNodes = 1;
			  ponderPos.qNodes = 0;
		  
			  /* Spent some time to use full window qsearch and 
			     find the top two scores to determine if theMove 
			     is an easy move. */
			  if (ponderWToMove)
			    { 
			      checkable = 
				isSqAttackableByB(ponderPos.wKSq, &ponderPos,
						  &others);
			      canCheck = isSqAttackableByW(ponderPos.bKSq, 
							   &ponderPos, 
							   &others);
			    }
			  else 
			    {
			      checkable = 
				isSqAttackableByW(ponderPos.bKSq, &ponderPos, 
						  &others);
			      canCheck = isSqAttackableByB(ponderPos.wKSq, 
							   &ponderPos, 
							   &others);
			    }

			  score = iterate(ponderWToMove, &ponderPos, 
					  ponderPlyData, pv, hash,
					  &others, &tc, allMoves, nMoves, 
					  origCE, &nPMicroS, 0, 9999, theMove,
					  &ponderMove, &ponderReply);
			  pos.iteration = ponderPos.iteration;

			  /* too deep now */
			  if (ponderPos.iteration >= maxPly-1)
			    {
			      pthread_mutex_lock(&goMutex);
			      ponderMove = (BB) 0;
			      go = goRest;
			      pthread_mutex_unlock(&goMutex);
			    }
			}
		    }
		}
	      else   /* nMoves == 0 */
		{
		  pthread_mutex_lock(&goMutex);
		  ponderMove = (BB) 0;
		  go = goRest;
		  pthread_mutex_unlock(&goMutex);
		}
#ifdef WINDOWS
	      clock1 = 1000000.0/CLOCKS_PER_SEC*clock();
#else
	      gettimeofday(&tv, &tz);
	      clock1 = (BB) tv.tv_sec*1000000 + (BB) tv.tv_usec;
#endif
	      timePondered = clock1 - tc.clock0;
	    }
	}
#endif  /* _REENTRANT */
    }
  
  /* All's well that ends well */
  free(EGTB_cache);
  /* Using exiting() here disturbed lint. */
  free(hash[white]);
  free(hash[black]);
  fclose(fd);
  exit(0);
}
