/* move.c */
/* Last modified: April 1, 2007 */


static ui pieceAtSq(ui sq, sPos *pos)
{
  BB Sq;

  ASSERT(sq >= A1 && sq <= H8);
  ASSERT(pos != NULL);
  ASSERT(!(pos->WBoard & pos->BBoard));

  Sq = OneMask[sq];
  if (Sq & pos->WPBoard) return wP;
  if (Sq & pos->BPBoard) return bP;
  if (Sq & pos->WNBoard) return wN;
  if (Sq & pos->BNBoard) return bN;
  if (Sq & pos->WBBoard) return wB;
  if (Sq & pos->BBBoard) return bB;
  if (Sq & pos->WRBoard) return wR;
  if (Sq & pos->BRBoard) return bR;
  if (Sq & pos->WQBoard) return wQ;
  if (Sq & pos->BQBoard) return bQ;

  if (sq == pos->wKSq) return wK;
  if (sq == pos->bKSq) return bK;

  return none;
}


ui testMove(ui wToMove, ui ply, BB move, sPos *pos, sPlyData *plyData, 
	    sOthers *others)
{
  ui notOK;

  ASSERT(wToMove == false || wToMove == true);
  ASSERT(ply >= 0 && ply <= maxPly);
  ASSERT(from(move) >= A1 && from(move) <= H8 && 
	 to(move) >= A1 && to(move) <= H8 &&
	 from(move) != to(move));
  ASSERT(piece(move) < 13);
  ASSERT(pos != NULL);
  ASSERT(plyData != NULL);
  ASSERT(others != NULL);
  ASSERT(!(pos->WBoard & pos->BBoard));

  makeMove(wToMove, ply, move, pos, plyData, others);
  if (wToMove) 
    notOK = (getBAttacker(pos->wKSq, pos, &plyData[ply], others) & 2);
  else
    notOK = (getWAttacker(pos->bKSq, pos, &plyData[ply], others) & 2);
  unmakeMove(wToMove, ply, move, pos, plyData);

  return !notOK;
}


/* Use see() (Static Exchange Evaluation) */
ui qcaptures(ui wToMove, ui ply, BB *moveList, sPos *pos, 
	    sPlyData *plyData, sOthers *others)
{
  BB None, From, To, Them, move;
  ui sq, t;
  ui c;
  int tValue, mVVLVAValue;
  ui count = 0;

  ASSERT(wToMove == false || wToMove == true);
  ASSERT(ply >= 0 && ply <= maxPly);
  ASSERT(moveList != NULL);
  ASSERT(pos != NULL);
  ASSERT(plyData != NULL);
  ASSERT(others != NULL);
  ASSERT(!(pos->WBoard & pos->BBoard));

  None = ~(pos->WBoard | pos->BBoard);

  *moveList = 0;

  if (wToMove)
    {
      Them = pos->BBoard;

      /* pawn */
      if (pos->WPBoard)
	{
	  From = pos->WPBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      if (OneMask[sq] & others->Row[6])   /* promotions */
		{
		  if (OneMask[sq+8] & None)
		    {
		      move = sq | ((sq+8) << 6) | (wP << 12);
		      mVVLVAValue = qValue - pValue;
		      count++;
		      *moveList++ = move | (wQ << 24) | 
			((BB)(mVVLVAValue+mateScore) << 30);
		    }
		}
	      To = others->WPCaptures[sq] & Them;
	      while (To)
		{
		  t = lastOne(To);
		  c = pieceAtSq(t, pos);
		  ASSERT(c < 13);
		  tValue = see(1, sq, t, pos, others);
		  move = sq | (t << 6) | (wP << 12) | (c << 18);
		  if (OneMask[sq] & others->Row[6])   /* promotions */
		    {
		      mVVLVAValue = qValue - pValue + tValue;
		      count++;
		      *moveList++ = move | (wQ << 24) |
			((BB)(mVVLVAValue+mateScore) << 30);
		    }
		  else
	            {
		      if (tValue >= -1)       /* bValue is 1 > nValue */
			{
			  count++;
			  *moveList++ = move | ((BB)(tValue+mateScore) << 30);
			}
		    }
		  To &= NilMask[t];
		}
	      /* en-passant */
	      if (plyData[ply].EP & others->WPCaptures[sq])
		{
		  move = sq | (firstOne(plyData[ply].EP) << 6)| (wP << 12);
		  mVVLVAValue = pValue;
	          count++;
	          *moveList++ = move | ((BB)(mVVLVAValue+mateScore) << 30);
		}
	      From &= NilMask[sq];
	    }
	}

      /* knight */
      if (pos->WNBoard)
	{
	  From = pos->WNBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      To = others->NMoves[sq] & Them;
	      while (To)
		{
		  t = lastOne(To);
		  c = pieceAtSq(t, pos);
		  ASSERT(c < 13);
		  tValue = see(1, sq, t, pos, others);
		  if (tValue >= -1)       /* bValue is 1 > nValue */
		    {
		      move = sq | (t << 6) | (wN << 12) | (c << 18);
		      count++;
		      *moveList++ = move | ((BB)(tValue+mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* bishop */
      if (pos->WBBoard)
	{
	  From = pos->WBBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      To = bishopAttacks(sq, pos) & Them;
	      while (To)
		{
		  t = lastOne(To);
		  c = pieceAtSq(t, pos);
		  ASSERT(c < 13);
		  tValue = see(1, sq, t, pos, others);
		  if (tValue >= -1)       /* bValue is 1 > nValue */
		    {
		      move = sq | (t << 6) | (wB << 12) | (c << 18);
		      count++;
		      *moveList++ = move | ((BB)(tValue+mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* rook */
      if (pos->WRBoard)
	{
	  From = pos->WRBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      To = rookAttacks(sq, pos) & Them;
	      while (To)
		{
		  t = lastOne(To);
		  c = pieceAtSq(t, pos);
		  ASSERT(c < 13);
		  tValue = see(1, sq, t, pos, others);
		  if (tValue >= -1)       /* bValue is 1 > nValue */
		    {
		      move = sq | (t << 6) | (wR << 12) | (c << 18);
		      count++;
		      *moveList++ = move | ((BB)(tValue+mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* queen */
      if (pos->WQBoard)
	{
	  From = pos->WQBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      To = (rookAttacks(sq, pos) | 
		    bishopAttacks(sq, pos)) 
		& Them;
	      while (To)
		{
		  t = lastOne(To);
		  c = pieceAtSq(t, pos);
		  ASSERT(c < 13);
		  tValue = see(1, sq, t, pos, others);
		  if (tValue >= -1)       /* bValue is 1 > nValue */
		    {
		      move = sq | (t << 6) | (wQ << 12) | (c << 18);
		      count++;
		      *moveList++ = move | ((BB)(tValue+mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* king */
      /* There is one king, no need to if or loop it */
      sq = pos->wKSq;
      To = others->KMoves[sq] & Them;
      while (To)
	{
	  t = lastOne(To);
	  c = pieceAtSq(t, pos);
	  ASSERT(c < 13);
	  tValue = value[c];
	  move = sq | (t << 6) | (wK << 12) | (c << 18);
	  count++;
          *moveList++ = move | ((BB)(tValue+mateScore) << 30);
	  To &= NilMask[t];
	}
    }
  else
    {
      Them = pos->WBoard;

      /* pawn */
      if (pos->BPBoard)
	{
	  From = pos->BPBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      if (OneMask[sq] & others->Row[1])   /* promotions */
		{
		  if (OneMask[sq-8] & None)
		    {
		      move = sq | ((sq-8) << 6) | (bP << 12);
		      mVVLVAValue = qValue - pValue;
		      count++;
		      *moveList++ = move | (bQ << 24) |
			((BB)(mVVLVAValue+mateScore) << 30);
		    }
		}
	      To = others->BPCaptures[sq] & Them;
	      while (To)
		{
		  t = firstOne(To);
		  c = pieceAtSq(t, pos);
		  ASSERT(c < 13);
		  tValue = see(0, sq, t, pos, others);
		  move = sq | (t << 6) | (bP << 12) | (c << 18);
		  if (OneMask[sq] & others->Row[1])   /* promotions */
		    {
		      mVVLVAValue = qValue - pValue + tValue;
		      count++;
		      *moveList++ = move | (bQ << 24) |
		        ((BB)(mVVLVAValue+mateScore) << 30);
		    }
		  else
	      	    {
		      if (tValue >= -1)       /* bValue is 1 > nValue */
			{
			  count++;
			  *moveList++ = move | ((BB)(tValue+mateScore) << 30);
			}
		    }
		  To &= NilMask[t];
		}
	      /* en-passant */
	      if (plyData[ply].EP & others->BPCaptures[sq])
		{
		  move = sq | (firstOne(plyData[ply].EP) << 6) | (bP << 12);
		  mVVLVAValue = pValue;
	          count++;
	          *moveList++ = move | ((BB)(mVVLVAValue+mateScore) << 30);
		}
	      From &= NilMask[sq];
	    }
	}

      /* knight */
      if (pos->BNBoard)
	{
	  From = pos->BNBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      To = others->NMoves[sq] & Them;
	      while (To)
		{
		  t = firstOne(To);
		  c = pieceAtSq(t, pos);
		  ASSERT(c < 13);
		  tValue = see(0, sq, t, pos, others);
		  if (tValue >= -1)       /* bValue is 1 > nValue */
		    {
		      move = sq | (t << 6) | (bN << 12) | (c << 18);
		      count++;
		      *moveList++ = move | ((BB)(tValue+mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* bishop */
      if (pos->BBBoard)
	{
	  From = pos->BBBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      To = bishopAttacks(sq, pos) & Them;
	      while (To)
		{
		  t = firstOne(To);
		  c = pieceAtSq(t, pos);
		  ASSERT(c < 13);
		  tValue = see(0, sq, t, pos, others);
		  if (tValue >= -1)       /* bValue is 1 > nValue */
		    {
		      move = sq | (t << 6) | (bB << 12) | (c << 18);
		      count++;
		      *moveList++ = move | ((BB)(tValue+mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* rook */
      if (pos->BRBoard)
	{
	  From = pos->BRBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      To = rookAttacks(sq, pos) & Them;
	      while (To)
		{
		  t = firstOne(To);
		  c = pieceAtSq(t, pos);
		  ASSERT(c < 13);
		  tValue = see(0, sq, t, pos, others);
		  if (tValue >= -1)       /* bValue is 1 > nValue */
		    {
		      move = sq | (t << 6) | (bR << 12) | (c << 18);
		      count++;
		      *moveList++ = move | ((BB)(tValue+mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* queen */
      if (pos->BQBoard)
	{
	  From = pos->BQBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      To = (rookAttacks(sq, pos) | 
		    bishopAttacks(sq, pos)) 
		& Them;
	      while (To)
		{
		  t = firstOne(To);
		  c = pieceAtSq(t, pos);
		  ASSERT(c < 13);
		  tValue = see(0, sq, t, pos, others);
		  if (tValue >= -1)       /* bValue is 1 > nValue */
		    {
		      move = sq | (t << 6) | (bQ << 12) | (c << 18);
		      count++;
		      *moveList++ = move | ((BB)(tValue+mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* king */
      /* There is one king, no need to if or loop it */
      sq = pos->bKSq;
      To = others->KMoves[sq] & Them;
      while (To)
	{
	  t = lastOne(To);
	  c = pieceAtSq(t, pos);
	  ASSERT(c < 13);
	  tValue = value[c];
	  move = sq | (t << 6) | (bK << 12) | (c << 18);
	  count++;
          *moveList++ = move | ((BB)(tValue+mateScore) << 30);
	  To &= NilMask[t];
	}
    }

  return count;
}


/* Use see() (Static Exchange Evaluation) */
ui captures(ui wToMove, ui ply, BB *moveList, sPos *pos, 
	    sPlyData *plyData, sOthers *others, ui pseudolegal)
{
  BB None, From, To, Them, move;
  ui sq, t;
  ui c;
  int tValue, mVVLVAValue;
  ui count = 0;

  ASSERT(wToMove == false || wToMove == true);
  ASSERT(ply >= 0 && ply <= maxPly);
  ASSERT(moveList != NULL);
  ASSERT(pos != NULL);
  ASSERT(plyData != NULL);
  ASSERT(others != NULL);
  ASSERT(!(pos->WBoard & pos->BBoard));

  None = ~(pos->WBoard | pos->BBoard);

  *moveList = 0;

  if (wToMove)
    {
      Them = pos->BBoard;

      /* pawn */
      if (pos->WPBoard)
	{
	  From = pos->WPBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      if (OneMask[sq] & others->Row[6])   /* promotions */
		{
		  if (OneMask[sq+8] & None)
		    {
		      move = sq | ((sq+8) << 6) | (wP << 12);
		      if (pseudolegal || testMove(wToMove, ply+1, move, pos, plyData, 
						  others))
			{
			  mVVLVAValue = qValue - pValue;
			  count++;
			  *moveList++ = move | (wQ << 24) | 
			    ((BB)(mVVLVAValue+mateScore) << 30);

			  mVVLVAValue = rValue - pValue;
			  count++;
			  *moveList++ = move | (wR << 24) |
			    ((BB)(mVVLVAValue+mateScore) << 30);
			      
			  mVVLVAValue = bValue - pValue;
			  count++;
			  *moveList++ = move | (wB << 24) |
			    ((BB)(mVVLVAValue+mateScore) << 30);

			  mVVLVAValue = nValue - pValue;
			  count++;
			  *moveList++ = move | (wN << 24) |
			    ((BB)(mVVLVAValue+mateScore) << 30);
			}
		    }
		}
	      To = others->WPCaptures[sq] & Them;
	      while (To)
		{
		  t = lastOne(To);
		  c = pieceAtSq(t, pos);
		  ASSERT(c < 13);
		  tValue = see(1, sq, t, pos, others);
#ifdef VERIFY
		  if (c == bK) 
		    {
		      printf("bK is captured by wP in captures().\n");
		      exit(EXIT_FAILURE);
		    }
#endif
		  move = sq | (t << 6) | (wP << 12) | (c << 18);
		  if (pseudolegal || testMove(wToMove, ply+1, move, pos, plyData, 
					      others))
		    {
		      if (OneMask[sq] & others->Row[6])   /* promotions */
			{
			  mVVLVAValue = qValue - pValue + tValue;
			  count++;
			  *moveList++ = move | (wQ << 24) |
			    ((BB)(mVVLVAValue+mateScore) << 30);

			  mVVLVAValue = rValue - pValue + tValue;
			  count++;
			  *moveList++ = move | (wR << 24) |
			    ((BB)(mVVLVAValue+mateScore) << 30);
			  
			  mVVLVAValue = bValue - pValue + tValue;
			  count++;
			  *moveList++ = move | (wB << 24) |
			    ((BB)(mVVLVAValue+mateScore) << 30);
			  
			  mVVLVAValue = nValue - pValue + tValue;
			  count++;
			  *moveList++ = move | (wN << 24) |
			    ((BB)(mVVLVAValue+mateScore) << 30);
			}
		      else
			{
			  count++;
			  *moveList++ = move |
			    ((BB)(tValue+mateScore) << 30);
			}
		    }
		  To &= NilMask[t];
		}
	      /* en-passant */
	      if (plyData[ply].EP & others->WPCaptures[sq])
		{
		  move = sq | (firstOne(plyData[ply].EP) << 6)| (wP << 12);
		  if (pseudolegal || 
		      testMove(wToMove, ply+1, move, pos, plyData, others)) 
		    {
		      mVVLVAValue = pValue;
		      count++;
		      *moveList++ = move |
			((BB)(mVVLVAValue+mateScore) << 30);
		    }
		}
	      From &= NilMask[sq];
	    }
	}

      /* knight */
      if (pos->WNBoard)
	{
	  From = pos->WNBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      To = others->NMoves[sq] & Them;
	      while (To)
		{
		  t = lastOne(To);
		  c = pieceAtSq(t, pos);
		  ASSERT(c < 13);
		  tValue = see(1, sq, t, pos, others);
#ifdef VERIFY
		  if (c == bK) 
		    {
		      printf("bK is captured by wN in captures().\n");
		      exit(EXIT_FAILURE);
		    }
#endif
		  move = sq | (t << 6) | (wN << 12) | (c << 18);
		  if (pseudolegal || testMove(wToMove, ply+1, move, pos, plyData, 
					      others))
		    {
		      count++;
		      *moveList++ = move |
			((BB)(tValue+mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* bishop */
      if (pos->WBBoard)
	{
	  From = pos->WBBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      To = bishopAttacks(sq, pos) & Them;
	      while (To)
		{
		  t = lastOne(To);
		  c = pieceAtSq(t, pos);
		  ASSERT(c < 13);
		  tValue = see(1, sq, t, pos, others);
#ifdef VERIFY
		  if (c == bK) 
		    {
		      printf("bK is captured by wB in captures().\n");
		      exit(EXIT_FAILURE);
		    }
#endif
		  move = sq | (t << 6) | (wB << 12) | (c << 18);
		  if (pseudolegal || testMove(wToMove, ply+1, move, pos, plyData, 
					      others))
		    {
		      count++;
		      *moveList++ = move |
			((BB)(tValue+mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* rook */
      if (pos->WRBoard)
	{
	  From = pos->WRBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      To = rookAttacks(sq, pos) & Them;
	      while (To)
		{
		  t = lastOne(To);
		  c = pieceAtSq(t, pos);
		  ASSERT(c < 13);
		  tValue = see(1, sq, t, pos, others);
#ifdef VERIFY
		  if (c == bK) 
		    {
		      printf("bK is captured by wR in captures().\n");
		      exit(EXIT_FAILURE);
		    }
#endif
		  move = sq | (t << 6) | (wR << 12) | (c << 18);
		  if (pseudolegal || testMove(wToMove, ply+1, move, pos, plyData, 
					      others))
		    {
		      count++;
		      *moveList++ = move |
			((BB)(tValue+mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* queen */
      if (pos->WQBoard)
	{
	  From = pos->WQBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      To = (rookAttacks(sq, pos) | 
		    bishopAttacks(sq, pos)) 
		& Them;
	      while (To)
		{
		  t = lastOne(To);
		  c = pieceAtSq(t, pos);
		  ASSERT(c < 13);
		  tValue = see(1, sq, t, pos, others);
#ifdef VERIFY
		  if (c == bK) 
		    {
		      printf("bK is captured by wQ in captures().\n");
		      exit(EXIT_FAILURE);
		    }
#endif
		  move = sq | (t << 6) | (wQ << 12) | (c << 18);
		  if (pseudolegal || testMove(wToMove, ply+1, move, pos, plyData, 
					      others))
		    {
		      count++;
		      *moveList++ = move |
			((BB)(tValue+mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* king */
      /* There is one king, no need to if or loop it */
      sq = pos->wKSq;
      To = others->KMoves[sq] & Them;
      while (To)
	{
	  t = lastOne(To);
	  c = pieceAtSq(t, pos);
	  ASSERT(c < 13);
	  tValue = value[c];
#ifdef VERIFY
	  if (c == bK) 
	    {
	      printf("bK is captured by wK in captures().\n");
	      exit(EXIT_FAILURE);
	    }
#endif
	  move = sq | (t << 6) | (wK << 12) | (c << 18);
	  if (pseudolegal || testMove(wToMove, ply+1, move, pos, plyData, others))
	    {
	      count++;
	      *moveList++ = move | 
		((BB)(tValue+mateScore) << 30);
	    }
	  To &= NilMask[t];
	}
    }
  else
    {
      Them = pos->WBoard;

      /* pawn */
      if (pos->BPBoard)
	{
	  From = pos->BPBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      if (OneMask[sq] & others->Row[1])   /* promotions */
		{
		  if (OneMask[sq-8] & None)
		    {
		      move = sq | ((sq-8) << 6) | (bP << 12);
		      if (pseudolegal || testMove(wToMove, ply+1, move, pos, plyData, 
						  others))
			{
			  mVVLVAValue = qValue - pValue;
			  count++;
			  *moveList++ = move | (bQ << 24) |
			    ((BB)(mVVLVAValue+mateScore) << 30);

			  mVVLVAValue = rValue - pValue;
			  count++;
			  *moveList++ = move | (bR << 24) |
			    ((BB)(mVVLVAValue+mateScore) << 30);
			  
			  mVVLVAValue = bValue - pValue;
			  count++;
			  *moveList++ = move | (bB << 24) |
			    ((BB)(mVVLVAValue+mateScore) << 30);
			  
			  mVVLVAValue = nValue - pValue;
			  count++;
			  *moveList++ = move | (bN << 24) |
			    ((BB)(mVVLVAValue+mateScore) << 30);
			}
		    }
		}
	      To = others->BPCaptures[sq] & Them;
	      while (To)
		{
		  t = firstOne(To);
		  c = pieceAtSq(t, pos);
		  ASSERT(c < 13);
		  tValue = see(0, sq, t, pos, others);
#ifdef VERIFY
		  if (c == wK) 
		    {
		      printf("wK is captured by bP in captures().\n");
		      exit(EXIT_FAILURE);
		    }
#endif
		  move = sq | (t << 6) | (bP << 12) | (c << 18);
		  if (pseudolegal || testMove(wToMove, ply+1, move, pos, plyData, 
					      others)) 
		    {
		      if (OneMask[sq] & others->Row[1])   /* promotions */
			{
			  mVVLVAValue = qValue - pValue + tValue;
			  count++;
			  *moveList++ = move | (bQ << 24) |
			    ((BB)(mVVLVAValue+mateScore) << 30);

			  mVVLVAValue = rValue - pValue + tValue;
			  count++;
			  *moveList++ = move | (bR << 24) |
			    ((BB)(mVVLVAValue+mateScore) << 30);
			  
			  mVVLVAValue = bValue - pValue + tValue;
			  count++;
			  *moveList++ = move | (bB << 24) |
			    ((BB)(mVVLVAValue+mateScore) << 30);
			  
			  mVVLVAValue = nValue - pValue + tValue;
			  count++;
			  *moveList++ = move | (bN << 24) |
			    ((BB)(mVVLVAValue+mateScore) << 30);
			}
		      else
			{
			  count++;
			  *moveList++ = move |
			    ((BB)(tValue+mateScore) << 30);
			}
		    }
		  To &= NilMask[t];
		}
	      /* en-passant */
	      if (plyData[ply].EP & others->BPCaptures[sq])
		{
		  move = sq | (firstOne(plyData[ply].EP) << 6) | (bP << 12);
		  if (pseudolegal || 
		      testMove(wToMove, ply+1, move, pos, plyData, others)) 
		    {
		      mVVLVAValue = pValue;
		      count++;
		      *moveList++ = move |
			((BB)(mVVLVAValue+mateScore) << 30);
		    }
		}
	      From &= NilMask[sq];
	    }
	}

      /* knight */
      if (pos->BNBoard)
	{
	  From = pos->BNBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      To = others->NMoves[sq] & Them;
	      while (To)
		{
		  t = firstOne(To);
		  c = pieceAtSq(t, pos);
		  ASSERT(c < 13);
		  tValue = see(0, sq, t, pos, others);
#ifdef VERIFY
		  if (c == wK) 
		    {
		      printf("wK is captured by bN in captures().\n");
		      exit(EXIT_FAILURE);
		    }
#endif
		  move = sq | (t << 6) | (bN << 12) | (c << 18);
		  if (pseudolegal || testMove(wToMove, ply+1, move, pos, plyData, 
					      others)) 
		    {
		      count++;
		      *moveList++ = move |
			((BB)(tValue+mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* bishop */
      if (pos->BBBoard)
	{
	  From = pos->BBBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      To = bishopAttacks(sq, pos) & Them;
	      while (To)
		{
		  t = firstOne(To);
		  c = pieceAtSq(t, pos);
		  ASSERT(c < 13);
		  tValue = see(0, sq, t, pos, others);
#ifdef VERIFY
		  if (c == wK) 
		    {
		      printf("wK is captured by bB in captures().\n");
		      exit(EXIT_FAILURE);
		    }
#endif
		  move = sq | (t << 6) | (bB << 12) | (c << 18);
		  if (pseudolegal || testMove(wToMove, ply+1, move, pos, plyData, 
					      others)) 
		    {
		      count++;
		      *moveList++ = move |
			((BB)(tValue+mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* rook */
      if (pos->BRBoard)
	{
	  From = pos->BRBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      To = rookAttacks(sq, pos) & Them;
	      while (To)
		{
		  t = firstOne(To);
		  c = pieceAtSq(t, pos);
		  ASSERT(c < 13);
		  tValue = see(0, sq, t, pos, others);
#ifdef VERIFY
		  if (c == wK) 
		    {
		      printf("wK is captured by bR in captures().\n");
		      exit(EXIT_FAILURE);
		    }
#endif
		  move = sq | (t << 6) | (bR << 12) | (c << 18);
		  if (pseudolegal || testMove(wToMove, ply+1, move, pos, plyData, 
					      others)) 
		    {
		      count++;
		      *moveList++ = move |
			((BB)(tValue+mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* queen */
      if (pos->BQBoard)
	{
	  From = pos->BQBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      To = (rookAttacks(sq, pos) | 
		    bishopAttacks(sq, pos)) 
		& Them;
	      while (To)
		{
		  t = firstOne(To);
		  c = pieceAtSq(t, pos);
		  ASSERT(c < 13);
		  tValue = see(0, sq, t, pos, others);
#ifdef VERIFY
		  if (c == wK) 
		    {
		      printf("wK is captured by bQ in captures().\n");
		      exit(EXIT_FAILURE);
		    }
#endif
		  move = sq | (t << 6) | (bQ << 12) | (c << 18);
		  if (pseudolegal || testMove(wToMove, ply+1, move, pos, plyData, 
					      others))
		    {
		      count++;
		      *moveList++ = move |
			((BB)(tValue+mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* king */
      /* There is one king, no need to if or loop it */
      sq = pos->bKSq;
      To = others->KMoves[sq] & Them;
      while (To)
	{
	  t = lastOne(To);
	  c = pieceAtSq(t, pos);
	  ASSERT(c < 13);
	  tValue = value[c];
#ifdef VERIFY
	  if (c == wK)
	    {
	      printf("wK is captured by bK in captures().\n");
	      exit(EXIT_FAILURE);
	    }
#endif
	  move = sq | (t << 6) | (bK << 12) | (c << 18);
	  if (pseudolegal || 
	      testMove(wToMove, ply+1, move, pos, plyData, others))
	    {
	      count++;
	      *moveList++ = move |
		((BB)(tValue+mateScore) << 30);
	    }
	  To &= NilMask[t];
	}
    }

  return count;
}


ui nonCaptures(ui wToMove, ui ply, BB *moveList, sPos *pos, sPlyData *plyData, 
	       sOthers *others, ui pseudolegal)
{
  BB None, From, To, Them, move;
  ui sq, t;
  ui count = 0;

  ASSERT(wToMove == false || wToMove == true);
  ASSERT(ply >= 0 && ply <= maxPly);
  ASSERT(moveList != NULL);
  ASSERT(pos != NULL);
  ASSERT(plyData != NULL);
  ASSERT(others != NULL);
  ASSERT(!(pos->WBoard & pos->BBoard));

  None = ~(pos->WBoard | pos->BBoard);

  *moveList = 0;

  if (wToMove)
    {
      Them = ~(pos->WBoard | pos->BBoard);

      /* pawn */
      if (pos->WPBoard)
	{
	  From = pos->WPBoard & ~others->Row[6];        /* not a promotion */
	  while (From)
	    {
	      sq = firstOne(From);
	      if (OneMask[sq+8] & Them)
		{
		  t = sq+8;
		  move = sq | (t << 6) | (wP << 12);
		  if (pseudolegal || testMove(wToMove, ply+1, move, pos, plyData, 
					      others)) 
		    {
		      count++;
		      *moveList++ = move |
			((BB)(mateScore) << 30);
		    }
		  if ((OneMask[sq] & others->Row[1]) && (OneMask[sq+16] & Them))
		    {
		      t = sq+16;
		      move = sq | (t << 6) | (wP << 12);
		      if (pseudolegal || testMove(wToMove, ply+1, move, pos, plyData, 
						  others))
			{
			  count++;
			  *moveList++ = move |
			    ((BB)(mateScore) << 30);
			}
		    }
		}
	      From &= NilMask[sq];
	    }
	}

      /* knight */
      if (pos->WNBoard)
	{
	  From = pos->WNBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      To = others->NMoves[sq] & Them;
	      while (To)
		{
		  t = lastOne(To);
		  move = sq | (t << 6) | (wN << 12);
		  if (pseudolegal || testMove(wToMove, ply+1, move, pos, plyData, 
					      others))
		    {
		      count++;
		      *moveList++ = move |
			((BB)(mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* rook */
      if (pos->WRBoard)
	{
	  From = pos->WRBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      To = rookAttacks(sq, pos) & Them;
	      while (To)
		{
		  t = lastOne(To);
		  move = sq | (t << 6) | (wR << 12);
		  if (pseudolegal || testMove(wToMove, ply+1, move, pos, plyData, 
					      others))
		    {
		      count++;
		      *moveList++ = move |
			((BB)(mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* bishop */
      if (pos->WBBoard)
	{
	  From = pos->WBBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      To = bishopAttacks(sq, pos) & Them;
	      while (To)
		{
		  t = lastOne(To);
		  move = sq | (t << 6) | (wB << 12);
		  if (pseudolegal || testMove(wToMove, ply+1, move, pos, plyData, 
					      others))
		    {
		      count++;
		      *moveList++ = move |
			((BB)(mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* queen */
      if (pos->WQBoard)
	{
	  From = pos->WQBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      To = (rookAttacks(sq, pos) | 
		    bishopAttacks(sq, pos)) 
		& Them;
	      while (To)
		{
		  t = lastOne(To);
		  move = sq | (t << 6) | (wQ << 12);
		  if (pseudolegal || testMove(wToMove, ply+1, move, pos, plyData, 
					      others))
		    {
		      count++;
		      *moveList++ = move |
			((BB)(mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* king */
      /* There is one king, no need to if or loop it */
      sq = pos->wKSq;

      if (plyData[ply].wCanCastle > 0)
	{
	  /* long castling */
	  if ((plyData[ply].wCanCastle & 2) &&
	      OneMask[A1] & pos->WRBoard &&
	      OneMask[B1] & None &&
	      OneMask[C1] & None &&
	      OneMask[D1] & None &&
	      !(getBAttacker(C1, pos, &plyData[ply], others) & 2) &&
	      !(getBAttacker(D1, pos, &plyData[ply], others) & 2) &&
	      !(getBAttacker(E1, pos, &plyData[ply], others) & 2))
	    {
	      count++;
	      *moveList++ = (BB)(E1 | (C1 << 6) | (wK << 12)) |
		((BB)(mateScore) << 30);
	    }
	  /* short castling */
	  if ((plyData[ply].wCanCastle & 1) &&
	      OneMask[H1] & pos->WRBoard &&
	      OneMask[G1] & None &&
	      OneMask[F1] & None &&
	      !(getBAttacker(G1, pos, &plyData[ply], others) & 2) &&
	      !(getBAttacker(F1, pos, &plyData[ply], others) & 2) &&
	      !(getBAttacker(E1, pos, &plyData[ply], others) & 2))
	    {
	      count++;
	      *moveList++ = (BB)(E1 | (G1 << 6) | (wK << 12)) |
		((BB)(mateScore) << 30);
	    }
	}
      /* Normal move */
      To = others->KMoves[sq] & Them;
      while (To)
	{
	  t = lastOne(To);
	  if (pseudolegal || 
	      !(getBAttacker(t, pos, &plyData[ply], others) & 2))
	    {
	      move = sq | (t << 6) | (wK << 12);
	      if (pseudolegal || testMove(wToMove, ply+1, move, pos, plyData, others))
		{
		  count++;
		  *moveList++ = move |
		    ((BB)(mateScore) << 30);
		}
	    }
	  To &= NilMask[t];
	}
    }
  else
    {
      Them = ~(pos->WBoard | pos->BBoard);

      /* pawn */
      if (pos->BPBoard)
	{
	  From = pos->BPBoard & ~others->Row[1];        /* not a promotion */
	  while (From)
	    {
	      sq = lastOne(From);
	      if (OneMask[sq-8] & Them)
		{
		  t = sq-8;
		  move = sq | (t << 6) | (bP << 12);
		  if (pseudolegal || testMove(wToMove, ply+1, move, pos, plyData, 
					      others))
		    {
		      count++;
		      *moveList++ = move |
			((BB)(mateScore) << 30);
		    }
		  if ((OneMask[sq] & others->Row[6]) && 
		      (OneMask[sq-16] & Them))
		    {
		      t = sq-16;
		      move = sq | (t << 6) | (bP << 12);
		      if (pseudolegal || testMove(wToMove, ply+1, move, pos, plyData, 
						  others))
			{
			  count++;
			  *moveList++ = move |
			    ((BB)(mateScore) << 30);
			}
		    }
		}
	      From &= NilMask[sq];
	    }
	}

      /* knight */
      if (pos->BNBoard)
	{
	  From = pos->BNBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      To = others->NMoves[sq] & Them;
	      while (To)
		{
		  t = firstOne(To);
		  move = sq | (t << 6) | (bN << 12);
		  if (pseudolegal || testMove(wToMove, ply+1, move, pos, plyData, 
					      others))
		    {
		      count++;
		      *moveList++ = move |
			((BB)(mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* rook */
      if (pos->BRBoard)
	{
	  From = pos->BRBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      To = rookAttacks(sq, pos) & Them;
	      while (To)
		{
		  t = firstOne(To);
		  move = sq | (t << 6) | (bR << 12);
		  if (pseudolegal || testMove(wToMove, ply+1, move, pos, plyData, 
					      others))
		    {
		      count++;
		      *moveList++ = move |
			((BB)(mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* bishop */
      if (pos->BBBoard)
	{
	  From = pos->BBBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      To = bishopAttacks(sq, pos) & Them;
	      while (To)
		{
		  t = firstOne(To);
		  move = sq | (t << 6) | (bB << 12);
		  if (pseudolegal || testMove(wToMove, ply+1, move, pos, plyData, 
					      others))
		    {
		      count++;
		      *moveList++ = move |
			((BB)(mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* queen */
      if (pos->BQBoard)
	{
	  From = pos->BQBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      To = (rookAttacks(sq, pos) | 
		    bishopAttacks(sq, pos)) 
		& Them;
	      while (To)
		{
		  t = firstOne(To);
		  move = sq | (t << 6) | (bQ << 12);
		  if (pseudolegal || testMove(wToMove, ply+1, move, pos, plyData, 
					      others))
		    {
		      count++;
		      *moveList++ = move |
			((BB)(mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* king */
      /* There is one king, no need to if or loop it */
      sq = pos->bKSq;

      if (plyData[ply].bCanCastle > 0)
	{
	  /* long castling */
	  if ((plyData[ply].bCanCastle & 2) &&
	      OneMask[A8] & pos->BRBoard &&
	      OneMask[B8] & None &&
	      OneMask[C8] & None &&
	      OneMask[D8] & None &&
	      !(getWAttacker(C8, pos, &plyData[ply], others) & 2) &&
	      !(getWAttacker(D8, pos, &plyData[ply], others) & 2) &&
	      !(getWAttacker(E8, pos, &plyData[ply], others) & 2))
	    {
	      count++;
	      *moveList++ = (BB)(E8 | (C8 << 6) | (bK << 12)) |
		((BB)(mateScore) << 30);
	    }
	  /* short castling */
	  if ((plyData[ply].bCanCastle & 1) &&
	      OneMask[H8] & pos->BRBoard &&
	      OneMask[G8] & None &&
	      OneMask[F8] & None &&
	      !(getWAttacker(G8, pos, &plyData[ply], others) & 2) &&
	      !(getWAttacker(F8, pos, &plyData[ply], others) & 2) &&
	      !(getWAttacker(E8, pos, &plyData[ply], others) & 2))
	    {
	      count++;
	      *moveList++ = (BB)(E8 | (G8 << 6) | (bK << 12)) |
		((BB)(mateScore) << 30);
	    }
	}
      /* Normal move */
      To = others->KMoves[sq] & Them;
      while (To)
	{
	  t = lastOne(To);
	  if (pseudolegal || 
	      !(getWAttacker(t, pos, &plyData[ply], others) & 2))
	    {
	      move = sq | (t << 6) | (bK << 12);
	      if (pseudolegal || 
                  testMove(wToMove, ply+1, move, pos, plyData, others))
		{
		  count++;
		  *moveList++ = move |
		    ((BB)(mateScore) << 30);
		}
	    }
	  To &= NilMask[t];
	}
    }

  return count;
}


void makeMove(ui wToMove, ui ply, BB move, sPos *pos, sPlyData *plyData, 
	      sOthers *others)
{
#ifdef VERIFY
  sPos thisPos;
  BB key = 0;
#endif
  ui p, f, t, c;
  char castleFlag = '-';

  BB *pWAttacker = (BB *) plyData[ply].wAttacker;
  BB *pBAttacker = (BB *) plyData[ply].bAttacker;

  ASSERT(wToMove == false || wToMove == true);
  ASSERT(ply >= 0 && ply <= maxPly);
  ASSERT(pos != NULL);
  ASSERT(plyData != NULL);
  ASSERT(others != NULL);
  ASSERT(!(pos->WBoard & pos->BBoard));

  plyData[ply] = plyData[ply-1];
  plyData[ply].count50Moves++;
  if (plyData[ply].EP) plyData[ply].key ^= 
			 others->ePKey[firstOne(plyData[ply].EP)];
  plyData[ply].EP = 0;

  /* Attack tables */
  pWAttacker[0] =   pWAttacker[1] =   pWAttacker[2] =   pWAttacker[3] = 
    pWAttacker[4] =   pWAttacker[5] =   pWAttacker[6] =   pWAttacker[7] = 0; 
  pBAttacker[0] =   pBAttacker[1] =   pBAttacker[2] =   pBAttacker[3] = 
    pBAttacker[4] =   pBAttacker[5] =   pBAttacker[6] =   pBAttacker[7] = 0; 

  f = from(move);
  t = to(move);
  p = piece(move);
  c = captured(move);
  ASSERT(f >= A1 && f <= H8 && t >= A1 && t <= H8 && f != t);
  ASSERT(p < 13);
  ASSERT(c < 13);

#ifdef VERIFY
  plyData[ply].piece[f] = none;
  plyData[ply].piece[t] = plyData[ply-1].piece[f];
#endif
  if (wToMove)
    {
      pos->WBoard &= NilMask[f];
      pos->WBoard |= OneMask[t];
    }
  else
    {
      pos->BBoard &= NilMask[f];
      pos->BBoard |= OneMask[t];
    }

  /* moved piece */
  switch(p)
    {
    case wP:
      plyData[ply].count50Moves = 0;
      pos->WPBoard &= NilMask[f];
      plyData[ply].key ^= others->wPKey[f];
      pos->WPBoard |= OneMask[t];
      plyData[ply].key ^= others->wPKey[t];
      if (t-f == 16) 
	{
	  plyData[ply].EP = OneMask[t-8];
	  plyData[ply].key ^= others->ePKey[t-8];
	}
      else if (OneMask[t] & plyData[ply-1].EP)
	{
#ifdef VERIFY
	  plyData[ply].piece[t-8] = none;
#endif
	  pos->BPBoard &= NilMask[t-8];
	  plyData[ply].key ^= others->bPKey[t-8];
	  pos->BBoard &= NilMask[t-8];
	  pos->wPCount[column(f)]--;
	  pos->wPCount[column(t)]++;
	  pos->bPCount[column(t)]--;
	}
      else if (c)
	{
	  pos->wPCount[column(f)]--;
	  pos->wPCount[column(t)]++;
	}
      if (promotedTo(move)) pos->wPCount[column(t)]--;
      break;
    case wN:
      pos->WNBoard &= NilMask[f];
      plyData[ply].key ^= others->wNKey[f];
      pos->WNBoard |= OneMask[t];
      plyData[ply].key ^= others->wNKey[t];
      break;
    case wB:
      pos->WBBoard &= NilMask[f];
      plyData[ply].key ^= others->wBKey[f];
      pos->WBBoard |= OneMask[t];
      plyData[ply].key ^= others->wBKey[t];
      break;
    case wR:
      if (plyData[ply].wCanCastle > 0)
	{
	  if (f == A1) plyData[ply].wCanCastle &= 1;
	  else if (f == H1) plyData[ply].wCanCastle &= 2;
	  plyData[ply].key ^= others->wCanCastleKey[plyData[ply-1].wCanCastle];
	  plyData[ply].key ^= others->wCanCastleKey[plyData[ply].wCanCastle];
	}
      pos->WRBoard &= NilMask[f];
      plyData[ply].key ^= others->wRKey[f];
      pos->WRBoard |= OneMask[t];
      plyData[ply].key ^= others->wRKey[t];
      break;
    case wQ:
      pos->WQBoard &= NilMask[f];
      plyData[ply].key ^= others->wQKey[f];
      pos->WQBoard |= OneMask[t];
      plyData[ply].key ^= others->wQKey[t];
      break;
    case wK:
      plyData[ply].key ^= others->wKKey[f];
      pos->wKSq = t;
      plyData[ply].key ^= others->wKKey[t];
      if (plyData[ply].wCanCastle > 0)
	{
	  /* long castling */
	  if (f == E1 && t == C1)
	    {
#ifdef VERIFY
	      plyData[ply].piece[A1] = none;
	      plyData[ply].piece[D1] = wR;
#endif
	      pos->WBoard &= NilMask[A1];
	      pos->WBoard |= OneMask[D1];
	      pos->WRBoard &= NilMask[A1];
	      plyData[ply].key ^= others->wRKey[A1];
	      pos->WRBoard |= OneMask[D1];
	      plyData[ply].key ^= others->wRKey[D1];
	      plyData[ply].wCanCastle = -(int) ply;
	      castleFlag = 'Q';
	    }
	  /* short castling */
	  else if (f == E1 && t == G1)
	    {
#ifdef VERIFY
	      plyData[ply].piece[H1] = none;
	      plyData[ply].piece[F1] = wR;
#endif
	      pos->WBoard &= NilMask[H1];
	      pos->WBoard |= OneMask[F1];
	      pos->WRBoard &= NilMask[H1];
	      plyData[ply].key ^= others->wRKey[H1];
	      pos->WRBoard |= OneMask[F1];
	      plyData[ply].key ^= others->wRKey[F1];
	      plyData[ply].wCanCastle = -(int) ply;
	      castleFlag = 'K';
	    }
	  else plyData[ply].wCanCastle = 0;
	  plyData[ply].key ^= others->wCanCastleKey[plyData[ply-1].wCanCastle];
	  plyData[ply].key ^= others->wCanCastleKey[0];
	}
      break;
    case bP:
      plyData[ply].count50Moves = 0;
      pos->BPBoard &= NilMask[f];
      plyData[ply].key ^= others->bPKey[f];
      pos->BPBoard |= OneMask[t];
      plyData[ply].key ^= others->bPKey[t];
      if (f-t == 16) 
	{
	  plyData[ply].EP = OneMask[t+8];
	  plyData[ply].key ^= others->ePKey[t+8];
	}
      else if (OneMask[t] & plyData[ply-1].EP)
	{
#ifdef VERIFY
	  plyData[ply].piece[t+8] = none;
#endif
	  pos->WPBoard &= NilMask[t+8];
	  plyData[ply].key ^= others->wPKey[t+8];
	  pos->WBoard &= NilMask[t+8];
	  pos->bPCount[column(f)]--;
	  pos->bPCount[column(t)]++;
	  pos->wPCount[column(t)]--;
	}
      else if (c)
	{
	  pos->bPCount[column(f)]--;
	  pos->bPCount[column(t)]++;
	}
      if (promotedTo(move)) pos->bPCount[column(t)]--;
      break;
    case bN:
      pos->BNBoard &= NilMask[f];
      plyData[ply].key ^= others->bNKey[f];
      pos->BNBoard |= OneMask[t];
      plyData[ply].key ^= others->bNKey[t];
      break;
    case bB:
      pos->BBBoard &= NilMask[f];
      plyData[ply].key ^= others->bBKey[f];
      pos->BBBoard |= OneMask[t];
      plyData[ply].key ^= others->bBKey[t];
      break;
    case bR:
      if (plyData[ply].bCanCastle > 0)
	{
	  if (f == A8) plyData[ply].bCanCastle &= 1;
	  else if (f == H8) plyData[ply].bCanCastle &= 2;
	  plyData[ply].key ^= others->bCanCastleKey[plyData[ply-1].bCanCastle];
	  plyData[ply].key ^= others->bCanCastleKey[plyData[ply].bCanCastle];
	}
      pos->BRBoard &= NilMask[f];
      plyData[ply].key ^= others->bRKey[f];
      pos->BRBoard |= OneMask[t];
      plyData[ply].key ^= others->bRKey[t];
      break;
    case bQ:
      pos->BQBoard &= NilMask[f];
      plyData[ply].key ^= others->bQKey[f];
      pos->BQBoard |= OneMask[t];
      plyData[ply].key ^= others->bQKey[t];
      break;
    case bK:
      plyData[ply].key ^= others->bKKey[f];
      pos->bKSq = t;
      plyData[ply].key ^= others->bKKey[t];
      if (plyData[ply].bCanCastle > 0)
	{
	  /* long castling */
	  if (f == E8 && t == C8)
	    {
#ifdef VERIFY
	      plyData[ply].piece[A8] = none;
	      plyData[ply].piece[D8] = bR;
#endif
	      pos->BBoard &= NilMask[A8];
	      pos->BBoard |= OneMask[D8];
	      pos->BRBoard &= NilMask[A8];
	      plyData[ply].key ^= others->bRKey[A8];
	      pos->BRBoard |= OneMask[D8];
	      plyData[ply].key ^= others->bRKey[D8];
	      plyData[ply].bCanCastle = -(int) ply;
	      castleFlag = 'q';
	    }
	  /* short castling */
	  else if (f == E8 && t == G8)
	    {
#ifdef VERIFY
	      plyData[ply].piece[H8] = none;
	      plyData[ply].piece[F8] = bR;
#endif
	      pos->BBoard &= NilMask[H8];
	      pos->BBoard |= OneMask[F8];
	      pos->BRBoard &= NilMask[H8];
	      plyData[ply].key ^= others->bRKey[H8];
	      pos->BRBoard |= OneMask[F8];
	      plyData[ply].key ^= others->bRKey[F8];
	      plyData[ply].bCanCastle = -(int) ply;
	      castleFlag = 'k';
	    }
	  else plyData[ply].bCanCastle = 0;
	  plyData[ply].key ^= others->bCanCastleKey[plyData[ply-1].bCanCastle];
	  plyData[ply].key ^= others->bCanCastleKey[0];
	}
      break;
    default:
      break;
    }

  /* captured piece */
  if (c)
    {
      ASSERT(c != wK && c != bK);
      plyData[ply].count50Moves = 0;
      switch(c)
	{
	case wP:
	  pos->WBoard &= NilMask[t];
	  pos->WPBoard &= NilMask[t];
	  plyData[ply].key ^= others->wPKey[t];
	  pos->wPCount[column(t)]--;
	  break;
	case wN:
	  pos->WBoard &= NilMask[t];
	  pos->WNBoard &= NilMask[t];
	  plyData[ply].key ^= others->wNKey[t];
	  break;
	case wB:
	  pos->WBoard &= NilMask[t];
	  pos->WBBoard &= NilMask[t];
	  plyData[ply].key ^= others->wBKey[t];
	  break;
	case wR:
	  pos->WBoard &= NilMask[t];
	  pos->WRBoard &= NilMask[t];
	  plyData[ply].key ^= others->wRKey[t];
	  break;
	case wQ:
	  pos->WBoard &= NilMask[t];
	  pos->WQBoard &= NilMask[t];
	  plyData[ply].key ^= others->wQKey[t];
	  break;
	case wK:
	  printf("makeMove: What the heck?  %u-%u\n", f, t);
	  display(pos);
	  printf("makeMove: What the heck?  wK at %u is captured.\n", t);
	  break;
	case bP:
	  pos->BBoard &= NilMask[t];
	  pos->BPBoard &= NilMask[t];
	  plyData[ply].key ^= others->bPKey[t];
	  pos->bPCount[column(t)]--;
	  break;
	case bN:
	  pos->BBoard &= NilMask[t];
	  pos->BNBoard &= NilMask[t];
	  plyData[ply].key ^= others->bNKey[t];
	  break;
	case bB:
	  pos->BBoard &= NilMask[t];
	  pos->BBBoard &= NilMask[t];
	  plyData[ply].key ^= others->bBKey[t];
	  break;
	case bR:
	  pos->BBoard &= NilMask[t];
	  pos->BRBoard &= NilMask[t];
	  plyData[ply].key ^= others->bRKey[t];
	  break;
	case bQ:
	  pos->BBoard &= NilMask[t];
	  pos->BQBoard &= NilMask[t];
	  plyData[ply].key ^= others->bQKey[t];
	  break;
	case bK:
	  printf("makeMove: What the heck?  %u-%u\n", f, t);
	  display(pos);
	  printf("makeMove: What the heck?  bK at %u is captured.\n", t);
	  break;
	default:
	  break;
	}
    }

  /* promoted piece */
  switch(promotedTo(move))
    {
    case none:
      break;
    case wN:
#ifdef VERIFY
      plyData[ply].piece[t] = wN;
#endif
      pos->WPBoard &= NilMask[t];
      plyData[ply].key ^= others->wPKey[t];
      pos->WNBoard |= OneMask[t];
      plyData[ply].key ^= others->wNKey[t]; 
     break;
    case wB:
#ifdef VERIFY
      plyData[ply].piece[t] = wB;
#endif
      pos->WPBoard &= NilMask[t];
      plyData[ply].key ^= others->wPKey[t];
      pos->WBBoard |= OneMask[t];
      plyData[ply].key ^= others->wBKey[t];
      break;
    case wR:
#ifdef VERIFY
      plyData[ply].piece[t] = wR;
#endif
      pos->WPBoard &= NilMask[t];
      plyData[ply].key ^= others->wPKey[t];
      pos->WRBoard |= OneMask[t];
      plyData[ply].key ^= others->wRKey[t];
      break;
    case wQ:
#ifdef VERIFY
      plyData[ply].piece[t] = wQ;
#endif
      pos->WPBoard &= NilMask[t];
      plyData[ply].key ^= others->wPKey[t];
      pos->WQBoard |= OneMask[t];
      plyData[ply].key ^= others->wQKey[t];
      break;
    case bN:
#ifdef VERIFY
      plyData[ply].piece[t] = bN;
#endif
      pos->BPBoard &= NilMask[t];
      plyData[ply].key ^= others->bPKey[t];
      pos->BNBoard |= OneMask[t];
      plyData[ply].key ^= others->bNKey[t];
      break;
    case bB:
#ifdef VERIFY
      plyData[ply].piece[t] = bB;
#endif
      pos->BPBoard &= NilMask[t];
      plyData[ply].key ^= others->bPKey[t];
      pos->BBBoard |= OneMask[t];
      plyData[ply].key ^= others->bBKey[t];
      break;
    case bR:
#ifdef VERIFY
      plyData[ply].piece[t] = bR;
#endif
      pos->BPBoard &= NilMask[t];
      plyData[ply].key ^= others->bPKey[t];
      pos->BRBoard |= OneMask[t];
      plyData[ply].key ^= others->bRKey[t];
      break;
    case bQ:
#ifdef VERIFY
      plyData[ply].piece[t] = bQ;
#endif
      pos->BPBoard &= NilMask[t];
      plyData[ply].key ^= others->bPKey[t];
      pos->BQBoard |= OneMask[t];
      plyData[ply].key ^= others->bQKey[t];
      break;
    default:
      break;
    }

  /* update rotated bitboards */
  pos->RotatedBoard90 &= NilMask[rotate90[f]];
  pos->RotatedBoard45l &= NilMask[rotate45l[f]];
  pos->RotatedBoard45r &= NilMask[rotate45r[f]];
  if (!c)
    {
      pos->RotatedBoard90 |= OneMask[rotate90[t]];
      pos->RotatedBoard45l |= OneMask[rotate45l[t]];
      pos->RotatedBoard45r |= OneMask[rotate45r[t]];
    }

  /* update rotated bitboards due to rook movement in castling */
  switch (castleFlag)
    {
    case '-':
      break;
    case 'K':
      pos->RotatedBoard90 &= NilMask[rotate90[H1]];
      pos->RotatedBoard45l &= NilMask[rotate45l[H1]];
      pos->RotatedBoard45r &= NilMask[rotate45r[H1]];
      pos->RotatedBoard90 |= OneMask[rotate90[F1]];
      pos->RotatedBoard45l |= OneMask[rotate45l[F1]];
      pos->RotatedBoard45r |= OneMask[rotate45r[F1]];
      break;
    case 'Q':
      pos->RotatedBoard90 &= NilMask[rotate90[A1]];
      pos->RotatedBoard45l &= NilMask[rotate45l[A1]];
      pos->RotatedBoard45r &= NilMask[rotate45r[A1]];
      pos->RotatedBoard90 |= OneMask[rotate90[D1]];
      pos->RotatedBoard45l |= OneMask[rotate45l[D1]];
      pos->RotatedBoard45r |= OneMask[rotate45r[D1]];
      break;
    case 'k':
      pos->RotatedBoard90 &= NilMask[rotate90[H8]];
      pos->RotatedBoard45l &= NilMask[rotate45l[H8]];
      pos->RotatedBoard45r &= NilMask[rotate45r[H8]];
      pos->RotatedBoard90 |= OneMask[rotate90[F8]];
      pos->RotatedBoard45l |= OneMask[rotate45l[F8]];
      pos->RotatedBoard45r |= OneMask[rotate45r[F8]];
      break;
    case 'q':
      pos->RotatedBoard90 &= NilMask[rotate90[A8]];
      pos->RotatedBoard45l &= NilMask[rotate45l[A8]];
      pos->RotatedBoard45r &= NilMask[rotate45r[A8]];
      pos->RotatedBoard90 |= OneMask[rotate90[D8]];
      pos->RotatedBoard45l |= OneMask[rotate45l[D8]];
      pos->RotatedBoard45r |= OneMask[rotate45r[D8]];
      break;
    default:
      break;
    }

  /* update rotated bitboards due to disappearing pawn in en-passant */
  if (OneMask[t] & plyData[ply-1].EP)
    {
      if (p == wP)
	{
	  pos->RotatedBoard90 &= NilMask[rotate90[t-8]];
	  pos->RotatedBoard45l &= NilMask[rotate45l[t-8]];
	  pos->RotatedBoard45r &= NilMask[rotate45r[t-8]];
	}
      else if (p == bP)
	{
	  pos->RotatedBoard90 &= NilMask[rotate90[t+8]];
	  pos->RotatedBoard45l &= NilMask[rotate45l[t+8]];
	  pos->RotatedBoard45r &= NilMask[rotate45r[t+8]];
	}
    }

  if (pos->repHead + ply < 101)
    pos->repKey[pos->repHead + ply - 1] = plyData[ply].key;

#ifdef VERIFY
  key = scanBoard(plyData[ply].piece, plyData[ply].EP,
		  plyData[ply].wCanCastle, plyData[ply].bCanCastle, &thisPos, 
		  others);

  if (pos->WBoard != thisPos.WBoard)
    {
      displayBitBoard(thisPos.WBoard);
      printf("ERROR %s: WBoard don't match in makeMove().\n", pos->posID);
      displayBitBoard(pos->WBoard);
      exit(EXIT_FAILURE);
    }
  if (pos->BBoard != thisPos.BBoard)
    {
      displayBitBoard(thisPos.BBoard);
      printf("ERROR %u-%u: BBoard don't match in makeMove().\n", f, t);
      displayBitBoard(pos->BBoard);
      exit(EXIT_FAILURE);
    }
  if (pos->WPBoard != thisPos.WPBoard)
    {
      displayBitBoard(thisPos.WPBoard);
      printf("ERROR %s: WPBoard don't match in makeMove().\n", pos->posID);
      displayBitBoard(pos->WPBoard);
      exit(EXIT_FAILURE);
    }
  if (pos->WNBoard != thisPos.WNBoard)
    {
      displayBitBoard(thisPos.WNBoard);
      printf("ERROR %s: WNBoard don't match in makeMove().\n", pos->posID);
      displayBitBoard(pos->WNBoard);
      exit(EXIT_FAILURE);
    }
  if (pos->WBBoard != thisPos.WBBoard)
    {
      displayBitBoard(thisPos.WBBoard);
      printf("ERROR %s: WBBoard don't match in makeMove().\n", pos->posID);
      displayBitBoard(pos->WBBoard);
      exit(EXIT_FAILURE);
    }
  if (pos->WRBoard != thisPos.WRBoard)
    {
      displayBitBoard(thisPos.WRBoard);
      printf("ERROR %s: WRBoard don't match in makeMove().\n", pos->posID);
      displayBitBoard(pos->WRBoard);
      exit(EXIT_FAILURE);
    }
  if (pos->WQBoard != thisPos.WQBoard)
    {
      displayBitBoard(thisPos.WQBoard);
      printf("ERROR %s: WQBoard don't match in makeMove().\n", pos->posID);
      displayBitBoard(pos->WQBoard);
      exit(EXIT_FAILURE);
    }
  if (pos->BPBoard != thisPos.BPBoard)
    {
      displayBitBoard(thisPos.BPBoard);
      printf("ERROR %s: BPBoard don't match in makeMove().\n", pos->posID);
      displayBitBoard(pos->BPBoard);
      exit(EXIT_FAILURE);
    }
  if (pos->BNBoard != thisPos.BNBoard)
    {
      displayBitBoard(thisPos.BNBoard);
      printf("ERROR %s: BNBoard don't match in makeMove().\n", pos->posID);
      displayBitBoard(pos->BNBoard);
      exit(EXIT_FAILURE);
    }
  if (pos->BBBoard != thisPos.BBBoard)
    {
      displayBitBoard(thisPos.BBBoard);
      printf("ERROR %s: BBBoard don't match in makeMove().\n", pos->posID);
      displayBitBoard(pos->BBBoard);
      exit(EXIT_FAILURE);
    }
  if (pos->BRBoard != thisPos.BRBoard)
    {
      displayBitBoard(thisPos.BRBoard);
      printf("ERROR %s: BRBoard don't match in makeMove().\n", pos->posID);
      displayBitBoard(pos->BRBoard);
      exit(EXIT_FAILURE);
    }
  if (pos->BQBoard != thisPos.BQBoard)
    {
      displayBitBoard(thisPos.BQBoard);
      printf("ERROR %s: BQBoard don't match in makeMove().\n", pos->posID);
      displayBitBoard(pos->BQBoard);
      exit(EXIT_FAILURE);
    }
  if (plyData[ply].key != key)
    {
      display(pos);
      printf("ERROR %s: key don't match in makeMove().\n", pos->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->wPCount[0] != thisPos.wPCount[0])
    {
      display(pos);
      printf("ERROR %s: wPCount[0] don't match in makeMove().\n", pos->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->wPCount[1] != thisPos.wPCount[1])
    {
      display(pos);
      printf("ERROR %s: wPCount[1] don't match in makeMove().\n", pos->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->wPCount[2] != thisPos.wPCount[2])
    {
      display(pos);
      printf("ERROR %s: wPCount[2] don't match in makeMove().\n", pos->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->wPCount[3] != thisPos.wPCount[3])
    {
      display(pos);
      printf("ERROR %s: wPCount[3] don't match in makeMove().\n", pos->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->wPCount[4] != thisPos.wPCount[4])
    {
      display(pos);
      printf("ERROR %s: wPCount[4] don't match in makeMove().\n", pos->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->wPCount[5] != thisPos.wPCount[5])
    {
      display(pos);
      printf("ERROR %s: wPCount[5] don't match in makeMove().\n", pos->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->wPCount[6] != thisPos.wPCount[6])
    {
      display(pos);
      printf("ERROR %s: wPCount[6] don't match in makeMove().\n", pos->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->wPCount[7] != thisPos.wPCount[7])
    {
      display(pos);
      printf("ERROR %s: wPCount[7] don't match in makeMove().\n", pos->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->bPCount[0] != thisPos.bPCount[0])
    {
      display(pos);
      printf("ERROR %s: bPCount[0] don't match in makeMove().\n", pos->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->bPCount[1] != thisPos.bPCount[1])
    {
      display(pos);
      printf("ERROR %s: bPCount[1] don't match in makeMove().\n", pos->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->bPCount[2] != thisPos.bPCount[2])
    {
      display(pos);
      printf("ERROR %s: bPCount[2] don't match in makeMove().\n", pos->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->bPCount[3] != thisPos.bPCount[3])
    {
      display(pos);
      printf("ERROR %s: bPCount[3] don't match in makeMove().\n", pos->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->bPCount[4] != thisPos.bPCount[4])
    {
      display(pos);
      printf("ERROR %s: bPCount[4] don't match in makeMove().\n", pos->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->bPCount[5] != thisPos.bPCount[5])
    {
      display(pos);
      printf("ERROR %s: bPCount[5] don't match in makeMove().\n", pos->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->bPCount[6] != thisPos.bPCount[6])
    {
      display(pos);
      printf("ERROR %s: bPCount[6] don't match in makeMove().\n", pos->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->bPCount[7] != thisPos.bPCount[7])
    {
      display(pos);
      printf("ERROR %s: bPCount[7] don't match in makeMove().\n", pos->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->RotatedBoard90 != thisPos.RotatedBoard90)
    {
      displayBitBoard(thisPos.RotatedBoard90);
      printf("ERROR %s: RotatedBoard90 don't match in makeMove().\n", 
	     pos->posID);
      displayBitBoard(pos->RotatedBoard90);
      exit(EXIT_FAILURE);
    }
  if (pos->RotatedBoard45l != thisPos.RotatedBoard45l)
    {
      displayBitBoard(thisPos.RotatedBoard45l);
      printf("ERROR %s: RotatedBoard45l don't match in makeMove().\n", 
	     pos->posID);
      displayBitBoard(pos->RotatedBoard45l);
      exit(EXIT_FAILURE);
    }
  if (pos->RotatedBoard45r != thisPos.RotatedBoard45r)
    {
      displayBitBoard(thisPos.RotatedBoard45r);
      printf("ERROR %s: RotatedBoard45r don't match in makeMove().\n", 
	     pos->posID);
      displayBitBoard(pos->RotatedBoard45r);
      exit(EXIT_FAILURE);
    }
#endif
}


void unmakeMove(ui wToMove, ui ply, BB move, sPos *pos, sPlyData *plyData)
{
  ui f, t, p, c;
  char castleFlag = '-';

  ASSERT(wToMove == false || wToMove == true);
  ASSERT(ply >= 0 && ply <= maxPly);
  ASSERT(pos != NULL);
  ASSERT(plyData != NULL);
  ASSERT(!(pos->WBoard & pos->BBoard));

  f = from(move);
  t = to(move);
  p = piece(move);
  c = captured(move);
  ASSERT(f >= A1 && f <= H8 && t >= A1 && t <= H8 && f != t);
  ASSERT(p < 13);
  ASSERT(c < 13);

  if (c != none)
    {
      if (wToMove) pos->BBoard |= OneMask[t];
      else pos->WBoard |= OneMask[t];
      
      switch (c)
	{
	case wP: pos->WPBoard |= OneMask[t];  break;
	case wN: pos->WNBoard |= OneMask[t];  break;
	case wB: pos->WBBoard |= OneMask[t];  break;
	case wR: pos->WRBoard |= OneMask[t];  break;
	case wQ: pos->WQBoard |= OneMask[t];  break;
	case bP: pos->BPBoard |= OneMask[t];  break;
	case bN: pos->BNBoard |= OneMask[t];  break;
	case bB: pos->BBBoard |= OneMask[t];  break;
	case bR: pos->BRBoard |= OneMask[t];  break;
	case bQ: pos->BQBoard |= OneMask[t];  break;
	default: break;
	}
    }

  if (wToMove)
    {
      pos->WBoard |= OneMask[f];
      pos->WBoard &= NilMask[t];
    }
  else
    {
      pos->BBoard |= OneMask[f];
      pos->BBoard &= NilMask[t];
    }

  /* promoted piece */
  switch(promotedTo(move))
    {
    case none:
      break;
    case wN:
      pos->WPBoard |= OneMask[t];
      pos->WNBoard &= NilMask[t];
     break;
    case wB:
      pos->WPBoard |= OneMask[t];
      pos->WBBoard &= NilMask[t];
      break;
    case wR:
      pos->WPBoard |= OneMask[t];
      pos->WRBoard &= NilMask[t];
      break;
    case wQ:
      pos->WPBoard |= OneMask[t];
      pos->WQBoard &= NilMask[t];
      break;
    case bN:
      pos->BPBoard |= OneMask[t];
      pos->BNBoard &= NilMask[t];
      break;
    case bB:
      pos->BPBoard |= OneMask[t];
      pos->BBBoard &= NilMask[t];
      break;
    case bR:
      pos->BPBoard |= OneMask[t];
      pos->BRBoard &= NilMask[t];
      break;
    case bQ:
      pos->BPBoard |= OneMask[t];
      pos->BQBoard &= NilMask[t];
      break;
    default:
      break;
    }

  /* moved piece */
  switch(p)
    {
    case wP:
      pos->WPBoard |= OneMask[f];
      pos->WPBoard &= NilMask[t];
      if (OneMask[t] & plyData[ply-1].EP)
	{
	  pos->BPBoard |= OneMask[t-8];
	  pos->BBoard |= OneMask[t-8];
	  pos->wPCount[column(f)]++;
	  pos->wPCount[column(t)]--;
	  pos->bPCount[column(t)]++;
	}
      else if (c)
	{
	  pos->wPCount[column(f)]++;
	  pos->wPCount[column(t)]--;
	}
      if (promotedTo(move)) pos->wPCount[column(t)]++;
      break;
    case wN:
      pos->WNBoard |= OneMask[f];
      pos->WNBoard &= NilMask[t];
      break;
    case wB:
      pos->WBBoard |= OneMask[f];
      pos->WBBoard &= NilMask[t];
      break;
    case wR:
      pos->WRBoard |= OneMask[f];
      pos->WRBoard &= NilMask[t];
      break;
    case wQ:
      pos->WQBoard |= OneMask[f];
      pos->WQBoard &= NilMask[t];
      break;
    case wK:
      pos->wKSq = f;
      if (plyData[ply-1].wCanCastle > 0)
	{
	  /* long castling */
	  if (f == E1 && t == C1)
	    {
	      pos->WBoard |= OneMask[A1];
	      pos->WBoard &= NilMask[D1];
	      pos->WRBoard |= OneMask[A1];
	      pos->WRBoard &= NilMask[D1];
	      castleFlag = 'Q';
	    }
	  /* short castling */
	  else if (f == E1 && t == G1)
	    {
	      pos->WBoard |= OneMask[H1];
	      pos->WBoard &= NilMask[F1];
	      pos->WRBoard |= OneMask[H1];
	      pos->WRBoard &= NilMask[F1];
	      castleFlag = 'K';
	    }
	}
      break;
    case bP:
      pos->BPBoard |= OneMask[f];
      pos->BPBoard &= NilMask[t];
      if (OneMask[t] & plyData[ply-1].EP)
	{
	  pos->WPBoard |= OneMask[t+8];
	  pos->WBoard |= OneMask[t+8];
	  pos->bPCount[column(f)]++;
	  pos->bPCount[column(t)]--;
	  pos->wPCount[column(t)]++;
	}
      else if (c)
	{
	  pos->bPCount[column(f)]++;
	  pos->bPCount[column(t)]--;
	}
      if (promotedTo(move)) pos->bPCount[column(t)]++;
      break;
    case bN:
      pos->BNBoard |= OneMask[f];
      pos->BNBoard &= NilMask[t];
      break;
    case bB:
      pos->BBBoard |= OneMask[f];
      pos->BBBoard &= NilMask[t];
      break;
    case bR:
      pos->BRBoard |= OneMask[f];
      pos->BRBoard &= NilMask[t];
      break;
    case bQ:
      pos->BQBoard |= OneMask[f];
      pos->BQBoard &= NilMask[t];
      break;
    case bK:
      pos->bKSq = f;
      if (plyData[ply-1].bCanCastle > 0)
	{
	  /* long castling */
	  if (f == E8 && t == C8)
	    {
	      pos->BBoard |= OneMask[A8];
	      pos->BBoard &= NilMask[D8];
	      pos->BRBoard |= OneMask[A8];
	      pos->BRBoard &= NilMask[D8];
	      castleFlag = 'q';
	    }
	  /* short castling */
	  else if (f == E8 && t == G8)
	    {
	      pos->BBoard |= OneMask[H8];
	      pos->BBoard &= NilMask[F8];
	      pos->BRBoard |= OneMask[H8];
	      pos->BRBoard &= NilMask[F8];
	      castleFlag = 'k';
	    }
	}
      break;
    default:
      break;
    }

  /* captured piece */
  ASSERT(c != wK && c != bK);
  switch(c)
    {
    case none:
      break;
    case wP:
      pos->WBoard |= OneMask[t];
      pos->WPBoard |= OneMask[t];
      pos->wPCount[column(t)]++;
      break;
    case wN:
      pos->WBoard |= OneMask[t];
      pos->WNBoard |= OneMask[t];
      break;
    case wB:
      pos->WBoard |= OneMask[t];
      pos->WBBoard |= OneMask[t];
      break;
    case wR:
      pos->WBoard |= OneMask[t];
      pos->WRBoard |= OneMask[t];
      break;
    case wQ:
      pos->WBoard |= OneMask[t];
      pos->WQBoard |= OneMask[t];
      break;
    case wK:
      printf("unmakeMove: What the heck?  %u-%u\n", f, t);
      display(pos);
      printf("unmakeMove: What the heck?  wK at %u is captured.\n", t);
      break;
    case bP:
      pos->BBoard |= OneMask[t];
      pos->BPBoard |= OneMask[t];
      pos->bPCount[column(t)]++;
      break;
    case bN:
      pos->BBoard |= OneMask[t];
      pos->BNBoard |= OneMask[t];
      break;
    case bB:
      pos->BBoard |= OneMask[t];
      pos->BBBoard |= OneMask[t];
      break;
    case bR:
      pos->BBoard |= OneMask[t];
      pos->BRBoard |= OneMask[t];
      break;
    case bQ:
      pos->BBoard |= OneMask[t];
      pos->BQBoard |= OneMask[t];
      break;
    case bK:
      printf("unmakeMove: What the heck?  %u-%u\n", f, t);
      display(pos);
      printf("unmakeMove: What the heck?  bK at %u is captured.\n", t);
      break;
    default:
      break;
    }

  /* update rotated bitboards */
  pos->RotatedBoard90 |= OneMask[rotate90[f]];
  pos->RotatedBoard45l |= OneMask[rotate45l[f]];
  pos->RotatedBoard45r |= OneMask[rotate45r[f]];
  if (!c)
    {
      pos->RotatedBoard90 &= NilMask[rotate90[t]];
      pos->RotatedBoard45l &= NilMask[rotate45l[t]];
      pos->RotatedBoard45r &= NilMask[rotate45r[t]];
    }

  /* update rotated bitboards due to rook movement in castling */
  switch (castleFlag)
    {
    case '-':
      break;
    case 'K':
      pos->RotatedBoard90 |= OneMask[rotate90[H1]];
      pos->RotatedBoard45l |= OneMask[rotate45l[H1]];
      pos->RotatedBoard45r |= OneMask[rotate45r[H1]];
      pos->RotatedBoard90 &= NilMask[rotate90[F1]];
      pos->RotatedBoard45l &= NilMask[rotate45l[F1]];
      pos->RotatedBoard45r &= NilMask[rotate45r[F1]];
      break;
    case 'Q':
      pos->RotatedBoard90 |= OneMask[rotate90[A1]];
      pos->RotatedBoard45l |= OneMask[rotate45l[A1]];
      pos->RotatedBoard45r |= OneMask[rotate45r[A1]];
      pos->RotatedBoard90 &= NilMask[rotate90[D1]];
      pos->RotatedBoard45l &= NilMask[rotate45l[D1]];
      pos->RotatedBoard45r &= NilMask[rotate45r[D1]];
      break;
    case 'k':
      pos->RotatedBoard90 |= OneMask[rotate90[H8]];
      pos->RotatedBoard45l |= OneMask[rotate45l[H8]];
      pos->RotatedBoard45r |= OneMask[rotate45r[H8]];
      pos->RotatedBoard90 &= NilMask[rotate90[F8]];
      pos->RotatedBoard45l &= NilMask[rotate45l[F8]];
      pos->RotatedBoard45r &= NilMask[rotate45r[F8]];
      break;
    case 'q':
      pos->RotatedBoard90 |= OneMask[rotate90[A8]];
      pos->RotatedBoard45l |= OneMask[rotate45l[A8]];
      pos->RotatedBoard45r |= OneMask[rotate45r[A8]];
      pos->RotatedBoard90 &= NilMask[rotate90[D8]];
      pos->RotatedBoard45l &= NilMask[rotate45l[D8]];
      pos->RotatedBoard45r &= NilMask[rotate45r[D8]];
      break;
    default:
      break;
    }

  /* update rotated bitboards due to disappearing pawn in en-passant */
  if (OneMask[t] & plyData[ply-1].EP)
    {
      if (p == wP)
	{
	  pos->RotatedBoard90 |= OneMask[rotate90[t-8]];
	  pos->RotatedBoard45l |= OneMask[rotate45l[t-8]];
	  pos->RotatedBoard45r |= OneMask[rotate45r[t-8]];
	}
      else if (p == bP)
	{
	  pos->RotatedBoard90 |= OneMask[rotate90[t+8]];
	  pos->RotatedBoard45l |= OneMask[rotate45l[t+8]];
	  pos->RotatedBoard45r |= OneMask[rotate45r[t+8]];
	}
    }
}


/* Black is checking me, I want to save my king. */
ui outOfBCheck(ui ply, ui kSq, BB Checkers, BB *moveList, sPos *pos, 
	       sPlyData *plyData, sOthers *others)
{
  BB Rank = (BB) 0, Fyle = (BB) 0, Diagonal = (BB) 0;
  BB From = (BB) 0, To = (BB) 0, Them = (BB) 0, move = (BB) 0;
  ui attacker1 = 64, attacker2 = 64;

  ui capture = 1;
  ui block = 1;
  ui sq, t, c, ok;
  ui kRank, kFyle;
  int tValue, mVVLVAValue;
  ui count = 0;

  ASSERT(ply >= 0 && ply <= maxPly);
  ASSERT(kSq >= A1 && kSq <= H8);
  ASSERT(moveList != NULL);
  ASSERT(pos != NULL);
  ASSERT(plyData != NULL);
  ASSERT(others != NULL);
  ASSERT(!(pos->WBoard & pos->BBoard));

  kRank = row(kSq);
  kFyle = column(kSq);

  /* 
   * count how many pieces are attacking the king
   * if there are more than one then move king
   * else if attakers are non slider then try
   * 1. capture the attacker1
   * 2. move the king
   * else try     
   * 1. capture the attacker1
   * 2. move the king (including captures)
   * 3. block
  */

  if (bitCount(Checkers) == 2)
    {
      capture = block = 0;

      attacker1 = firstOne(Checkers);
      switch (pieceAtSq(attacker1, pos))
	{
	case bP:
	case bN:
	  break;
	case bB:
	  Diagonal = others->BMoves[attacker1] & others->BMoves[kSq];
	  break;
	case bR:
	  if (row(attacker1) == kRank) Rank = others->Row[kRank];
	  else if (column(attacker1) == kFyle) Fyle = others->Column[kFyle];
	  break;
	case bQ:
	  if (row(attacker1) == kRank) Rank = others->Row[kRank];
	  else if (column(attacker1) == kFyle) Fyle = others->Column[kFyle];
	  else Diagonal = others->BMoves[attacker1] & others->BMoves[kSq];
	  break;
	default:
	  break;
	}
      
      attacker2 = lastOne(Checkers);
      switch (pieceAtSq(attacker2, pos))
	{
	case bP:
	case bN:
	  break;
	case bB:
	  Diagonal = others->BMoves[attacker2] & others->BMoves[kSq];
	  break;
	case bR:
	  if (row(attacker2) == kRank) Rank = others->Row[kRank];
	  else if (column(attacker2) == kFyle) Fyle = others->Column[kFyle];
	  break;
	case bQ:
	  if (row(attacker2) == kRank) Rank = others->Row[kRank];
	  else if (column(attacker2) == kFyle) Fyle = others->Column[kFyle];
	  else Diagonal = others->BMoves[attacker2] & others->BMoves[kSq];
	  break;
	default:
	  break;
	}
    }
  else 
    {
      attacker1 = firstOne(Checkers);
      c = pieceAtSq(attacker1, pos);
      ASSERT(c < 13);
      switch (c)
	{
	case bP:
	case bN:
	  block = 0;
	  break;
	case bB:
	  Diagonal = others->BMoves[attacker1] & others->BMoves[kSq];
	  break;
	case bR:
	  if (row(attacker1) == kRank) Rank = others->Row[kRank];
	  else if (column(attacker1) == kFyle) Fyle = others->Column[kFyle];
	  break;
	case bQ:
	  if (row(attacker1) == kRank) Rank = others->Row[kRank];
	  else if (column(attacker1) == kFyle) Fyle = others->Column[kFyle];
	  else Diagonal = others->BMoves[attacker1] & others->BMoves[kSq];
	  break;
	default:
	  break;
	}
    }

  /* capture the attacker1 */
  if (capture)
    {
      /* pawn */
      if (pos->WPBoard)
	{
	  t = attacker1;
	  From = pos->WPBoard & others->BPCaptures[t];
	  while (From)
	    {
	      sq = firstOne(From);
	      tValue = see(1, sq, t, pos, others);
#ifdef VERIFY
	      if (t == pos->bKSq) 
		{
		  printf("bK is captured by wP in outOfBCheck().\n");
		  exit(EXIT_FAILURE);
		}
#endif
	      move = sq | (t << 6) | (wP << 12) | (c << 18);
	      if (!pinner(white, sq, pos->wKSq, pos, others))
		{
		  if (OneMask[sq] & others->Row[6])   /* promotions */
		    {
		      mVVLVAValue = qValue + tValue - pValue;
		      count++;
		      *moveList++ = move | (wQ << 24) |
			((BB)(mVVLVAValue+mateScore) << 30);
		      
		      mVVLVAValue = rValue + tValue - pValue;
		      count++;
		      *moveList++ = move | (wR << 24) |
			((BB)(mVVLVAValue+mateScore) << 30);
		      
		      mVVLVAValue = bValue + tValue - pValue;
		      count++;
		      *moveList++ = move | (wB << 24) |
			((BB)(mVVLVAValue+mateScore) << 30);
		      
		      mVVLVAValue = nValue + tValue - pValue;
		      count++;
		      *moveList++ = move | (wN << 24) |
			((BB)(mVVLVAValue+mateScore) << 30);
		    }
		  else
		    {
		      count++;
		      *moveList++ = move | ((BB)(tValue+mateScore) << 30);
		    }
		}
	      From &= NilMask[sq];
	    }

	  /* en-passant */
	  if (plyData[ply].EP)
	    {
	      t = firstOne(plyData[ply].EP);
	      if (t-attacker1 == 8)
		{
		  From = pos->WPBoard & others->BPCaptures[t];
		  while (From)
		    {
		      sq = firstOne(From);
		      move = sq | (t << 6) | (wP << 12);
                      if (!pinner(white, sq, pos->wKSq, pos, others))
			{
			  mVVLVAValue = pValue;
			  count++;
			  *moveList++ = move |
			    ((BB)(mVVLVAValue+mateScore) << 30);
			}		      
		      From &= NilMask[sq];
		    }
		}
	    }
	}

      /* knight */
      if (pos->WNBoard)
	{
	  t = attacker1;
	  From = pos->WNBoard & others->NMoves[t];
	  while (From)
	    {
	      sq = firstOne(From);
	      tValue = see(1, sq, t, pos, others);
#ifdef VERIFY
	      if (t == pos->bKSq) 
		{
		  printf("bK is captured by wN in outOfBCheck().\n");
		  exit(EXIT_FAILURE);
		}
#endif
	      move = sq | (t << 6) | (wN << 12) | (c << 18);
	      if (!pinner(white, sq, pos->wKSq, pos, others))
		{
		  count++;
		  *moveList++ = move | ((BB)(tValue+mateScore) << 30);
		}
	      From &= NilMask[sq];
	    }
	}


      /* bishop */
      if (pos->WBBoard)
	{
	  t = attacker1;
	  From = pos->WBBoard & bishopAttacks(t, pos);
	  while (From)
	    {
	      sq = firstOne(From);
	      tValue = see(1, sq, t, pos, others);
#ifdef VERIFY
	      if (t == pos->bKSq) 
		{
		  printf("bK is captured by wB in outOfBCheck().\n");
		  exit(EXIT_FAILURE);
		}
#endif
	      move = sq | (t << 6) | (wB << 12) | (c << 18);
	      if (!pinner(white, sq, pos->wKSq, pos, others))
		{
		  count++;
		  *moveList++ = move | ((BB)(tValue+mateScore) << 30);
		}
	      From &= NilMask[sq];
	    }
	}

      /* rook */
      if (pos->WRBoard)
	{
	  t = attacker1;
	  From = pos->WRBoard & rookAttacks(t, pos);
	  while (From)
	    {
	      sq = firstOne(From);
	      tValue = see(1, sq, t, pos, others);
#ifdef VERIFY
	      if (t == pos->bKSq) 
		{
		  printf("bK is captured by wR in outOfBCheck().\n");
		  exit(EXIT_FAILURE);
		}
#endif
	      move = sq | (t << 6) | (wR << 12) | (c << 18);
	      if (!pinner(white, sq, pos->wKSq, pos, others))
		{
		  count++;
		  *moveList++ = move | ((BB)(tValue+mateScore) << 30);
		}
	      From &= NilMask[sq];
	    }
	}

      /* queen */
      if (pos->WQBoard)
	{
	  t = attacker1;
	  From = pos->WQBoard & 
	    (bishopAttacks(t, pos) | rookAttacks(t, pos));
	  while (From)
	    {
	      sq = firstOne(From);
	      tValue = see(1, sq, t, pos, others);
#ifdef VERIFY
	      if (t == pos->bKSq) 
		{
		  printf("bK is captured by wQ in outOfBCheck().\n");
		  exit(EXIT_FAILURE);
		}
#endif
	      move = sq | (t << 6) | (wQ << 12) | (c << 18);
	      if (!pinner(white, sq, pos->wKSq, pos, others))
		{
		  count++;
		  *moveList++ = move | ((BB)(tValue+mateScore) << 30);
		}
	      From &= NilMask[sq];
	    }
	} 
   }


  /* block */
  if (block)
    {
      switch (c)
	{
	/* the beauty of rotated bitboard */
	case bB: 
	  Them = bishopAttacks(kSq, pos) &  
	    bishopAttacks(attacker1, pos);
	  break;
	case bR: 
	  Them = rookAttacks(kSq, pos) &  
	    rookAttacks(attacker1, pos);
	  break;
	case bQ: 
	  /* be careful here */
	  if (row(attacker1) == row(kSq) || column(attacker1) == column(kSq))
	    Them = rookAttacks(kSq, pos) &  
	      rookAttacks(attacker1, pos);
	  else
	  Them = bishopAttacks(kSq, pos) &  
	    bishopAttacks(attacker1, pos);
	  break;
	default:
	  break;
	}

      /* pawn */
      if (pos->WPBoard)
	{
	  From = pos->WPBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      if (OneMask[sq+8] & Them)
		{
		  t = sq+8;
		  move = sq | (t << 6) | (wP << 12);
	          if (!pinner(white, sq, pos->wKSq, pos, others))
		    {
		      if (OneMask[sq] & others->Row[6])
			{
			  mVVLVAValue = qValue - pValue;
			  count++;
			  *moveList++ = move | (wQ << 24) | 
			    ((BB)(mVVLVAValue+mateScore) << 30);

			  mVVLVAValue = rValue - pValue;
			  count++;
			  *moveList++ = move | (wR << 24) |
			    ((BB)(mVVLVAValue+mateScore) << 30);
			      
			  mVVLVAValue = bValue - pValue;
			  count++;
			  *moveList++ = move | (wB << 24) |
			    ((BB)(mVVLVAValue+mateScore) << 30);

			  mVVLVAValue = nValue - pValue;
			  count++;
			  *moveList++ = move | (wN << 24) |
			    ((BB)(mVVLVAValue+mateScore) << 30);
			}
		      else
			{
			  count++;
			  *moveList++ = move | ((BB)(mateScore) << 30);
			}
		    }
		}

		  if ((OneMask[sq] & others->Row[1]) && 
		      (OneMask[sq+16] & Them) &&
		      (!(OneMask[sq+8] & (pos->WBoard | pos->BBoard))))
		    {
		      t = sq+16;
		      move = sq | (t << 6) | (wP << 12);
	              if (!pinner(white, sq, pos->wKSq, pos, others))
			{
			  count++;
			  *moveList++ = move | ((BB)(mateScore) << 30);
			}
		    }


	      From &= NilMask[sq];
	    }
	}

      /* knight */
      if (pos->WNBoard)
	{
	  From = pos->WNBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      To = others->NMoves[sq] & Them;
	      while (To)
		{
		  t = lastOne(To);
		  move = sq | (t << 6) | (wN << 12);
	          if (!pinner(white, sq, pos->wKSq, pos, others))
		    {
		      count++;
		      *moveList++ = move | ((BB)(mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* bishop */
      if (pos->WBBoard)
	{
	  From = pos->WBBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      To = bishopAttacks(sq, pos) & Them;
	      while (To)
		{
		  t = lastOne(To);
		  move = sq | (t << 6) | (wB << 12);
	          if (!pinner(white, sq, pos->wKSq, pos, others))
		    {
		      count++;
		      *moveList++ = move | ((BB)(mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* rook */
      if (pos->WRBoard)
	{
	  From = pos->WRBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      To = rookAttacks(sq, pos) & Them;
	      while (To)
		{
		  t = lastOne(To);
		  move = sq | (t << 6) | (wR << 12);
	          if (!pinner(white, sq, pos->wKSq, pos, others))
		    {
		      count++;
		      *moveList++ = move | ((BB)(mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* queen */
      if (pos->WQBoard)
	{
	  From = pos->WQBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      To = (rookAttacks(sq, pos) | 
		    bishopAttacks(sq, pos)) 
		& Them;
	      while (To)
		{
		  t = lastOne(To);
		  move = sq | (t << 6) | (wQ << 12);
	          if (!pinner(white, sq, pos->wKSq, pos, others))
		    {
		      count++;
		      *moveList++ = move | ((BB)(mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}
    }


  /* move the king, including captures, should be tried in all cases */
  sq = kSq;

  To = others->KMoves[sq] & ~pos->WBoard;
  while (To)
    {
      t = lastOne(To);
      c = pieceAtSq(t, pos);
      ASSERT(c < 13);
      move = sq | (t << 6) | (wK << 12) | (c << 18);
      ok = true;
      if (t != attacker1 && t != attacker2)
	{
	  if ((Rank | Fyle | Diagonal) & OneMask[t]) ok = false; 
	}
      if (ok && !(getBAttacker(t, pos, &plyData[ply], others) & 2))
	{
	  count++;
	  *moveList++ = move | ((BB)(value[c]+mateScore) << 30);
	}
      To &= NilMask[t];
    }

  return count;
}


/* White is checking me, I bant to save my king. */
ui outOfWCheck(ui ply, ui kSq, BB Checkers, BB *moveList, sPos *pos, 
	       sPlyData *plyData, sOthers *others)
{
  BB Rank = (BB) 0, Fyle = (BB) 0, Diagonal = (BB) 0;
  BB From = (BB) 0, To = (BB) 0, Them = (BB) 0, move = (BB) 0;
  ui attacker1 = 64, attacker2 = 64;

  ui capture = 1;
  ui block = 1;
  ui sq, t, c, ok;
  ui kRank, kFyle;
  int tValue, mVVLVAValue;
  ui count = 0;

  ASSERT(ply >= 0 && ply <= maxPly);
  ASSERT(kSq >= A1 && kSq <= H8);
  ASSERT(moveList != NULL);
  ASSERT(pos != NULL);
  ASSERT(plyData != NULL);
  ASSERT(others != NULL);
  ASSERT(!(pos->WBoard & pos->BBoard));

  kRank = row(kSq);
  kFyle = column(kSq);

  /* 
   * count how many pieces are attacking the king
   * if there are more than one then move king
   * else if attakers are non slider then try
   * 1. capture the attacker1
   * 2. move the king
   * else try     
   * 1. capture the attacker1
   * 2. move the king (including captures)
   * 3. block
  */

  if (bitCount(Checkers) == 2)
    {
      capture = block = 0;

      attacker1 = firstOne(Checkers);
      switch (pieceAtSq(attacker1, pos))
	{
	case wP:
	case wN:
	  break;
	case wB:
	  Diagonal = others->BMoves[attacker1] & others->BMoves[kSq];
	  break;
	case wR:
	  if (row(attacker1) == kRank) Rank = others->Row[kRank];
	  else if (column(attacker1) == kFyle) Fyle = others->Column[kFyle];
	  break;
	case wQ:
	  if (row(attacker1) == kRank) Rank = others->Row[kRank];
	  else if (column(attacker1) == kFyle) Fyle = others->Column[kFyle];
	  else Diagonal = others->BMoves[attacker1] & others->BMoves[kSq];
	  break;
	default:
	  break;
	}
      
      attacker2 = lastOne(Checkers);
      switch (pieceAtSq(attacker2, pos))
	{
	case wP:
	case wN:
	  break;
	case wB:
	  Diagonal = others->BMoves[attacker2] & others->BMoves[kSq];
	  break;
	case wR:
	  if (row(attacker2) == kRank) Rank = others->Row[kRank];
	  else if (column(attacker2) == kFyle) Fyle = others->Column[kFyle];
	  break;
	case wQ:
	  if (row(attacker2) == kRank) Rank = others->Row[kRank];
	  else if (column(attacker2) == kFyle) Fyle = others->Column[kFyle];
	  else Diagonal = others->BMoves[attacker2] & others->BMoves[kSq];
	  break;
	default:
	  break;
	}
    }
  else 
    {
      attacker1 = lastOne(Checkers);
      c = pieceAtSq(attacker1, pos);
      ASSERT(c < 13);
      switch (c)
	{
	case wP:
	case wN:
	  block = 0;
	  break;
	case wB:
	  Diagonal = others->BMoves[attacker1] & others->BMoves[kSq];
	  break;
	case wR:
	  if (row(attacker1) == kRank) Rank = others->Row[kRank];
	  else if (column(attacker1) == kFyle) Fyle = others->Column[kFyle];
	  break;
	case wQ:
	  if (row(attacker1) == kRank) Rank = others->Row[kRank];
	  else if (column(attacker1) == kFyle) Fyle = others->Column[kFyle];
	  else Diagonal = others->BMoves[attacker1] & others->BMoves[kSq];
	  break;
	default:
	  break;
	}
    }

  /* capture the attacker1 */
  if (capture)
    {
      /* pawn */
      if (pos->BPBoard)
	{
	  t = attacker1;
	  From = pos->BPBoard & others->WPCaptures[t];
	  while (From)
	    {
	      sq = lastOne(From);
	      tValue = see(0, sq, t, pos, others);
#ifdef VERIFY
	      if (t == pos->wKSq) 
		{
		  printf("wK is captured wy bP in outOfWCheck().\n");
		  exit(EXIT_FAILURE);
		}
#endif
	      move = sq | (t << 6) | (bP << 12) | (c << 18);
	      if (!pinner(black, sq, pos->bKSq, pos, others))
		{
		  if (OneMask[sq] & others->Row[1])   /* promotions */
		    {
		      mVVLVAValue = qValue + tValue - pValue;
		      count++;
		      *moveList++ = move | (bQ << 24) |
			((BB)(mVVLVAValue+mateScore) << 30);
		      
		      mVVLVAValue = rValue + tValue - pValue;
		      count++;
		      *moveList++ = move | (bR << 24) |
			((BB)(mVVLVAValue+mateScore) << 30);
		      
		      mVVLVAValue = bValue + tValue - pValue;
		      count++;
		      *moveList++ = move | (bB << 24) |
			((BB)(mVVLVAValue+mateScore) << 30);
		      
		      mVVLVAValue = nValue + tValue - pValue;
		      count++;
		      *moveList++ = move | (bN << 24) |
			((BB)(mVVLVAValue+mateScore) << 30);
		    }
		  else
		    {
		      count++;
		      *moveList++ = move | ((BB)(tValue+mateScore) << 30);
		    }
		}
	      From &= NilMask[sq];
	    }

	  /* en-passant */
	  if (plyData[ply].EP)
	    {
	      t = lastOne(plyData[ply].EP);
	      if (attacker1-t == 8)
		{
		  From = pos->BPBoard & others->WPCaptures[t];
		  while (From)
		    {
		      sq = lastOne(From);
		      move = sq | (t << 6) | (bP << 12);
	              if (!pinner(black, sq, pos->bKSq, pos, others))
			{
			  mVVLVAValue = pValue;
			  count++;
			  *moveList++ = move |
			    ((BB)(mVVLVAValue+mateScore) << 30);
			}		      
		      From &= NilMask[sq];
		    }
		}
	    }
	}

      /* knight */
      if (pos->BNBoard)
	{
	  t = attacker1;
	  From = pos->BNBoard & others->NMoves[t];
	  while (From)
	    {
	      sq = lastOne(From);
	      tValue = see(0, sq, t, pos, others);
#ifdef VERIFY
	      if (t == pos->wKSq) 
		{
		  printf("wK is captured wy bN in outOfWCheck().\n");
		  exit(EXIT_FAILURE);
		}
#endif
	      move = sq | (t << 6) | (bN << 12) | (c << 18);
	      if (!pinner(black, sq, pos->bKSq, pos, others))
		{
		  count++;
		  *moveList++ = move | ((BB)(tValue+mateScore) << 30);
		}
	      From &= NilMask[sq];
	    }
	}


      /* bishop */
      if (pos->BBBoard)
	{
	  t = attacker1;
	  From = pos->BBBoard & bishopAttacks(t, pos);
	  while (From)
	    {
	      sq = lastOne(From);
	      tValue = see(0, sq, t, pos, others);
#ifdef VERIFY
	      if (t == pos->wKSq) 
		{
		  printf("wK is captured by bB in outOfWCheck().\n");
		  exit(EXIT_FAILURE);
		}
#endif
	      move = sq | (t << 6) | (bB << 12) | (c << 18);
	      if (!pinner(black, sq, pos->bKSq, pos, others))
		{
		  count++;
		  *moveList++ = move | ((BB)(tValue+mateScore) << 30);
		}
	      From &= NilMask[sq];
	    }
	}

      /* rook */
      if (pos->BRBoard)
	{
	  t = attacker1;
	  From = pos->BRBoard & rookAttacks(t, pos);
	  while (From)
	    {
	      sq = lastOne(From);
	      tValue = see(0, sq, t, pos, others);
#ifdef VERIFY
	      if (t == pos->wKSq) 
		{
		  printf("wK is captured wy bR in outOfWCheck().\n");
		  exit(EXIT_FAILURE);
		}
#endif
	      move = sq | (t << 6) | (bR << 12) | (c << 18);
	      if (!pinner(black, sq, pos->bKSq, pos, others))
		{
		  count++;
		  *moveList++ = move | ((BB)(tValue+mateScore) << 30);
		}
	      From &= NilMask[sq];
	    }
	}

      /* queen */
      if (pos->BQBoard)
	{
	  t = attacker1;
	  From = pos->BQBoard & 
	    (bishopAttacks(t, pos) | rookAttacks(t, pos));
	  while (From)
	    {
	      sq = lastOne(From);
	      tValue = see(0, sq, t, pos, others);
#ifdef VERIFY
	      if (t == pos->wKSq) 
		{
		  printf("wK is captured wy bQ in outOfWCheck().\n");
		  exit(EXIT_FAILURE);
		}
#endif
	      move = sq | (t << 6) | (bQ << 12) | (c << 18);
	      if (!pinner(black, sq, pos->bKSq, pos, others))
		{
		  count++;
		  *moveList++ = move | ((BB)(tValue+mateScore) << 30);
		}
	      From &= NilMask[sq];
	    }
	} 
   }


  /* block */
  if (block)
    {
      switch (c)
	{
	/* the weauty of rotated witwoard */
	case wB: 
	  Them = bishopAttacks(kSq, pos) &  
	    bishopAttacks(attacker1, pos);
	  break;
	case wR: 
	  Them = rookAttacks(kSq, pos) &  
	    rookAttacks(attacker1, pos);
	  break;
	case wQ: 
	  /* we careful here */
	  if (row(attacker1) == row(kSq) || column(attacker1) == column(kSq))
	    Them = rookAttacks(kSq, pos) &  
	      rookAttacks(attacker1, pos);
	  else
	  Them = bishopAttacks(kSq, pos) &  
	    bishopAttacks(attacker1, pos);
	  break;
	default:
	  break;
	}

      /* pawn */
      if (pos->BPBoard)
	{
	  From = pos->BPBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      if (OneMask[sq-8] & Them)
		{
		  t = sq-8;
		  move = sq | (t << 6) | (bP << 12);
  	          if (!pinner(black, sq, pos->bKSq, pos, others))
		    {
		      if (OneMask[sq] & others->Row[1])
			{
			  mVVLVAValue = qValue - pValue;
			  count++;
			  *moveList++ = move | (bQ << 24) | 
			    ((BB)(mVVLVAValue+mateScore) << 30);

			  mVVLVAValue = rValue - pValue;
			  count++;
			  *moveList++ = move | (bR << 24) |
			    ((BB)(mVVLVAValue+mateScore) << 30);
			      
			  mVVLVAValue = bValue - pValue;
			  count++;
			  *moveList++ = move | (bB << 24) |
			    ((BB)(mVVLVAValue+mateScore) << 30);

			  mVVLVAValue = nValue - pValue;
			  count++;
			  *moveList++ = move | (bN << 24) |
			    ((BB)(mVVLVAValue+mateScore) << 30);
			}
		      else
			{
			  count++;
			  *moveList++ = move | ((BB)(mateScore) << 30);
			}
		    }
		}

		  if ((OneMask[sq] & others->Row[6]) && 
		      (OneMask[sq-16] & Them) &&
		      (!(OneMask[sq-8] & (pos->WBoard | pos->BBoard))))
		    {
		      t = sq-16;
		      move = sq | (t << 6) | (bP << 12);
  	              if (!pinner(black, sq, pos->bKSq, pos, others))
			{
			  count++;
			  *moveList++ = move | ((BB)(mateScore) << 30);
			}
		    }


	      From &= NilMask[sq];
	    }
	}

      /* knight */
      if (pos->BNBoard)
	{
	  From = pos->BNBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      To = others->NMoves[sq] & Them;
	      while (To)
		{
		  t = firstOne(To);
		  move = sq | (t << 6) | (bN << 12);
  	          if (!pinner(black, sq, pos->bKSq, pos, others))
		    {
		      count++;
		      *moveList++ = move | ((BB)(mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* bishop */
      if (pos->BBBoard)
	{
	  From = pos->BBBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      To = bishopAttacks(sq, pos) & Them;
	      while (To)
		{
		  t = firstOne(To);
		  move = sq | (t << 6) | (bB << 12);
  	          if (!pinner(black, sq, pos->bKSq, pos, others))
		    {
		      count++;
		      *moveList++ = move | ((BB)(mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* rook */
      if (pos->BRBoard)
	{
	  From = pos->BRBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      To = rookAttacks(sq, pos) & Them;
	      while (To)
		{
		  t = firstOne(To);
		  move = sq | (t << 6) | (bR << 12);
  	          if (!pinner(black, sq, pos->bKSq, pos, others))
		    {
		      count++;
		      *moveList++ = move | ((BB)(mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* queen */
      if (pos->BQBoard)
	{
	  From = pos->BQBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      To = (rookAttacks(sq, pos) | 
		    bishopAttacks(sq, pos)) 
		& Them;
	      while (To)
		{
		  t = firstOne(To);
		  move = sq | (t << 6) | (bQ << 12);
  	          if (!pinner(black, sq, pos->bKSq, pos, others))
		    {
		      count++;
		      *moveList++ = move | ((BB)(mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}
    }

  /* move the king, including captures, should be tried in all cases */
  sq = kSq;

  To = others->KMoves[sq] & ~pos->BBoard;
  while (To)
    {
      t = firstOne(To);
      c = pieceAtSq(t, pos);
      ASSERT(c < 13);
      move = sq | (t << 6) | (bK << 12) | (c << 18);
      ok = true;
      if (t != attacker1 && t != attacker2)
	{
	  if ((Rank | Fyle | Diagonal) & OneMask[t]) ok = false; 
	}
      if (ok && !(getWAttacker(t, pos, &plyData[ply], others) & 2))
	{
	  count++;
	  *moveList++ = move | ((BB)(value[c]+mateScore) << 30);
	}
      To &= NilMask[t];
    }

  return count;
}


/* Make moves to a certain depth to calculate how many legal positions there 
 * are.  Useful for checking the correctness and speed of move generation.
 * These are the verified depth versus perft numbers from
 * http://www.albert.nu/default.asp?sub=programs/default.asp?sub=dperft/ \
 *        main.htm
 * 1  	20  	   	
 * 2 	400 	  	
 * 3 	8902 	  	
 * 4 	197281 	  	
 * 5 	4865609 	
 * 6 	119060324 	
 * 7 	3195901860 	
 * 8 	84998978956 	
 * 9 	2439530234167 	
 * 10 	69352859712417  
 */
void perft(ui wToMove, ui ply, ui depth, sPos *pos, sPlyData *plyData, 
	   sOthers *others)
{
  BB winningCapture[32], nonCapture[128], allMoves[128];
  ui i, noOfCaptures, noOfNonCaptures, nMoves;

  ASSERT(wToMove == false || wToMove == true);
  ASSERT(ply >= 0 && ply <= maxPly);
  ASSERT(depth >= 0 && depth <= maxPly);
  ASSERT(pos != NULL);
  ASSERT(plyData != NULL);
  ASSERT(others != NULL);
  ASSERT(!(pos->WBoard & pos->BBoard));

  /* try captures */
  noOfCaptures = captures(wToMove, ply, winningCapture, pos, 
			  plyData, others, 0);
  /* try non-captures */
  noOfNonCaptures = nonCaptures(wToMove, ply, nonCapture, pos, 
				plyData, others, 0);
  if (noOfCaptures+noOfNonCaptures == 0) return;

  /* Copying all moves to allMoves[]. */
  nMoves = 0;
  if (noOfCaptures > 0)
    for (i = 0; i < noOfCaptures; i++)
      allMoves[nMoves++] = winningCapture[i];
  if (noOfNonCaptures > 0)
    for (i = 0; i < noOfNonCaptures; i++)
      allMoves[nMoves++] = nonCapture[i];

  /* go deeper */
  for (i = 0; i < nMoves; i++)
    {
      makeMove(wToMove, ply+1, allMoves[i], pos, plyData, others);
      if (depth == 1) pos->sNodes++;
      else perft(wToMove^1, ply+1, depth-1, pos, plyData, others);
      unmakeMove(wToMove, ply+1, allMoves[i], pos, plyData);
    }

  return;
}
