/* aliases.h */
/* Last modified: April 19, 2006 */


#include <stdlib.h>

/* type definitions */
typedef unsigned long long BB;
typedef unsigned int ui;
typedef unsigned char uc;
typedef struct HashTable sHash;
typedef struct Others sOthers;
typedef struct Position sPos;
typedef struct PlyData sPlyData;
typedef struct TimeCheck sTC;


/* from fruit */
#if DEBUG
#  define ASSERT(a) { if (!(a)) { \
  printf("ERROR in %s line %d: \"" #a "\" failed\n",__FILE__,__LINE__); \
    exit(EXIT_FAILURE); }}
#else
#  define ASSERT(a)
#endif


#if !defined WINDOWS
/* common utility */
#define min(a,b)  (((a) > (b)) ? (b):(a))
#define max(a,b)  (((a) < (b)) ? (b):(a))
#endif

/* board utilities */
#define row(a) ((a) >> 3)
#define column(a) ((a) & 7)
#define rowDistance(f,t) (abs((int)(column(t)-column(f))))
#define columnDistance(f,t) (abs((int)(row(t)-row(f))))
#define distance(f,t) (max(rowDistance((f),(t)),columnDistance((f),(t))))

/* in move */
#define from(a) ((ui)((a) & 63))                                /*  6 bits */
#define to(a) ((ui)(((a) >> 6) & 63))                           /*  6 bits */
#define piece(a) ((ui)(((a) >> 12) & 63))                       /*  6 bits */
#define captured(a) ((ui)(((a) >> 18) & 63))                    /*  6 bits */
#define promotedTo(a) ((ui)(((a) >> 24) & 63))                  /*  6 bits */
#define score(a) ((int)((((a) >> 30) & 0xFFFF) - mateScore))    /* 16 bits */
#define draw(a) ((ui)(((a) >> 46) & 7))                         /*  3 bits */

/* for *Attacks() */
#define shiftToRow(B,i) ((B)>>(8*(i)))&255
/* Squares attacked by Rook at sq horizontally */
#define hRookAttacks(sq,pos) \
(RankAttack[(sq)][shiftToRow((pos)->WBoard | (pos)->BBoard, row((sq)))])
/* Squares attacked by Rook at sq vertically */
#define vRookAttacks(sq,pos) \
(RankAttack90[(sq)][shiftToRow((pos)->RotatedBoard90, column((sq)))])
/* Squares attacked by Rook at sq */
#define rookAttacks(sq,pos) \
((RankAttack[(sq)][shiftToRow((pos)->WBoard | \
    (pos)->BBoard, row((sq)))]) | \
 (RankAttack90[(sq)] \
    [shiftToRow((pos)->RotatedBoard90, column((sq)))]))
/* Squares attacked by Bishop at sq along left diagonal */
#define lBishopAttacks(sq,pos) \
(RankAttack45l[(sq)] \
   [shiftToRow((pos)->RotatedBoard45l, rank45l[(sq)])])
/* Squares attacked by Bishop at sq along right diagonal */
#define rBishopAttacks(sq,pos) \
(RankAttack45r[(sq)] \
   [shiftToRow((pos)->RotatedBoard45r, rank45r[(sq)])])
/* Squares attacked by Bishop at sq */
#define bishopAttacks(sq,pos) \
((RankAttack45l[(sq)] \
    [shiftToRow((pos)->RotatedBoard45l, rank45l[(sq)])]) | \
 (RankAttack45r[(sq)] \
    [shiftToRow((pos)->RotatedBoard45r, rank45r[(sq)])]))


/* for end game table base */
/*
  Service macro - initialize squares of the particular piece as well as
  counter for that piece. Note: dual initialization saves some time when
  TB is present, but waste for non-present TB. If we often will call
  probing function for an absent TB, maybe we shall split that code.
*/

#define  vInitSqCtr(rgCtr, rgSquares, i, bitboard) {             \
  int  cPieces=0;                                                \
  BB Temp=(bitboard);                                            \
  while (Temp) {                                                 \
    const squaret sq=firstOne (Temp);                            \
    (rgSquares)[(i)*C_PIECES+cPieces]=sq;                        \
    cPieces++;                                                   \
    Temp &= NilMask[sq];                                         \
  }                                                              \
  (rgCtr)[(i)]=cPieces;                                          \
}
/* values.h */
/* Last modified: June 28, 2006 */


#define version "060628"


/* logic and characters */
#define true 1
#define false 0
#define newLine 10

/* maximum number of words sent to runCommand() */
#define WORDS 12

/* except mateScore, the others are for game listings */
#define infinity     32768
#define mateScore    30000
#define almostScore  29000
#define drawScore    25000
#define forcedScore  20000
#define onlyScore    15000
#define openingScore 10000


/* piece values */
#define pValue 100
#define nValue 300
#define bValue 301         /* mainly so that we can tell bishop from knight */
#define rValue 500
#define qValue 900
#define endGameValue 1400  /* rValue + qValue */
#define middleGameValue 2227  /* value of all pieces minus queen */
#define kValue mateScore


/* search related */
#define NPMS 0.3  /* estimated nodes per microsecond */
#define maxMove 256
#define minPly 4
#define maxPly 64
#define easyScore 200
#define easyFactor 3
#define halfWindow 40
#define drawCountLimit 25
#define drawOfferRepeat 5
#define drawMargin 25
#define resignCountLimit 5
#define resignValue 900
#define pruneMargin 300
#define allNode -1
#define PVNode 0
#define cutNode 1
#define nonEndgame 0
#define endgame 1


/* eval related */
#define BONUS00    3 /* knight mobility bonus per possible move */
#define BONUS01  100 /* knight unassailable at D5/E5/D4/E4 */
#define BONUS02   10 /* knight protected at D5/E5/D4/E4 */
#define BONUS03  200 /* knight unassailable at D6/E6/D3/E3 */
#define BONUS04   20 /* knight protected at D6/E6/D3/E3 */
#define BONUS05    3 /* bishop mobility bonus per possible move */
#define BONUS06   10 /* semi open file */
#define BONUS07   35 /* rook on 7th rank */
#define BONUS08   10 /* queen on 7th rank */
#define BONUS09    3 /* king good neighborhood bonus per friendly piece */
#define BONUS10   40 /* rook behind passed pawn */
#define BONUS11   10 /* pawn pair */
#define BONUS12   25 /* both side pawns, bishop versus knight */
#define BONUS13   15 /* winning in end game, king chases opponent king */
#define BONUS14   10 /* king sheltered from pesky rook */
#define BONUS15   20 /* open file */

#define PENALTY00    25 /* piled pawn */
#define PENALTY01    20 /*. pawn isolani */
#define PENALTY02    30 /* bishop blocking D2/E2/D7/E7 */
#define PENALTY03    25 /* rook obstructed by king */
#define PENALTY04    25 /* rook obstructed by pawn */
#define PENALTY05    10 /* king too advanced penalty per rank */
#define PENALTY06    10 /* missing pawn 1 in front of king */
#define PENALTY07     7 /* missing pawn 2 in front of king */
#define PENALTY08     3 /* missing pawn 3 in front of king */
#define PENALTY09     5 /* missing pawn 1 outside king */
#define PENALTY10     4 /* missing pawn 2 outside king */
#define PENALTY11     2 /* missing pawn 3 outside king */
#define PENALTY12     3 /* missing pawn 1 inside king */
#define PENALTY13     2 /* missing pawn 2 inside king */
#define PENALTY14     1 /* missing pawn 3 inside king */
#define PENALTY15    10 /* B2/G2/B7/G7 hole */
#define PENALTY16    20 /* losing castling right */
#define PENALTY17    25 /* prevent frivolous minor piece sacrifice for two pawns */
#define PENALTY18    10 /* king in open file */
#define PENALTY19     5 /* outside king is open file */
#define PENALTY20     3 /* inside king is open file */
#define PENALTY21    70 /* double bonus reduction due to passed pawn */


/* For SAN notation */
#define rxx "12345678"
#define cxx "abcdefgh"
#define sanChars   " PNBRQKPNBRQK"
#define boardChars "-FNBRQKpnbrqk"


/* "E4" is easier to be remembered than "28" */
#define A1  0
#define B1  1
#define C1  2
#define D1  3
#define E1  4
#define F1  5
#define G1  6
#define H1  7
#define A2  8
#define B2  9
#define C2 10
#define D2 11
#define E2 12
#define F2 13
#define G2 14
#define H2 15
#define A3 16
#define B3 17
#define C3 18
#define D3 19
#define E3 20
#define F3 21
#define G3 22
#define H3 23
#define A4 24
#define B4 25
#define C4 26
#define D4 27
#define E4 28
#define F4 29
#define G4 30
#define H4 31
#define A5 32
#define B5 33
#define C5 34
#define D5 35
#define E5 36
#define F5 37
#define G5 38
#define H5 39
#define A6 40
#define B6 41
#define C6 42
#define D6 43
#define E6 44
#define F6 45
#define G6 46
#define H6 47
#define A7 48
#define B7 49
#define C7 50
#define D7 51
#define E7 52
#define F7 53
#define G7 54
#define H7 55
#define A8 56
#define B8 57
#define C8 58
#define D8 59
#define E8 60
#define F8 61
#define G8 62
#define H8 63

/* colors */
#define white 0
#define black 1

/* pieces */
#define none 0
#define wP 1
#define wN 2
#define wB 3
#define wR 4
#define wQ 5
#define wK 6
#define bP 7
#define bN 8
#define bB 9
#define bR 10
#define bQ 11
#define bK 12


/* flags */
#define flagDraw3  1
#define flagDraw13 13
#define flagDraw50 2


/* hash tables */
#define bookSize 65535        /* 2^16 - 1*/
#define EXACT        1
#define FAIL_LOW     2
#define FAIL_HIGH    4
/* constants.h */
/* Last modified: February 22, 2006 */


static const BB OneMask[64] = {((BB) 01),
                        ((BB) 01) << 1,
                        ((BB) 01) << 2,
                        ((BB) 01) << 3,
                        ((BB) 01) << 4,
                        ((BB) 01) << 5,
                        ((BB) 01) << 6,
                        ((BB) 01) << 7,
                        ((BB) 01) << 8,
                        ((BB) 01) << 9,
                        ((BB) 01) << 10,
                        ((BB) 01) << 11,
                        ((BB) 01) << 12,
                        ((BB) 01) << 13,
                        ((BB) 01) << 14,
                        ((BB) 01) << 15,
                        ((BB) 01) << 16,
                        ((BB) 01) << 17,
                        ((BB) 01) << 18,
                        ((BB) 01) << 19,
                        ((BB) 01) << 20,
                        ((BB) 01) << 21,
                        ((BB) 01) << 22,
                        ((BB) 01) << 23,
                        ((BB) 01) << 24,
                        ((BB) 01) << 25,
                        ((BB) 01) << 26,
                        ((BB) 01) << 27,
                        ((BB) 01) << 28,
                        ((BB) 01) << 29,
                        ((BB) 01) << 30,
                        ((BB) 01) << 31,
                        ((BB) 01) << 32,
                        ((BB) 01) << 33,
                        ((BB) 01) << 34,
                        ((BB) 01) << 35,
                        ((BB) 01) << 36,
                        ((BB) 01) << 37,
                        ((BB) 01) << 38,
                        ((BB) 01) << 39,
                        ((BB) 01) << 40,
                        ((BB) 01) << 41,
                        ((BB) 01) << 42,
                        ((BB) 01) << 43,
                        ((BB) 01) << 44,
                        ((BB) 01) << 45,
                        ((BB) 01) << 46,
                        ((BB) 01) << 47,
                        ((BB) 01) << 48,
                        ((BB) 01) << 49,
                        ((BB) 01) << 50,
                        ((BB) 01) << 51,
                        ((BB) 01) << 52,
                        ((BB) 01) << 53,
                        ((BB) 01) << 54,
                        ((BB) 01) << 55,
                        ((BB) 01) << 56,
                        ((BB) 01) << 57,
                        ((BB) 01) << 58,
                        ((BB) 01) << 59,
                        ((BB) 01) << 60,
                        ((BB) 01) << 61,
                        ((BB) 01) << 62,
                        ((BB) 01) << 63};

static const BB NilMask[64] = {~((BB) 01),
                        ~(((BB) 01) << 1),
                        ~(((BB) 01) << 2),
                        ~(((BB) 01) << 3),
                        ~(((BB) 01) << 4),
                        ~(((BB) 01) << 5),
                        ~(((BB) 01) << 6),
                        ~(((BB) 01) << 7),
                        ~(((BB) 01) << 8),
                        ~(((BB) 01) << 9),
                        ~(((BB) 01) << 10),
                        ~(((BB) 01) << 11),
                        ~(((BB) 01) << 12),
                        ~(((BB) 01) << 13),
                        ~(((BB) 01) << 14),
                        ~(((BB) 01) << 15),
                        ~(((BB) 01) << 16),
                        ~(((BB) 01) << 17),
                        ~(((BB) 01) << 18),
                        ~(((BB) 01) << 19),
                        ~(((BB) 01) << 20),
                        ~(((BB) 01) << 21),
                        ~(((BB) 01) << 22),
                        ~(((BB) 01) << 23),
                        ~(((BB) 01) << 24),
                        ~(((BB) 01) << 25),
                        ~(((BB) 01) << 26),
                        ~(((BB) 01) << 27),
                        ~(((BB) 01) << 28),
                        ~(((BB) 01) << 29),
                        ~(((BB) 01) << 30),
                        ~(((BB) 01) << 31),
                        ~(((BB) 01) << 32),
                        ~(((BB) 01) << 33),
                        ~(((BB) 01) << 34),
                        ~(((BB) 01) << 35),
                        ~(((BB) 01) << 36),
                        ~(((BB) 01) << 37),
                        ~(((BB) 01) << 38),
                        ~(((BB) 01) << 39),
                        ~(((BB) 01) << 40),
                        ~(((BB) 01) << 41),
                        ~(((BB) 01) << 42),
                        ~(((BB) 01) << 43),
                        ~(((BB) 01) << 44),
                        ~(((BB) 01) << 45),
                        ~(((BB) 01) << 46),
                        ~(((BB) 01) << 47),
                        ~(((BB) 01) << 48),
                        ~(((BB) 01) << 49),
                        ~(((BB) 01) << 50),
                        ~(((BB) 01) << 51),
                        ~(((BB) 01) << 52),
                        ~(((BB) 01) << 53),
                        ~(((BB) 01) << 54),
                        ~(((BB) 01) << 55),
                        ~(((BB) 01) << 56),
                        ~(((BB) 01) << 57),
                        ~(((BB) 01) << 58),
                        ~(((BB) 01) << 59),
                        ~(((BB) 01) << 60),
                        ~(((BB) 01) << 61),
                        ~(((BB) 01) << 62),
                        ~(((BB) 01) << 63)};

static const uc rotate90[64] = {0,  8, 16, 24, 32, 40, 48, 56,
			  1,  9, 17, 25, 33, 41, 49, 57,
			  2, 10, 18, 26, 34, 42, 50, 58,
			  3, 11, 19, 27, 35, 43, 51, 59,
			  4, 12, 20, 28, 36, 44, 52, 60,
			  5, 13, 21, 29, 37, 45, 53, 61,
			  6, 14, 22, 30, 38, 46, 54, 62,
			  7, 15, 23, 31, 39, 47, 55, 63};

static const uc rotate45l[64] = {0,  8, 16, 24, 32, 40, 48, 56,
			   9, 17, 25, 33, 41, 49, 57,  1,
			   18, 26, 34, 42, 50, 58,  2, 10,
			   27, 35, 43, 51, 59,  3, 11, 19,
			   36, 44, 52, 60,  4, 12, 20, 28,
			   45, 53, 61,  5, 13, 21, 29, 37,
			   54, 62,  6, 14, 22, 30, 38, 46,
			   63,  7, 15, 23, 31, 39, 47, 55};

static const uc rotate45r[64] = {56, 48, 40, 32, 24, 16,  8,  0,
			   1, 57, 49, 41, 33, 25, 17,  9,
			   10,  2, 58, 50, 42, 34, 26, 18,
			   19, 11,  3, 59, 51, 43, 35, 27,
			   28, 20, 12,  4, 60, 52, 44, 36,
			   37, 29, 21, 13,  5, 61, 53, 45,
			   46, 38, 30, 22, 14,  6, 62, 54,
			   55, 47, 39, 31, 23, 15,  7, 63};

static const uc rank45l[64] = {0, 1, 2, 3, 4, 5, 6, 7,
			 1, 2, 3, 4, 5, 6, 7, 0,
			 2, 3, 4, 5, 6, 7, 0, 1,
			 3, 4, 5, 6, 7, 0, 1, 2,
			 4, 5, 6, 7, 0, 1, 2, 3,
			 5, 6, 7, 0, 1, 2, 3, 4,
			 6, 7, 0, 1, 2, 3, 4, 5,
			 7, 0, 1, 2, 3, 4, 5, 6};

static const uc rank45r[64] = {7, 6, 5, 4, 3, 2, 1, 0,
			  0, 7, 6, 5, 4, 3, 2, 1,
			  1, 0, 7, 6, 5, 4, 3, 2,
			  2, 1, 0, 7, 6, 5, 4, 3,
			  3, 2, 1, 0, 7, 6, 5, 4,
			  4, 3, 2, 1, 0, 7, 6, 5,
			  5, 4, 3, 2, 1, 0, 7, 6,
			  6, 5, 4, 3, 2, 1, 0, 7};

static const ui wPGoal[8] = {A8, B8, C8, D8, E8, F8, G8, H8};
static const ui bPGoal[8] = {A1, B1, C1, D1, E1, F1, G1, H1};

static const int fMargin[64] = 
  {300,   300,   300,   300,   300,   300,   300,   300,
   250,   200,   150,   100,    50,     0,   -50,  -100,
  -150,  -200,  -250,  -300,  -350,  -400,  -450,  -500,
  -550,  -600,  -650,  -700,  -750,  -800,  -850,  -900,
  -950, -1000, -1050, -1100, -1150, -1200, -1250, -1300,
 -1350, -1400, -1450, -1500, -1550, -1600, -1650, -1700,
 -1750, -1800, -1850, -1900, -1950, -2000, -2050, -2100,
 -2150, -2200, -2250, -2300, -2350, -2400, -2450, -2500};

static const BB ScoreBoard = 
           (((BB) 01) << 30) | (((BB) 01) << 31) | (((BB) 01) << 32) | 
           (((BB) 01) << 33) | (((BB) 01) << 34) | (((BB) 01) << 35) | 
           (((BB) 01) << 36) | (((BB) 01) << 37) | (((BB) 01) << 38) | 
           (((BB) 01) << 39) | (((BB) 01) << 40) | (((BB) 01) << 41) | 
           (((BB) 01) << 42) | (((BB) 01) << 43) | (((BB) 01) << 44) |
           (((BB) 01) << 45);

static const int value[13] = {0, pValue, nValue, bValue, rValue, qValue, 
			      kValue, pValue, nValue, bValue, rValue, qValue, 
			      kValue};

#ifdef BONUSES_PENALTIES
extern int PBonus, NBonus, BBonus, RBonus, QBonus, KBonus;
extern int PPenalty, NPenalty, BPenalty, RPenalty, QPenalty, KPenalty;
extern int PEndBonus, NEndBonus, BEndBonus, REndBonus, QEndBonus, KEndBonus;
extern int PEndPenalty, NEndPenalty, BEndPenalty, REndPenalty, QEndPenalty, KEndPenalty;
#endif
/* eval.h */
/* Last modified: June 17, 2006 */


/* passed pawn pair bonus based on rank */
static int pppBonus[] = {0, 32, 64, 125, 250, 500, 750};


/* multiplier, etc */
#define KING_RACE_FACTOR                       2


/* Initial king safety counter depending on where the king is */
static ui wKsfCounter[] = {2, 0, 0, 0, 0, 0, 0, 2,
		    2, 1, 1, 1, 1, 1, 1, 2,
		    4, 3, 3, 3, 3, 3, 3, 4,
		    4, 4, 4, 4, 4, 4, 4, 4,
		    4, 4, 4, 4, 4, 4, 4, 4,
		    4, 4, 4, 4, 4, 4, 4, 4,
		    4, 4, 4, 4, 4, 4, 4, 4,
		    4, 4, 4, 4, 4, 4, 4, 4};

static ui bKsfCounter[] = {4, 4, 4, 4, 4, 4, 4, 4,
		    4, 4, 4, 4, 4, 4, 4, 4,
		    4, 4, 4, 4, 4, 4, 4, 4,
		    4, 4, 4, 4, 4, 4, 4, 4,
		    4, 4, 4, 4, 4, 4, 4, 4,
		    4, 3, 3, 3, 3, 3, 3, 4,
		    2, 1, 1, 1, 1, 1, 1, 2,
		    2, 0, 0, 0, 0, 0, 0, 2};

/* Additional king safety counter depends on the 
   combination of opponent pieces attacking king */
static ui ksfFlagToCounter[] =
  {/* N are for knight or bishop
      . P N N R R R R Q Q Q Q Q Q Q Q K K K K K K K K K K K K K K K K
      P   P N N   P N N R R R R   P N N R R R R Q Q Q Q Q Q Q Q  
      P       P   P N N       P   P N N   P N N R R R R 
      P               P       P   P N N        
      P */
    0,0,0,0,0,0,1,1,0,1,2,2,2,3,3,3,0,0,0,0,1,1,2,2,2,3,3,3,3,3,3,3};

/* Now translate the counter into a real value */
static ui ksfTable[] = {  0,   2,   3,   6,  12,  18,  25,  37,  50,  75,
		   100, 125, 150, 175, 200, 225, 250, 275, 300, 325,
		   350, 375, 400, 425, 450, 475, 500, 525, 550, 575,
		   600, 600, 600, 600, 600, 600, 600, 600, 600, 600};
/* prototypes.h */
/* Last modified: June 9, 2006 */


#ifdef WINDOWS
#include <time.h>
#else
#include <sys/time.h>
#endif


/* attack.c */
void initRankAttacks(void);
ui isSqAttackedByWP(ui sq, sPos *pos, sOthers *others);
ui isSqAttackedByWN(ui sq, sPos *pos, sOthers *others);
ui isSqAttackedByWK(ui sq, sPos *pos, sOthers *others);
ui isInCheckByW(ui sq, BB *Attackers, sPos *pos, sOthers *others);
ui isSqAttackableByW(ui sq, sPos *pos, sOthers *others);
ui isSqAttackedByBP(ui sq, sPos *pos, sOthers *others);
ui isSqAttackedByBN(ui sq, sPos *pos, sOthers *others);
ui isSqAttackedByBK(ui sq, sPos *pos, sOthers *others);
ui isInCheckByB(ui sq, BB *Attackers, sPos *pos, sOthers *others);
ui isSqAttackableByB(ui sq, sPos *pos, sOthers *others);
ui getWAttacker(ui sq, sPos *pos, sPlyData *thisPlyData, sOthers *others);
ui getBAttacker(ui sq, sPos *pos, sPlyData *thisPlyData, sOthers *others);
ui pinner(ui pinneeColor, ui pinnee, ui pinKing, sPos *pos, 
	  sOthers *others);
BB ambusher(ui sq, int direction, sPos *pos, sOthers *others);
int valueAtSq(ui sq, sPos *pos);
int see(ui wToMove, ui sq, ui t, sPos *pos, sOthers *others);


/* book.c */
void readPGN(char *file, char *player, int numberOfMoves);
void whiteFEN(sPos *pos, sPlyData *plyData, sHash **hash, sOthers *others);
void blackFEN(sPos *pos, sPlyData *plyData, sHash **hash, sOthers *others);
void readTextBooks(sPos *pos, sPlyData *plyData, sHash **hash, 
		   sOthers *others);
void readBinaryBooks(void);
BB openingBook(sPlyData *thisPlyData, sOthers *others);


/* command.c */
void getCommand(sPos *pos, sPlyData *plyData, sHash **hash, 
		sOthers *others, ui acceptDraw, FILE *fd);
ui runCommand(char *word0, char *word1, char *word2, char *word3,
	      char *word4, char *word5, char *word6, char *word7,
	      char *word8, char *word9, char *word10, char *word11,
	      sPos *pos, sPlyData *plyData, sHash **hash, 
	      sOthers *others, ui acceptDraw, FILE *fd);


/* egtbprobe.c */
int EGTBProbe (int *score, sPos *pos, sPlyData *thisPlyData);


/* eval.c */
int evalMaterial(sPos *thiPos, sOthers *others, int *wPieceScore, 
		 int *bPieceScore, int *mScore, int *doubleBonus);
int eval(ui ply, sPos *pos, sPlyData *plyData, sOthers *others, int lScore,
	 int wPieceScore, int bPieceScore, int alpha, int doubleBonus);
ui noPotentialPPawn(sPos *pos);
int bonusFor7thRank(ui side, ui rank, ui fyle, sPos *pos);
void revalWOpening(BB move, sPos *pos, sOthers *others);
void revalBOpening(BB move, sPos *pos, sOthers *others);
void reval(BB move, sPos *pos, sOthers *others);
ui isDrawByRule(ui ply, sPos *pos, sPlyData *thisPlyData, sOthers *others);


/* hash.c */
void hashStore(ui ply, ui depth, int score, ui from, ui to, ui mateTreat,
	       ui hashFlag, sPlyData *thisPlyData, sHash **hash, 
	       sOthers *others);
int hashProbe(ui ply, ui depth, int *score, ui *from1, 
	      ui *to2, ui *mateThreat, ui *hashFlag, 
	      sPlyData *thisPlyData, sHash **hash, sOthers *others);


/* init.c */
void initBitBoards(sOthers *others);
void init(sPos *pos, sPlyData *plyData, sHash **hash, sOthers *others);
void initKeys(sOthers *others);
void clearHash(sHash **hash, sOthers *others);
void finalizeInit(ui *piece, sPos *pos, sPlyData *thisPlyData,
		  sOthers *others);
void initPosBoard(sPos *pos, sPlyData *thisPlyData, sOthers *others);
BB scanBoard(ui *piece, BB EP, int wCanCastle, int bCanCastle,
	       sPos *pos, sOthers *others);
void readSqvalue(sOthers *others);


/* move.c */
ui testMove(ui ply, BB move, sPos *pos, sPlyData *plyData, sOthers *others);
ui captures(ui ply, BB *moveList, int quiescent, sPos *pos, sPlyData *plyData,
	    sOthers *others, ui psedolegal);
ui nonCaptures(ui ply, BB *moveList, sPos *pos, sPlyData *plyData, 
	       sOthers *others, ui pseudolegal);
void makeMove(ui ply, BB move, sPos *pos, sPlyData *plyData, sOthers *others);
void unmakeMove(ui ply, BB move, sPos *pos, sPlyData *plyData);
ui outOfBCheck(ui ply, ui kSq, BB Checkers, BB *moveList, sPos *pos, 
	       sPlyData *plyData, sOthers *others);
ui outOfWCheck(ui ply, ui kSq, BB Checkers, BB *moveList, sPos *pos, 
	       sPlyData *plyData, sOthers *others);
void perft(ui ply, ui depth, sPos *pos, sPlyData *plyData, sOthers *others);


/* pmRandom.c */
void setRandomSeed(unsigned long initial_seed);
unsigned long random31(void);
unsigned long random32(void);
BB random64(void);


/* search.c */
int qsearch(ui ply, ui qDepth, int alpha, int beta, int lMargin, sPos *pos, 
	    sPlyData *plyData, sOthers *others, sTC *tc, ui nullMode, 
	    ui isPVNode);
int search(ui ply, int alpha, int beta, ui depth, sPos *pos, sPlyData *plyData,
	   sHash **hash, sOthers *others, sTC *tc, 
	   ui nullOK, ui nullMode, ui centiExtension, ui isPVNode);
int rsearch(int alpha, int beta, ui depth, sPos *pos, sPlyData *plyData,
	    sHash **hash, sOthers *others, sTC *tc, 
	    BB* allMoves, ui nMoves, ui origCE);


/* util.c */
void initFLOne(void);
void san(ui ply, BB move, char *s, sPos *pos, sPlyData *plyData, 
	 sOthers *others);
void algebraic(BB move, char *s);
void display(sPos thispos);
#ifdef VERIFY
void displayBitBoard(BB BitBoard);
void displayBitRank(ui bitRank);
#endif
void setBoard(char* s1, char s2, char* s3, char* s4, char* s5, char* s6,
	      sPos *pos, sPlyData *thisPlyData, sOthers *others);
void getFEN(sPos *pos, sPlyData *thisPlyData, sOthers *others, char *fen);
void reverse(sPos *pos, sPlyData *thisPlyData, sOthers *others, char *fen);
void pick(ui head, ui tail, BB *moveList);
void closing(FILE *fd, sOthers *others, char *result, char *comment);
void exiting(FILE *fd, sHash **hash, sOthers *others);


/* bitstuffs.s */
ui firstOne(BB BitBoard);
ui lastOne(BB BitBoard);
ui bitCount(BB BitBoard);


/* egtb.cpp */
int FTbSetCacheSize(void *EGTB_cache, int EGTB_cache_size);
int IInitializeTb(char *pszPath);
int IDescFindFromCounters (int *rgiCounters);


#ifdef LINUX
#if !defined(WINDOWS)
/* <sys/time.h>, declare it to please splint */
//int gettimeofday(struct timeval *tv, struct timezone *tz);
#endif
#endif
/* variables.h */
/* Last modified: April 21, 2006 */


#ifdef WINDOWS
#include <time.h>
#else
#include <sys/time.h>
#endif


/* 16 bytes */
struct BookTable
{
  BB move;
  ui key;
  unsigned short no;
  unsigned short count;
};
static struct BookTable wBookTable[(bookSize+1)], bBookTable[(bookSize+1)];


/* 32 bytes */
struct TimeCheck
{
  double timeBudget;        /* in micro seconds */
#ifdef WINDOWS
  double clock0;
#else
  BB clock0;
#endif
  BB nodeInc, nodeCheckPoint;
};


/* 32 bytes */
struct HashTable
{
  BB key;
  BB stuffs;
  BB keySave;
  BB stuffsSave;
};


/* 708 bytes - 704 bytes will be ideal */
struct Position
{
  BB WBoard, BBoard, WPBoard, BPBoard, WRBoard, BRBoard, 
    WNBoard, BNBoard, WBBoard, BBBoard, WQBoard, BQBoard;
  BB RotatedBoard90, RotatedBoard45l, RotatedBoard45r;
  ui wKSq, bKSq;
  ui wPCount[8], bPCount[8];
  ui killerFrom[maxPly], killerTo[maxPly];
  int wScore;
};


/* 424 bytes */
struct PlyData
{
#ifdef VERIFY
  ui piece[64];
#endif
  /****************************************
   * Attack tables based on Rebel's       *
   * bit    0: not calculated/calculated  *
   * bit    1: not attacked/attacked      *
   * bit    2: not used                   * 
   * bit    3: attack by pawn             *
   * bit    4: attack by knight or bishop *
   * bit    5: attack by rook             *
   * bit    6: attack by queen            *
   * bit    7: attack by king             *
   ****************************************/
  unsigned char wAttacker[64], bAttacker[64];
  BB key;
  BB EP;
  BB pv[maxPly];
  ui pvLength;
  ui wToMove;
  int wCanCastle, bCanCastle;  /* 01  : can castle short  
				* 02  : can castle long 
				* 00  : lost castling right
				* -ply: castled on ply ply
				*/
  ui count50Moves;
};


/* 28792 bytes - crazy and messy */
struct Others
{
  double timeLeft0, timeLeft, oTimeLeft, timeInc;  /* in centiSecond */

  /* all the bitboards */
  /* 5632 bytes */
  BB Up1Of[64], Up7Of[64], Up8Of[64], Up9Of[64];
  BB Down1Of[64], Down7Of[64], Down8Of[64], Down9Of[64];
  BB SideOf[64], Up8SideOf[64], Down8SideOf[64];

  /* 3072 bytes */
  BB OneUp[64], TwoUp[64], OneDown[64], TwoDown[64], OneSide[64], TwoSide[64];

  /* 64 bytes */
  BB QueenSide, FarQueenSide, FNQueenSide, FSQueenSide;
  BB KingSide, FarKingSide, FNKingSide, FSKingSide;

  /* 3072 bytes */
  BB WPCaptures[64] , BPCaptures[64];
  BB NMoves[64], BMoves[64], KMoves[64], KHood[64];

  /* 128 bytes */
  BB Row[8], Column[8];

  /* 64 bytes */
  BB Edge, InnerEdge, Dark, Light;
  BB qNodes, sNodes, nodes;
  /* Put all true/false global values into this variable
   *        bit        description
   *          0        xboard
   *          1        forced
   *          2        game is already listed
   *          3        randomize
   *          4        search terminated
   *          5        program terminated
   *          6        book disabled
   *          7        search almost terminated
   *          8        drawish
   *          9        pawn king square rule was used
   *         10        timeReserved mode
   */
  BB Flags;

  /* square value tables */
  /* 3584 bytes */
  int wPSqvalue[64], wNSqvalue[64], wBSqvalue[64];
  int wRSqvalue[64], wQSqvalue[64], wKSqvalue[64];
  int bPSqvalue[64], bNSqvalue[64], bBSqvalue[64];
  int bRSqvalue[64], bQSqvalue[64], bKSqvalue[64];
  int wPassedPSqvalue[64], bPassedPSqvalue[64];

  /* 1024 bytes */
  BB plyNodes[128];

  /* all the hash keys */
  /* 7680 bytes */
  BB wPKey[64], wNKey[64], wBKey[64], wRKey[64], wQKey[64], wKKey[64];
  BB bPKey[64], bNKey[64], bBKey[64], bRKey[64], bQKey[64], bKKey[64];
  BB ePKey[64], wCanCastleKey[4], bCanCastleKey[4];

  /* 128 bytes */
  char direction[64][64];

  /* 816 bytes */
  BB repKey[102];

  /* 487 bytes */
  ui iAm;
  /* WARNING: others->moveNumber starts from 0, not 1 like the real move 
   * number.  It needs to start from 0 because it's used as an index of
   * gameListing[].
   */
  ui moveNumber, repHead;
  ui movesPerTC;
  ui EGTBlimit;  /* maximum pieces in our endgame tablebase */
  ui failBook, openingNo;
  ui iteration, searchDepth;
  ui hashSize, cacheSize;
  ui rOpening;
  ui rootStage;
  int deltaRating;
  char opponentName[80];
  char posID[80], solution[80], myMove[8];
  /* for debugging: "nodeNumber_ply_hashKeyBeforeTheMove_wToMove_sanMove" */
  char gameID[80], logDate[7], fullDate[16];
  char egtbDir[80];
};
