/* pmRandom.c */


/*
 *
 * This program is public domain and was written by William S. England
 * (Oct 1988).  It is based on an article by:
 *
 * Stephen K. Park and Keith W. Miller. RANDOM NUMBER GENERATORS:
 * GOOD ONES ARE HARD TO FIND. Communications of the ACM,
 * New York, NY.,October 1988 p.1192

Modifications:
   Sun Feb 10 18:20:38 PST 1991
      WSE, modified for replacement of random number object file
      under unix and for use with Perl.
   Wed Jun 12 02:20:43 EST 2002
      Johanes Suhardjo, the original function returns 31 bit random integer.
      Added functions that return 32 bit and 64 bit random integers.

 The following is a portable c program for generating random numbers.
 The modulus and multipilier have been extensively tested and should
 not be changed except by someone who is a professional Lehmer generator
 writer.  THIS GENERATOR REPRESENTS THE MINIMUM STANDARD AGAINST WHICH
 OTHER GENERATORS SHOULD BE JUDGED. ("Quote from the referanced article's
 authors. WSE" )
*/


#define	mm  (unsigned long)2147483647
#define	qq  (unsigned long)127773

#define	aa (unsigned int)16807
#define	rr (unsigned int)2836

/*
** F(z)	= (az)%m
**	= az-m(az/m)
**
** F(z)  = G(z)+mT(z)
** G(z)  = a(z%q)- r(z/q)
** T(z)  = (z/q) - (az/m)
**
** F(z)  = a(z%q)- rz/q+ m((z/q) - a(z/m))
** 	 = a(z%q)- rz/q+ m(z/q) - az
*/

/*
**
*/
static unsigned long seed;

void setRandomSeed(unsigned long initial_seed)
{
  seed = initial_seed; 
}
/*
**
*/
unsigned long random31()
{
  register unsigned int lo, hi;
  register int test;

  hi = seed/qq;
  lo = seed%qq;

  test = aa*lo - rr*hi;
  
  if (test > 0) seed = test;
  else seed = test+mm;
  
  return seed;
}
/*
**
*/
unsigned long random32()
{
  return random31()+random31();
}
/*
**
*/
unsigned long long random64()
{
  unsigned long r1, r2;

  r1 = random32();
  r2 = random32();
  return r1 | (unsigned long long) r2<<32;
}
