/* An attempt to mathematically score a version of parrot.
   The components:
   1. match: 5 minutes per 40 moves games against select opponents
   2. bulletMatch: 1 minutes per 40 moves games against the same opponents
   3. testSuite: testOP, testMP, testMT, testEP, testET, 10 positions each

   1. match
      If 1000 games are played, that's it, score = match_result.
      If less than 1000 games are played, use the match function 
              p1 = 100 - 58.122 * exp(-0.0040625 * x1)
      where x1 is the number of games.  The match function is based on a 
      function where error decays with the number of games.  It is curve fit
      of the match results of parrot-050516 against smallpotato, faile, and
      drunkenmaster between May 16, 2005 and May 26, 2005.  The assumption is
      that 1000 games yield an almost perfect result.  See parrot.m for the 
      MATLAB least square fit calculation.
              score_from_match = p1/100 * match_result
   
   2. bulletMatch
              p2 = 10 - 5.8122 * exp(-0.0040625 * x2)
      where x2 is the number of games.
	      if (p1 + p2) > 100, p2 = 100 - p1
              score_from_bulletMatch = p2/100 * bulletMatch_result * c2
      where c2 is a calibration factor.  As of May 27, 2005,
              c2 = 50.4 / 47.4
      where 50.4% and 47.4% are the results of 1000 game match and bulletMatch,
      respectively.

   3. testSuite
              if (p1 + p2) < 1000
              p3 = 100 - p1 - p2
              score_from_testSuite = p3/100 * testSuite_score * c3
      where c3 is a calibration factor.  As of May 27, 2005,
              c3 = 50.4 / 53.0
      where 53.0% is the output of runtest.

   Finally
              Final score = score_from_match + score_from_bulletMatch + 
	                    score_from_testSuite

   Notes:
   a. match is not accurate, that's why we have the match function with error
      that decays with number of games.
   b. bulletMatch is even less accurate, so it contribute up to 10% only.
   c. testSuite is not accurate and sometimes is not even relevant.  For 
      example, if we change the opening books, testSuite score doesn't change.
      Try not to rely on it.  In the cases it is not relevant, I think we 
      should just play enough games in match and bulletMatch (435 and 1000
      games, respectively).
   d. The calibration factors or maybe the match function itself have to be 
      adjusted from time to time, especially if the opponents change.

   Johanes Suhardjo - May 27, 2005.
   Updated with new match/test results: December 15, 2005.
*/


#include <stdio.h>


main()
{
  int x1, x2;
  double c2, c3;
  double match_result, bulletMatch_result, testSuite_result;
  double p1, p2, p3;
  double score_from_match = 0.0, score_from_bulletMatch = 0.0, 
    score_from_testSuite = 0.0;

  /* data */
  x1 = 460;
  match_result = 50.4;
  x2 = 1006;
  bulletMatch_result = 47.4;
  testSuite_result = 53.0;

  /* calibration factors */
  c2 = 50.4 / 47.4;
  c3 = 50.4 / 53.0;

 /* score from match */
  p1 = 100 - 58.122 * exp(-0.0040625 * x1);
  score_from_match = p1/100 * match_result;
  printf("Score from match: %2.1f%%\n", score_from_match);

  if (x1 >= 1000) 
    {
      printf("Final score: %2.1f%%\n", score_from_match);
      exit(0);
    }

  /* score from bulletMatch */
  p2 = 10 - 5.8122 * exp(-0.0040625 * x2);
  if ((p1 + p2) > 100) p2 = 100 - p1;
  score_from_bulletMatch = p2/100 * bulletMatch_result * c2;
  printf("Score from bulletMatch: %2.1f%%\n", score_from_bulletMatch);

  if (p1 + p2 > 100) 
    {
      printf("Final score: %2.1f%%\n", 
	     score_from_match + score_from_bulletMatch);
      exit(0);
    }
  else p3 = 100 - p1 - p2;
  printf("p1=%2.2f p2=%2.2f p3=%2.2f\n", p1, p2, p3);

  score_from_testSuite = p3/100 * testSuite_result * c3;
  printf("Score from testSuite: %2.1f%%\n", score_from_testSuite);

  printf("Final score = %2.2f\n", score_from_match + score_from_bulletMatch + 
	 score_from_testSuite);

  exit(0);
}
