/* main.c */
/* Last modified: June 9, 2006 */


/* parrot - Johanes Suhardjo (johanes@nd.edu) */
/* http://www.nd.edu/~johanes/parrot.html */
/* This program is a freeware, but if you use part of my code 
 * in your own program, do me a favor, acknowledge it. */


#include <stdio.h>
#include <ctype.h>
#include <signal.h>
#include <string.h>
#include <time.h>

#if !defined WINDOWS
#include <stdlib.h>
#include <strings.h>
#include <unistd.h>
#else
#define strcasecmp stricmp
#endif

#include "parrot.h"

#ifdef BONUSES_PENALTIES
int PBonus, NBonus, BBonus, RBonus, QBonus, KBonus;
int PPenalty, NPenalty, BPenalty, RPenalty, QPenalty, KPenalty;
int PEndBonus, NEndBonus, BEndBonus, REndBonus, QEndBonus, KEndBonus;
int PEndPenalty, NEndPenalty, BEndPenalty, REndPenalty, QEndPenalty, KEndPenalty;
#endif


#ifdef SPEED_TEST
/* supposed to be very fast for small a, but still slower than bitstuffs.s */
int myBitCount(BB a)
{
  int c = 0;

  while (a)
    {
      c++;
      a &= a-1;
    }

  return c;
}
#endif /* SPEED_TEST */


int main(int argc, char **argv)
{
  BB winningCapture[32], nonCapture[128];
  BB allMoves[128];

  BB Checkers;
  BB lastMove, theMove, lastPV;
  BB lastNodes;
  double timeReserved;
  double bFactor = 3.0;
  double nPMicroS = NPMS;
  int timebookExtra = 50;
  ui timebookSpan = 10;

  ui count, wordNo;
  int alpha, beta;
  ui i, j, noOfCaptures, noOfNonCaptures, nMoves, dtm;
  ui checkers, checkable, canCheck;
  int score = 0, gameEnd, lastScore = 0;
  int mScore = 0, doubleBonus = 0, wPieceScore, bPieceScore, minPieceScore;

  int topScore, secondScore;
  int ok = false;
  ui lastOpeningNo = 0;
  ui origCE;
  ui mNum, wTM;
  int temp;
  ui movesLeft;
  ui drawCount = 0, acceptDraw = false, resignCount = 0;
  char line[80];

  char word[WORDS][80];

  char lastC, c;
  char s[8];
  /* use Nalimov's endgame tablebase */
  void  *EGTB_cache;
  time_t when;
  struct tm *timeStr;
#ifdef WINDOWS
  double clock1;
#else
  struct timeval tv;
  struct timezone tz;
  BB clock1;
#endif
  FILE *fdin;
  FILE *fd;
  char fname[256];

  sPlyData plyData[maxPly], pvPlyData[maxPly];
  sHash *hash[2];
  sPos pos, pvPos;
  sOthers *others, pvOthers;
  struct TimeCheck tc;


  printf("parrot version %s\n", version);


  if(!(others = (sOthers *) malloc (sizeof(sOthers))))
    {
      printf("Can't malloc others");
      exit(0);
    }

  /* today's date */
  time(&when);
  timeStr = localtime(&when);
  strftime(others->logDate, 7, "%m%d%y", timeStr);
  sprintf (fname, "list.%s", others->logDate);
  fd = fopen(fname, "a+");

  /* reading configuration file */
  if ((fdin = fopen("parrot.cfg", "r")) == NULL)
    {
      printf("Can't read parrot.cfg.  Quiting.");
      exit(0);
    }
  while (fgets(line, 80, fdin) != NULL)
    {
      if (line[0] != '#' && line[0] != '\n' && line[0] != '\r')
	{
	  wordNo = -1;
	  count = 0;
	  lastC = ' ';
	  /* Better not assume that word[][] is always zero */
	  for (i = 0; i < WORDS; i++)
	    word[i][0] = '\0';

	  /* trap EOF which gives segmentation fault if not trapped */
	  i = 0;
	  c = line[i++];
	  do
	    {
	      if (isspace((int) lastC) && isspace((int) c)) continue;
	      else if (isspace((int) lastC) && !(isspace((int) c)))
		{
		  wordNo++;
		  count = 0;
		  word[wordNo][count++] = c;
		  lastC = c;
		  continue;
		}
	      else if (!(isspace((int) lastC)) && (isspace((int) c)))
		{
		  word[wordNo][count] = '\0';
		  /* Limit WORDS tokens per customer */
		  if (wordNo == WORDS) break; 
		  lastC = ' ';
		  continue;
		}
	      else
		{
		  word[wordNo][count++] = c;
		  lastC = c;
		}
	    }
	  while ((c = line[i++]) != '\0');
	  
	  if (!runCommand(word[0], word[1], word[2], word[3], word[4], 
			  word[5], word[6], word[7], word[8], word[9], 
			  word[10], word[11], &pos, plyData, hash, 
			  others, acceptDraw, fd))
	      printf("%s in parrot.cfg: no such command\n", word[0]);
	}
    }
      
  if(!(hash[white] = (sHash *) malloc(sizeof(sHash) * (others->hashSize+1))))
    {
      printf("Can't malloc hash");
      exit(0);
    }  

  if(!(hash[black] = (sHash *) malloc(sizeof(sHash) * (others->hashSize+1))))
    {
      printf("Can't malloc hash");
      exit(0);
    }  

  /**************************************************************/
 

  /* use Nalimov's endgame tablebase */
  temp = 2*sizeof(sHash)*(others->hashSize+1)/1024/1024;
  printf("hash table size: %d MBytes.\n", temp);
  EGTB_cache = (void *) 0;
  EGTB_cache=malloc((size_t) others->cacheSize);
  if (EGTB_cache)
    {
      others->EGTBlimit = (ui) IInitializeTb(others->egtbDir);
      ok = FTbSetCacheSize(EGTB_cache, (int)others->cacheSize);
    }
  if (ok) 
    {
      printf("EGTB cache memory: %uM bytes.\n", others->cacheSize/1024/1024);
      printf("EGTB directory: %s\n", others->egtbDir); 
    }
  else printf("ERROR in initializing EGTB\n");


  memset(&pos, 0, sizeof(pos));
  memset((void *) plyData, 0, maxPly*sizeof(plyData[0]));
  memset((void *) others->repKey, 0, sizeof(others->repKey));
  init(&pos, plyData, hash, others);
  /* bitboard used in clearing score from a move */
  readBinaryBooks();

#ifdef BONUSES_PENALTIES
  PBonus = NBonus = BBonus = RBonus = QBonus = KBonus = -mateScore;
  PPenalty = NPenalty = BPenalty = RPenalty = QPenalty = KPenalty = mateScore;
  PEndBonus = NEndBonus = BEndBonus = REndBonus = QEndBonus = KEndBonus = -mateScore;
  PEndPenalty = NEndPenalty = BEndPenalty = REndPenalty = QEndPenalty = KEndPenalty = mateScore;
#endif

  display(pos);
  others->rootStage = nonEndgame;
  while (true)
    {
      origCE = 0;
      getCommand(&pos, plyData, hash, others, acceptDraw, fd);

      if (others->Flags & OneMask[5]) break;

      if (!(others->Flags & OneMask[1])) /* not forced */ 
	{
	  gameEnd = false;
	  others->iteration = 0;

	  /* time control */
	  others->Flags &= NilMask[4];    /* search neither terminated */
	  others->Flags &= NilMask[7];    /*     nor almost terminated */
#ifdef WINDOWS
	  tc.clock0 = 1000000.0/CLOCKS_PER_SEC*clock();
#else
	  gettimeofday(&tv, &tz);
	  tc.clock0 = (BB) tv.tv_sec*1000000 + (BB) tv.tv_usec;
#endif
	  /* try to budget time close to opponent */
	  if (others->movesPerTC == 0) movesLeft = 1;
	  else
	    {
	      if ((others->moveNumber+1) % others->movesPerTC == 0)
		movesLeft = 1;
	      else movesLeft = (others->movesPerTC+1) - 
		     ((others->moveNumber+1) % others->movesPerTC);
	    }
#ifdef NUNN
	  others->Flags &= NilMask[3];    /* don't randomize */
	  others->Flags |= OneMask[6];    /* don't use opening book */
	  others->rOpening = 0;   /* don't evaluate base on a random opening */
	  timebookExtra = 0;
#endif
#ifdef HANDICAPPED_OPPONENT
	  if (others->Flags & OneMask[10]) /* timeReserved mode */
	    tc.timeBudget = (10000*others->timeLeft/movesLeft)/2;
	  else
	    {
	      tc.timeBudget = (10000*others->timeLeft/movesLeft);
	      timebookExtra = 0;
	    }
#else
	  if (others->Flags & OneMask[10]) /* timeReserved mode */
	    tc.timeBudget = (10000*others->timeLeft/movesLeft
			     + 10000*(others->timeLeft - others->oTimeLeft))/2;
	  else
	    {
	      tc.timeBudget = (10000*others->timeLeft/movesLeft);
	      timebookExtra = 0;
	    }
	  if (tc.timeBudget <= 0) 
	    tc.timeBudget = 10000*others->timeLeft/movesLeft;
#endif
	  /* timebook idea from crafty */
	  if (others->failBook >= 3 && others->failBook < (3+timebookSpan))
	    tc.timeBudget += (tc.timeBudget*(timebookExtra/100) *
			      ((3+timebookSpan)-others->failBook)/10);

	  /* crunch time: less than 1 second per move left or 7 moves left */
	  if (others->timeLeft/movesLeft < 100 || 
	      movesLeft <= 7)
	    {
	      others->Flags |= OneMask[7];
	      /* try not to be behind in time */
#ifdef HANDICAPPED_OPPONENT
	      tc.timeBudget /= 2;
#else
	      if (others->timeLeft <= others->oTimeLeft) 
		tc.timeBudget = 100000;        /* 0.1 seconds */
	      else if (others->movesPerTC > 1)
		tc.timeBudget = 5000*(others->timeLeft - others->oTimeLeft);
	      else
		{
		  tc.timeBudget = 10000*(others->timeLeft - others->oTimeLeft);
		  /* if possible, substract 1 second for safety */
		  if (tc.timeBudget > 1000000) tc.timeBudget -= 1000000;
		}
#endif
	      timeReserved = 0;
	    }
	  else
	    if (others->Flags & OneMask[10]) 
	      {
		timeReserved = 
		  ((10000*others->timeLeft/movesLeft) - tc.timeBudget) / 10;
		if (timeReserved < 0) timeReserved = 0;
	      }
	    else timeReserved = 0;
	  printf("Time budget = %.0f with %.0f reserved\n", 
		 tc.timeBudget/10000, timeReserved/10000);

	  /* shoot to check 100 times per move */
	  if (nPMicroS > 4*NPMS) nPMicroS = 4*NPMS;  /* avoid extremes */
	  tc.nodeCheckPoint = (BB)(nPMicroS * tc.timeBudget / 10);

	  /* Is this position a draw by rule? */
	  switch(isDrawByRule(0, &pos, &plyData[0], others))
	    {
	    case 0:
	      break;
	    case 3:
	      if (others->Flags & OneMask[0])
		printf("1/2-1/2 {repetition}\n");
	      else
		{
		  closing(fd, others, "1/2-1/2", "repetition");
		  exiting(fd, hash, others);
		}
	      break;
	    case 13:
	      if (others->Flags & OneMask[0])
		printf("1/2-1/2 {not enough material}\n");
	      else
		{
		  closing(fd, others, "1/2-1/2", "not enough material");
		  exiting(fd, hash, others);
		}
	      break;
	    case 50:
	      if (others->Flags & OneMask[0]) 
		printf("1/2-1/2 {50 move rule}\n");
	      else
		{
		  closing(fd, others, "1/2-1/2", "50 move rule");
		  exiting(fd, hash, others);
		}
	      break;
	    default:
	      break;
	    }

	  /* Try opening book.  Three strikes and you're out. */
	  theMove = 0;
	  if (others->failBook < 3 && !(others->Flags & OneMask[6]))
	    theMove = openingBook(&plyData[0], others);

	  if (theMove)
	    {
	      lastOpeningNo = others->openingNo;
	    }
	  else
	    {
	      others->failBook++;
	      others->openingNo = lastOpeningNo;

	      /* Move generation */
	      Checkers = (BB) 0;
	      if (plyData[0].wToMove)
		checkers = isInCheckByB(pos.wKSq, &Checkers, &pos, others);
	      else 
		checkers = isInCheckByW(pos.bKSq, &Checkers, &pos, others);
	      if (checkers) origCE += 50;

	      nMoves = 0;
	      if (checkers)
		{
		  if (plyData[0].wToMove) 
		    nMoves = outOfBCheck(0, pos.wKSq, Checkers, allMoves, 
					 &pos, plyData, others);
		  else
		    nMoves = outOfWCheck(0, pos.bKSq, Checkers, allMoves, 
					 &pos, plyData, others);
#ifdef VERIFY
		  noOfCaptures = captures(0, winningCapture, 0, &pos, plyData, 
					  others, 0);
		  noOfNonCaptures = nonCaptures(0, nonCapture, &pos, plyData, 
						others, 0);
		  if (noOfCaptures+noOfNonCaptures != nMoves)
		    {
		      display(pos);
		      for (j = 0; j < noOfCaptures; j++) 
			printf("ERROR in outOf?Check() in main() C%d. %d-%d\n",
			       j+1, from(winningCapture[j]), 
			       to(winningCapture[j]));
		      for (j = 0; j < noOfNonCaptures; j++) 
			printf("ERROR in outOf?Check() in main() N%d. %d-%d\n",
			       j+1, from(nonCapture[j]), to(nonCapture[j]));
		      for (j = 0; j < nMoves; j++) 
			printf("ERROR in outOf?Check() in main() A%d. %d-%d\n",
			       j+1, from(allMoves[j]), to(allMoves[j]));
		      exit(EXIT_FAILURE);
		    }
#endif
		}
	      else
		{
		  /* try captures */
		  noOfCaptures = captures(0, winningCapture, 0, &pos, plyData,
					  others, 0);
		  /* try non-captures */
		  noOfNonCaptures = nonCaptures(0, nonCapture, &pos, plyData, 
						others, 0);
		  
		  /* Copying all moves to allMoves[]. */
		  if (noOfCaptures > 0)
		    for (i = 0; i < noOfCaptures; i++)
		      allMoves[nMoves++] = winningCapture[i];
		  if (noOfNonCaptures > 0)
		    for (i = 0; i < noOfNonCaptures; i++)
		      allMoves[nMoves++] = nonCapture[i];
		}
	      
	      if (nMoves == 1)
		{
		  theMove = allMoves[0];
		}
	      else if (nMoves > 0)
		{
		  /* search loop */
		  others->sNodes = 1;
		  others->qNodes = 0;
		  lastNodes = others->sNodes + others->qNodes;
#ifdef WINDOWS
		  clock1 = 1000000.0/CLOCKS_PER_SEC*clock();
#else
		  gettimeofday(&tv, &tz);
		  clock1 = (BB) tv.tv_sec*1000000 + (BB) tv.tv_usec;
#endif

		  /* Spent some time to use full window qsearch and find the
		     top two scores to determine if theMove is an easy move. */
		  if (plyData[0].wToMove)
		    { 
		      checkable = isSqAttackableByB(pos.wKSq, &pos, others);
		      canCheck = isSqAttackableByW(pos.bKSq, &pos, others);
		    }
		  else 
		    {
		      checkable = isSqAttackableByW(pos.bKSq, &pos, others);
		      canCheck = isSqAttackableByB(pos.wKSq, &pos, others);
		    }
		  if (checkable || canCheck)
		    {
		      if (checkable) origCE += 13;
		      if (canCheck) origCE += 13;
		    }
		  else
		    {
		      topScore = secondScore = -mateScore;

		      for (i = 0; i < nMoves; i++)
			{
			  makeMove(1, allMoves[i], &pos, plyData, others);
			  score = -qsearch(1, 0, -mateScore, mateScore, 
					   onlyScore, &pos, plyData,
					   others, &tc, 0, PVNode);
			  ASSERT(score >= -mateScore && 
				 score <= mateScore);
			  /* tag on the scores on the moves so they can be 
			     sorted */
			  allMoves[i] = (allMoves[i] & ~ScoreBoard) |
			    ((BB)(score+mateScore) << 30);
			  
			  if (score > topScore)
			    {
			      secondScore = topScore;
			      topScore = score;
			    }
			  else if (score > secondScore) 
			    secondScore = score;
			  unmakeMove(1, allMoves[i], &pos, plyData);
			}

		      if (topScore > secondScore + easyScore && 
		      tc.timeBudget > 1000000)
			{
			  printf("Easy move, tc.timeBudget is reduced from %u ",
				 (ui)(tc.timeBudget/1000000));
			  tc.timeBudget /= easyFactor;
			  printf("to %u seconds.\n", 
				 (ui)(tc.timeBudget/1000000));
			}
		    }
		  
		  for (i = 0; i < nMoves+1; i++) others->plyNodes[i] = 0;
		  alpha = -mateScore;
		  beta = mateScore;
		  theMove = lastMove = plyData[0].pv[0] = allMoves[0];

		  /* Are we changing from nonEndgame to endgame or v.v.? */
		  score = evalMaterial(&pos, others, &wPieceScore, 
				       &bPieceScore, &mScore, &doubleBonus);
		  ASSERT(score >= -mateScore && score <= mateScore);
		  if ((wPieceScore + bPieceScore) >= 0) 
		    minPieceScore = -bPieceScore;
		  else minPieceScore = wPieceScore;
		  ASSERT(minPieceScore >= 0 && minPieceScore <= 3902);
		  if (minPieceScore <= endGameValue) 
		    {
		      if (others->rootStage == nonEndgame)
			clearHash(hash, others);
		      others->rootStage = endgame;
		    }
		  else
		    {
		      if (others->rootStage == endgame)
			clearHash(hash, others);
		      others->rootStage = nonEndgame;
		    }

		  while (!(others->Flags & OneMask[4]) && 
			 others->iteration < others->searchDepth &&
			 (double) ((clock1-tc.clock0) * bFactor) < tc.timeBudget)
		    {
		      others->iteration++;
#ifdef TREEDUMP
		      printf("============================================\n");
		      printf("             Iteration %d\n", others->iteration);
		      printf("============================================\n");
#endif
		      tc.nodeInc = (BB)(nPMicroS * tc.timeBudget / 10);
		      if (others->Flags & OneMask[7]) tc.nodeInc /= 10;
		      
		      ASSERT(others->iteration <= maxPly);
		      score = rsearch(alpha, beta, others->iteration, &pos, 
				      plyData, hash, others, &tc,
				      allMoves, nMoves, origCE);
		      ASSERT(score >= -mateScore && score <= mateScore);
		      
		      /* if out of bound, re-search with a proper bound */
		      if (!(others->Flags & OneMask[4]) && (score <= alpha))
			{
			  printf("score too low: %d<%d<%d\n", 
				 alpha, score, beta);
			  /* tend to make weak moves here, add time to think */
			  if (others->iteration > minPly && timeReserved > 0)
			    tc.timeBudget += timeReserved;
			  ASSERT(others->iteration < maxPly);
			  score = rsearch(-mateScore, score+1, others->iteration, 
					  &pos, plyData, hash, others, &tc, 
					  allMoves, nMoves, origCE);
			  ASSERT(score >= -mateScore && score <= mateScore);
			}
		      else if (!(others->Flags & OneMask[4]) && (score >= beta))
			{
			  printf("score too high: %d<%d<%d\n", 
				 alpha, score, beta);
			  /* take care of instability (if fail low here) */
			  lastPV = plyData[0].pv[0];
			  lastScore = score;
			  ASSERT(others->iteration < maxPly);
			  score = rsearch(score-1, mateScore, others->iteration, 
					  &pos, plyData, hash, others, &tc,
					  allMoves, nMoves, origCE);
			  ASSERT(score >= -mateScore && score <= mateScore);
			  /* take care of instability (if fail low here) */
			  if (plyData[0].pvLength == 0)
			    {
			      printf("fail low after fail high at root, use lastMove\n");
			      plyData[0].pvLength = 1;
			      plyData[0].pv[0] = lastPV;
			      score = lastScore;
			    }
			}
		      
		      /* the best move so far */
		      if (others->Flags & OneMask[4]) 
			{
			  theMove = lastMove;
			  score = score(lastMove);
			  ASSERT(score >= -mateScore && score <= mateScore);
			}
		      else theMove = (plyData[0].pv[0] & ~ScoreBoard) | 
			     ((BB)(score+mateScore) << 30);
		      lastMove = theMove;
		      
		      /* narrow dowm the search space */
		      if (score > -mateScore+halfWindow)
			alpha = score - halfWindow;
		      else alpha = -mateScore;
		      if (score < mateScore-halfWindow)
			beta = score + halfWindow;
		      else beta = mateScore;


		      if (alpha < -mateScore) alpha = -mateScore;
		      if (beta > mateScore) beta = mateScore;
		      
#ifdef WINDOWS
		      clock1 = 1000000.0/CLOCKS_PER_SEC*clock();
#else
		      gettimeofday(&tv, &tz);
		      clock1 = (BB) tv.tv_sec*1000000 + (BB) tv.tv_usec;
#endif
		      
		      if (others->iteration >= minPly)
			{
			  /* Michael Yee corrected the following lines to 
			     enable xboard's "show thinking" option. */ 
			  printf("%u", others->iteration);
			  printf(" %d", score);
			  /* in centisecond */
			  printf(" %u", (ui) ((clock1-tc.clock0)/10000));
			  printf(" %.0f ", 
				 (double)(others->sNodes + others->qNodes));
			  mNum = others->moveNumber+1;
			  wTM = plyData[0].wToMove;
			  
			  /* print the PV */
			  pvPos = pos;
			  for (j = 0; j < maxPly; j++)
			    pvPlyData[j] = plyData[j];
			  pvOthers = *others;
			  if (others->Flags & OneMask[4])
			    pvPlyData[0].pvLength = 1;
			  for (j = 0; j < pvPlyData[0].pvLength; j++)
			    {
			      if (wTM) printf("%u. ", mNum);
			      else if (j == 0) printf("%u. ... ", mNum);
			      san(j, pvPlyData[0].pv[j], s, &pvPos, pvPlyData, 
				  &pvOthers);
			      printf("%s ", s);
			      wTM ^= 1;
			      if (wTM) mNum++;
			      makeMove(j+1, pvPlyData[0].pv[j], &pvPos, 
				       pvPlyData, &pvOthers);
			    }
			  if (others->Flags & OneMask[4]) printf(" CUT");
			  printf("\n");
			  fflush(stdout);
			}
		      ASSERT(others->moveNumber < maxMove);
#ifdef WINDOWS
		      clock1 = 1000000.0/CLOCKS_PER_SEC*clock();
#else
		      gettimeofday(&tv, &tz);
		      clock1 = (BB) tv.tv_sec*1000000 + (BB) tv.tv_usec;
#endif
		      nPMicroS = (double) (others->sNodes + others->qNodes) /
			(clock1-tc.clock0);
		      if (nPMicroS > 2*NPMS) nPMicroS = 2*NPMS;
		      bFactor = (double) (others->sNodes + others->qNodes) / 
			lastNodes;
		      if (bFactor < 1.0) bFactor = 1.0;
		      if (bFactor < 3.0 && movesLeft <= 7) 
			bFactor = 3.0;
		      if (bFactor > 6.0) bFactor = 6.0;
		      lastNodes = others->sNodes + others->qNodes;
		      if (lastNodes == 0) lastNodes = 1;
		      
		      /* find a mate, enough already */
		      if (score > mateScore-1000) break;
		    }
		}
	      else
		{
		  if (others->iAm == white)
		    {
		      if (getBAttacker(pos.wKSq, &pos, &plyData[0], others) & 2)
			{
			  if (others->Flags & OneMask[0])
			    printf("0-1 {Black mates}\n");
			  else
			    {
			      closing(fd, others, "0-1", "Black mates");
			      exiting(fd, hash, others);
			    }
			}
		      else
			{
			  if (others->Flags & OneMask[0])
			    printf("1/2-1/2 {stalemate}\n");
			  else
			    {
			      closing(fd, others, 
				      "1/2-1/2", "stalemate");
			      exiting(fd, hash, others);
			    }
			}
		    }
		  else
		    {
		      if (getWAttacker(pos.bKSq, &pos, &plyData[0], others) & 2)
			{
			  if (others->Flags & OneMask[0])
			    printf("1-0 {White mates}\n");
			  else
			    {
			      closing(fd, others, "1-0", "White mates");
			      exiting(fd, hash, others);
			    }
			}
		      else
			{
			  if (others->Flags & OneMask[0])
			    printf("1/2-1/2 {stalemate}\n");
			  else
			    {
			      closing(fd, others, 
				      "1/2-1/2", "stalemate");
			      exiting(fd, hash, others);
			    }
			}
		    }
		  
		  gameEnd = true;
		}	      
	    }
	  
	  if (!gameEnd)
	    {
	      /* write to the game listing */
	      san(0, theMove, s, &pos, plyData, others);
	      if (plyData[0].wToMove) 
		{
		      if (others->moveNumber == 0)
			{
			  /* listing headers */
			  fprintf(fd, "[Event \"%s\"]\n", others->fullDate);
			  if (others->iAm == white) 
			    fprintf(fd, "[White \"parrot\"] ");
			  else fprintf(fd, "[Black \"parrot\"] ");
			  fprintf(fd, "        score");
			  fprintf(fd, " opening");
			  fprintf(fd, " depth");
			  fprintf(fd, "\n");
			}
		      fprintf(fd, "white");
		}
	      else fprintf(fd, "black");
	      fprintf(fd, "%4u.", (others->moveNumber+1));
	      fprintf(fd, " %8s", s);
	      if (others->iteration > 0)
		{
		  if (others->iAm == white) fprintf(fd, "\t%6d w", score);
		  else fprintf(fd, "\t%6d b", score);
		  fprintf(fd, " %5u", others->openingNo);
		  fprintf(fd, " %5u", others->iteration);
		  if (strlen(others->solution) != 0) 
		    fprintf(fd, "   %s", others->solution);
		  else fprintf(fd, "   0x%016llX", plyData[0].key);
		} 
	      fprintf(fd, "\n");
	      fflush(fd);

	      /* it's time to make the move now */
	      algebraic(theMove, s);
	      printf("\nmove %s\n", s);
	      
#ifdef BONUSES_PENALTIES
	      printf("PBonus = %d\n", PBonus);
	      printf("NBonus = %d\n", NBonus);
	      printf("BBonus = %d\n", BBonus);
	      printf("RBonus = %d\n", RBonus);
	      printf("QBonus = %d\n", QBonus);
	      printf("KBonus = %d\n", KBonus);
	      
	      printf("PPenalty = %d\n", PPenalty);
	      printf("NPenalty = %d\n", NPenalty);
	      printf("BPenalty = %d\n", BPenalty);
	      printf("RPenalty = %d\n", RPenalty);
	      printf("QPenalty = %d\n", QPenalty);
	      printf("KPenalty = %d\n", KPenalty);

	      printf("PEndBonus = %d\n", PEndBonus);
	      printf("NEndBonus = %d\n", NEndBonus);
	      printf("BEndBonus = %d\n", BEndBonus);
	      printf("REndBonus = %d\n", REndBonus);
	      printf("QEndBonus = %d\n", QEndBonus);
	      printf("KEndBonus = %d\n", KEndBonus);

	      printf("PEndPenalty = %d\n", PEndPenalty);
	      printf("NEndPenalty = %d\n", NEndPenalty);
	      printf("BEndPenalty = %d\n", BEndPenalty);
	      printf("REndPenalty = %d\n", REndPenalty);
	      printf("QEndPenalty = %d\n", QEndPenalty);
	      printf("KEndPenalty = %d\n", KEndPenalty);
#endif

	      sprintf(others->gameID, "%s_%04u", others->fullDate, 
		      others->openingNo);
	      printf("gameID = %s, search depth = %u, %.0f/%.0f nodes, score: %d\n", 
		     others->gameID, others->iteration, (double) others->qNodes, 
		     (double) (others->sNodes + others->qNodes), score(theMove));
	      if (strlen(others->solution)) sprintf(others->myMove, "%s", s);
	      
	      /* it's time to make the move now */
	      makeMove(1, theMove, &pos, plyData, others);
	      if (!(others->Flags & OneMask[0])) display(pos);
#ifdef WINDOWS
	      clock1 = 1000000.0/CLOCKS_PER_SEC*clock();
#else
	      gettimeofday(&tv, &tz);
	      clock1 = (BB) tv.tv_sec*1000000 + (BB) tv.tv_usec;
#endif
	      others->timeLeft -= (clock1 - tc.clock0)/10000;
	      others->timeLeft += others->timeInc;
	      others->oTimeLeft = others->timeLeft - tc.timeBudget/10000;

	      if (others->moveNumber < 30)
		{
		  drawCount = 0;
		  if (score <= -resignValue) acceptDraw = true;
		  else acceptDraw = false;
		  resignCount = 0;
		}
	      else
		{
		  if (abs(score(theMove)) <= drawMargin)
		    {
		      drawCount++;
		      /* keep the status quo of acceptDraw */
		      resignCount = 0;
		    }
		  else if (score <= -resignValue)
		    {
		      drawCount = 0;
		      acceptDraw = true; /* Opponent offers draw?  Why not? */
		      resignCount++;
		    }
		  else
		    {
		      drawCount = 0;
		      acceptDraw = false;
		      resignCount = 0;
		    }
		}

	      /* After making the move, is this position a draw by rule? */
	      switch(draw(theMove))
		{
		case flagDraw3:
		  if (others->Flags & OneMask[0]) 
		    printf("1/2-1/2 {repetition}\n");
		  else
		    {
		      closing(fd, others, "1/2-1/2", "repetition");
		      exiting(fd, hash, others);
		    }
		  break;
		case flagDraw50:
		  if (others->Flags & OneMask[0]) 
		    printf("1/2-1/2 {50 move rule}\n");
		  else
		    {
		      closing(fd, others, "1/2-1/2", "50 move rule");
		      exiting(fd, hash, others);
		    }
		  break;
		default:
		  break;
		}

	      /* After making the move, can I checkmate? */
	      if (score(theMove) == mateScore) 
		{
		  if (others->iAm == white)
		    {
		      if (others->Flags & OneMask[0]) 
			printf("1-0 {White mates}\n");
		      else
			{
			  closing(fd, others, "1-0", "White mates");
			  exiting(fd, hash, others);
			}
		    }
		  else
		    {
		      if (others->Flags & OneMask[0]) 
			printf("0-1 {Black mates}\n");
		      else
			{
			  closing(fd, others, "0-1", "Black mates");
			  exiting(fd, hash, others);
			}
		    }
		}

	      /* After making the move, should we offer a draw? */
	      if (abs(score(theMove)) <= drawMargin &&
		  others->moveNumber >= 30 &&           /* not a short game */
		  movesLeft > 7 &&         /* opponent doesn't need to rush */
		  others->oTimeLeft/movesLeft >= 100)
		{
		  drawCount++;
		  if (drawCount >= drawCountLimit && 
		      plyData[0].count50Moves >= 2*drawOfferRepeat)
		    {
		      if (others->Flags & OneMask[0]) 
#ifdef FICS
			printf("tellics draw\n");
#else
			printf("1/2-1/2 {offer draw}\n");
#endif
		      else
			{
			  closing(fd, others, "1/2-1/2", "offer draw");
			  exiting(fd, hash, others);
			}
		      /* if draw offer is refused, try to be not obnoxious */
		      drawCount -= drawOfferRepeat;
		      /* it is okay to accept draw if opponent offers it */
		      acceptDraw = true;
		    }
		}

	      /* After making the move, should I resign? */
	      if (score(theMove) <= -resignValue && /* we're in a bad shape */
		  others->moveNumber >= 30 &&           /* not a short game */
		  movesLeft > 7 &&         /* opponent doesn't need to rush */
		  others->oTimeLeft/movesLeft >= 100)
		{
		  resignCount++;
		  if (resignCount >= resignCountLimit)
		    {
		      if (others->iAm == white)
			{
			  if (others->Flags & OneMask[0]) 
			    printf("0-1 {resign}\n");
			  else
			    {
			      closing(fd, others, "0-1", "resign");
			      exiting(fd, hash, others);
			    }
			}
		      else
			{
			  if (others->Flags & OneMask[0]) 
			    printf("1-0 {resign}\n");
			  else
			    {
			      closing(fd, others, "1-0", "resign");
			      exiting(fd, hash, others);
			    }
			}
		    }
		}
	      
	      if (score(theMove) > mateScore-1000)
		{
		  dtm = (ui) (mateScore-score(theMove))/2;
		  if (dtm > 0) 
		    printf("Announcing checkmate in %u moves\n", dtm);
		}

	      /* Reset */
#ifdef UNDO_REMOVE
	      if (plyData[0].wToMove)
		{
		  gameListing[others->moveNumber].wPlyData = plyData[0];
		  gameListing[others->moveNumber].wFailBook = others->failBook;
		}
	      else
		{
		  gameListing[others->moveNumber].bPlyData = plyData[0];
		  gameListing[others->moveNumber].bFailBook = others->failBook;
		}
#endif
	      plyData[0] = plyData[1];
	      if (plyData[0].wToMove) others->moveNumber++;
	      ASSERT(others->moveNumber < maxMove);
	      if (movesLeft == 1)
		{
		  others->timeLeft += others->timeLeft0;
		  others->oTimeLeft += others->timeLeft0;
		}
	      if (plyData[0].count50Moves == 0) others->repHead = 0;
	      else others->repHead++;
	    }
	}
      
      else /* forced */
	{
	  others->iAm ^= 1;
	}
    }

  /* All's well that ends well */
  free(EGTB_cache);
  /* Using exiting() here disturbed lint. */
  free(hash[white]);
  free(hash[black]);
  free(others);
  fclose(fd);
  exit(0);
}
/* attack.c */
/* Last modified: March 13, 2006 */


static BB RankAttack[64][256], RankAttack90[64][256];
static BB RankAttack45l[64][256], RankAttack45r[64][256]; 
static ui column45l[64] = {0,  0,  0,  0,  0,  0,  0,  0,
		       1,  1,  1,  1,  1,  1,  1,  1,
		       2,  2,  2,  2,  2,  2,  2,  2,
		       3,  3,  3,  3,  3,  3,  3,  3,
		       4,  4,  4,  4,  4,  4,  4,  4,
		       5,  5,  5,  5,  5,  5,  5,  5,
		       6,  6,  6,  6,  6,  6,  6,  6,
		       7,  7,  7,  7,  7,  7,  7,  7};
static ui column45r[64] = {0,  0,  0,  0,  0,  0,  0,  0,
		       1,  1,  1,  1,  1,  1,  1,  1,
		       2,  2,  2,  2,  2,  2,  2,  2,
		       3,  3,  3,  3,  3,  3,  3,  3,
		       4,  4,  4,  4,  4,  4,  4,  4,
		       5,  5,  5,  5,  5,  5,  5,  5,
		       6,  6,  6,  6,  6,  6,  6,  6,
		       7,  7,  7,  7,  7,  7,  7,  7};
static ui inverse45l[64] = {0, 15, 22, 29, 36, 43, 50, 57,
			1,  8, 23, 30, 37, 44, 51, 58,
			2,  9, 16, 31, 38, 45, 52, 59,
			3, 10, 17, 24, 39, 46, 53, 60,
			4, 11, 18, 25, 32, 47, 54, 61,
			5, 12, 19, 26, 33, 40, 55, 62,
			6, 13, 20, 27, 34, 41, 48, 63,
			7, 14, 21, 28, 35, 42, 49, 56};
static ui inverse45r[64] = {7,  8, 17, 26, 35, 44, 53, 62,
			6, 15, 16, 25, 34, 43, 52, 61,
			5, 14, 23, 24, 33, 42, 51, 60,
			4, 13, 22, 31, 32, 41, 50, 59,
			3, 12, 21, 30, 39, 40, 49, 58,
			2, 11, 20, 29, 38, 47, 48, 57,
			1, 10, 19, 28, 37, 46, 55, 56,
			0,  9, 18, 27, 36, 45, 54, 63};


/*
 * Find the attacks from <sq> on a 8-bit vector supplied as <bitRank>.
 * <bitRank> represents a rank, but it can be rotated to represent a 
 * file or diagonal.  From crafty.
 */
static ui attackedBitRank(ui sq, ui bitRank)
{
  ui start;
  ui attacked = 0;

  ASSERT(sq <= H8);

  /* find attacks to right of <sq> */
  if (sq < 7) 
    {
      start = 1 << (sq+1);
      while (start < 256) 
	{
	  attacked = attacked | start;
	  if (bitRank & start) break;
	  start = start << 1;
	}
    }

  /* find attacks to left of <sq> */
  if (sq > 0) 
    {
      start = 1 << (sq-1);
      while (start > 0) 
	{
	  attacked = attacked | start;
	  if (bitRank & start) break;
	  start = start >> 1;
	}
    }

  return (attacked & 255);
}


void initRankAttacks()
{
  ui firstOne8bit[256];
  ui i, j, sq, t, t45;
  ui attacked;

  /* Index of first bit in an 8-bit value */
  firstOne8bit[0]=8;
  for (i=0; i<256; i++)
    {
      for (j=0; j<8; j++)
	{
	  if (i & (1<<j))
	    {
	      firstOne8bit[i] = j;
	      break;
	    }
	}
    }

  /* Precalculate all possible attacks from <sq> on a 8-bit vector */
  for (sq=0; sq<64; sq++)
    {
      for (i=0; i<256; i++) 
	{
	  attacked = attackedBitRank(column(sq), i);
	  RankAttack[sq][i] = 0;
	  /* mobility[sq][i] = 0; */
	  while (attacked) 
	    {
	      t = firstOne8bit[attacked];
	      /* This is a neat trick.  (sq&56) "slides" sq to first row. */
	      RankAttack[sq][i] |= OneMask[(sq&56) + t];
	      attacked &= ~(1<<t);
	    }
	}
      /* mobility[sq][i] = PopCnt(RankAttack[sq][i]); */
    }

  /* Precalculate all possible attacks from <sq> on a 8-bit vector, rotated
     90 degrees. */
  for (sq=0; sq<64; sq++) 
    {
      for (i=0; i<256; i++) 
	{
	  attacked = attackedBitRank(row(sq), i);
	  RankAttack90[sq][i] = 0;
	  /* mobility90[sq][i] = 0; */
	  while (attacked) 
	    {
	      t = firstOne8bit[attacked];
	      RankAttack90[sq][i] |= OneMask[rotate90[((sq&7)<<3) + t]];
	      attacked &= ~(1<<t);
	    }
	}
      /* mobility90[sq][i]=PopCnt(RankAttack90[sq][i]); */
    }

  /* Precalculate all possible attacks from <sq> on a 8-bit vector, rotated
     45 degrees left. */
  for (sq=0; sq<64; sq++) 
    {
      for (i=0; i<256; i++) 
	{
	  attacked = attackedBitRank(column45l[sq], i);
	  RankAttack45l[sq][i] = 0;
	  /* mobility45l[sq][i] = 0; */
	  while (attacked) 
	    {
	      t = firstOne8bit[attacked];
	      t45 = inverse45l[((rotate45l[sq])&56) + t];
	      if ((row(sq)+column(sq)) == (row(t45)+column(t45)))
		RankAttack45l[sq][i] |= OneMask[t45];
	      attacked &= ~(1<<t);
	    }
	}
      /* mobility45l[sq][i]=PopCnt(RankAttack45l[sq][i]); */
    }

  /* Precalculate all possible attacks from <sq> on a 8-bit vector, rotated
     45 degrees right. */
  for (sq=0; sq<64; sq++) 
    {
      for (i=0; i<256; i++) 
	{
	  attacked = attackedBitRank(column45r[sq], i);
	  RankAttack45r[sq][i] = 0;
	  /* mobility45r[sq][i] = 0; */
	  while (attacked) 
	    {
	      t = firstOne8bit[attacked];
	      t45 = inverse45r[((rotate45r[sq])&56) + t];
	      if ((row(sq)-column(sq)) == (row(t45)-column(t45)))
		RankAttack45r[sq][i] |= OneMask[t45];
	      attacked &= ~(1<<t);
	    }
      /* mobility45r[sq][i]=PopCnt(RankAttack45r[sq][i]); */
	}
    }
}


/* Is sq attacked by White Pawn? */
ui isSqAttackedByWP(ui sq, sPos *pos, sOthers *others)
{
  ASSERT(sq <= H8);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  if (others->BPCaptures[sq] & pos->WPBoard) return 1;
  return 0;
}


/* Is sq attacked by White Knight? */
ui isSqAttackedByWN(ui sq, sPos *pos, sOthers *others)
{
  ASSERT(sq <= H8);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  if (others->NMoves[sq] & pos->WNBoard) return 2;
  return 0;
}


/* Is sq attacked by White Bishop? */
static ui isSqAttackedByWB(ui sq, sPos *pos)
{
  ASSERT(sq <= H8);
  ASSERT(pos != NULL);

  if (bishopAttacks(sq, pos) & pos->WBBoard) return 2;
  return 0;
}


/* Is sq attacked by White Rook? */
static ui isSqAttackedByWR(ui sq, sPos *pos)
{
  ASSERT(sq <= H8);
  ASSERT(pos != NULL);

  if (rookAttacks(sq, pos) & pos->WRBoard) return 4;
  return 0;
}


/* Is sq attacked by White Queen? */
static ui isSqAttackedByWQ(ui sq, sPos *pos)
{
  ASSERT(sq <= H8);
  ASSERT(pos != NULL);

  if (rookAttacks(sq, pos) & pos->WQBoard) return 8;
  if (bishopAttacks(sq, pos) & pos->WQBoard) return 8;
  return 0;
}


/* Is sq attacked by White King? */
ui isSqAttackedByWK(ui sq, sPos *pos, sOthers *others)
{
  ASSERT(sq <= H8);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  if (others->KMoves[sq] & OneMask[pos->wKSq]) return 16;
  return 0;
}


/* Form a bitboard of White pieces that are checking Black king at sq. */
ui isInCheckByW(ui sq, BB *Attackers, sPos *pos, sOthers *others)
{
  ui checkers = 0;
  BB Temp = 0;

  ASSERT(sq <= H8);
  ASSERT(Attackers != NULL);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  /* pawn and knight can't check at the same time */
  if ((Temp = (others->BPCaptures[sq] & pos->WPBoard)))
    {
      checkers++;
      *Attackers |= Temp;
    }
  else if ((Temp = (others->NMoves[sq] & pos->WNBoard)))
    {
      checkers++;
      *Attackers |= Temp;
    }

  /* rook direction first, faster than bishop direction */
  if ((Temp = rookAttacks(sq, pos) & (pos->WRBoard | pos->WQBoard)))
    {
      checkers++;
      *Attackers |= Temp;
    }
  
  /* can't have more than 2 checkers */
  if (checkers < 2 && 
      ((Temp = bishopAttacks(sq, pos) & 
	(pos->WBBoard | pos->WQBoard))))
    {
      checkers++;
      *Attackers |= Temp;
    }

  return checkers;
}


/* Can White make a move to attack sq? */
/* IMPORTANT: this function does not take into account attack from king */
ui isSqAttackableByW(ui sq, sPos *pos, sOthers *others)
{
  BB Temp;
  ui i;

  ASSERT(sq <= H8);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  /* knight */
  Temp = others->NMoves[sq] & ~pos->WBoard;
  while (Temp)
    {
      i = firstOne(Temp);
      if (pos->WNBoard & others->NMoves[i]) return 1;
      Temp &= NilMask[i];
    }

  /* bishop */
  Temp = bishopAttacks(sq, pos) & ~pos->WBoard;
  while (Temp)
    {
      i = firstOne(Temp);
      if (isSqAttackedByWB(i, pos)) return 1;
      Temp &= NilMask[i];
    }

  /* rook */
  Temp = rookAttacks(sq, pos) & ~pos->WBoard;
  while (Temp)
   {
     i = firstOne(Temp);
     if (isSqAttackedByWR(i, pos)) return 1;
     Temp &= NilMask[i];
    }


  /* queen */
  Temp = (bishopAttacks(sq, pos) |
	  rookAttacks(sq, pos)) 
    & ~pos->WBoard;
  while (Temp)
    {
      i = firstOne(Temp);
      if (isSqAttackedByWQ(i, pos)) return 1;
      Temp &= NilMask[i];
    }

  /* pawn */
  Temp = others->BPCaptures[sq] & ~pos->WBoard;
  while (Temp)
    {
      i = firstOne(Temp);
      if (!(OneMask[i] & pos->BBoard))
	{
	  if ((row(i) > 2) && (OneMask[i-8] & pos->WPBoard)) return 1;
	  if ((row(i) == 4) &&
	      !(OneMask[i-8] & (pos->WBoard | pos->BBoard)) &&
	      (OneMask[i-16] & pos->WPBoard))
	    return 1;
	}
      if (pos->WPBoard & others->BPCaptures[i]) return 1;
      Temp &= NilMask[i];
    }

  return 0;
}


/* Is sq attacked by Black Pawn? */
ui isSqAttackedByBP(ui sq, sPos *pos, sOthers *others)
{
  ASSERT(sq <= H8);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  if (others->WPCaptures[sq] & pos->BPBoard) return 1;
  return 0;
}


/* Is sq attacked by Black Knight? */
ui isSqAttackedByBN(ui sq, sPos *pos, sOthers *others)
{
  ASSERT(sq <= H8);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  if (others->NMoves[sq] & pos->BNBoard) return 2;
  return 0;
}


/* Is sq attacked by Black Bishop? */
static ui isSqAttackedByBB(ui sq, sPos *pos)
{
  ASSERT(sq <= H8);
  ASSERT(pos != NULL);

  if (bishopAttacks(sq, pos) & pos->BBBoard) return 2;
  return 0;
}


/* Is sq attacked by Black Rook? */
static ui isSqAttackedByBR(ui sq, sPos *pos)
{
  ASSERT(sq <= H8);
  ASSERT(pos != NULL);

  if (rookAttacks(sq, pos) & pos->BRBoard) return 4;
  return 0;
}


/* Is sq attacked by Black Queen? */
static ui isSqAttackedByBQ(ui sq, sPos *pos)
{
  ASSERT(sq <= H8);
  ASSERT(pos != NULL);

  if (rookAttacks(sq, pos) & pos->BQBoard) return 8;
  if (bishopAttacks(sq, pos) & pos->BQBoard) return 8;
  return 0;
}


/* Is sq attacked by Black King? */
ui isSqAttackedByBK(ui sq, sPos *pos, sOthers *others)
{
  ASSERT(sq <= H8);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  if (others->KMoves[sq] & OneMask[pos->bKSq]) return 16;
  return 0;
}


/* Form a bitboard of Black pieces that are checking White king at sq. */
ui isInCheckByB(ui sq, BB *Attackers, sPos *pos, sOthers *others)
{
  ui checkers = 0;
  BB Temp = 0;

  ASSERT(sq <= H8);
  ASSERT(Attackers != NULL);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  /* pawn and knight can't check at the same time */
  if ((Temp = (others->WPCaptures[sq] & pos->BPBoard)))
    {
      checkers++;
      *Attackers |= Temp;
    }
  else if ((Temp = (others->NMoves[sq] & pos->BNBoard)))
    {
      checkers++;
      *Attackers |= Temp;
    }

  /* rook direction first, faster than bishop direction */
  if ((Temp = rookAttacks(sq, pos) & 
      (pos->BRBoard | pos->BQBoard)))
    {
      checkers++;
      *Attackers |= Temp;
    }
  
  /* can't have more than 2 checkers */
  if (checkers < 2 && 
      ((Temp = bishopAttacks(sq, pos) & 
       (pos->BBBoard | pos->BQBoard))))
    {
      checkers++;
      *Attackers |= Temp;
    }

  return checkers;
}


/* Can Black make a move to attack sq? */
/* IMPORTANT: this function does not take into account attack from king */
ui isSqAttackableByB(ui sq, sPos *pos, sOthers *others)
{
  BB Temp;
  ui i;

  ASSERT(sq <= H8);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  /* knight */
  Temp = others->NMoves[sq] & ~pos->BBoard;
  while (Temp)
    {
      i = firstOne(Temp);
      if (pos->BNBoard & others->NMoves[i]) return 1;
      Temp &= NilMask[i];
    }

  /* bishop */
  Temp = bishopAttacks(sq, pos) & ~pos->BBoard;
  while (Temp)
    {
      i = firstOne(Temp);
      if (isSqAttackedByBB(i, pos)) return 1;
      Temp &= NilMask[i];
    }

  /* rook */
  Temp = rookAttacks(sq, pos) & ~pos->BBoard;
  while (Temp)
    {
      i = firstOne(Temp);
      if (isSqAttackedByBR(i, pos)) return 1;
      Temp &= NilMask[i];
    }

  /* queen */
  Temp = (bishopAttacks(sq, pos) |
	  rookAttacks(sq, pos))
	  & ~pos->BBoard;
  while (Temp)
    {
      i = firstOne(Temp);
      if (isSqAttackedByBQ(i, pos)) return 1;
      Temp &= NilMask[i];
    }


  /* pawn */
  Temp = others->WPCaptures[sq] & ~pos->BBoard;
  while (Temp)
    {
      i = firstOne(Temp);
      if (!(OneMask[i] & pos->WBoard))
	{
	  if ((row(i) < 5) && (OneMask[i+8] & pos->BPBoard)) return 1;
	  if ((row(i) == 3) &&
	      !(OneMask[i+8] & (pos->WBoard | pos->BBoard)) &&
	      (OneMask[i+16] & pos->BPBoard))
	    return 1;
	}
      if (pos->BPBoard & others->WPCaptures[i]) return 1;
      Temp &= NilMask[i];
    }

  return 0;
}


/* Some sort of caching for wAttacker[sq] */
ui getWAttacker(ui sq, sPos *pos, sPlyData *thisPlyData, sOthers *others)
{
  ASSERT(sq <= H8);
  ASSERT(thisPlyData != NULL);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  if (thisPlyData->wAttacker[sq] & 1) return (ui) thisPlyData->wAttacker[sq];
  else
    {
      thisPlyData->wAttacker[sq] = 1;
      if (isSqAttackedByWP(sq, pos, others)) 
	thisPlyData->wAttacker[sq] |= 8;
      if (isSqAttackedByWN(sq, pos, others)) 
	thisPlyData->wAttacker[sq] |= 16;
      if (isSqAttackedByWB(sq, pos)) 
	thisPlyData->wAttacker[sq] |= 16;
      if (isSqAttackedByWR(sq, pos)) 
	thisPlyData->wAttacker[sq] |= 32;
      if (isSqAttackedByWQ(sq, pos)) 
	thisPlyData->wAttacker[sq] |= 64;
      if (isSqAttackedByWK(sq, pos, others)) 
	thisPlyData->wAttacker[sq] |= 128;
      if (thisPlyData->wAttacker[sq] > 1) 
	thisPlyData->wAttacker[sq] |= 2;
      return (ui) thisPlyData->wAttacker[sq];
    }
}


/* Some sort of caching for bAttacker[sq] */
ui getBAttacker(ui sq, sPos *pos, sPlyData *thisPlyData, sOthers *others)
{
  ASSERT(sq <= H8);
  ASSERT(thisPlyData != NULL);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  if (thisPlyData->bAttacker[sq] & 1) return (ui) thisPlyData->bAttacker[sq];
  else
    {
      thisPlyData->bAttacker[sq] = 1;
      if (isSqAttackedByBP(sq, pos, others)) 
	thisPlyData->bAttacker[sq] |= 8;
      if (isSqAttackedByBN(sq, pos, others)) 
	thisPlyData->bAttacker[sq] |= 16;
      if (isSqAttackedByBB(sq, pos)) 
	thisPlyData->bAttacker[sq] |= 16;
      if (isSqAttackedByBR(sq, pos)) 
	thisPlyData->bAttacker[sq] |= 32;
      if (isSqAttackedByBQ(sq, pos)) 
	thisPlyData->bAttacker[sq] |= 64;
      if (isSqAttackedByBK(sq, pos, others)) 
	thisPlyData->bAttacker[sq] |= 128;
      if (thisPlyData->bAttacker[sq] > 1) 
	thisPlyData->bAttacker[sq] |= 2;
      return (ui) thisPlyData->bAttacker[sq];
    }
}


/* Find what piece (pinner) is pinning pinnee to pinKing. */
ui pinner(ui pinneeColor, ui pinnee, ui pinKing, sPos *pos, sOthers *others)
{
  BB AttackBoard, Line;
  ui r, c;

  ASSERT(pinneeColor == white || pinneeColor == black);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  r = row(pinKing);
  c = column(pinKing);

  switch (others->direction[pinKing][pinnee])
    {
    case 0: return none;
    case 1:
    case -1:
      AttackBoard = hRookAttacks(pinnee, pos);
      /* pinnee and pinKing have to be connected though */
      if (AttackBoard & OneMask[pinKing])
	{
	  Line = AttackBoard & others->Row[r];
	  if (pinneeColor == white)
	    {
	      if (Line & pos->BRBoard) return bR;
	      else if (Line & pos->BQBoard) return bQ;
	    }
	  else
	    {
	      if (Line & pos->WRBoard) return wR;
	      else if (Line & pos->WQBoard) return wQ;
	    }
	}
      break;
    case 8:
    case -8:
      AttackBoard = vRookAttacks(pinnee, pos);
      /* pinnee and pinKing have to be connected though */
      if (AttackBoard & OneMask[pinKing])
	{
	  Line = AttackBoard & others->Column[c];
	  if (pinneeColor == white)
	    {
	      if (Line & pos->BRBoard) return bR;
	      else if (Line & pos->BQBoard) return bQ;
	    }
	  else
	    {
	      if (Line & pos->WRBoard) return wR;
	      else if (Line & pos->WQBoard) return wQ;
	    }
	}
      break;
    case 7:
      AttackBoard = lBishopAttacks(pinnee, pos);
      /* pinnee and pinKing have to be connected though */
      if (AttackBoard & OneMask[pinKing])
	{
	  Line = others->Up7Of[pinnee] & AttackBoard;
	  if (pinneeColor == white)
	    {
	      if (Line & pos->BBBoard) return bB;
	      else if (Line & pos->BQBoard) return bQ;
	    }
	  else
	    {
	      if (Line & pos->WBBoard) return wB;
	      else if (Line & pos->WQBoard) return wQ;
	    }
	}
      break;
    case -7:
      AttackBoard = lBishopAttacks(pinnee, pos);
      /* pinnee and pinKing have to be connected though */
      if (AttackBoard & OneMask[pinKing])
	{
	  Line = others->Down7Of[pinnee] & AttackBoard;
	  if (pinneeColor == white)
	    {
	      if (Line & pos->BBBoard) return bB;
	      else if (Line & pos->BQBoard) return bQ;
	    }
	  else
	    {
	      if (Line & pos->WBBoard) return wB;
	      else if (Line & pos->WQBoard) return wQ;
	    }
	}
      break;
    case 9:
      AttackBoard = rBishopAttacks(pinnee, pos);
      /* pinnee and pinKing have to be connected though */
      if (AttackBoard & OneMask[pinKing])
	{
	  Line = others->Up9Of[pinnee] & AttackBoard;
	  if (pinneeColor == white)
	    {
	      if (Line & pos->BBBoard) return bB;
	      else if (Line & pos->BQBoard) return bQ;
	    }
	  else
	    {
	      if (Line & pos->WBBoard) return wB;
	      else if (Line & pos->WQBoard) return wQ;
	    }
	}
      break;
    case -9:
      AttackBoard = rBishopAttacks(pinnee, pos);
      /* pinnee and pinKing have to be connected though */
      if (AttackBoard & OneMask[pinKing])
	{
	  Line = others->Down9Of[pinnee] & AttackBoard;
	  if (pinneeColor == white)
	    {
	      if (Line & pos->BBBoard) return bB;
	      else if (Line & pos->BQBoard) return bQ;
	    }
	  else
	    {
	      if (Line & pos->WBBoard) return wB;
	      else if (Line & pos->WQBoard) return wQ;
	    }
	}
      break;
    }

  return none;
}


/* Return the bitboard of a sliding pieces in a direction from sq */
BB ambusher(ui sq, int direction, sPos *pos, sOthers *others)
{
  BB Attacker;

  ASSERT(sq <= H8);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  switch (direction)
    {
    case 1:
      Attacker = pos->WRBoard | pos->WQBoard | pos->BRBoard | pos->BQBoard;
      return Attacker & hRookAttacks(sq, pos) & others->Up1Of[sq];
    case -1:
      Attacker = pos->WRBoard | pos->WQBoard | pos->BRBoard | pos->BQBoard;
      return Attacker & hRookAttacks(sq, pos) & others->Down1Of[sq];
    case 8:
      Attacker = pos->WRBoard | pos->WQBoard | pos->BRBoard | pos->BQBoard;
      return Attacker & vRookAttacks(sq, pos) & others->Up8Of[sq];
    case -8:
      Attacker = pos->WRBoard | pos->WQBoard | pos->BRBoard | pos->BQBoard;
      return Attacker & vRookAttacks(sq, pos) & others->Down8Of[sq];
    case 7:
      Attacker = pos->WBBoard | pos->WQBoard | pos->BBBoard | pos->BQBoard;
      return Attacker & lBishopAttacks(sq, pos) & others->Up7Of[sq];
    case -7:
      Attacker = pos->WBBoard | pos->WQBoard | pos->BBBoard | pos->BQBoard;
      return Attacker & lBishopAttacks(sq, pos) & others->Down7Of[sq];
    case 9:
      Attacker = pos->WBBoard | pos->WQBoard | pos->BBBoard | pos->BQBoard;
      return Attacker & rBishopAttacks(sq, pos) & others->Up9Of[sq];
    case -9:
      Attacker = pos->WBBoard | pos->WQBoard | pos->BBBoard | pos->BQBoard;
      return Attacker & rBishopAttacks(sq, pos) & others->Down9Of[sq];
    default:
      break;
    }

  return (BB) 0;
}


/* Return the value of a piece at sq */
int valueAtSq(ui sq, sPos *pos)
{
  BB Sq;

  ASSERT(sq <= H8);
  ASSERT(pos != NULL);

  Sq = OneMask[sq];
  if (Sq & pos->WPBoard) return pValue;
  if (Sq & pos->BPBoard) return pValue;
  if (Sq & pos->WNBoard) return nValue;
  if (Sq & pos->BNBoard) return nValue;
  if (Sq & pos->WBBoard) return bValue;
  if (Sq & pos->BBBoard) return bValue;
  if (Sq & pos->WRBoard) return rValue;
  if (Sq & pos->BRBoard) return rValue;
  if (Sq & pos->WQBoard) return qValue;
  if (Sq & pos->BQBoard) return qValue;

  if (sq == pos->wKSq) return kValue;
  if (sq == pos->bKSq) return kValue;

  return none;
}


/* Static Exchange Evaluation, championed by Bob Hyatt */
/* Probably works better if the capturer is not a king */
int see(ui wToMove, ui sq, ui t, sPos *pos, sOthers *others)
{
  int scoreArray[16];
  BB Attacker, Temp;
  ui index = 0;
  int lastValue;
  int direction;
  ui i;

  ASSERT(sq <= H8);
  ASSERT(t <= H8);
  ASSERT(sq != t);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  /* collect all attackers, including ambusher behind sq */
  Attacker = 
    (others->WPCaptures[t] & pos->BPBoard) |
    (others->BPCaptures[t] & pos->WPBoard) |
    (others->NMoves[t] & (pos->WNBoard | pos->BNBoard)) |
    (bishopAttacks(t, pos) & 
     (pos->WBBoard | pos->BBBoard | pos->WQBoard | pos->BQBoard)) |
    (rookAttacks(t, pos) & 
     (pos->WRBoard | pos->BRBoard | pos->WQBoard | pos->BQBoard)) |
    (others->KMoves[t] & (OneMask[pos->wKSq] | OneMask[pos->bKSq]));

  /* first capture */
  scoreArray[index] = valueAtSq(t, pos);
  ASSERT(scoreArray[index] > 0 && scoreArray[index] <= qValue);
  lastValue = valueAtSq(sq, pos);
  Attacker &= NilMask[sq];
  direction = others->direction[t][sq];
  if (direction) Attacker |= ambusher(sq, direction, pos, others);
  wToMove ^= 1;

  /* go through all attackers to fill scoreArray[], smaller ones first */
  while (Attacker)
    {
      Temp = (BB) 0;

      if (wToMove)
	{
	  if ((Temp = (pos->WPBoard & Attacker))) sq = firstOne(Temp);
	  else if ((Temp = (pos->WNBoard & Attacker))) sq = firstOne(Temp);
	  else if ((Temp = (pos->WBBoard & Attacker))) sq = firstOne(Temp);
	  else if ((Temp = (pos->WRBoard & Attacker))) sq = firstOne(Temp);
	  else if ((Temp = (pos->WQBoard & Attacker))) sq = firstOne(Temp);
	  else if (OneMask[pos->wKSq] & Attacker) 
	    {
	      if (Attacker & pos->BBoard) break;
	      sq = pos->wKSq;
	    }
	  else break;        /* otherwise canibalism */
	}
      else
	{
	  if ((Temp = (pos->BPBoard & Attacker))) sq = firstOne(Temp);
	  else if ((Temp = (pos->BNBoard & Attacker))) sq = firstOne(Temp);
	  else if ((Temp = (pos->BBBoard & Attacker))) sq = firstOne(Temp);
	  else if ((Temp = (pos->BRBoard & Attacker))) sq = firstOne(Temp);
	  else if ((Temp = (pos->BQBoard & Attacker))) sq = firstOne(Temp);
	  else if (OneMask[pos->bKSq] & Attacker) 
	    {
	      if (Attacker & pos->WBoard) break;
	      sq = pos->bKSq;
	    }
	  else break;        /* otherwise canibalism */
	}

      index++;
      ASSERT(index < 16);
      scoreArray[index] = lastValue - scoreArray[index-1];
      lastValue = valueAtSq(sq, pos);
      Attacker &= NilMask[sq];
      direction = others->direction[t][sq];
      if (direction) Attacker |= ambusher(sq, direction, pos, others);
      wToMove ^= 1;
    }

  /* Now the tricky part to "minimax" the values */
  for (i = index; i > 0; i--)
    if (scoreArray[i] > -scoreArray[i-1]) scoreArray[i-1] = - scoreArray[i];

  return scoreArray[0];
}
/* book.c */
/* Last modified: June 9, 2006 */


/* Read a PGN file, pick a game where our player wins, 
 * and put the first 20 moves into white or black opening book.
 * If the player is "allPlayers", then it takes all players' games. 
 */
void readPGN(char *file, char *player, int numberOfMoves)
{
  int i, j, iA = 0, iZ = 0;
  int inNumber = 1, inMove1 = 0, inMove2 = 0;
  int commentNo = 0, numberNo = 0, wOpeningNo = 0, bOpeningNo = 0;
  int maxBookMoves;
  int win = 13;                /* 1: 1-0, 0: 1/2-1/2, -1: 0-1 */
  int colorPlayed = 13;        /* white or black */
  int number[256];
  char c, comment[20][512];
  char line[512], move1[256][8], move2[256][8], numberStr[6];
  FILE *fdin, *fdout;

 /* How many openings are there in our opening books? */
  fdin = fopen("white", "r");
  while (fgets(line, 512, fdin))
    {
      if (strstr(line, "OpeningNo"))
	{
	  i = 0;  j = 12;
	  do 
	    {
	      numberStr[i++] = line[j++];
	    } while (line[j] != '"');

	  numberStr[i] = '\0';
	  wOpeningNo = atoi(numberStr);
	}
    }
  fclose(fdin);
  wOpeningNo++;

  fdin = fopen("black", "r");
  while (fgets(line, 512, fdin))
    {
      if (strstr(line, "OpeningNo"))
	{
	  i = 0;  j = 12;
	  do 
	    {
	      numberStr[i++] = line[j++];
	    } while (line[j] != '"');

	  numberStr[i] = '\0';
	  bOpeningNo = atoi(numberStr);
	}
    }
  fclose(fdin);
  bOpeningNo++;

  fdin = fopen(file, "r");
  while (fgets(line, 512, fdin))
    {
      if (line[0] == '[') 
	{
	  strcpy(comment[commentNo++], line);

	  if (strstr(line, "Event ") && commentNo > 1)
	    {
	      /* write to opening book */
	      if (!strcmp(player, "allPlayers"))
		{
		  if (win == 1) colorPlayed = white;
		  else if (win == -1) colorPlayed = black;
		}
	      if ((win == 1 && colorPlayed == white) || 
		  (win == -1 && colorPlayed == black))
		{
		  if (win == 1) fdout = fopen("white", "a+");
		  else fdout = fopen("black", "a+");
		  
		  if (win == 1)
		    fprintf(fdout, "[OpeningNo \"%d\"]\n", wOpeningNo++);
		  else
		    fprintf(fdout, "[OpeningNo \"%d\"]\n", bOpeningNo++);
		  maxBookMoves = numberOfMoves;
		  if (numberNo < maxBookMoves) maxBookMoves = numberNo;
		  for (i = 0; i < commentNo; i++) 
		    fprintf(fdout, "%s", comment[i]);
		  for (i = 0; i < maxBookMoves; i++)
		    fprintf (fdout, "%d. %s %s\n", 
			     number[i], move1[i], move2[i]);
		  fprintf(fdout, "\n");
		  fclose(fdout);
		}
	      
	      /* initialize control variables for next opening */
	      inNumber = 1, inMove1 = inMove2 = 0;
	      commentNo = numberNo = 0;
	      win = colorPlayed = 13;
	    }
	  
	  /* who wins? */
	  if (strstr(line, "Result"))
	    {
	      if (strstr(line, "1-0")) win = 1;
	      else if (strstr(line, "1/2")) win = 0;
	      if (strstr(line, "0-1")) win = -1;
	    }

	  /* Does our player wins this game? */
	  if (strstr(line, player))
	    {
	      if (strstr(line, "White ")) colorPlayed = white;
	      else if (strstr(line, "Black ")) colorPlayed = black;
	    }
	}
      else 
	{
	  /* add a space to line so my algorithm below works as it is */
	  iZ = (int) strlen(line);
	  if (iZ > 0 && line[iZ-2] != ' ')/* the last character is a newline */
	    {
	      line[iZ] = '\0';
	      line[iZ-1] = line[iZ-2];
	      line[iZ-2] = ' ';
	    } 
	  
	  iA = iZ = 0;
	  while ((c = line[iZ++]) != '\0')
	    {
	      if (inNumber && (c == '.'))
		{
		  i = 0;
		  for (j = iA; j < iZ-1; j++) numberStr[i++] = line[j];
		  numberStr[i++] = '\0';
		  number[numberNo] = atoi(numberStr);
		  iA = iZ;
		  while ((c = line[iZ]) == ' ') iA = ++iZ;

		  inNumber = 0; inMove1 = 1;
		}
	      
	      if (inMove1 && (c == ' '))
		{
		  i = 0;
		  for (j = iA; j < iZ-1; j++) move1[numberNo][i++] = line[j];
		  move1[numberNo][i++] = '\0';
		  iA = iZ;
		  while ((c = line[iZ]) == ' ') iA = ++iZ;

		  inMove1 = 0; inMove2 = 1;
		}
	      
	      if (inMove2 && (c == ' '))
		{
		  i = 0;
		  for (j = iA; j < iZ-1; j++) move2[numberNo][i++] = line[j];
		  move2[numberNo][i++] = '\0';
		  iA = iZ;
		  while ((c = line[iZ]) == ' ') iA = ++iZ;

		  inMove2 = 0; inNumber = 1;
		  numberNo++;
		}
	    }
	}
    }     
  fclose(fdin);


  /* Take care of the last game because the file doesn't end with Event */
  /* write to opening book */
  if (!strcmp(player, "allPlayers"))
    {
      if (win == 1) colorPlayed = white;
      else if (win == -1) colorPlayed = black;
    }
  if ((win == 1 && colorPlayed == white) || 
      (win == -1 && colorPlayed == black))
    {
      if (win == 1) fdout = fopen("white", "a+");
      else fdout = fopen("black", "a+");
      
      if (win == 1)
	fprintf(fdout, "[OpeningNo \"%d\"]\n", wOpeningNo++);
      else
	fprintf(fdout, "[OpeningNo \"%d\"]\n", bOpeningNo++);
      maxBookMoves = numberOfMoves;
      if (numberNo < maxBookMoves) maxBookMoves = numberNo;
      for (i = 0; i < commentNo; i++) 
	fprintf(fdout, "%s", comment[i]);
      for (i = 0; i < maxBookMoves; i++)
	fprintf (fdout, "%d. %s %s\n", 
		 number[i], move1[i], move2[i]);
      fprintf(fdout, "\n");
      fclose(fdout);
    }
}


/* Read a white file which contains games where white wins.  
 * Print out the FEN of a position 2 moves before white wins material.
 */
void whiteFEN(sPos *pos, sPlyData *plyData, sHash **hash, sOthers *others)
{
  int no = 0, number, printed;
  char line[512], numberStr[8], move1[8], move2[8], s[8], fen[2*maxMove][80];
  FILE *fdin;
  ui i, j, ply = 0, wCMoves, nCMoves, winMaterial = 0;
  size_t k;
  BB winningCapture[32], nonCapture[128];
  BB theMove;
  int wPieceScore, bPieceScore, mScore = 0, doubleBonus = 0;  /* not used */

  fdin = fopen("white", "r");
  while (fgets(line, 512, fdin) && ply < 2*maxMove)
    {
      if (line[0] == '[' || line[0] == '\n' || line[0] == '\r') 
	{
	  if (strstr(line, "OpeningNo"))
	    {
	      init(pos, plyData, hash, others);
	      ply = 0;
	      printed = false;

	      i = 0;  j = 12;
	      do 
		{
		  numberStr[i++] = line[j++];
		} while (line[j] != '"');

	      numberStr[i] = '\0';
	      no = atoi(numberStr);
	    }
	}
      else 
	{      
	  sscanf (line, "%d. %s %s", &number, move1, move2);

	  /* white's move */
	  theMove = 0;
	  /* Try winning captures */
	  wCMoves = nCMoves = 0;
	  wCMoves = captures(ply, winningCapture, 0, pos, plyData, others, 0);
	  if (wCMoves > 0) 
	    {
	      for (j = 0; j < wCMoves; j++)
		{
		  san(ply, winningCapture[j], s, pos, plyData, others);
		  /* remove '+' or '#' */
		  for (k = strlen(s); k > 1; k--)
		    if (s[k] == '+' || s[k] == '#') s[k] = '\0';
		  for (k = strlen(move1); k > 1; k--)
		    if (move1[k] == '+' || move1[k] == '#') move1[k] = '\0';
		  if (!strcmp(move1, s)) theMove = winningCapture[j];
		}
	    }
	  /* Try non captures */
	  if (theMove == 0)
	    {
	      nCMoves = nonCaptures(ply, nonCapture, pos, plyData, others, 0);
	      if (nCMoves > 0) 
		{
		  for (j = 0; j < nCMoves; j++)
		    {
		      san(ply, nonCapture[j], s, pos, plyData, others);
		      /* remove '+' or '#' */
		      for (k = strlen(s); k > 1; k--)
			if (s[k] == '+' || s[k] == '#') s[k] = '\0';
		      for (k = strlen(move1); k > 1; k--)
			if (move1[k] == '+' || move1[k] == '#') 
			  move1[k] = '\0';
		      if (!strcmp(move1, s)) theMove = nonCapture[j];
		    }
		}
	    }
	  if (theMove == 0)
	    {
	      if (plyData[0].wToMove)
		printf("ERROR in white no %d: %d. %s\n",
		       no, number, move1); 
	      else
		printf("ERROR in black no %d: %d. %s\n",
		       no, number, move1); 
	      exit(EXIT_FAILURE);
	    }

	  /* make the move, go to the next ply */
	  makeMove(++ply, theMove, pos, plyData, others);
	  /* uncomment this if there's a problem with the book 
	     display(*pos); */

	  /* black's move */
	  theMove = 0;
	  /* Try winning captures */
	  wCMoves = nCMoves = 0;
	  wCMoves = captures(ply, winningCapture, 0, pos, plyData, others, 0);
	  if (wCMoves > 0) 
	    {
	      for (j = 0; j < wCMoves; j++)
		{
		  san(ply, winningCapture[j], s, pos, plyData, others);
		  /* remove '+' or '#' */
		  for (k = strlen(s); k > 1; k--)
		    if (s[k] == '+' || s[k] == '#') s[k] = '\0';
		  for (k = strlen(move2); k > 1; k--)
		    if (move2[k] == '+' || move2[k] == '#') move2[k] = '\0';
		  if (!strcmp(move2, s)) theMove = winningCapture[j];
		}
	    }
	  /* Try non captures */
	  if (theMove == 0)
	    {
	      nCMoves = nonCaptures(ply, nonCapture, pos, plyData, others, 0);
	      if (nCMoves > 0) 
		{
		  for (j = 0; j < nCMoves; j++)
		    {
		      san(ply, nonCapture[j], s, pos, plyData, others);
		      /* remove '+' or '#' */
		      for (k = strlen(s); k > 1; k--)
			if (s[k] == '+' || s[k] == '#') s[k] = '\0';
		      for (k = strlen(move2); k > 1; k--)
			if (move2[k] == '+' || move2[k] == '#') 
			  move2[k] = '\0';
		      if (!strcmp(move2, s)) theMove = nonCapture[j];
		    }
		}
	    }
	  if (theMove == 0)
	    {
	      if (plyData[0].wToMove)
		printf("ERROR in white no %d: %d. ... %s\n",
		       no, number, move2); 
	      else
		printf("ERROR in black no %d: %d. ... %s\n",
		       no, number, move2); 
	      exit(EXIT_FAILURE);
	    }

	  /* make the move, go to the next ply */
	  makeMove(++ply, theMove, pos, plyData, others);
	  /* uncomment this if there's a problem with the book 
	     display(*pos); */

	  /* save the FEN for this position */
	  getFEN(pos, &plyData[ply], others, fen[ply]);
	  /* if white wins material for 3 moves, print FEN from 5 moves ago */
	  if (evalMaterial(pos, others, &wPieceScore, &bPieceScore, &mScore,
			   &doubleBonus) 
	      >= 100) 
	    winMaterial++;
	  else winMaterial = 0;
	  /* 16 to avoid a gambit */ 
	  if (winMaterial == 3 && !printed && ply > 16+10)
	    {
	      printed = true;
	      printf("%s %d %d\n", fen[ply-10], no, number-5);
	    }
	}
    } 
  fclose(fdin);
}


/* Read a black file which contains games where black wins.  
 * Print out the FEN of a position 2 moves before black wins material.
 */
void blackFEN(sPos *pos, sPlyData *plyData, sHash **hash, sOthers *others)
{
  int no = 0, number, printed;
  char line[512], numberStr[8], move1[8], move2[8], s[8], fen[2*maxMove][80];
  FILE *fdin;
  ui i, j, ply = 0, wCMoves, nCMoves, winMaterial = 0;
  size_t k;
  BB winningCapture[32], nonCapture[128];
  BB theMove;
  int wPieceScore, bPieceScore, mScore = 0, doubleBonus = 0;  /* not used */
  fdin = fopen("black", "r");
  while (fgets(line, 512, fdin) && ply < 2*maxMove)
    {
      if (line[0] == '[' || line[0] == '\n' || line[0] == '\r') 
	{
	  if (strstr(line, "OpeningNo"))
	    {
	      init(pos, plyData, hash, others);
	      printed = false;
	      ply = 0;

	      i = 0;  j = 12;
	      do 
		{
		  numberStr[i++] = line[j++];
		} while (line[j] != '"');

	      numberStr[i] = '\0';
	      no = atoi(numberStr);
	    }
	}
      else 
	{      
	  sscanf (line, "%d. %s %s", &number, move1, move2);

	  /* black's move */
	  theMove = 0;
	  /* Try winning captures */
	  wCMoves = nCMoves = 0;
	  wCMoves = captures(ply, winningCapture, 0, pos, plyData, others, 0);
	  if (wCMoves > 0) 
	    {
	      for (j = 0; j < wCMoves; j++)
		{
		  san(ply, winningCapture[j], s, pos, plyData, others);
		  /* remove '+' or '#' */
		  for (k = strlen(s); k > 1; k--)
		    if (s[k] == '+' || s[k] == '#') s[k] = '\0';
		  for (k = strlen(move1); k > 1; k--)
		    if (move1[k] == '+' || move1[k] == '#') move1[k] = '\0';
		  if (!strcmp(move1, s)) theMove = winningCapture[j];
		}
	    }
	  /* Try non captures */
	  if (theMove == 0)
	    {
	      nCMoves = nonCaptures(ply, nonCapture, pos, plyData, others, 0);
	      if (nCMoves > 0) 
		{
		  for (j = 0; j < nCMoves; j++)
		    {
		      san(ply, nonCapture[j], s, pos, plyData, others);
		      /* remove '+' or '#' */
		      for (k = strlen(s); k > 1; k--)
			if (s[k] == '+' || s[k] == '#') s[k] = '\0';
		      for (k = strlen(move1); k > 1; k--)
			if (move1[k] == '+' || move1[k] == '#') 
			  move1[k] = '\0';
		      if (!strcmp(move1, s)) theMove = nonCapture[j];
		    }
		}
	    }
	  if (theMove == 0)
	    {
	      if (plyData[0].wToMove)
		printf("ERROR in black no %d: %d. %s\n",
		       no, number, move1); 
	      else
		printf("ERROR in black no %d: %d. %s\n",
		       no, number, move1); 
	      exit(EXIT_FAILURE);
	    }

	  /* make the move, go to the next ply */
	  makeMove(++ply, theMove, pos, plyData, others);
	  /* uncomment this if there's a problem with the book 
	     display(*pos); */


	  /* save the FEN for this position */
	  getFEN(pos, &plyData[ply], others, fen[ply]);
	  /* if black wins material for 3 moves, print FEN from 5 moves ago */
	  if (evalMaterial(pos, others, &wPieceScore, &bPieceScore, &mScore,
			   &doubleBonus) 
	      <= -100) 
	    winMaterial++;
	  else winMaterial = 0;
	  /* 16 to avoid a gambit */ 
	  if (winMaterial == 3 && !printed && ply > 16+10)
	    {
	      printed = true;
	      printf("%s %d %d\n", fen[ply-10], no, number-5);
	    }

	  /* black's move */
	  theMove = 0;
	  /* Try winning captures */
	  wCMoves = nCMoves = 0;
	  wCMoves = captures(ply, winningCapture, 0, pos, plyData, others, 0);
	  if (wCMoves > 0) 
	    {
	      for (j = 0; j < wCMoves; j++)
		{
		  san(ply, winningCapture[j], s, pos, plyData, others);
		  /* remove '+' or '#' */
		  for (k = strlen(s); k > 1; k--)
		    if (s[k] == '+' || s[k] == '#') s[k] = '\0';
		  for (k = strlen(move2); k > 1; k--)
		    if (move2[k] == '+' || move2[k] == '#') move2[k] = '\0';
		  if (!strcmp(move2, s)) theMove = winningCapture[j];
		}
	    }
	  /* Try non captures */
	  if (theMove == 0)
	    {
	      nCMoves = nonCaptures(ply, nonCapture, pos, plyData, others, 0);
	      if (nCMoves > 0) 
		{
		  for (j = 0; j < nCMoves; j++)
		    {
		      san(ply, nonCapture[j], s, pos, plyData, others);
		      /* remove '+' or '#' */
		      for (k = strlen(s); k > 1; k--)
			if (s[k] == '+' || s[k] == '#') s[k] = '\0';
		      for (k = strlen(move2); k > 1; k--)
			if (move2[k] == '+' || move2[k] == '#') 
			  move2[k] = '\0';
		      if (!strcmp(move2, s)) theMove = nonCapture[j];
		    }
		}
	    }
	  if (theMove == 0)
	    {
	      if (plyData[0].wToMove)
		printf("ERROR in black no %d: %d. ... %s\n",
		       no, number, move2); 
	      else
		printf("ERROR in black no %d: %d. ... %s\n",
		       no, number, move2); 
	      exit(EXIT_FAILURE);
	    }

	  /* make the move, go to the next ply */
	  makeMove(++ply, theMove, pos, plyData, others);
	  /* uncomment this if there's a problem with the book 
	     display(*pos); */
	}
    } 
  fclose(fdin);
}


void readTextBooks(sPos *pos, sPlyData *plyData, sHash **hash, sOthers *others)
{
  int no = 0;
  int number, done;
  char line[512], numberStr[8], move1[8], move2[8], s[8];
  FILE *fdin, *fout;
  ui i, j, ply = 0, wCMoves, nCMoves, posNo = 0, uniqueNo = 0, dot = 0;
  size_t k;
  BB winningCapture[32], nonCapture[128];
  BB theMove;
  struct BookTable *bookPtr;
#ifdef WINDOWS
  double c0, c1;
#else
  struct timeval tv;
  struct timezone tz;
  long c0, c1;
#endif

#ifdef WINDOWS
  c0 = 1000000.0/CLOCKS_PER_SEC*clock();
#else
  gettimeofday(&tv, &tz);
  c0 = tv.tv_sec * 1000000;
#endif

  /* initialize book tables */
  memset((void *) wBookTable, 0, (bookSize+1)*sizeof(struct BookTable));
  memset((void *) bBookTable, 0, (bookSize+1)*sizeof(struct BookTable));

  printf("Reading opening books: ");

  /* white book */
  no = 0;
  fdin = fopen("white", "r");
  while (fgets(line, 512, fdin))
    {
      if (line[0] == '[' || line[0] == '\n' || line[0] == '\r') 
	{
	  if (strstr(line, "OpeningNo"))
	    {
	      init(pos, plyData, hash, others);
	      ply = 0;

	      i = 0;  j = 12;
	      do 
		{
		  numberStr[i++] = line[j++];
		} while (line[j] != '"');

	      numberStr[i] = '\0';
	      no = atoi(numberStr);
	    }
	}
      else 
	{
	  sscanf (line, "%d. %s %s", &number, move1, move2);

	  /* white's move */
	  theMove = 0;
	  /* Try winning captures */
	  wCMoves = nCMoves = 0;
	  wCMoves = captures(ply, winningCapture, 0, pos, plyData, others, 0);
	  if (wCMoves > 0) 
	    {
	      for (j = 0; j < wCMoves; j++)
		{
		  san(ply, winningCapture[j], s, pos, plyData, others);
		  /* remove '+' or '#' */
		  for (k = strlen(s); k > 1; k--)
		    if (s[k] == '+' || s[k] == '#') s[k] = '\0';
		  for (k = strlen(move1); k > 1; k--)
		    if (move1[k] == '+' || move1[k] == '#') move1[k] = '\0';
		  if (!strcmp(move1, s)) theMove = winningCapture[j];
		}
	    }
	  /* Try non captures */
	  if (theMove == 0)
	    {
	      nCMoves = nonCaptures(ply, nonCapture, pos, plyData, others, 0);
	      if (nCMoves > 0) 
		{
		  for (j = 0; j < nCMoves; j++)
		    {
		      san(ply, nonCapture[j], s, pos, plyData, others);
		      /* remove '+' or '#' */
		      for (k = strlen(s); k > 1; k--)
			if (s[k] == '+' || s[k] == '#') s[k] = '\0';
		      for (k = strlen(move1); k > 1; k--)
			if (move1[k] == '+' || move1[k] == '#') 
			  move1[k] = '\0';
		      if (!strcmp(move1, s)) theMove = nonCapture[j];
		    }
		}
	    }
	  if (theMove == 0)
	    {
	      if (plyData[0].wToMove)
		printf("ERROR in white no %d: %d. %s\n",
		       no, number, move1); 
	      else
		printf("ERROR in black no %d: %d. %s\n",
		       no, number, move1); 
	      exit(EXIT_FAILURE);
	    }

	  /* store in book hash table */
	  done = 0;
	  /* no rehashing, keep filling in the blank spaces */
	  for (bookPtr = &wBookTable[plyData[ply].key & (bookSize)];
	       bookPtr->count;
	       bookPtr++)
	    {
	      /* the move is already there */
	      if (bookPtr->key == (ui) plyData[ply].key &&
		  bookPtr->move == theMove)
		{
		  bookPtr->no = (unsigned short) no;
		  bookPtr->count++;
		  done = 1;
		  break;
		}
	      /* wrap around to the beginning of the table */
	      if (bookPtr == &wBookTable[bookSize])
		bookPtr = &wBookTable[0];
	    }
	  /* the move isn't there, but there's a blank space */
	  if (!done)
	    {
	      bookPtr->key = (ui) plyData[ply].key;
	      bookPtr->move = theMove;
	      bookPtr->no = (unsigned short) no;
	      bookPtr->count++;
	      uniqueNo++;
	    }
	  
	  /* make the move, go to the next ply */
	  makeMove(++ply, theMove, pos, plyData, others);
	  /* uncomment this if there's a problem with the book 
	     display(*pos); */

	  /* some statistics and nicities */
	  posNo++;  dot++;
	  if (dot > 65)
	    {
	      dot = 0;
	      printf("w");  fflush(stdout);
	    }

	  /* black's move */
	  theMove = 0;
	  /* Try winning captures */
	  wCMoves = nCMoves = 0;
	  wCMoves = captures(ply, winningCapture, 0, pos, plyData, others, 0);
	  if (wCMoves > 0) 
	    {
	      for (j = 0; j < wCMoves; j++)
		{
		  san(ply, winningCapture[j], s, pos, plyData, others);
		  /* remove '+' or '#' */
		  for (k = strlen(s); k > 1; k--)
		    if (s[k] == '+' || s[k] == '#') s[k] = '\0';
		  for (k = strlen(move2); k > 1; k--)
		    if (move2[k] == '+' || move2[k] == '#') move2[k] = '\0';
		  if (!strcmp(move2, s)) theMove = winningCapture[j];
		}
	    }
	  /* Try non captures */
	  if (theMove == 0)
	    {
	      nCMoves = nonCaptures(ply, nonCapture, pos, plyData, others, 0);
	      if (nCMoves > 0) 
		{
		  for (j = 0; j < nCMoves; j++)
		    {
		      san(ply, nonCapture[j], s, pos, plyData, others);
		      /* remove '+' or '#' */
		      for (k = strlen(s); k > 1; k--)
			if (s[k] == '+' || s[k] == '#') s[k] = '\0';
		      for (k = strlen(move2); k > 1; k--)
			if (move2[k] == '+' || move2[k] == '#') 
			  move2[k] = '\0';
		      if (!strcmp(move2, s)) theMove = nonCapture[j];
		    }
		}
	    }
	  if (theMove == 0)
	    {
	      if (plyData[0].wToMove)
		printf("ERROR in white no %d: %d. ... %s\n",
		       no, number, move2); 
	      else
		printf("ERROR in black no %d: %d. ... %s\n",
		       no, number, move2); 
	      exit(EXIT_FAILURE);
	    }

	  /* make the move, go to the next ply */
	  makeMove(++ply, theMove, pos, plyData, others);
	  /* uncomment this if there's a problem with the book 
	     display(*pos); */
	}
    }
  fclose(fdin);

  /* black book */
  no = 0;
  fdin = fopen("black", "r");
  while (fgets(line, 512, fdin))
    {
      if (line[0] == '[' || line[0] == '\n') 
	{
	  if (strstr(line, "OpeningNo"))
	    {
	      init(pos, plyData, hash, others);
	      ply = 0;

	      i = 0;  j = 12;
	      do 
		{
		  numberStr[i++] = line[j++];
		} while (line[j] != '"');

	      numberStr[i] = '\0';
	      no = atoi(numberStr);
	    }
	}
      else 
	{
	  sscanf (line, "%d. %s %s", &number, move1, move2);

	  /* black's move */
	  theMove = 0;
	  /* Try winning captures */
	  wCMoves = nCMoves = 0;
	  wCMoves = captures(ply, winningCapture, 0, pos, plyData, others, 0);
	  if (wCMoves > 0) 
	    {
	      for (j = 0; j < wCMoves; j++)
		{
		  san(ply, winningCapture[j], s, pos, plyData, others);
		  /* remove '+' or '#' */
		  for (k = strlen(s); k > 1; k--)
		    if (s[k] == '+' || s[k] == '#') s[k] = '\0';
		  for (k = strlen(move1); k > 1; k--)
		    if (move1[k] == '+' || move1[k] == '#') move1[k] = '\0';
		  if (!strcmp(move1, s)) theMove = winningCapture[j];
		}
	    }
	  /* Try non captures */
	  if (theMove == 0)
	    {
	      nCMoves = nonCaptures(ply, nonCapture, pos, plyData, others, 0);
	      if (nCMoves > 0) 
		{
		  for (j = 0; j < nCMoves; j++)
		    {
		      san(ply, nonCapture[j], s, pos, plyData, others);
		      /* remove '+' or '#' */
		      for (k = strlen(s); k > 1; k--)
			if (s[k] == '+' || s[k] == '#') s[k] = '\0';
		      for (k = strlen(move1); k > 1; k--)
			if (move1[k] == '+' || move1[k] == '#') 
			  move1[k] = '\0';
		      if (!strcmp(move1, s)) theMove = nonCapture[j];
		    }
		}
	    }
	  if (theMove == 0)
	    {
	      if (plyData[0].wToMove)
		printf("ERROR in black no %d: %d. %s\n",
		       no, number, move1); 
	      else
		printf("ERROR in black no %d: %d. %s\n",
		       no, number, move1); 
	      exit(EXIT_FAILURE);
	    }

	  /* make the move, go to the next ply */
	  makeMove(++ply, theMove, pos, plyData, others);
	  /* uncomment this if there's a problem with the book 
	     display(*pos); */

	  /* black's move */
	  theMove = 0;
	  /* Try winning captures */
	  wCMoves = nCMoves = 0;
	  wCMoves = captures(ply, winningCapture, 0, pos, plyData, others, 0);
	  if (wCMoves > 0) 
	    {
	      for (j = 0; j < wCMoves; j++)
		{
		  san(ply, winningCapture[j], s, pos, plyData, others);
		  /* remove '+' or '#' */
		  for (k = strlen(s); k > 1; k--)
		    if (s[k] == '+' || s[k] == '#') s[k] = '\0';
		  for (k = strlen(move2); k > 1; k--)
		    if (move2[k] == '+' || move2[k] == '#') move2[k] = '\0';
		  if (!strcmp(move2, s)) theMove = winningCapture[j];
		}
	    }
	  /* Try non captures */
	  if (theMove == 0)
	    {
	      nCMoves = nonCaptures(ply, nonCapture, pos, plyData, others, 0);
	      if (nCMoves > 0) 
		{
		  for (j = 0; j < nCMoves; j++)
		    {
		      san(ply, nonCapture[j], s, pos, plyData, others);
		      /* remove '+' or '#' */
		      for (k = strlen(s); k > 1; k--)
			if (s[k] == '+' || s[k] == '#') s[k] = '\0';
		      for (k = strlen(move2); k > 1; k--)
			if (move2[k] == '+' || move2[k] == '#') 
			  move2[k] = '\0';
		      if (!strcmp(move2, s)) theMove = nonCapture[j];
		    }
		}
	    }
	  if (theMove == 0)
	    {
	      if (plyData[0].wToMove)
		printf("ERROR in black no %d: %d. ... %s\n",
		       no, number, move2); 
	      else
		printf("ERROR in black no %d: %d. ... %s\n",
		       no, number, move2); 
	      exit(EXIT_FAILURE);
	    }

	  /* store in book hash table */
	  done = 0;
	  /* no rehashing, keep filling in the blank spaces */
	  for (bookPtr = &bBookTable[plyData[ply].key & (bookSize)];
	       bookPtr->count;
	       bookPtr++)
	    {
	      /* the move is already there */
	      if (bookPtr->key == (ui) plyData[ply].key &&
		  bookPtr->move == theMove)
		{
		  bookPtr->no = (unsigned short) no;
		  bookPtr->count++;
		  done = 1;
		  break;
		}
	      /* wrap around to the beginning of the table */
	      if (bookPtr == &bBookTable[bookSize])
		bookPtr = &bBookTable[0];
	    }
	  /* the move isn't there, but there's a blank space */
	  if (!done)
	    {
	      bookPtr->key = (ui) plyData[ply].key;
	      bookPtr->move = theMove;
	      bookPtr->no = (unsigned short) no;
	      bookPtr->count++;
	      uniqueNo++;
	    }
	  
	  /* make the move, go to the next ply */
	  makeMove(++ply, theMove, pos, plyData, others);
	  /* uncomment this if there's a problem with the book 
	     display(*pos); */

	  /* some statistics and nicities */
	  posNo++;  dot++;
	  if (dot > 65)
	    {
	      dot = 0;
	      printf("b");  fflush(stdout);
	    }
	}
    }
  fclose(fdin);

#ifdef WINDOWS
  c1 = 1000000.0/CLOCKS_PER_SEC*clock();
#else
  gettimeofday(&tv, &tz);
  c1 = tv.tv_sec * 1000000;
#endif

  printf("\n%u positions in %g seconds\n", posNo, (float)(c1-c0)/1000000);
  printf("Unique positions: %2.2f percent of bookSize\n",
	 (posNo*100.0/(bookSize*2)));

  /* write the books to binary files */
  fout = fopen("white.bin", "w");
  fwrite(wBookTable, sizeof(struct BookTable), (bookSize+1), fout);
  fclose(fout);
  fout = fopen("black.bin", "w");
  fwrite(bBookTable, sizeof(struct BookTable), (bookSize+1), fout);
  fclose(fout);
}


void readBinaryBooks()
{
  FILE *fdin;

  /* read the books from binary files */
  if ((fdin = fopen("white.bin", "r")) != NULL)
    {
      fread(wBookTable, sizeof(struct BookTable), (bookSize+1), fdin);
      fclose(fdin);
    }
  else printf("Playing without white opening book.\n");
  if ((fdin = fopen("black.bin", "r")) != NULL)
    {
      fread(bBookTable, sizeof(struct BookTable), (bookSize+1), fdin);
      fclose(fdin);
    }
  else printf("Playing without black opening book.\n");
}


/* return a move from the opening book or zero */
BB openingBook(sPlyData *thisPlyData, sOthers *others)
{
  struct BookTable *bookPtr;
  int moveNo = 0, r;
  unsigned short count = 0;

  if (thisPlyData->wToMove)
    {
      bookPtr = &wBookTable[thisPlyData->key & (bookSize)];
      /* counting beans */
      for (bookPtr = &wBookTable[thisPlyData->key & (bookSize)];
	   bookPtr->count;
	   bookPtr++)
	{
	  if (bookPtr->key == (ui) thisPlyData->key)
	    {
	      moveNo++;
	      count += bookPtr->count;
	    }
	}

      /* picking a bean */
      if (moveNo == 0) return 0;
      else if (moveNo == 1) 
	{
	  for (bookPtr = &wBookTable[thisPlyData->key & (bookSize)];
	       bookPtr->count;
	       bookPtr++)
	    {
	      if (bookPtr->key == (ui) thisPlyData->key)
		{
		  others->openingNo = (ui) bookPtr->no;
		  return bookPtr->move;
		}
	    }
	}
      else
	{
	  if (others->Flags & OneMask[3]) r = (int) random32() % (count);
	  else r = 0;
	  for (bookPtr = &wBookTable[thisPlyData->key & (bookSize)];
	       bookPtr->count;
	       bookPtr++)
	    {
	      if (bookPtr->key == (ui) thisPlyData->key)
		{
		  r -= bookPtr->count;
		  if (r < 1) 
		    {
		      others->openingNo = (ui) bookPtr->no;
		      return bookPtr->move;
		    }
		}
	    }
	}
    }
  else
    {
      /* counting beans */
      for (bookPtr = &bBookTable[thisPlyData->key & (bookSize)];
	   bookPtr->count;
	   bookPtr++)
	{
	  if (bookPtr->key == (ui) thisPlyData->key)
	    {
	      moveNo++;
	      count += bookPtr->count;
	    }
	}

      /* picking a bean */
      if (moveNo == 0) return 0;
      else if (moveNo == 1)
	{
	  for (bookPtr = &bBookTable[thisPlyData->key & (bookSize)];
	       bookPtr->count;
	       bookPtr++)
	    {
	      if (bookPtr->key == (ui) thisPlyData->key)
		{
		  others->openingNo = (ui) bookPtr->no;
		  return bookPtr->move;
		}
	    }
	}
      else
	{
	  if (others->Flags & OneMask[3]) r = (int) random32() % (count);
	  else r = 0;
	  for (bookPtr = &bBookTable[thisPlyData->key & (bookSize)];
	       bookPtr->count;
	       bookPtr++)
	    {
	      if (bookPtr->key == (ui) thisPlyData->key)
		{
		  r -= bookPtr->count;
		  if (r < 1)
		    {
		      others->openingNo = (ui) bookPtr->no;
		      return bookPtr->move;
		    }
		}
	    }
	}
    }
	  
  return 0;
}
/* command.c */
/* Last modified: June 9, 2006 */


/* Get command, based on xboard's engine-intf.txt. */
void getCommand(sPos *pos, sPlyData *plyData, sHash **hash, 
		sOthers *others, ui acceptDraw, FILE *fd)
{
  int i, commentLine;
  ui j, count, ok, go;
  int wordNo;
  char lastC, c;
  char word[WORDS][80];
  ui wCMoves, nCMoves;
  BB winningCapture[32], nonCapture[128];
  BB theMove;
  char s[8];

  go = false;
  while (!go)
    {
      if (!(others->Flags & OneMask[0])) printf("What now? ");

      wordNo = -1;
      count = 0;
      lastC = ' ';
      commentLine = false;
      /* Better not assume that word[][] is always zero */
      for (i = 0; i < WORDS; i++)
	word[i][0] = '\0';

      /* trap EOF which gives segmentation fault if not trapped */
      c = getc(stdin);
      if (!(others->Flags & OneMask[0])) printf("%c", c);
      if (c == EOF) 
	{
	  printf("\nend of stdin\n");
	  free(hash[white]);
	  free(hash[black]);
	  free(others);
	  exit(EXIT_FAILURE);
	}

      do
	{
	  /* It's a comment, ignore the rest of the line. */
	  if ((wordNo == -1) && (count == 0) && (c == '#'))
	    {
	      commentLine = true;
	    }

	  if (!commentLine)
	    {
	      if (isspace((int) lastC) && isspace((int) c)) continue;
	      else if (isspace((int) lastC) && !(isspace((int) c)))
		{
		  wordNo++;
		  count = 0;
		  word[wordNo][count++] = c;
		  lastC = c;
		  continue;
		}
	      else if (!(isspace((int) lastC)) && (isspace((int) c)))
		{
		  word[wordNo][count] = '\0';
		  /* Limit WORDS tokens per customer */
		  if (wordNo == WORDS) break; 
		  lastC = ' ';
		  continue;
		}
	      else
		{
		  word[wordNo][count++] = c;
		  lastC = c;
		}
	    }
	  if (!(others->Flags & OneMask[0]) && c != '#') printf("%c", c);
	}
      while ((c=getc(stdin)) != newLine);
      if (!(others->Flags & OneMask[0])) printf("\n");

      /* Processing the command now */
      if (wordNo >= 0)
	{
	  word[wordNo][count] = '\0';
#ifdef DEBUG
	  for (i = 0; i <= wordNo; i++) 
	    printf("Word #%d is %s.\n", i, word[i]);
#endif

	  ok = runCommand(word[0], word[1], word[2], word[3], word[4], 
			  word[5], word[6], word[7], word[8], word[9], 
			  word[10], word[11], pos, plyData, hash, 
			  others, acceptDraw, fd);
	  if (ok == 2) go = true;
	  else go = false;
	  if (!ok)
	    {/*
	       MOVE
	       See below for the syntax of moves. If the move is illegal, prui 
	       an error message; see the section "Commands from the engine to 
	       xboard".  If the move is legal and in turn, make it. If not in 
	       force mode, stop the opponent's clock, start the engine's clock,
	       start thinking, and eventually make a move.

	       When xboard sends your engine a move, it normally sends 
	       coordinate algebraic notation. Examples:

	       Normal moves:            e2e4
	       Pawn promotion:          e7e8q
	       Castling:                e1g1, e1c1, e8g8, e8c8
	       Bughouse/crazyhouse drop:P@h3
	       ICS Wild 0/1 castling:   d1f1, d1b1, d8f8, d8b8
	       FischerRandom castling:  O-O, O-O-O (oh, not zero)

	       Beginning in protocol version 2, you can use the feature command
	       to select SAN (standard algebraic notation) instead; for 
	       example, e4, Nf3, exd5, Bxf7+, Qxf7#, e8=Q, O-O, or P@h3. Note 
	       that the last form, P@h3, is a extension to the PGN standard's 
	       definition of SAN, which does not support bughouse or 
	       crazyhouse.

	       xboard doesn't reliably detect illegal moves, because it does 
	       not keep track of castling unavailability due to king or rook 
	       moves, or en passant availability. If xboard sends an illegal 
	       move, send back an error message so that xboard can retract it 
	       and inform the user; see the section "Commands from the engine 
	       to xboard".

	       e2e4, e7e8q, e1g1, e1c1, e8g8, e8c8, e4, exd5, or e8=Q
	     */
	      theMove = 0;
	      /* Try winning captures */
	      wCMoves = nCMoves = 0;
	      wCMoves = captures(0, winningCapture, 0, pos, plyData, 
				 others, 0);
	      if (wCMoves > 0)
		{
		  for (j = 0; j < wCMoves; j++)
		    {
		      algebraic(winningCapture[j], s);
		      if (!strcmp(word[0], s)) 
			theMove = winningCapture[j];
		      else
			{
			  size_t k;
			  
			  san(0, winningCapture[j], s, pos, plyData, others);
			  /* remove '+' or '#' */
			  for (k = strlen(s); k > 1; k--)
			    if (s[k] == '+' || s[k] == '#') s[k] = '\0';
			  for (k = strlen(word[0]); k > 1; k--)
			    if (word[0][k] == '+' || word[0][k] == '#') 
			      word[0][k] = '\0';
			  if (!strcmp(word[0], s)) 
			    theMove = winningCapture[j];
			}
		    }
		}
	      /* Try non captures */
	      if (theMove == 0)
		{
		  nCMoves = nonCaptures(0, nonCapture, pos, plyData, 
					others, 0);
		  if (nCMoves > 0) 
		    {
		      for (j = 0; j < nCMoves; j++)
			{
			  algebraic(nonCapture[j], s);
			  if (!strcmp(word[0], s)) 
			    theMove = nonCapture[j];
			  else
			    {
			      size_t k;
			      
			      san(0, nonCapture[j], s, pos, plyData, others);
			      /* remove '+' or '#' */
			      for (k = strlen(s); k > 1; k--)
				if (s[k] == '+' || s[k] == '#') s[k] = '\0';
			      for (k = strlen(word[0]); k > 1; k--)
				if (word[0][k] == '+' || word[0][k] == '#') 
				  word[0][k] = '\0';
			      if (!strcmp(word[0], s)) 
				theMove = nonCapture[j];
			    }
			}
		    }
		}
	      if (theMove == 0)
		{
#ifdef DEBUG
		  display(*pos);
		  printf("%u captures, %u non captures\n", wCMoves, nCMoves);
		  for (j = 0; j < wCMoves; j++)
		    {
		      san(0, winningCapture[j], s, pos, plyData, others);
		      printf("Winning capture #%u: %s\n", j, s);
		    }
		  if (nCMoves > 0) 
		    {
		      for (j = 0; j < nCMoves; j++)
			{
			  san(0, nonCapture[j], s, pos, plyData, others);
			  printf("Noncapture #%u: %s\n", j, s);
			}
		    }
#endif
		  printf("illegal move: %s\n", word[0]);
		}
	      else
		{
		  go = true;

		  /* write to the game listing */
		  san(0, theMove, s, pos, plyData, others);
		  if (plyData[0].wToMove)
		    {
		      if (others->moveNumber == 0)
			{
			  /* listing headers */
			  fprintf(fd, "[Event \"%s\"]\n", others->fullDate);
			  if (others->iAm == white) 
			    fprintf(fd, "[White \"parrot\"] ");
			  else fprintf(fd, "[Black \"parrot\"] ");
			  fprintf(fd, "        score");
			  fprintf(fd, " opening");
			  fprintf(fd, " depth");
			  fprintf(fd, "\n");
			}
		      fprintf(fd, "white");
		    }
		  else fprintf(fd, "black");
		  fprintf(fd, "%4u.", (others->moveNumber+1));
		  fprintf(fd, " %8s", s);
		  if (strlen(others->solution) != 0) 
		    fprintf(fd, "\t                       %s", 
			    others->solution); 
		  fprintf(fd, "\n");
		  fflush(fd);

		  /* it's time to make the move now */
		  makeMove(1, theMove, pos, plyData, others);

		  /* Reset */
#ifdef UNDO_REMOVE
		  if (plyData[0].wToMove)
		    {
		      gameListing[others->moveNumber].wPlyData = plyData[0];
		      gameListing[others->moveNumber].wFailBook = 
			others->failBook;
		    }
		  else
		    {
		      gameListing[others->moveNumber].bPlyData = plyData[0];
		      gameListing[others->moveNumber].bFailBook = 
			others->failBook;
		    }
#endif
		  plyData[0] = plyData[1];
		  if (plyData[0].wToMove) others->moveNumber++;
		  ASSERT(others->moveNumber < maxMove);
		  if (plyData[0].count50Moves == 0) others->repHead = 0;
		  else others->repHead++;
		  display(*pos);
		}
	    }
	}
    }
}


ui runCommand(char *word0, char *word1, char *word2, char *word3,
	      char *word4, char *word5, char *word6, char *word7,
	      char *word8, char *word9, char *word10, char *word11,
	      sPos *pos, sPlyData *plyData, sHash **hash, 
	      sOthers *others, ui acceptDraw, FILE *fd)
{
  ui i, j;
  ui ok;   /* ok=0: no such command, ok=2: go = true in getCommand() */
  char wordArray[WORDS][80], comment[256];
  double timePerMove;
  int tempSize, reqSize;
  char fen[80];
  int score, wPieceScore, bPieceScore, mScore = 0, doubleBonus = 0;

  ok = 0;
  if (!strcasecmp(word0, "time"))
    {/*
       Set a clock that always belongs to the engine. N is a number in
       centiseconds (units of 1/100 second). Even if the engine changes
       to playing the opposite color, this clock remains with the 
       engine.
     */
      ok = 1;
      others->timeLeft = atof(word1);   /* in centi seconds */ 
    }
  else if (!strcasecmp(word0, "otim"))
    {/*
       Set a clock that always belongs to the opponent. N is a 
       number in centiseconds (units of 1/100 second). Even if the 
       opponent changes to playing the opposite color, this clock 
       remains with the opponent.
       
       If needed for purposes of board display in force mode (where
       the engine is not participating in the game) the time clock 
       should be associated with the last color that the engine was
       set to play, the otim clock with the opposite color.
       
       Beginning in protocol version 2, if you can't handle the 
       time and otim commands, you can use the "feature" command to
       disable them; see below.  The following techniques from 
       older protocol versions also work: You can ignore the time 
       and otim commands (that is, treat them as no-ops), or send 
       back "Error (unknown command): time" the first time you see 
       "time".
     */
      ok = 1;
      others->oTimeLeft = atof(word1);   /* in centi seconds */ 

      /* 
       * Correction for the delay in the way xboard is reporting otim
       * For example, in a 40 moves per 5 minutes game, when it is white's 
       * turn to move on Move 41, xboard reports otim without the additional
       * 5 minutes for the next 40 moves.  Hence, parrot has to add it.
       * WARNING: others->moveNumber starts from 0, not 1 like the real move 
       * number.  It needs to start from 0 because it's used as an index of
       * gameListing[].
       */
      if (others->movesPerTC > 0 &&
	  ((others->moveNumber+1) % others->movesPerTC == 0) &&
          others->iAm == white && others->moveNumber > 0)
        {
          others->oTimeLeft += others->timeLeft0;
        }
     }
  else if (!strcasecmp(word0, "white"))
    {/*
       (This command is obsolete as of protocol version 2, but is still
       sent in some situations to accommodate older engines unless you 
       disable it with the feature command.) Set White on move. Set the
       engine to play Black. Stop clocks.
     */
      ok = 1;
      plyData[0].wToMove = true;
      others->iAm = black;
      /* in case I get "white go" */
      if (!strcasecmp(word1, "go"))
	{
	  others->iAm ^= 1;
	  plyData[0].wToMove = others->iAm^1;
	  others->Flags &= NilMask[1];   /* not forced */
	  ok = 2;
	}
    }
  else if (!strcasecmp(word0, "black"))
    {/*
       (This command is obsolete as of protocol version 2, but is still
       sent in some situations to accommodate older engines unless you 
       disable it with the feature command.) Set Black on move. Set the
       engine to play White. Stop clocks.
     */
      ok = 1;
      plyData[0].wToMove = false;
      others->iAm = white;
      /* in case I get "black go" */
      if (!strcasecmp(word1, "go"))
	{
	  others->iAm ^= 1;
	  plyData[0].wToMove = others->iAm^1;
	  others->Flags &= NilMask[1];   /* not forced */
	  ok = 2  ;
	}
    }
  else if (!strcasecmp(word0, "result"))
    {/*
       After the end of each game, xboard will send you a result 
       command. You can use this command to trigger learning. RESULT is
       either 1-0, 0-1, 1/2-1/2, or *, indicating whether white won, 
       black won, the game was a draw, or the game was unfinished. The 
       COMMENT string is purely a human-readable comment; its content 
       is unspecified and subject to change. In ICS mode, it is passed 
       through from ICS uninterpreted.  Example:
       
       result 1-0 {White mates}
       
       Here are some notes on interpreting the "result" command. Some 
       apply only to playing on ICS ("Zippy" mode).
       
       If you won but did not just play a mate, your opponent must have
       resigned or forfeited. If you lost but were not just mated, you
       probably forfeited on time, or perhaps the operator resigned 
       manually.  If there was a draw for some nonobvious reason, 
       perhaps your opponent called your flag when he had insufficient 
       mating material (or vice versa), or perhaps the operator agreed 
       to a draw manually.
       
       You will get a result command even if you already know the game 
       ended -- for example, after you just checkmated your opponent. 
       In fact, if you send the "RESULT {COMMENT}" command (discussed 
       below), you will simply get the same thing fed back to you with 
       "result" tacked in front. You might not always get a "result *" 
       command, however. In particular, you won't get one in local 
       chess engine mode when the user stops playing by selecting 
       Reset, Edit Game, Exit or the like.
     */
      ok = 1;
      j = 0;
      strcpy(wordArray[2], word2);
      strcpy(wordArray[3], word3);
      strcpy(wordArray[4], word4);
      strcpy(wordArray[5], word5);
      strcpy(wordArray[6], word6);
      strcpy(wordArray[7], word7);
      strcpy(wordArray[8], word8);
      strcpy(wordArray[9], word9);
      strcpy(wordArray[10], word10);
      strcpy(wordArray[11], word11);
      for (i = 2; i < WORDS; i++)
	{
	  if (strlen(wordArray[i]))
	    {
	      size_t k;
	      
	      for (k = 0; k < strlen(wordArray[i]); k++)
		comment[j++] = wordArray[i][k];
	      comment[j++] = ' ';
	    }
	}
      comment[j] = '\0';
      closing(fd, others, word1, comment);
    }
  else if (!strcasecmp(word0, "force"))
    {/*
       Set the engine to play neither color ("force mode").  Stop 
       clocks. The engine should check that moves received in force 
       mode are legal and made in the proper turn, but should not 
       think, ponder, or make move of its own.
     */
      ok = 1;
      others->Flags |= OneMask[1];   /* forced */
    }
  else if (!strcasecmp(word0, "go"))
    {/*
       Leave force mode and set the engine to play the color that is on
       move.  Associate the engine's clock with the color that is on 
       move, the opponent's clock with the color that is not on move. 
       Start the engine's clock. Start thinking and eventually make a 
       move.
     */
      ok = 1;
      others->iAm ^= 1;
      plyData[0].wToMove = others->iAm^1;
      others->Flags &= NilMask[1];   /* not forced */
      ok = 2;
    }
  else if (!strcasecmp(word0, "level"))
    {/*
       level moversPerTC timeLeft0 timeInc
       timeLeft0 is in minutes
     */
      ok = 1;
      others->movesPerTC = (ui) atoi(word1);
      others->timeLeft0 = atof(word2) * 6000; /* in centi seconds */ 
      others->timeLeft = others->timeLeft0;
      if (others->movesPerTC == 0) timePerMove = others->timeLeft/40;
      else timePerMove = others->timeLeft/others->movesPerTC;
      /* others->oTimeLeft = others->timeLeft - timePerMove; 
	 what is the crazy idea? */
      others->timeInc = atof(word3) * 100;    /* in centi seconds */ 
    }
  else if (!strcasecmp(word0, "st"))
    {/*
       Set timeLeft in seconds.
       The commands "level" and "st" are not to be used together.
     */
      ok = 1;
      others->movesPerTC = 1;
      others->timeLeft = atof(word1) * 100;
      timePerMove = others->timeLeft/others->movesPerTC;
      others->oTimeLeft = others->timeLeft - timePerMove;
      others->timeInc = 0;                      /* in centi seconds */ 
    }
  else if (!strcasecmp(word0, "sd"))
    {/*
       The engine should limit its thinking to DEPTH ply.
     */
      ok = 1;
      others->searchDepth = (ui) atoi(word1);
    }
  else if (!strcasecmp(word0, "solution"))
    {/*
       Copy test position information to solution[] string.
     */
      ok = 1;
      sprintf(others->solution, "%s %s %s", word1, word2, word3);
    }
  else if (!strcasecmp(word0, "position"))
    {/*
       Read test position information.  Copy it to solution[] string.
       Do some initializations.
       WARNING: "position" has to be used before "setboard".
     */
      ok = 1;
      init(pos, plyData, hash, others);
      others->Flags &= NilMask[3];    /* don't randomize */
      others->Flags |= OneMask[6];    /* don't use opening book */
      others->rOpening = 0;       /* don't evaluate base on a random opening */
      others->Flags &= NilMask[10];     /* not in timeReserved mode */
      sprintf(others->solution, "%s %s %s", word1, word2, word3);
    }
  else if (!strcasecmp(word0, "printMove"))
    {/*
       The end of a position in testset
     */
      ok = 1;
      printf("%s myMove %s\n", others->solution, others->myMove);
    }
  else if (!strcasecmp(word0, "setboard"))
    {/*
       The setboard command is the new way to set up positions, 
       beginning in protocol version 2. It is not used unless it has 
       been selected with the feature command. Here FEN is a position 
       in Forsythe-Edwards Notation, as defined in the PGN standard.
       
       Illegal positions: Note that either setboard or edit can be used
       to send an illegal position to the engine. The user can create 
       any position with xboard's Edit Position command (even, say, an 
       empty board, or a board with 64 white kings and no black ones). 
       If your engine receives a position that it considers illegal, I 
       suggest that you send the response "tellusererror Illegal 
       position", and then respond to any attempted move with "Illegal 
       move" until the next new, edit, or setboard command
     */
      ok = 1;
      setBoard(word1, word2[0], word3, word4, word5, word6, pos, 
	       &plyData[0], others);
      plyData[1] = plyData[0];
    }
  else if (!strcasecmp(word0, "getfen"))
    {
      ok = 1;
      getFEN(pos, &plyData[0], others, fen);
      printf("%s\n", fen);
    }
  else if (!strcasecmp(word0, "reverse"))
    {
      ok = 1;
      reverse(pos, &plyData[0], others, fen);
      plyData[1] = plyData[0];
      printf("%s\n", fen);
    }
  else if (!strcasecmp(word0, "draw"))
    {/*
       The engine's opponent offers the engine a draw. To accept the 
       draw, send "offer draw". To decline, ignore the offer (that is, 
       send nothing". If you're playing on ICS, it's possible for the 
       draw offer to have been withdrawn by the time you accept it, so 
       don't assume the game is over because you accept a draw offer. 
       Continue playing until xboard tells you the game is over. See 
       also "offer draw" below.
     */

      /* acceptDraw is set in main() */
      if (acceptDraw == true)
	{
	  if (others->Flags & OneMask[0]) printf("offer draw\n");
	}

      /* Is this position a draw by rule? */
      switch(isDrawByRule(0, pos, &plyData[0], others))
	{
	case 0:
	  break;
	case 3:
	  if (others->Flags & OneMask[0]) printf("offer draw\n");
	  break;
	case 13:
	  if (others->Flags & OneMask[0]) printf("offer draw\n");
	  break;
	case 50:
	  if (others->Flags & OneMask[0]) printf("offer draw\n");
	  break;
	default:
	  break;
	}
      ok = 1;
    }
  else if (!strcasecmp(word0, "hard"))
    {/*
       Turn on pondering (thinking on the opponent's time, also known 
       as "permanent brain"). xboard will not make any assumption about
       what your default is for pondering or whether "new" affects this
       setting.
     */
      ok = 1;
    }
  else if (!strcasecmp(word0, "easy"))
    {/*
       Turn off pondering.
     */
      ok = 1;
    }
  else if (!strcasecmp(word0, "new"))
    {/*
       Reset the board to the standard chess starting position. Set 
       White on move. Leave force mode and set the engine to play 
       Black. Associate the engine's clock with Black and the 
       opponent's clock with White. Reset clocks and time controls to 
       the start of a new game. Stop clocks. Do not ponder on this 
       move, even if pondering is on. Remove any search depth limit 
       previously set by the sd command.
     */
      ok = 1;
      init(pos, plyData, hash, others);
      printf("tellics seek\n");  /* put out a challenge on FICS */  
    }
  else if (!strcasecmp(word0, "rating"))
    {/*
       In ICS mode, xboard obtains the ICS opponent's rating from the
       "Creating:" message that appears before each game. (This message
       may not appear on servers using outdated versions of the FICS 
       code.) In Zippy mode, it sends these ratings on to the chess 
       engine using the "rating" command. The chess engine's own rating
       comes first, and if either opponent is not rated, his rating is 
       given as 0. In the future this command may also be used in other
       modes, if ratings are known.  Example:
       
       rating 2600 1500
     */
      ok = 1;
      others->deltaRating = atoi(word2) - atoi(word1);
    }
  else if (!strcasecmp(word0, "name"))
    {/*
       This command informs the engine of its opponent's name. When the
       engine is playing on a chess server, xboard obtains the 
       opponent's name from the server. When the engine is playing 
       locally against a human user, xboard obtains the user's login 
       name from the local operating system.  When the engine is 
       playing locally against another engine, xboard uses either the 
       other engine's filename or the name that the other engine 
       supplied in the myname option to the feature command. By 
       default, xboard uses the name command only when the engine is 
       playing on a chess server. Beginning in protocol version 2, you 
       can change this with the name option to the feature command; see
       below.
     */
      ok = 1;
      strcpy(others->opponentName, word1);
    }
  else if (!strcasecmp(word0, "computer"))
    {/*
       The opponent is also a computer chess engine. Some engines alter
       their playing style when they receive this command.
     */
      ok = 1;
    }
  else if (!strcasecmp(word0, "post"))
    {/*
       Turn on thinking/pondering output. See Thinking Output section.
     */
      ok = 1;
    }
  else if (!strcasecmp(word0, "nopost"))
    {/*
       Turn off thinking/pondering output.
     */
      ok = 1;
    }
  else if (!strcasecmp(word0, "playother"))
    {/*
       (This command is new in protocol version 2. It is not sent 
       unless you enable it with the feature command.) Leave force mode
       and set the engine to play the color that is not on move. 
       Associate the opponent's clock with the color that is on move, 
       the engine's clock with the color that is not on move. Start the
       opponent's clock. If pondering is enabled, the engine should 
       begin pondering. If the engine later receives a move, it should 
       start thinking and eventually reply.
     */
      ok = 1;
    }
  else if (!strcasecmp(word0, "?"))
    {/*
       Move now. If your engine is thinking, it should move 
       immediately; otherwise, the command should be ignored (treated 
       as a no-op"). It is permissible for your engine to always ignore
       the ? command. The only bad consequence is that xboard's Move 
       Now menu command will do nothing.
       
       It is also permissible for your engine to move immediately if it
       gets any command while thinking, as long as it processes the 
       command right after moving, but it's preferable if you don't do 
       this. For example, xboard may send post, nopost, easy, hard, 
       force, quit, or other commands while the engine is on move.
     */
      ok = 1;
    }
  else if (!strcasecmp(word0, "exit"))
    {/*
       The chess engine should immediately exit. This command is used 
       when xboard is itself exiting, and also between games if the 
       -xreuse command line option is given (or -xreuse2 for the second
       engine"). See also Signals above.
     */
      ok = 1;
      exiting(fd, hash, others);
    }
  else if (!strcasecmp(word0, "quit"))
    {/*
       The chess engine should immediately exit. This command is used 
       when xboard is itself exiting, and also between games if the 
       -xreuse command line option is given (or -xreuse2 for the second
       engine"). See also Signals above.
     */
      ok = 1; 
      exiting(fd, hash, others);
    }
  else if (!strcasecmp(word0, "protover"))
    {/*
       protover N
       Beginning in protocol version 2 (in which N=2"), this command 
       will be sent immediately after the "xboard" command. If you 
       receive some other command immediately after "xboard" (such as 
       "new""), you can assume that protocol version 1 is in use. The 
       "protover" command is the only new command that xboard always 
       sends in version 2. All other new commands to the engine are 
       sent only if the engine first enables them with the "feature" 
       command. Protocol versions will always be simple integers so 
       that they can easily be compared.
       
       Your engine should reply to the protover command by sending the
       "feature" command (see below) with the list of non-default 
       feature settings that you require, if any.
       
       Your engine should never refuse to run due to receiving a higher
       protocol version number than it is expecting! New protocol 
       versions will always be compatible with older ones by default; 
       the larger version number is simply a hui that additional 
       "feature" command options added in later protocol versions may 
       be accepted.
     */
      ok = 1;
    }
  else if (!strcasecmp(word0, "accepted rejected"))
    {/*
       These commands may be sent to your engine in reply to the 
       "feature" command; see its documentation below.
     */
      ok = 1;
    }
  else if (!strcasecmp(word0, "xboard"))
    {/*
       This command will be sent once immediately after your engine 
       process is started. You can use it to put your engine into 
       "xboard mode" if that is needed. If your engine prints a prompt 
       to ask for user input, you must turn off the prompt and output a
       newline when the "xboard" command comes in.
     */
      ok = 1;
      setbuf(stdout, NULL);        /* Immediately write to stdout */
      /* Prevent xboard interrupt from killing parrotx */
      signal(SIGINT, SIG_IGN);
      signal (SIGTERM, SIG_IGN);
      others->Flags |= OneMask[0];
    }
  else if (!strcasecmp(word0, "readPGN"))
    {/*
       readPGN PGNFileName playerName numberOfMoves
       Read a PGN file, pick a game where our player wins, and put the first 
       numberOfMoves moves into white or black opening book.  If the player
       is "allPlayers", then it takes all players' games. 
     */
      ok = 1;
      readPGN(word1, word2, atoi(word3));
    }
  else if (!strcasecmp(word0, "whiteFEN"))
    {/* 
       Read white file created by readPGN.  Print out the
       FEN of positions 2 plies before white wins material.
     */
      ok = 1;
      whiteFEN(pos, plyData, hash, others);
    }
  else if (!strcasecmp(word0, "blackFEN"))
    {/* 
       Read black file created by readPGN.  Print out the
       FEN of positions 2 plies before black wins material.
     */
      ok = 1;
      blackFEN(pos, plyData, hash, others);
    }
  else if (!strcasecmp(word0, "readTextBooks"))
    {/* 
       Read white and black files created by readPGN.  Process them 
       and save the openings in binary files white.bin and black.bin.
     */
      ok = 1;
      readTextBooks(pos, plyData, hash, others);
    }
  else if (!strcasecmp(word0, "bk"))
    {/*
       If the user selects "Book" from the xboard menu, xboard will 
       send your engine the command "bk". You can send any text you 
       like as the response, as long as each line begins with a blank 
       space or tab (\t character, and you send an empty line at the 
       end. The text pops up in a modal information dialog.
     */
      ok = 1;
    }
  else if (!strcasecmp(word0, "pause resume"))
    {/*
       (These commands are new in protocol version 2 and will not be 
       sent unless feature pause=1 is set. At this writing, xboard 
       actually does not use the commands at all, but it or other 
       interfaces may use them in the future.) The "pause" command puts
       the engine into a special state where it does not think, ponder,
       or otherwise consume significant CPU time. The current thinking 
       or pondering (if any) is suspended and both player's clocks are 
       stopped. The only command that the interface may send to the 
       engine while it is in the paused state is "resume". The paused 
       thinking or pondering (if any) resumes from exactly where it 
       left off, and the clock of the player on move resumes running 
       from where it stopped.
     */
      ok = 1;
    }
  else if (!strcasecmp(word0, "usermove MOVE"))
    {/*
       By default, moves are sent to the engine without a command name;
       the notation is just sent as a line by itself. Beginning in 
       protocol version 2, you can use the feature command to cause the
       command name "usermove" to be sent before the move. Example: 
       "usermove e2e4".
     */
      ok = 1;
    }
  else if (!strcasecmp(word0, "ping N"))
    {/*
       In this command, N is a decimal number. When you receive the 
       command, reply by sending the string pong N, where N is the same
       number you received. Important: You must not reply to a "ping" 
       command until you have finished executing all commands that you 
       received before it.  Pondering does not count; if you receive a 
       ping while pondering, you should reply immediately and continue 
       pondering. Because of the way xboard uses the ping command, if 
       you implement the other commands in this protocol, you should 
       never see a "ping" command when it is your move; however, if you
       do, you must not send the "pong" reply to xboard until after you
       send your move. For example, xboard may send "?" immediately 
       followed by "ping". If you implement the "?" command, you will 
       have moved by the time you see the subsequent ping command.  
       Similarly, xboard may send a sequence like "force", "new", 
       "ping". You must not send the pong response until after you have
       finished executing the "new" command and are ready for the new 
       game to start.
       
       The ping command is new in protocol version 2 and will not be 
       sent unless you enable it with the "feature" command. Its 
       purpose is to allow several race conditions that could occur in 
       previous versions of the protocol to be fixed, so it is highly 
       recommended that you implement it. It is especially important in
       simple engines that do not ponder and do not poll for input 
       while thinking, but it is needed in all engines.
     */
      ok = 1;
    }
  else if (!strcasecmp(word0, "ics HOSTNAME"))
    {/*
       If HOSTNAME is "-", the engine is playing against a local 
       opponent; otherwise, the engine is playing on an Internet Chess 
       Server (ICS) with the given hostname. This command is new in 
       protocol version 2 and is not sent unless the engine has enabled
       it with the "feature" command.  Example: "ics freechess.org"
     */
      ok = 1;
    }
  else if (!strcasecmp(word0, "display"))
    {
      ok = 1;
      display(*pos);
    }
#ifdef UNDO_REMOVE
  else if (!strcasecmp(word0, "undo"))
    {/*
       If the user asks to back up one move, xboard will send you the 
       "undo" command. xboard will not send this command without 
       putting you in "force" mode first, so you don't have to worry 
       about what should happen if the user asks to undo a move your 
       engine made. (GNU Chess 4 actually switches to playing the 
       opposite color in this case.)
     */
      ok = 1; 
      if (others->repHead > 0) others->repHead--;
      if ((plyData[0].wToMove)) 
	{
	  others->moveNumber--;
	  others->failBook = gameListing[others->moveNumber].wFailBook;
	  plyData[0] = gameListing[others->moveNumber].bPlyData;
	  unmakeMove(1, gameListing[others->moveNumber].bMove, pos, plyData);
	}
      else
	{
	  others->failBook = gameListing[others->moveNumber].bFailBook;
	  plyData[0] = gameListing[others->moveNumber].wPlyData;
	  unmakeMove(1, gameListing[others->moveNumber].wMove, pos, plyData);
	}
      others->iAm ^= 1;
      display(*pos);
    }
  else if (!strcasecmp(word0, "remove"))
    {/*
       If the user asks to retract a move, xboard will send you the 
       "remove" command. It sends this command only when the user is on
       move. Your engine should undo the last two moves (one for each 
       player) and continue playing the same color.
     */
      ok = 1;

      /* undo the last move */
      if (others->repHead > 0) others->repHead--;
      if ((plyData[0].wToMove)) 
	{
	  others->moveNumber--;
	  others->failBook = gameListing[others->moveNumber].wFailBook;
	  plyData[0] = gameListing[others->moveNumber].bPlyData;
	  unmakeMove(1, gameListing[others->moveNumber].bMove, pos, plyData);
	}
      else
	{
	  others->failBook = gameListing[others->moveNumber].bFailBook;
	  plyData[0] = gameListing[others->moveNumber].wPlyData;
	  unmakeMove(1, gameListing[others->moveNumber].wMove, pos, plyData);
	}
      others->iAm ^= 1;

      /* undo the move before the last move */
      if (others->repHead > 0) others->repHead--;
      if ((plyData[0].wToMove)) 
	{
	  others->moveNumber--;
	  plyData[0] = gameListing[others->moveNumber].bPlyData;
	  unmakeMove(1, gameListing[others->moveNumber].bMove, pos, plyData);
	}
      else
	{
	  plyData[0] = gameListing[others->moveNumber].wPlyData;
	  unmakeMove(1, gameListing[others->moveNumber].wMove, pos, plyData);
	}
      others->iAm ^= 1;
      display(*pos);
    }
#endif
  else if (!strcasecmp(word0, "hint"))
    {/*
       If the user asks for a hint, xboard sends your engine the 
       command "hint". Your engine should respond with "Hint: xxx", 
       where xxx is a suggested move. If there is no move to suggest, 
       you can ignore the hui command (that is, treat it as a no-op).
     */
      ok = 1;
    }
  else if (!strcasecmp(word0, "random"))
    {/*
       This command is specific to GNU Chess 4. You can either ignore 
       it completely (that is, treat it as a no-op) or implement it as 
       GNU Chess does. The command toggles "random" mode (that is, it 
       sets random = !random). In random mode, the engine adds a small 
       random value to its evaluation function to vary its play. The 
       "new" command sets random mode off.
     */
      /* "random on" */
      ok = 1;
      if (!strcasecmp(word1, "on")) 
	{
	  others->Flags |= OneMask[3];
	  /* choose an all purpose opening */
	  others->rOpening = 1 + random32() % 3;
	}
      else if (!strcasecmp(word1, "off")) 
	{
	  others->Flags &= NilMask[3];
	  /* don't evaluate base on a random opening */
	  others->rOpening = 0;  
	}
    }
  else if (!strcasecmp(word0, "edit"))
    {/*
       The edit command is the old way to set up positions. For 
       compatibility  with old engines, it is still used by default, 
       but new engines may prefer to use the feature command (see 
       below) to cause xboard to use setboard instead. The edit command
       puts the chess engine into a special mode, where it accepts the 
       following subcommands:
       c                change current piece color, initially white
       Pa4 (for example place pawn of current color on a4
       xa4 (for example empty the square a4 (not used by xboard)
       #                clear board
       .                leave edit mode
       See the Idioms section below for additional subcommands used in
       ChessBase's implementation of the protocol.
       
       The edit command does not change the side to move. To set up a
       black-on-move position, xboard uses the following command 
       sequence:
       
       new
       force
       a2a3
       edit
       <edit commands>
       .
       
       This sequence is used to avoid the "black" command, which is now
       considered obsolete and which many engines never did implement 
       as specified in this document.
       
       After an edit command is complete, if a king and a rook are on 
       their home squares, castling is assumed to be available to them.
       En passant capture is assumed to be illegal on the current move 
       regardless of the positions of the pawns. The clock for the 50 
       move rule starts at zero, and for purposes of the draw by 
       repetition rule, no prior positions are deemed to have occurred.
     */
      ok = 1;
    }
  else if (!strcasecmp(word0, "variant VARNAME"))
    {/*
       If the game is not standard chess, but a variant, this command 
       is sent after "new" and before the first move or "edit" command.
       Currently defined variant names are:
       
       wildcastle  Shuffle chess where king can castle from d file
       nocastle    Shuffle chess with no castling at all
       fischerandomFischer Random (not supported yet
       bughouse    Bughouse, ICC/FICS rules
       crazyhouse  Crazyhouse, ICC/FICS rules
       losers      Win by losing all pieces or getting mated (ICC)    
       suicide     Win by losing all pieces including king, or by 
       having fewer pieces when one player has no legal 
       moves (FICS)
       giveaway    Win by losing all pieces including king, or by 
       having no legal moves (ICC)
       twokings    Weird ICC wild 9
       kriegspiel  Kriegspiel (engines not supported
       atomic      Atomic
       3check      Win by giving check 3 times
       unknown     Unknown variant (not supported)
     */
      ok = 1;
    }
  else if (!strcasecmp(word0, "analyze"))
    {/*
       Enter analyze mode. See Analyze Mode section.
     */
      ok = 1;
    }
  else if (!strcasecmp(word0, "score"))
    {
      /*
	Show the score of the side to move.
      */
      ok = 1;
      score = evalMaterial(pos, others, &wPieceScore, &bPieceScore, &mScore,
			   &doubleBonus);
      ASSERT(score >= -mateScore && score <= mateScore);
      if (score != drawScore)
	{
	  score = eval(1, pos, plyData, others, score,
		       wPieceScore, bPieceScore, -mateScore, doubleBonus);
	  ASSERT(score >= -mateScore && score <= mateScore);
	}
      else score = 0;
      if (plyData[0].wToMove) 
	printf("score = %d (material: %d)\n", score, mScore);
      else printf("score = %d (material: %d)\n", -score, -mScore);
    }
  else if (!strcasecmp(word0, "perft"))
    {
      /*
	Make moves to a certain depth to calculate how many legal positions 
	there are. 
      */
#ifdef WINDOWS
      double clock0, clock1;
#else
      struct timeval tv;
      struct timezone tz;
      BB clock0, clock1;
#endif
      ok = 1;
      others->sNodes = 0;

#ifdef WINDOWS      
      clock0 = 1000000.0/CLOCKS_PER_SEC*clock();
#else
      gettimeofday(&tv, &tz);
      clock0 = (BB) tv.tv_sec*1000000 + (BB) tv.tv_usec;
#endif
      perft(0, (ui) atoi(word1), pos, plyData, others);
#ifdef WINDOWS      
      clock1 = 1000000.0/CLOCKS_PER_SEC*clock();
#else
      gettimeofday(&tv, &tz);
      clock1 = (BB) tv.tv_sec*1000000 + (BB) tv.tv_usec;
#endif
      printf("perft(%d) = %.0f (%0.2f knps)\n", atoi(word1), 
	     (double) others->sNodes, 
	     1000.0*others->sNodes/(clock1-clock0));
    }
  else if (!strcasecmp(word0, "hashsize"))
    {
      /*
	hashsize n where n, a power of 2, is hashsize in MBytes
      */
      ok = 1;
      reqSize = atoi(word1);
      /* hashsize of 2^14-1 corresponds reqSize of 1 MBytes */
      tempSize = 1;
      for (i = 14; i < 32; i++)
	{
	  if (tempSize > reqSize) break;
	  else 
	    {
	      j = i;
	      tempSize *= 2;
	    }
	}
      others->hashSize = 1;
      for (i = 1; i <= j; i++) others->hashSize *= 2;
      others->hashSize--;
    }
  else if (!strcasecmp(word0, "egtbcachesize"))
    {
      /* EGTB cache size, in MBytes */
      ok = 1;
      others->cacheSize = (ui)(atoi(word1)) * 1024 * 1024;
    }
  else if (!strcasecmp(word0, "egtbdir"))
    {/* EGTB directory */
      ok = 1;
      sprintf(others->egtbDir, "%s", word1);
    }
  else if (!strcasecmp(word0, "null"))
    {
      /*
	make a null move
      */
      ok = 1;
      plyData[1] = plyData[0];
      plyData[1].wToMove ^= 1;
      if (plyData[1].EP) 
	plyData[1].key ^= others->ePKey[firstOne(plyData[1].EP)];
      plyData[1].count50Moves = 0;
      plyData[1].EP = 0;

      /* Reset */
      plyData[0] = plyData[1];
      display(*pos);

      ok = 2;
    }

  return ok;
}
/* egtbprobe.c borrowed from crafty */
/* last modified 11/03/98 */


/*
*******************************************************************************
*                                                                             *
*  EGTBProbe() is the interface to the new tablebase code by Eugene Nalimov.  *
*  This is taken from crafty.                                                 *
*  This is called from search() when 5 or fewer pieces are left on the board. *
*                                                                             *
*******************************************************************************
*/

#define  XX  127
#define  C_PIECES  3  /* Maximum # of pieces of one color OTB */

typedef unsigned int INDEX;
typedef unsigned int squaret;

/* Those declarations necessary because parrot is C, not C++ program */

#if defined (_MSC_VER)
#define  TB_FASTCALL  __fastcall
#else
#define  TB_FASTCALL
#endif

  typedef  int  color;
# define  x_colorWhite  0
# define  x_colorBlack  1
# define  x_colorNeutral  2
# define COLOR_DECLARED

# define  x_pieceNone    0
# define  x_piecePawn    1
# define  x_pieceKnight  2
# define  x_pieceBishop  3
# define  x_pieceRook    4
# define  x_pieceQueen   5
# define  x_pieceKing    6
# define PIECES_DECLARED
typedef  signed char tb_t;

#define pageL       65536
#define tbbe_ssL    ((pageL-4)/2)
#define bev_broken  (tbbe_ssL+1)  /* illegal or busted */
#define bev_mi1     tbbe_ssL      /* mate in 1 move */
#define bev_mimin   1               /* mate in max moves */
#define bev_draw    0               /* draw */
#define bev_limax   (-1)            /* mated in max moves */
#define bev_li0     (-tbbe_ssL)   /* mated in 0 moves */

typedef INDEX (TB_FASTCALL * PfnCalcIndex)
    (squaret*, squaret*, squaret, int fInverse);

extern int IDescFindFromCounters (int*);
extern int FRegisteredFun (int, color);
extern PfnCalcIndex PfnIndCalcFun (int, color);
extern int TB_FASTCALL L_TbtProbeTable (int, color, INDEX);

#define PfnIndCalc PfnIndCalcFun
#define FRegistered FRegisteredFun

int EGTBProbe (int *score, sPos *pos, sPlyData *thisPlyData) 
{
  int       rgiCounters[10], iTb, fInvert;
  color     side;
  squaret   rgsqWhite[C_PIECES*5+1], rgsqBlack[C_PIECES*5+1];
  squaret   *psqW, *psqB, sqEnP;
  INDEX     ind;
  int       tbValue;

/*
 ----------------------------------------------------------
|                                                          |
|   initialize counters and piece arrays so the probe code |
|   can compute the modified Godel number.                 |
|                                                          |
 ----------------------------------------------------------
*/
  vInitSqCtr(rgiCounters, rgsqWhite, 0, pos->WPBoard);
  vInitSqCtr(rgiCounters, rgsqWhite, 1, pos->WNBoard);
  vInitSqCtr(rgiCounters, rgsqWhite, 2, pos->WBBoard);
  vInitSqCtr(rgiCounters, rgsqWhite, 3, pos->WRBoard);
  vInitSqCtr(rgiCounters, rgsqWhite, 4, pos->WQBoard);
  vInitSqCtr(rgiCounters+5, rgsqBlack, 0, pos->BPBoard);
  vInitSqCtr(rgiCounters+5, rgsqBlack, 1, pos->BNBoard);
  vInitSqCtr(rgiCounters+5, rgsqBlack, 2, pos->BBBoard);
  vInitSqCtr(rgiCounters+5, rgsqBlack, 3, pos->BRBoard);
  vInitSqCtr(rgiCounters+5, rgsqBlack, 4, pos->BQBoard);
/*
 ----------------------------------------------------------
|                                                          |
|   quick early exit.  is the tablebase for the current    |
|   set of pieces registered?                              |
|                                                          |
 ----------------------------------------------------------
*/
  iTb=IDescFindFromCounters (rgiCounters);
  if (!iTb) return(0);
/*
 ----------------------------------------------------------
|                                                          |
|   yes, finish setting up to probe the tablebase.  if     |
|   black is the "winning" side (more pieces) then we need |
|   to "invert" the pieces in the lists.                   |
|                                                          |
 ----------------------------------------------------------
*/
  rgsqWhite[C_PIECES*5]=pos->wKSq;
  rgsqBlack[C_PIECES*5]=pos->bKSq;
  if (iTb > 0) {
    side=thisPlyData->wToMove ? x_colorWhite : x_colorBlack;
    fInvert=0;
    psqW=rgsqWhite;
    psqB=rgsqBlack;
  }
  else {
    side=thisPlyData->wToMove ? x_colorBlack : x_colorWhite;
    fInvert=1;
    psqW=rgsqBlack;
    psqB=rgsqWhite;
    iTb=-iTb;
  }
/*
 ----------------------------------------------------------
|                                                          |
|   now check to see if this particular tablebase for this |
|   color to move is registered.                           |
|                                                          |
 ----------------------------------------------------------
*/
  if (!FRegistered (iTb, side)) return(0);
  if (thisPlyData->EP) sqEnP = firstOne(thisPlyData->EP);
  else sqEnP = XX;
  ind=PfnIndCalc (iTb, side) (psqW, psqB, sqEnP, fInvert);
  tbValue=L_TbtProbeTable(iTb, side, ind);
  if (bev_broken == tbValue) return(0);
/*
 ---------------------------------------------------------------
|                                                               |
|   now convert to correct mateScore range to match the value   |
|   parrot uses.                                                |
|                                                               |
 ---------------------------------------------------------------
*/
  if (tbValue > 0)
    *score=mateScore+2*(-bev_mi1+tbValue-1);
  else if (tbValue < 0)
    *score=-mateScore+2*(bev_mi1+tbValue);
  else
    *score=0;

  return(1);
}
/* eval.c */
/* Last modified: June 28, 2006 */


/********************** Leaf evaluation ***********************/
int evalMaterial(sPos *pos, sOthers *others, int *wPieceScore, 
		 int *bPieceScore, int *mScore, int *doubleBonus)
{
  int wPawnScore, bPawnScore;
  int pawnDiff, score0 = 0; 
  ui uselessMaterial, dontDouble;

  ASSERT(pos != NULL);
  ASSERT(others != NULL);
  ASSERT(wPieceScore != NULL);
  ASSERT(bPieceScore != NULL);
  ASSERT(mScore != NULL);
  ASSERT(doubleBonus != NULL);

  /********************************************************************/
  /* material score                                                   */
  /********************************************************************/
  
  /* Don't forget to initialize them. */
  *wPieceScore = *bPieceScore = wPawnScore = bPawnScore = 0;

  /* non pawn white material */
  if (pos->WQBoard) *wPieceScore += bitCount(pos->WQBoard) * qValue;
  if (pos->WRBoard) *wPieceScore += bitCount(pos->WRBoard) * rValue;
  if (pos->WBBoard) *wPieceScore += bitCount(pos->WBBoard) * bValue;
  if (pos->WNBoard) *wPieceScore += bitCount(pos->WNBoard) * nValue;
  ASSERT(*wPieceScore <= mateScore);

  /* non pawn black material */
  if (pos->BQBoard) *bPieceScore -= bitCount(pos->BQBoard) * qValue;
  if (pos->BRBoard) *bPieceScore -= bitCount(pos->BRBoard) * rValue;
  if (pos->BBBoard) *bPieceScore -= bitCount(pos->BBBoard) * bValue;
  if (pos->BNBoard) *bPieceScore -= bitCount(pos->BNBoard) * nValue;
  ASSERT(*bPieceScore >= -mateScore);

  /* pawn material */
  if (pos->WPBoard) wPawnScore += bitCount(pos->WPBoard) * pValue;
  if (pos->BPBoard) bPawnScore -= bitCount(pos->BPBoard) * pValue;
  ASSERT(wPawnScore <= mateScore);
  ASSERT(bPawnScore >= -mateScore);

  /* Both sides do not have enough material to win */
  if (wPawnScore == 0 && bPawnScore == 0 && 
      *wPieceScore < rValue && *bPieceScore > -rValue)
    return drawScore;

  *mScore = *wPieceScore + *bPieceScore + wPawnScore + bPawnScore;


  /* white bishop pair */
  if (bitCount(pos->WBBoard) == 2) score0 += 25;
  /* black bishop pair */
  if (bitCount(pos->BBBoard) == 2) score0 -= 25;

  others->Flags &= NilMask[8];        /* clear drawish flag */
  /* rook endings are drawish */
  if (*wPieceScore == rValue && *bPieceScore == -rValue)
    {
      others->Flags |= OneMask[8];
      pawnDiff = wPawnScore + bPawnScore;
      if (pawnDiff > 0) score0 -= 75;
      else if (pawnDiff < 0) score0 += 75;
    }
  /* Only bishops of opposite color left, drawish */
  else if (*wPieceScore == bValue && *bPieceScore == -bValue)
    {
      if ((pos->WBBoard & others->Dark) && 
	  (pos->BBBoard & others->Light))
	{
	  others->Flags |= OneMask[8];
	  score0 -= *mScore/2;
	}
      else if ((pos->WBBoard & others->Light) && 
	       (pos->BBBoard & others->Dark))
	{
	  others->Flags |= OneMask[8];
	  score0 -= *mScore/2;
	}
    }
  else
    {
      /* no pawn?  One minor piece advantage is pretty useless. */
      uselessMaterial = false;
      if (wPawnScore == 0)
	{
	  if (*wPieceScore < rValue && *wPieceScore > 0) 
	    {
	      score0 -= 225;
	      uselessMaterial = true;
	    }
	  else if (*bPieceScore == -rValue && 
		   (*wPieceScore + *bPieceScore) > 50 &&
		   (*wPieceScore + *bPieceScore) < rValue)
	    {
	      score0 -= (*wPieceScore + *bPieceScore) * 3/4;
	      uselessMaterial = true;
	    }
	}
      if (bPawnScore == 0)
	{
	  if (*bPieceScore > -rValue && *bPieceScore < 0) 
	    {
	      score0 += 225;
	      uselessMaterial = true;
	    }
	  else if (*wPieceScore == rValue && 
		   (*wPieceScore + *bPieceScore) < -50 &&
		   (*wPieceScore + *bPieceScore) > -rValue)
	    {
	      score0 -= (*bPieceScore + *wPieceScore) * 3/4;
	      uselessMaterial = true;
	    }
	}
      
      if (!uselessMaterial)
	{
	  dontDouble = false;
	  /* One bishop with one wrong rook pawn is pretty useless */
	  if ((*wPieceScore + wPawnScore == bValue + pValue) && *mScore > 50)
	    {
	      if ((pos->WPBoard & others->Column[0]) && 
		  (pos->WBBoard & others->Dark))
		{
		  score0 = -*wPieceScore - wPawnScore + 50;
		  dontDouble = true;
		  others->Flags |= OneMask[8];
		}
	      else if ((pos->WPBoard & others->Column[7]) && 
		       (pos->WBBoard & others->Light))
		{
		  score0 = -*wPieceScore - wPawnScore + 50;
		  dontDouble = true;
		  others->Flags |= OneMask[8];
		}
	    }
	  if ((*bPieceScore + bPawnScore == -bValue - pValue) && *mScore < -50)
	    {
	      if ((pos->BPBoard & others->Column[0]) && 
		  (pos->BBBoard & others->Light))
		{
		  score0 = -*wPieceScore - wPawnScore - 50;
		  dontDouble = true;
		  others->Flags |= OneMask[8];
		}
	      else if ((pos->BPBoard & others->Column[7]) && 
		       (pos->BBBoard & others->Dark))
		{
		  score0 = -*wPieceScore - wPawnScore - 50;
		  dontDouble = true;
		  others->Flags |= OneMask[8];
		}
	    }
	  
	  if (!dontDouble)
	    {
	      /* if non pawn material is ahead, 
		 trade queen to avoid complications */
	      if ((*wPieceScore + *bPieceScore > 0) && !pos->BQBoard)
		*doubleBonus = (*wPieceScore + *bPieceScore);
	      else if ((*wPieceScore + *bPieceScore < 0) && !pos->WQBoard)
		*doubleBonus = (*wPieceScore + *bPieceScore);
	    }
	}
    }

  return *mScore + score0 + *doubleBonus;
}


#ifdef NOT
/* no potential passed pawn */
ui noPotentialPPawn(sPos *pos)
{
  ui i;

  ASSERT(pos != NULL);

  for (i = 0; i < 8; i++)
    {
      if (pos->wPCount[i] == 0)
	{
	  if (pos->bPCount[i] > 0) return true;
	}
      else if (pos->bPCount[i] == 0) return true;
      ASSERT((pos->wPCount[i] > 0 && pos->bPCount[i] > 0) ||
	     (pos->wPCount[i] == 0 && pos->bPCount[i] == 0));
    }
  return false;
}
#endif


int bonusFor7thRank(ui side, ui rank, ui fyle, sPos *pos)
{
  ASSERT(side == white || side == black);
  ASSERT(rank >= 0 && rank <= 7);
  ASSERT(fyle >= 0 && fyle <= 7);
  ASSERT(pos != NULL);

  if (side == white)
    {
      if ((rank == 6 && (pos->bKSq > H7)) ||
	  (rank == 1 && (pos->bKSq < A2)) ||
	  (fyle == 1 && (column(pos->bKSq) == 0)) ||
	  (fyle == 6 && (column(pos->bKSq) == 7)))
	return true;
    }
  else
    {
      if ((rank == 1 && (pos->wKSq < A2)) ||
	  (rank == 6 && (pos->wKSq > H7)) ||
	  (fyle == 1 && (column(pos->wKSq) == 0)) ||
	  (fyle == 6 && (column(pos->wKSq) == 7)))
	return true;
    }

  return false;
} 


static int wPBonus(ui wSleepy, int minPieceScore, sPos *pos, sOthers *others) 
{
  int score = 0;
  ui sq, fyle, ourPawns, theirPawns;
  BB Temp;
  ui i, n;

  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  /********************************************************************/
  /* White pawn score                                            */
  /********************************************************************/

  for (i = 0; i < 8; i++)
    {
      n = pos->wPCount[i];
      if (n)
	{
	  /* doubled or tripled or .... pawns */
	  if (n > 1) score -= (n-1) * PENALTY00;

	  if (wSleepy == 0 || minPieceScore <= middleGameValue)
	    {
	      sq = lastOne(pos->WPBoard & others->Column[i]);
	      ASSERT(sq <= H8);

	      /* passed pawns */
	      if (!(pos->BPBoard & others->Up8Of[sq]))
		{
		  theirPawns = 
		    bitCount(pos->BPBoard & others->Up8SideOf[sq]);
		  ASSERT(theirPawns <= 8);
		  if (!theirPawns) 
		    score += others->wPassedPSqvalue[sq];
		  else
		    {
		      ourPawns = bitCount(pos->WPBoard & 
					  (others->SideOf[sq] | 
					   others->Down8SideOf[sq]));
		      ASSERT(ourPawns <= 8);
		      if (ourPawns >= theirPawns)
			score += others->wPassedPSqvalue[sq];
		    }
		}
	    }
	}
    }
    
  Temp = pos->WPBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      ASSERT(sq <= H8);
      score += others->wPSqvalue[sq];
      /* isolani */
      fyle = column(sq);
      if ((fyle == 0 || pos->wPCount[fyle-1] == 0) &&
	  (fyle == 7 || pos->wPCount[fyle+1] == 0))
	score -= PENALTY01;

      Temp &= NilMask[sq];
    }

#ifdef BONUSES_PENALTIES
  if (score > PBonus) PBonus = score;
#endif

  ASSERT(score >= -mateScore && score <= mateScore);
  return score;
}


static int bPBonus(ui bSleepy, int minPieceScore, sPos *pos, sOthers *others)
{
  int score = 0;
  ui sq, fyle, ourPawns, theirPawns;
  BB Temp;
  ui i, n;

  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  /********************************************************************/
  /* Black pawn score                                            */
  /********************************************************************/

  for (i = 0; i < 8; i++)
    {
      n = pos->bPCount[i];
      if (n)
	{
	  /* doubled or tripled or .... pawns */
	  if (n > 1) score += (n-1) * PENALTY00;

	  if (bSleepy == 0 || minPieceScore <= middleGameValue)
	    {
	      sq = firstOne(pos->BPBoard & others->Column[i]);
	      ASSERT(sq <= H8);
	      
	      /* passed pawns */
	      if (!(pos->WPBoard & others->Down8Of[sq]))
		{
		  theirPawns = 
		    bitCount(pos->WPBoard & others->Down8SideOf[sq]);
		  ASSERT(theirPawns <= 8);
		  if (!theirPawns) 
		    score -= others->bPassedPSqvalue[sq];
		  else
		    {
		      ourPawns = bitCount(pos->BPBoard & 
					  (others->SideOf[sq] | 
					   others->Up8SideOf[sq]));
		      ASSERT(ourPawns <= 8);
		      if (ourPawns >= theirPawns) 
			score -= others->bPassedPSqvalue[sq];
		    }
		}
	    }
	}
    }
  
  Temp = pos->BPBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      ASSERT(sq <= H8);
      score -= others->bPSqvalue[sq];
      /* isolani */
      fyle = column(sq);
      if ((fyle == 0 || pos->bPCount[fyle-1] == 0) &&
	  (fyle == 7 || pos->bPCount[fyle+1] == 0))
	  score += PENALTY01;

      Temp &= NilMask[sq];
    }

  ASSERT(score >= -mateScore && score <= mateScore);
  return score;
}


static int wNBonus(ui wSleepy, sPos *pos, sOthers *others) 
{
  int score = 0;
  ui sq;
  BB Temp, AttackBoard;

  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  /********************************************************************/
  /* White knight score                                               */
  /********************************************************************/
  Temp = pos->WNBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      ASSERT(sq <= H8);
      score += others->wNSqvalue[sq];

      AttackBoard = others->NMoves[sq] & ~pos->WBoard;
      score += bitCount(AttackBoard) * BONUS00;
      ASSERT(score <= mateScore);

      if (wSleepy == 0)
	{
	  /* Knight outpost */
	  if (others->BPCaptures[sq] & pos->WPBoard)
	    {
	      if (sq == D5)
		{
		  if (bitCount(pos->BNBoard) == 0 && 
		      !(pos->BBBoard & others->Light) &&
		      !(pos->BPBoard & (OneMask[C7] | OneMask[E7]))) 
		    score += BONUS01;
		  else score += BONUS02;
		}
	      else if (sq == E5)
		{
		  if (bitCount(pos->BNBoard) == 0 && 
		      !(pos->BBBoard & others->Dark) &&
		      !(pos->BPBoard & (OneMask[D7] | OneMask[F7]))) 
		    score += BONUS01;
		  else score += BONUS02;
		}
	      else if (sq == D6)
		{
		  if (bitCount(pos->BNBoard) == 0 && 
		      !(pos->BBBoard & others->Dark)) 
		    score += BONUS03;
		  else score += BONUS04;
		}
	      else if (sq == E6)
		{
		  if (bitCount(pos->BNBoard) == 0 && 
		      !(pos->BBBoard & others->Light)) 
		    score += BONUS03;
		  else score += BONUS04;
		}
	    }
	}

      Temp &= NilMask[sq];
    }

#ifdef BONUSES_PENALTIES
  if (score > NBonus) NBonus = score;
#endif

  ASSERT(score >= -mateScore && score <= mateScore);
  return score;
}


static int bNBonus(ui bSleepy, sPos *pos, sOthers *others)
{
  int score = 0;
  ui sq;
  BB Temp, AttackBoard;

  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  /********************************************************************/
  /* Black knight score                                               */
  /********************************************************************/
  Temp = pos->BNBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      ASSERT(sq <= H8);
      score -= others->bNSqvalue[sq];

      AttackBoard = others->NMoves[sq] & ~pos->BBoard;
      score -= bitCount(AttackBoard) * BONUS00;
      ASSERT(score >= -mateScore);

      if (bSleepy == 0)
	{
	  /* Knight outpost */
	  if (others->WPCaptures[sq] & pos->BPBoard)
	    {
	      if (sq == D4)
		{
		  if (bitCount(pos->WNBoard) == 0 && 
		      !(pos->WBBoard & others->Dark) &&
		      !(pos->WPBoard & (OneMask[C2] | OneMask[E2]))) 
		    score -= BONUS01;
		  else score -= BONUS02;
		}
	      else if (sq == E4)
		{
		  if (bitCount(pos->WNBoard) == 0 && 
		      !(pos->WBBoard & others->Light) &&
		      !(pos->WPBoard & (OneMask[D2] | OneMask[F2]))) 
		    score -= BONUS01;
		  else score -= BONUS02;
		}
	      else if (sq == D3)
		{
		  if (bitCount(pos->WNBoard) == 0 && 
		      !(pos->WBBoard & others->Light)) 
		    score -= BONUS03;
		  else score -= BONUS04;
		}
	      else if (sq == E3)
		{
		  if (bitCount(pos->WNBoard) == 0 && 
		      !(pos->WBBoard & others->Dark)) 
		    score -= BONUS03;
		  else score -= BONUS04;
		}
	    }
	}

      Temp &= NilMask[sq];
    }

  ASSERT(score >= -mateScore && score <= mateScore);
  return score;
}


static int wBBonus(sPos *pos, sOthers *others) 
{
  int score = 0;
  ui sq;
  BB Temp, AttackBoard;

  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  /********************************************************************/
  /* White bishop score                                               */
  /********************************************************************/
  Temp = pos->WBBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      ASSERT(sq <= H8);
      score += others->wBSqvalue[sq];

      AttackBoard = bishopAttacks(sq, pos) & ~pos->WBoard;
      score += bitCount(AttackBoard) * BONUS05;
      ASSERT(score <= mateScore);

      Temp &= NilMask[sq];
    }

#ifdef BONUSES_PENALTIES
  if (score > BBonus) BBonus = score;
#endif

  ASSERT(score >= -mateScore && score <= mateScore);
  return score;
}


static int wBPenalty(sPos *pos) 
{
  int score = 0;
  ui sq;
  BB Temp;

  ASSERT(pos != NULL);

  /********************************************************************/
  /* White bishop score                                               */
  /********************************************************************/
  Temp = pos->WBBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      ASSERT(sq <= H8);

      if (sq == D3 && (pos->WPBoard & OneMask[D2])) score -= PENALTY02;
      else if (sq == E3 && (pos->WPBoard & OneMask[E2])) score -= PENALTY02; 

      Temp &= NilMask[sq];
    }

#ifdef BONUSES_PENALTIES
  if (score < BPenalty) BPenalty = score;
#endif

  ASSERT(score >= -mateScore && score <= mateScore);
  return score;
}


static int bBBonus(sPos *pos, sOthers *others)
{
  int score = 0;
  ui sq;
  BB Temp, AttackBoard;

  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  /********************************************************************/
  /* Black bishop score                                               */
  /********************************************************************/
  Temp = pos->BBBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      ASSERT(sq <= H8);
      score -= others->bBSqvalue[sq];

      AttackBoard = bishopAttacks(sq, pos) & ~pos->BBoard;
      score -= bitCount(AttackBoard) * BONUS05;
      ASSERT(score >= -mateScore);

      Temp &= NilMask[sq];
    }

  ASSERT(score >= -mateScore && score <= mateScore);
  return score;
}


static int bBPenalty(sPos *pos)
{
  int score = 0;
  ui sq;
  BB Temp;


  /********************************************************************/
  /* Black bishop score                                               */
  /********************************************************************/
  Temp = pos->BBBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      ASSERT(sq <= H8);

      if (sq == D6 && (pos->BPBoard & OneMask[D7])) score += PENALTY02; 
      else if (sq == E6 && (pos->BPBoard & OneMask[E7])) score += PENALTY02; 

      Temp &= NilMask[sq];
    }

  ASSERT(score >= -mateScore && score <= mateScore);
  return score;
}


static int wRBonus(ui wSleepy, int minPieceScore, sPos *pos, sOthers *others) 
{
  int score = 0;
  ui sq, fyle, rank;
  BB Temp;

  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  /********************************************************************/
  /* White rook score                                                 */
  /********************************************************************/
  Temp = pos->WRBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      ASSERT(sq <= H8);
      fyle = column(sq);
      rank = row(sq);
      ASSERT(fyle < 8 && rank < 8);
      score += others->wRSqvalue[sq];

      /* semi open or open file */
      if (!pos->wPCount[fyle])
	{
	  score += BONUS06;
	  if (!pos->bPCount[fyle]) score += BONUS15;
	}

      if (wSleepy == 0 || minPieceScore <= middleGameValue)
	{
	  /* Rook in 7th rank */
	  if ((bitCount(pos->WRBoard | pos->WQBoard) > 1) &&
	      bonusFor7thRank(white, rank, fyle, pos))
	    score += BONUS07;
	}

      Temp &= NilMask[sq];
    }

#ifdef BONUSES_PENALTIES
  if (score > RBonus) RBonus = score;
#endif

  ASSERT(score >= -mateScore && score <= mateScore);
  return score;
}


static int wRPenalty(sPos *pos)
{
  int score = 0;
  ui sq, fyle, rank;
  BB Temp;

  ASSERT(pos != NULL);

  /********************************************************************/
  /* White rook score                                                 */
  /********************************************************************/
  Temp = pos->WRBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      ASSERT(sq <= H8);
      fyle = column(sq);
      rank = row(sq);
      ASSERT(fyle < 8 && rank < 8);

      /* Rook obstructed by king */
      if (row(pos->wKSq) == rank && rank < 2)
	{
	  if (fyle < 2 && column(pos->wKSq) < 3 &&
	      column(pos->wKSq) > fyle)
	    score -= PENALTY03;
	  else if (fyle > 5 && column(pos->wKSq) < fyle &&
		   column(pos->wKSq) > 4)
	    score -= PENALTY03;
	}

      /* Rook obstructed by pawn */
      if (fyle == 0 && 
	  ((pos->WPBoard & OneMask[sq+1]) ||
	   (pos->WPBoard & OneMask[sq+2])))
	score -= PENALTY04;
      else if (fyle == 7 && 
	       ((pos->WPBoard & OneMask[sq-1]) ||
		(pos->WPBoard & OneMask[sq-2])))
	score -= PENALTY04;

      Temp &= NilMask[sq];
    }

#ifdef BONUSES_PENALTIES
  if (score < RPenalty) RPenalty = score;
#endif

  ASSERT(score >= -mateScore && score <= mateScore);
  return score;
}


static int bRBonus(ui bSleepy, int minPieceScore, sPos *pos, sOthers *others)
{
  int score = 0;
  ui sq, fyle, rank;
  BB Temp;

  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  /********************************************************************/
  /* Black rook score                                                 */
  /********************************************************************/
  Temp = pos->BRBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      ASSERT(sq <= H8);
      fyle = column(sq);
      rank = row(sq);
      ASSERT(fyle < 8 && rank < 8);
      score -= others->bRSqvalue[sq];

      /* semi open or open file */
      if (!pos->bPCount[fyle])
	{
	  score -= BONUS06;
	  if (!pos->wPCount[fyle]) score -= BONUS15;
	}

      if (bSleepy == 0 || minPieceScore <= middleGameValue)
	{
	  /* Rook in 7th rank */
	  if ((bitCount(pos->BRBoard | pos->BQBoard) > 1) &&
	      bonusFor7thRank(black, rank, fyle, pos))
	    score -= BONUS07;
	}

      Temp &= NilMask[sq];
    }

  ASSERT(score >= -mateScore && score <= mateScore);
  return score;
}


static int bRPenalty(sPos *pos)
{
  int score = 0;
  ui sq, fyle, rank;
  BB Temp;

  ASSERT(pos != NULL);

  /********************************************************************/
  /* Black rook score                                                 */
  /********************************************************************/
  Temp = pos->BRBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      ASSERT(sq <= H8);
      fyle = column(sq);
      rank = row(sq);
      ASSERT(fyle < 8 && rank < 8);

      /* Rook obstructed by king */
      if (row(pos->bKSq) == rank && rank > 5)
	{
	  if (fyle < 2 && column(pos->bKSq) < 3 &&
	      column(pos->bKSq) > fyle)
	    score += PENALTY03;
	  else if (fyle > 5 && column(pos->bKSq) < fyle &&
		   column(pos->bKSq) > 4)
	    score += PENALTY03;
	}

      /* Rook obstructed by pawn */
      if (fyle == 0 && 
	  ((pos->BPBoard & OneMask[sq+1]) ||
	   (pos->BPBoard & OneMask[sq+2])))
	score += PENALTY04;
      else if (fyle == 7 && 
	       ((pos->BPBoard & OneMask[sq-1]) ||
		(pos->BPBoard & OneMask[sq-2])))
	score += PENALTY04;

      Temp &= NilMask[sq];
    }

  ASSERT(score >= -mateScore && score <= mateScore);
  return score;
}


static int wQBonus(ui wSleepy, int minPieceScore, sPos *pos, sOthers *others) 
{
  int score = 0;
  ui sq, rank, fyle;
  BB Temp;

  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  /********************************************************************/
  /* White queen score                                                */
  /********************************************************************/
  Temp = pos->WQBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      ASSERT(sq <= H8);
      rank = row(sq);
      fyle = column(sq);
      ASSERT(rank < 8);
      score += others->wQSqvalue[sq];

      if (wSleepy == 0 || minPieceScore <= middleGameValue)
	{
	  /* Queen in 7th rank */
	  if ((bitCount(pos->WRBoard | pos->WQBoard) > 1) &&
	      bonusFor7thRank(white, rank, fyle, pos))
	    score += BONUS08;
	}

      Temp &= NilMask[sq];
    }

#ifdef BONUSES_PENALTIES
  if (score > QBonus) QBonus = score;
#endif

  ASSERT(score >= -mateScore && score <= mateScore); 
 return score;
}


static int bQBonus(ui bSleepy, int minPieceScore, sPos *pos, sOthers *others)
{
  int score = 0;
  ui sq, rank, fyle;
  BB Temp;

  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  /********************************************************************/
  /* Black queen score                                                */
  /********************************************************************/
  Temp = pos->BQBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      ASSERT(sq <= H8);
      rank = row(sq);
      fyle = column(sq);
      ASSERT(rank < 8);
      score -= others->bQSqvalue[sq];

      if (bSleepy == 0 || minPieceScore <= middleGameValue)
	{
	  /* Queen in 7th rank */
	  if ((bitCount(pos->BRBoard | pos->BQBoard) > 1) &&
	      bonusFor7thRank(black, rank, fyle, pos))
	    score -= BONUS08;
	}

      Temp &= NilMask[sq];
    }

  ASSERT(score >= -mateScore && score <= mateScore);
  return score;
}


static int wKBonus(sPos *pos, sOthers *others) 
{
  int score = 0;
  ui sq;

  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  /********************************************************************/
  /* White king safety score                                          */
  /********************************************************************/
  sq = pos->wKSq;
  ASSERT(sq <= H8);
  score += others->wKSqvalue[sq];

  if (pos->BQBoard)         /* opponent has queen */
    {
        /* Good neighborhood */
      if (sq == A1 || sq == B1 || sq == G1 || sq == H1)
	{
	  score += bitCount(others->KHood[sq] & 
			    (pos->WBoard & ~(pos->WPBoard))) * BONUS09;
	  ASSERT(score <= mateScore);
	}
    }

#ifdef BONUSES_PENALTIES
  if (score > KBonus) KBonus = score;
#endif

  ASSERT(score >= -mateScore && score <= mateScore);
  return score;
}


static int wKPenalty(ui ply, sPos *pos, sPlyData *plyData, sOthers *others) 
{
  int score = 0;
  ui sq, sq1, fyle, rank;
  BB Temp, AttackBoard;
  ui i;
  ui ksfCounter, ksfFlag;

  ASSERT(ply < maxPly);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);
  ASSERT(plyData != NULL);

  /********************************************************************/
  /* White king safety score                                          */
  /********************************************************************/
  sq = pos->wKSq;
  ASSERT(sq <= H8);

  AttackBoard = pos->BQBoard & others->KMoves[sq];
  if (AttackBoard &&             /* opponent's queen is at king's face */
      (getBAttacker(firstOne(AttackBoard), pos, &plyData[ply], others) & 2) &&
      getWAttacker(firstOne(AttackBoard), pos, &plyData[ply], others) == 131)
    {
      score -= ksfTable[39];
    }
  else if (pos->BQBoard)         /* opponent has queen */
    {
      /* Too advanced is dangerous */
      rank = row(sq);
      fyle = column(sq);
      ASSERT(fyle < 8 && rank < 8);
      score -= rank * PENALTY05;
      
      /* pawn cover */
      if (rank < 2)
	{
	  /* pawn cover in front of king */
	  if (!(OneMask[sq+8] & pos->WPBoard))
	    {
	      score -= PENALTY06;
	      if (!(OneMask[sq+16] & pos->WPBoard))
		{
		  score -= PENALTY07;
		  if (!(OneMask[sq+24] & pos->WPBoard))
		    {
		      score -= PENALTY08;
		      if (!pos->wPCount[fyle] && !pos->bPCount[fyle])
			score -= PENALTY18;
		    }
		}
	    }

	  if (fyle > 3)
	    {
	      /* pawn cover outside of king */
	      if (column(sq+1) > fyle)
		{
		  sq1 = sq+1;
		  if (!(OneMask[sq1+8] & pos->WPBoard))
		    {
		      score -= PENALTY09;
		      if (!(OneMask[sq1+16] & pos->WPBoard))
			{
			  score -= PENALTY10;
			  if (!(OneMask[sq1+24] & pos->WPBoard)) 
			    {
			      score -= PENALTY11;
			      if (!pos->wPCount[fyle] && !pos->bPCount[fyle])
				score -= PENALTY19;
			    }
			}
		    }
		}
	      if (column(sq+2) > fyle)
		{
		  sq1 = sq+2;
		  if (!(OneMask[sq1+8] & pos->WPBoard))
		    {
		      score -= PENALTY09;
		      if (!(OneMask[sq1+16] & pos->WPBoard))
			{
			  score -= PENALTY10;
			  if (!(OneMask[sq1+24] & pos->WPBoard)) 
			    {
			      score -= PENALTY11;
			      if (!pos->wPCount[fyle] && !pos->bPCount[fyle])
				score -= PENALTY19;
			    }
			}
		    }
		}
	      if (column(sq+3) > fyle)
		{
		  sq1 = sq+3;
		  if (!(OneMask[sq1+8] & pos->WPBoard))
		    {
		      score -= PENALTY09;
		      if (!(OneMask[sq1+16] & pos->WPBoard))
			{
			  score -= PENALTY10;
			  if (!(OneMask[sq1+24] & pos->WPBoard)) 
			    {
			      score -= PENALTY11;
			      if (!pos->wPCount[fyle] && !pos->bPCount[fyle])
				score -= PENALTY19;
			    }
			}
		    }
		}

	      /* pawn cover inside of king */
	      if (column(sq-1) < fyle && column(sq-1) > 3)
		{
		  sq1 = sq-1;
		  if (!(OneMask[sq1+8] & pos->WPBoard))
		    {
		      score -= PENALTY12;
		      if (!(OneMask[sq1+16] & pos->WPBoard))
			{
			  score -= PENALTY13;
			  if (!(OneMask[sq1+24] & pos->WPBoard)) 
			    {
			      score -= PENALTY14;
			      if (!pos->wPCount[fyle] && !pos->bPCount[fyle])
				score -= PENALTY20;
			    }
			}
		    }
		}
	      if (column(sq-2) < fyle && column(sq-2) > 3)
		{
		  sq1 = sq-2;
		  if (!(OneMask[sq1+8] & pos->WPBoard))
		    {
		      score -= PENALTY12;
		      if (!(OneMask[sq1+16] & pos->WPBoard))
			{
			  score -= PENALTY13;
			  if (!(OneMask[sq1+24] & pos->WPBoard)) 
			    {
			      score -= PENALTY14;
			      if (!pos->wPCount[fyle] && !pos->bPCount[fyle])
				score -= PENALTY20;
			    }
			}
		    }
		}
	      if (column(sq-3) < fyle && column(sq-3) > 3)
		{
		  sq1 = sq-3;
		  if (!(OneMask[sq1+8] & pos->WPBoard))
		    {
		      score -= PENALTY12;
		      if (!(OneMask[sq1+16] & pos->WPBoard))
			{
			  score -= PENALTY13;
			  if (!(OneMask[sq1+24] & pos->WPBoard)) 
			    {
			      score -= PENALTY14;
			      if (!pos->wPCount[fyle] && !pos->bPCount[fyle])
				score -= PENALTY20;
			    }
			}
		    }
		}
	      if (column(pos->bKSq) < 4) score += score;
	    }
	  else if (fyle < 4)
	    {
	      /* pawn cover outside of king */
	      if (column(sq-1) < fyle)
		{
		  sq1 = sq-1;
		  if (!(OneMask[sq1+8] & pos->WPBoard))
		    {
		      score -= PENALTY09;
		      if (!(OneMask[sq1+16] & pos->WPBoard))
			{
			  score -= PENALTY10;
			  if (!(OneMask[sq1+24] & pos->WPBoard)) 
			    {
			      score -= PENALTY11;
			      if (!pos->wPCount[fyle] && !pos->bPCount[fyle])
				score -= PENALTY19;
			    }
			}
		    }
		}
	      if (column(sq-2) < fyle)
		{
		  sq1 = sq-2;
		  if (!(OneMask[sq1+8] & pos->WPBoard))
		    {
		      score -= PENALTY09;
		      if (!(OneMask[sq1+16] & pos->WPBoard))
			{
			  score -= PENALTY10;
			  if (!(OneMask[sq1+24] & pos->WPBoard)) 
			    {
			      score -= PENALTY11;
			      if (!pos->wPCount[fyle] && !pos->bPCount[fyle])
				score -= PENALTY19;
			    }
			}
		    }
		}
	      if (column(sq-3) < fyle)
		{
		  sq1 = sq-3;
		  if (!(OneMask[sq1+8] & pos->WPBoard))
		    {
		      score -= PENALTY09;
		      if (!(OneMask[sq1+16] & pos->WPBoard))
			{
			  score -= PENALTY10;
			  if (!(OneMask[sq1+24] & pos->WPBoard)) 
			    {
			      score -= PENALTY11;
			      if (!pos->wPCount[fyle] && !pos->bPCount[fyle])
				score -= PENALTY19;
			    }
			}
		    }
		}

	      /* pawn cover inside of king */
	      if (column(sq+1) > fyle && column(sq+1) < 4)
		{
		  sq1 = sq+1;
		  if (!(OneMask[sq1+8] & pos->WPBoard))
		    {
		      score -= PENALTY12;
		      if (!(OneMask[sq1+16] & pos->WPBoard))
			{
			  score -= PENALTY13;
			  if (!(OneMask[sq1+24] & pos->WPBoard)) 
			    {
			      score -= PENALTY14;
			      if (!pos->wPCount[fyle] && !pos->bPCount[fyle])
				score -= PENALTY20;
			    }
			}
		    }
		}
	      if (column(sq+2) > fyle && column(sq+2) < 4)
		{
		  sq1 = sq+2;
		  if (!(OneMask[sq1+8] & pos->WPBoard))
		    {
		      score -= PENALTY12;
		      if (!(OneMask[sq1+16] & pos->WPBoard))
			{
			  score -= PENALTY13;
			  if (!(OneMask[sq1+24] & pos->WPBoard)) 
			    {
			      score -= PENALTY14;
			      if (!pos->wPCount[fyle] && !pos->bPCount[fyle])
				score -= PENALTY20;
			    }
			}
		    }
		}
	      if (column(sq+3) > fyle && column(sq+3) < 4)
		{
		  sq1 = sq+3;
		  if (!(OneMask[sq1+8] & pos->WPBoard))
		    {
		      score -= PENALTY12;
		      if (!(OneMask[sq1+16] & pos->WPBoard))
			{
			  score -= PENALTY13;
			  if (!(OneMask[sq1+24] & pos->WPBoard)) 
			    {
			      score -= PENALTY14;
			      if (!pos->wPCount[fyle] && !pos->bPCount[fyle])
				score -= PENALTY20;
			    }
			}
		    }
		}
	      if (column(pos->bKSq) > 3) score += score;
	    }
	}

      /* Penalty for B2 or G2 hole */
      if ((others->KMoves[sq] & OneMask[B2]) && 
	  !((pos->WPBoard | pos->WBBoard) & OneMask[B2])) 
	score -= PENALTY15;
      else if ((others->KMoves[sq] & OneMask[G2]) && 
	  !((pos->WPBoard | pos->WBBoard) & OneMask[G2])) 
	score -= PENALTY15;

      /* Losing castling right */
      if (plyData[0].wCanCastle > 0 && plyData[ply].wCanCastle == 0) 
	score -= PENALTY16;
      
      /* Rebel's evaluation.  See eval.h for explanation */
      Temp = others->KMoves[sq];  /* squares around the king */
      ksfFlag = 0;
      ksfCounter = wKsfCounter[sq]; /* initial king safety counter */
      while (Temp)
	{
	  i = firstOne(Temp);	
	  if (getBAttacker(i, pos, &plyData[ply], others) & 2)
	    {
	      ksfFlag |= plyData[ply].bAttacker[i];
	      ksfCounter++;
	      /* no friendly piece on square in front of king */
	      if (row(i) > rank && (!(OneMask[i] & pos->WBoard)))
		ksfCounter++;
	      /* Only king attacks the square */
	      if (getWAttacker(i, pos, &plyData[ply], others) == 131)
		ksfCounter++;
	    }
	  Temp &= NilMask[i];
	}
      ksfCounter += ksfFlagToCounter[ksfFlag >> 3];
      score -= ksfTable[ksfCounter];

      /* prevent frivolous minor piece sacrifice for two pawns */
      score -= bitCount(pos->BNBoard | pos->BBBoard) * PENALTY17;
      ASSERT(score >= -mateScore);
    }

#ifdef BONUSES_PENALTIES
  if (score < KPenalty) KPenalty = score;
#endif

  ASSERT(score >= -mateScore && score <= mateScore);
  return score;
}


static int bKBonus(sPos *pos, sOthers *others)
{
  int score = 0;
  ui sq;

  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  /********************************************************************/
  /* Black king safety score                                          */
  /********************************************************************/
  sq = pos->bKSq;
  ASSERT(sq <= H8);
  score -= others->bKSqvalue[sq];

  if (pos->WQBoard)         /* opponent has queen */
    {
      /* Good neighborhood */
      if (sq == A8 || sq == B8 || sq == G8 || sq == H8)
	{
	  score -= bitCount(others->KHood[sq] & 
			    (pos->BBoard & ~(pos->BPBoard))) * BONUS09;
	  ASSERT(score >= -mateScore);
	}
    }

  ASSERT(score >= -mateScore && score <= mateScore);
  return score;
}


static int bKPenalty(ui ply, sPos *pos, sPlyData *plyData, sOthers *others)
{
  int score = 0;
  ui sq, sq1, fyle, rank;
  BB Temp, AttackBoard;
  ui i;
  ui ksfCounter, ksfFlag;

  ASSERT(ply < maxPly);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);
  ASSERT(plyData != NULL);

  /********************************************************************/
  /* Black king safety score                                          */
  /********************************************************************/
  sq = pos->bKSq;
  ASSERT(sq <= H8);

  AttackBoard = pos->WQBoard & others->KMoves[sq];
  if (AttackBoard &&             /* opponent's queen is at king's face */
      (getWAttacker(firstOne(AttackBoard), pos, &plyData[ply], others) & 2) &&
      getBAttacker(firstOne(AttackBoard), pos, &plyData[ply], others) == 131)
    {
      score += ksfTable[39];
    }
  else if (pos->WQBoard)         /* opponent has queen */
    {
      /* Too advanced is dangerous */
      rank = row(sq);
      fyle = column(sq);
      ASSERT(fyle < 8 && rank < 8);
      score += (7-rank) * PENALTY05;
      
      /* pawn cover */
      if (rank > 5)
	{
	  if (!(OneMask[sq-8] & pos->BPBoard))
	    {
	      score +=  PENALTY06;
	      if (!(OneMask[sq-16] & pos->BPBoard))
		{
		  score +=  PENALTY07;
		  if (!(OneMask[sq-24] & pos->BPBoard))
		    {
		      score += PENALTY08;
		      if (!pos->wPCount[fyle] && !pos->bPCount[fyle])
			score += PENALTY18;
		    }
		}
	    }

	  if (fyle > 3)
	    {
	      /* pawn cover outside of king */
	      if (column(sq+1) > fyle)
		{
		  sq1 = sq+1;
		  if (!(OneMask[sq1-8] & pos->BPBoard))
		    {
		      score += PENALTY09;
		      if (!(OneMask[sq1-16] & pos->BPBoard))
			{
			  score += PENALTY10;
			  if (!(OneMask[sq1-24] & pos->BPBoard)) 
			    {
			      score += PENALTY11;
			      if (!pos->wPCount[fyle] && !pos->bPCount[fyle])
				score += PENALTY19;
			    }
			}
		    }
		}
	      if (column(sq+2) > fyle)
		{
		  sq1 = sq+2;
		  if (!(OneMask[sq1-8] & pos->BPBoard))
		    {
		      score += PENALTY09;
		      if (!(OneMask[sq1-16] & pos->BPBoard))
			{
			  score += PENALTY10;
			  if (!(OneMask[sq1-24] & pos->BPBoard)) 
			    {
			      score += PENALTY11;
			      if (!pos->wPCount[fyle] && !pos->bPCount[fyle])
				score += PENALTY19;
			    }
			}
		    }
		}
	      if (column(sq+3) > fyle)
		{
		  sq1 = sq+3;
		  if (!(OneMask[sq1-8] & pos->BPBoard))
		    {
		      score += PENALTY09;
		      if (!(OneMask[sq1-16] & pos->BPBoard))
			{
			  score += PENALTY10;
			  if (!(OneMask[sq1-24] & pos->BPBoard)) 
			    {
			      score += PENALTY11;
			      if (!pos->wPCount[fyle] && !pos->bPCount[fyle])
				score += PENALTY19;
			    }
			}
		    }
		}

	      /* pawn cover inside of king */
	      if (column(sq-1) < fyle && column(sq-1) > 3)
		{
		  sq1 = sq-1;
		  if (!(OneMask[sq1-8] & pos->BPBoard))
		    {
		      score += PENALTY12;
		      if (!(OneMask[sq1-16] & pos->BPBoard))
			{
			  score += PENALTY13;
			  if (!(OneMask[sq1-24] & pos->BPBoard)) 
			    {
			      score += PENALTY14;
			      if (!pos->wPCount[fyle] && !pos->bPCount[fyle])
				score += PENALTY20;
			    }
			}
		    }
		}
	      if (column(sq-2) < fyle && column(sq-2) > 3)
		{
		  sq1 = sq-2;
		  if (!(OneMask[sq1-8] & pos->BPBoard))
		    {
		      score += PENALTY12;
		      if (!(OneMask[sq1-16] & pos->BPBoard))
			{
			  score += PENALTY13;
			  if (!(OneMask[sq1-24] & pos->BPBoard)) 
			    {
			      score += PENALTY14;
			      if (!pos->wPCount[fyle] && !pos->bPCount[fyle])
				score += PENALTY20;
			    }
			}
		    }
		}
	      if (column(sq-3) < fyle && column(sq-3) > 3)
		{
		  sq1 = sq-3;
		  if (!(OneMask[sq1-8] & pos->BPBoard))
		    {
		      score += PENALTY12;
		      if (!(OneMask[sq1-16] & pos->BPBoard))
			{
			  score += PENALTY13;
			  if (!(OneMask[sq1-24] & pos->BPBoard)) 
			    {
			      score += PENALTY14;
			      if (!pos->wPCount[fyle] && !pos->bPCount[fyle])
				score += PENALTY20;
			    }
			}
		    }
		}
	      if (column(pos->wKSq) < 4) score += score;
	    }
	  else if (fyle < 4)
	    {
	      /* pawn cover outside of king */
	      if (column(sq-1) < fyle)
		{
		  sq1 = sq-1;
		  if (!(OneMask[sq1-8] & pos->BPBoard))
		    {
		      score += PENALTY09;
		      if (!(OneMask[sq1-16] & pos->BPBoard))
			{
			  score += PENALTY10;
			  if (!(OneMask[sq1-24] & pos->BPBoard)) 
			    {
			      score += PENALTY11;
			      if (!pos->wPCount[fyle] && !pos->bPCount[fyle])
				score += PENALTY19;
			    }
			}
		    }
		}
	      if (column(sq-2) < fyle)
		{
		  sq1 = sq-2;
		  if (!(OneMask[sq1-8] & pos->BPBoard))
		    {
		      score += PENALTY09;
		      if (!(OneMask[sq1-16] & pos->BPBoard))
			{
			  score += PENALTY10;
			  if (!(OneMask[sq1-24] & pos->BPBoard)) 
			    {
			      score += PENALTY11;
			      if (!pos->wPCount[fyle] && !pos->bPCount[fyle])
				score += PENALTY19;
			    }
			}
		    }
		}
	      if (column(sq-3) < fyle)
		{
		  sq1 = sq-3;
		  if (!(OneMask[sq1-8] & pos->BPBoard))
		    {
		      score += PENALTY09;
		      if (!(OneMask[sq1-16] & pos->BPBoard))
			{
			  score += PENALTY10;
			  if (!(OneMask[sq1-24] & pos->BPBoard)) 
			    {
			      score += PENALTY11;
			      if (!pos->wPCount[fyle] && !pos->bPCount[fyle])
				score += PENALTY19;
			    }
			}
		    }
		}

	      /* pawn cover inside of king */
	      if (column(sq+1) > fyle && column(sq+1) < 4)
		{
		  sq1 = sq+1;
		  if (!(OneMask[sq1-8] & pos->BPBoard))
		    {
		      score += PENALTY12;
		      if (!(OneMask[sq1-16] & pos->BPBoard))
			{
			  score += PENALTY13;
			  if (!(OneMask[sq1-24] & pos->BPBoard)) 
			    {
			      score += PENALTY14;
			      if (!pos->wPCount[fyle] && !pos->bPCount[fyle])
				score += PENALTY20;
			    }
			}
		    }
		}
	      if (column(sq+2) > fyle && column(sq+2) < 4)
		{
		  sq1 = sq+2;
		  if (!(OneMask[sq1-8] & pos->BPBoard))
		    {
		      score += PENALTY12;
		      if (!(OneMask[sq1-16] & pos->BPBoard))
			{
			  score += PENALTY13;
			  if (!(OneMask[sq1-24] & pos->BPBoard)) 
			    {
			      score += PENALTY14;
			      if (!pos->wPCount[fyle] && !pos->bPCount[fyle])
				score += PENALTY20;
			    }
			}
		    }
		}
	      if (column(sq+3) > fyle && column(sq+3) < 4)
		{
		  sq1 = sq+3;
		  if (!(OneMask[sq1-8] & pos->BPBoard))
		    {
		      score += PENALTY12;
		      if (!(OneMask[sq1-16] & pos->BPBoard))
			{
			  score += PENALTY13;
			  if (!(OneMask[sq1-24] & pos->BPBoard)) 
			    {
			      score += PENALTY14;
			      if (!pos->wPCount[fyle] && !pos->bPCount[fyle])
				score += PENALTY20;
			    }
			}
		    }
		}
	      if (column(pos->wKSq) > 3) score += score;
	    }
	}

      /* Penalty for B7 or G7 hole */
      if ((others->KMoves[sq] & OneMask[B7]) && 
	  !((pos->BPBoard | pos->BBBoard) & OneMask[B7])) 
	score += PENALTY15;
      else if ((others->KMoves[sq] & OneMask[G7]) && 
	  !((pos->BPBoard | pos->BBBoard) & OneMask[G7])) 
	score += PENALTY15;

      /* Losing castling right */
      if (plyData[0].bCanCastle > 0 && plyData[ply].bCanCastle == 0) 
	score += PENALTY16;
      
      /* Rebel's evaluation.  See eval.h for explanation */
      Temp = others->KMoves[sq];  /* squares around the king */
      ksfFlag = 0;
      ksfCounter = bKsfCounter[sq]; /* initial king safety counter */
      while (Temp)
	{
	  i = firstOne(Temp);
	  if (getWAttacker(i, pos, &plyData[ply], others) & 2)
	    {
	      ksfFlag |= plyData[ply].wAttacker[i];
	      ksfCounter++;
	      /* no friendly piece on square in front of king */
	      if (row(i) < rank && (!(OneMask[i] & pos->BBoard)))
		ksfCounter++;
	      /* Only king attacks the square */
	      if (getBAttacker(i, pos, &plyData[ply], others) == 131) 
		ksfCounter++;
	    }
	  Temp &= NilMask[i];
	}
      ksfCounter += ksfFlagToCounter[ksfFlag >> 3];
      score += ksfTable[ksfCounter];

      /* prevent frivolous minor piece sacrifice for two pawns */
      score += bitCount(pos->WNBoard | pos->WBBoard) * PENALTY17;
      ASSERT(score <= mateScore);
    }

  ASSERT(score >= -mateScore && score <= mateScore);
  return score;
}


static int wPEndBonus(int bPieceScore, sPos *pos, sPlyData *thisPlyData, 
		      sOthers *others, ui bothSidePawns, ui *ppCount)
{
  ui wPassedPSq[8];
  int score = 0;
  ui i, n, sq, ourPawns, theirPawns, pawnDistance, kingDistance;
  ui wDistance, bDistance, minDistance;
  ui rank;

  ASSERT(pos != NULL);
  ASSERT(others != NULL);
  ASSERT(thisPlyData != NULL);
  ASSERT(ppCount != NULL);

  /********************************************************************/
  /* White pawn score                                                 */
  /********************************************************************/

  /* This variable has to be initialized before the real loop. */
  for (i = 0; i < 8; i++) wPassedPSq[i] = 64;
  for (i = 0; i < 8; i++)
    {
      n = pos->wPCount[i];
      if (n)
	{
	  /* doubled or tripled or .... pawns */
	  if (n > 1) score -= (n-1) * PENALTY00;

	  sq = lastOne(pos->WPBoard & others->Column[i]);
	  ASSERT(sq <= H8);
	  rank = row(sq);
	  ASSERT(rank < 8);
	  wDistance = (ui) distance(sq, pos->wKSq);
	  bDistance = (ui) distance(sq, pos->bKSq);
	  ASSERT(wDistance < 8 && bDistance < 8);
	  if (wDistance < bDistance) minDistance = wDistance;
	  else minDistance = bDistance;

	  /* Any pawn likes black king, hates white king. */
	  score += rank * (bDistance - wDistance);
	  if (bPieceScore == 0)
	    {
	      score += KING_RACE_FACTOR * (bDistance - wDistance) *
		(7-minDistance);
	    }
	  /* passed pawns */
	  if (!(pos->BPBoard & others->Up8Of[sq]))
	    {
	      /* Passed pawn likes white king more, hates black king more. */
	      score += rank * (bDistance - wDistance);
	      if (bPieceScore == 0)
		{
		  score += KING_RACE_FACTOR * (bDistance - wDistance) *
		    (7-minDistance);
		}

	      theirPawns = 
		bitCount(pos->BPBoard & others->Up8SideOf[sq]);
	      ASSERT(theirPawns <= 8);
	      if (!theirPawns)
		{
		  *ppCount = *ppCount + 1;
		  wPassedPSq[i] = sq;
		  score += others->wPassedPSqvalue[sq] + rank * 
		    (bDistance - wDistance);
		  /* protected */
		  if (others->BPCaptures[sq] & pos->WPBoard) 
		    score += others->wPassedPSqvalue[sq] >> 1;
	      
		  /* pawn king square rule */
		  if (bPieceScore == 0)
		    {
		      pawnDistance = (ui) distance(sq, wPGoal[i]);
		      kingDistance = (ui) distance(pos->bKSq, wPGoal[i]);
		      ASSERT(pawnDistance < 8 && kingDistance < 8);
		      if (pawnDistance+1 < kingDistance) 
			{
			  others->Flags |= OneMask[9];
			  score += rank * pValue;
			}
		      else if (thisPlyData->wToMove && 
			       pawnDistance < kingDistance)
			{
			  others->Flags |= OneMask[9];
			  score += rank * pValue;
			}
		    }

		  /* pair of passed pawns */
		  ASSERT(i >= 0 && i <= 7);
		  ASSERT(sq >= A1 && sq <= H8);
		  if (i > 0 && (sq-1 == wPassedPSq[i-1])) 
		    {
		      /* *ppCount = *ppCount + 1; */
		      score += pppBonus[rank];
		    }

		  /* rook behind passed pawn */
		  if (pos->wPCount[i] == 1)
		    {
		      if (pos->WRBoard & others->Down8Of[sq]) 
			score += BONUS10;
		      if (pos->BRBoard & others->Down8Of[sq]) 
			score -= BONUS10;
		    }
		}
	      else
		{
		  ourPawns = bitCount(pos->WPBoard & 
			       (others->SideOf[sq] | others->Down8SideOf[sq]));
		  ASSERT(ourPawns <= 8);
		  if (ourPawns >= theirPawns) 
		    {
		      score += others->wPassedPSqvalue[sq] + rank * 
			(bDistance - wDistance);

		      /* protected */
		      if (others->BPCaptures[sq] & pos->WPBoard) 
			score += others->wPassedPSqvalue[sq] >> 1;
	      
		      /* rook behind passed pawn */
		      if (pos->wPCount[i] == 1)
			{
			  if (pos->WRBoard & others->Down8Of[sq]) 
			    score += BONUS10;
			  if (pos->BRBoard & others->Down8Of[sq]) 
			    score -= BONUS10;
			}
		    }
		}
	    }

	  /* strength in company */
	  if (others->SideOf[sq] & pos->WPBoard) score += rank*BONUS11;
	}
    }

  /* With pawns at both sides of the board, bishop is better than knight */
  if (bothSidePawns && pos->WBBoard && 
      bPieceScore > -rValue && !pos->BBBoard) 
    score += BONUS12;

#ifdef BONUSES_PENALTIES
  if (score > PEndBonus) PEndBonus = score;
#endif

  ASSERT(score >= -mateScore && score <= mateScore);
  return score;
}


static int bPEndBonus(int wPieceScore, sPos *pos, sPlyData *thisPlyData, 
		      sOthers *others, ui bothSidePawns, ui *ppCount)
{
  ui bPassedPSq[8];
  int score = 0;
  ui i, n, sq, ourPawns, theirPawns, pawnDistance, kingDistance;
  ui wDistance, bDistance, minDistance;
  ui rank;

  ASSERT(pos != NULL);
  ASSERT(others != NULL);
  ASSERT(thisPlyData != NULL);
  ASSERT(ppCount != NULL);

  /********************************************************************/
  /* Black pawn score                                                 */
  /********************************************************************/

  /* This variable has to be initialized before the real loop. */
  for (i = 0; i < 8; i++) bPassedPSq[i] = 64;
  for (i = 0; i < 8; i++)
    {
      n = pos->bPCount[i];
      if (n)
	{
	  /* doubled or tripled or .... pawns */
	  if (n > 1) score += (n-1) * PENALTY00;

	  sq = firstOne(pos->BPBoard & others->Column[i]);
	  ASSERT(sq <= H8);
	  rank = 7-row(sq);
	  ASSERT(rank < 8);
	  wDistance = (ui) distance(sq, pos->wKSq);
	  bDistance = (ui) distance(sq, pos->bKSq);
	  ASSERT(wDistance < 8 && bDistance < 8);
	  if (wDistance < bDistance) minDistance = wDistance;
	  else minDistance = bDistance;

	  /* Any pawn likes black king, hates white king. */
	  score -= rank * (wDistance - bDistance);
	  if (wPieceScore == 0)
	    {
	      score -= KING_RACE_FACTOR * (wDistance - bDistance) *
		(7-minDistance);
	    }
	  /* passed pawns */
	  if (!(pos->WPBoard & others->Down8Of[sq]))
	    {
	      /* Passed pawn likes black king more, hates white king more. */
	      score -= rank * (wDistance - bDistance);
	      if (wPieceScore == 0)
		{
		  score -= KING_RACE_FACTOR * (wDistance - bDistance) *
		    (7-minDistance);
		}

	      theirPawns = 
		bitCount(pos->WPBoard & others->Down8SideOf[sq]);
	      ASSERT(theirPawns <= 8);
	      if (!theirPawns)
		{
		  *ppCount = *ppCount + 1;
		  bPassedPSq[i] = sq;
		  score -= (others->bPassedPSqvalue[sq] + rank * 
			    (wDistance - bDistance));

		  /* protected */
		  if (others->WPCaptures[sq] & pos->BPBoard) 
		    score -= others->bPassedPSqvalue[sq] >> 1;
		  
		  /* pawn king square rule */
		  if (wPieceScore == 0)
		    {
		      pawnDistance = (ui) distance(sq, bPGoal[i]);
		      kingDistance = (ui) distance(pos->wKSq, bPGoal[i]);
		      ASSERT(pawnDistance < 8 && kingDistance < 8);
		      if (pawnDistance+1 < kingDistance) 
			{
			  others->Flags |= OneMask[9];
			  score -= rank * pValue;
			}
		      else if (!thisPlyData->wToMove && 
			       pawnDistance < kingDistance)
			{
			  others->Flags |= OneMask[9];
			  score -= rank * pValue;
			}
		    }

		  /* pair of passed pawns */
		  ASSERT(i >= 0 && i <= 7);
		  ASSERT(sq >= A1 && sq <= H8);
		  if (i > 0 && (sq-1 == bPassedPSq[i-1])) 
		    {
		      /* *ppCount = *ppCount + 1; */
		      score -= pppBonus[rank];
		    }

		  /* rook behind passed pawn */
		  if (pos->bPCount[i] == 1)
		    {
		      if (pos->WRBoard & others->Up8Of[sq]) 
			score += BONUS10;
		      if (pos->BRBoard & others->Up8Of[sq]) 
			score -= BONUS10;
		    }
		}
	      else
		{
		  ourPawns = bitCount(pos->BPBoard & 
			       (others->SideOf[sq] | others->Up8SideOf[sq]));
		  ASSERT(ourPawns <= 8);
		  if (ourPawns >= theirPawns) 
		    {
		      score -= (others->bPassedPSqvalue[sq] + rank * 
				(wDistance - bDistance));

		      /* protected */
		      if (others->WPCaptures[sq] & pos->BPBoard) 
			score -= others->bPassedPSqvalue[sq] >> 1;
	      
		      /* rook behind passed pawn */
		      if (pos->bPCount[i] == 1)
			{
			  if (pos->WRBoard & others->Up8Of[sq]) 
			    score += BONUS10;
			  if (pos->BRBoard & others->Up8Of[sq]) 
			    score -= BONUS10;
			}
		    }
		}
	    }

	  /* strength in company */
	  if (others->SideOf[sq] & pos->BPBoard) score -= rank*BONUS11;
	}
    }

  /* With pawns at both sides of the board, bishop is better than knight */
  if (bothSidePawns && pos->BBBoard &&
      wPieceScore < rValue && !pos->WBBoard) 
    score -= BONUS12;

  ASSERT(score >= -mateScore && score <= mateScore);
  return score;
}


static int wNEndBonus(sPos *pos, sOthers *others)
{
  int score = 0;
  ui sq;
  BB Temp, AttackBoard;

  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  /********************************************************************/
  /* White knight score                                                */
  /********************************************************************/

  if (pos->WPBoard)
    {
      Temp = pos->WNBoard;
      while (Temp)
	{
	  sq = firstOne(Temp);
	  ASSERT(sq <= H8);
	  score += others->wNSqvalue[sq];
	  AttackBoard = others->NMoves[sq] & ~pos->WBoard;
	  score += bitCount(AttackBoard) * BONUS00;
	  ASSERT(score <= mateScore);
	  Temp &= NilMask[sq];
	}
    }

#ifdef BONUSES_PENALTIES
  if (score > NEndBonus) NEndBonus = score;
#endif

  ASSERT(score >= -mateScore && score <= mateScore);
  return score;
}


static int bNEndBonus(sPos *pos, sOthers *others)
{
  int score = 0;
  ui sq;
  BB Temp, AttackBoard;

  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  /********************************************************************/
  /* Black knight score                                               */
  /********************************************************************/

  if (pos->BPBoard)
    {
      Temp = pos->BNBoard;
      while (Temp)
	{
	  sq = firstOne(Temp);
	  ASSERT(sq <= H8);
	  score -= others->bNSqvalue[sq];
	  AttackBoard = others->NMoves[sq] & ~pos->BBoard;
	  score -= bitCount(AttackBoard) * BONUS00;
	  ASSERT(score >= -mateScore);
	  Temp &= NilMask[sq];
	}
    }

  ASSERT(score >= -mateScore && score <= mateScore);
  return score;
}


static int wBEndBonus(sPos *pos, sOthers *others)
{
  int score = 0;
  ui sq;
  BB Temp, AttackBoard;

  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  /********************************************************************/
  /* White bishop score                                               */
  /********************************************************************/

  if (pos->WPBoard)
    {
      Temp = pos->WBBoard;
      while (Temp)
	{
	  sq = firstOne(Temp);
	  ASSERT(sq <= H8);
	  AttackBoard = bishopAttacks(sq, pos) & ~pos->WBoard;
	  score += bitCount(AttackBoard) * BONUS05;
	  ASSERT(score <= mateScore);
	  Temp &= NilMask[sq];
	}
    }

#ifdef BONUSES_PENALTIES
  if (score > BEndBonus) BEndBonus = score;
#endif

  ASSERT(score >= -mateScore && score <= mateScore);
  return score;
}


static int bBEndBonus(sPos *pos, sOthers *others)
{
  int score = 0;
  ui sq;
  BB Temp, AttackBoard;

  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  /********************************************************************/
  /* Black bishop score                                                */
  /********************************************************************/
  if (pos->BPBoard)
    {
      Temp = pos->BBBoard;
      while (Temp)
	{
	  sq = firstOne(Temp);
	  ASSERT(sq <= H8);
	  AttackBoard = bishopAttacks(sq, pos) & ~pos->BBoard;
	  score -= bitCount(AttackBoard) * BONUS05;
	  ASSERT(score >= -mateScore);
	  Temp &= NilMask[sq];
	}
    }

  ASSERT(score >= -mateScore && score <= mateScore);
  return score;
}


static int wREndBonus(sPos *pos)
{
  int score = 0;
  ui sq, rank, fyle;
  BB Temp;

  ASSERT(pos != NULL);

  /********************************************************************/
  /* White rook score                                                 */
  /********************************************************************/
  Temp = pos->WRBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      ASSERT(sq <= H8);
      rank = row(sq);
      fyle = column(sq);
      ASSERT(fyle < 8);

      /* semi open or open file */
      if (!pos->wPCount[fyle])
	{
	  if (!pos->bPCount[fyle]) score += BONUS15;
	}

      /* Rook in 7th rank */
      if (bonusFor7thRank(white, rank, fyle, pos)) score += BONUS07;

      Temp &= NilMask[sq];
    }

#ifdef BONUSES_PENALTIES
  if (score > REndBonus) REndBonus = score;
#endif

  ASSERT(score >= -mateScore && score <= mateScore);
  return score;
}


static int wREndPenalty(sPos *pos)
{
  int score = 0;
  ui sq, fyle;
  BB Temp;

  ASSERT(pos != NULL);

  /********************************************************************/
  /* White rook score                                                 */
  /********************************************************************/
  Temp = pos->WRBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      ASSERT(sq <= H8);
      fyle = column(sq);
      ASSERT(fyle < 8);

      /* Rook obstructed by pawn */
      if (fyle == 0 && 
	  ((pos->WPBoard & OneMask[sq+1]) ||
	   (pos->WPBoard & OneMask[sq+2])))
	score -= PENALTY04;
      else if (fyle == 7 && 
	       ((pos->WPBoard & OneMask[sq-1]) ||
		(pos->WPBoard & OneMask[sq-2])))
	score -= PENALTY04;

      Temp &= NilMask[sq];
    }

#ifdef BONUSES_PENALTIES
  if (score < REndPenalty) REndPenalty = score;
#endif

  ASSERT(score >= -mateScore && score <= mateScore);
  return score;
}


static int bREndBonus(sPos *pos)
{
  int score = 0;
  ui sq, rank, fyle;
  BB Temp;

  ASSERT(pos != NULL);

  /********************************************************************/
  /* Black rook score                                                 */
  /********************************************************************/
  Temp = pos->BRBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      ASSERT(sq <= H8);
      rank = row(sq);
      fyle = column(sq);
      ASSERT(fyle < 8);

      /* semi open or open file */
      if (!pos->bPCount[fyle])
	{
	  if (!pos->wPCount[fyle]) score -= BONUS15;
	}

      /* Rook in 7th rank */
      if (bonusFor7thRank(black, rank, fyle, pos)) score -= BONUS07;

      Temp &= NilMask[sq];
    }

  ASSERT(score >= -mateScore && score <= mateScore);
  return score;
}


static int bREndPenalty(sPos *pos)
{
  int score = 0;
  ui sq, fyle;
  BB Temp;

  ASSERT(pos != NULL);

  /********************************************************************/
  /* Black rook score                                                 */
  /********************************************************************/
  Temp = pos->BRBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      ASSERT(sq <= H8);
      fyle = column(sq);
      ASSERT(fyle < 8);

      /* Rook obstructed by pawn */
      if (fyle == 0 && 
	  ((pos->BPBoard & OneMask[sq+1]) ||
	   (pos->BPBoard & OneMask[sq+2])))
	score += PENALTY04;
      else if (fyle == 7 && 
	       ((pos->BPBoard & OneMask[sq-1]) ||
		(pos->BPBoard & OneMask[sq-2])))
	score += PENALTY04;

      Temp &= NilMask[sq];
    }

  ASSERT(score >= -mateScore && score <= mateScore);
  return score;
}


static int wKEndBonus(int wPieceScore, int bPieceScore, sPos *pos,
		      sOthers *others)
{
  int score = 0;
  ui fyle, rFile;

  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  /********************************************************************/
  /* White king score */
  /********************************************************************/

  /* if we can mate, then push opponent king */
  if (wPieceScore+bPieceScore >= rValue)
    score += (7-distance(pos->wKSq, pos->bKSq)) * BONUS13;

  /* Good neighborhood */
  if (pos->BQBoard  || bitCount(pos->BRBoard) == 2)
    {
      score += bitCount(others->KHood[pos->wKSq] & pos->WBoard) * BONUS09;
      ASSERT(score <= mateScore);
    }

  /* Shelter the king from that pesky rook */
  if (bitCount(pos->BRBoard) == 1)
    {
      rFile = column(firstOne(pos->BRBoard));
      fyle = column(pos->wKSq);
      if ((fyle+1 < rFile) && (pos->WPBoard & OneMask[pos->wKSq+1]))
	score += BONUS14;
      else if ((fyle > rFile+1) && (pos->WPBoard & OneMask[pos->wKSq-1]))
	score += BONUS14;
    }

#ifdef BONUSES_PENALTIES
  if (score > KEndBonus) KEndBonus = score;
#endif

  ASSERT(score >= -mateScore && score <= mateScore);
  return score;
}


static int bKEndBonus(int wPieceScore, int bPieceScore, sPos *pos,
		      sOthers *others)
{
  int score = 0;
  ui fyle, rFile;

  ASSERT(pos != NULL);
  ASSERT(others != NULL);

  /********************************************************************/
  /* Black king score */
  /********************************************************************/

  /* if we can mate, then push opponent king */
  if (wPieceScore+bPieceScore <= -rValue)
    score = (distance(pos->wKSq, pos->bKSq)-7) * BONUS13;

  /* Good neighborhood */
  if (pos->WQBoard  || bitCount(pos->WRBoard) == 2)
    {
      score -= bitCount(others->KHood[pos->bKSq] & pos->BBoard) * BONUS09;
      ASSERT(score >= -mateScore);
    }

  /* Shelter the king from that pesky rook */
  if (bitCount(pos->WRBoard) == 1)
    {
      rFile = column(firstOne(pos->WRBoard));
      fyle = column(pos->bKSq);
      if ((fyle+1 < rFile) && (pos->BPBoard & OneMask[pos->bKSq+1]))
	score -= BONUS14;
      else if ((fyle > rFile+1) && (pos->BPBoard & OneMask[pos->bKSq-1]))
	score -= BONUS14;
    }

  ASSERT(score >= -mateScore && score <= mateScore);
  return score;
}


/* Is this position a draw by rule? */
ui isDrawByRule(ui ply, sPos *pos, sPlyData *thisPlyData, sOthers *others) 
{
  ui wCantWin = false, bCantWin = false;
  int i, stillGoing;
  ui happenings = 1;

  ASSERT(ply < maxPly);
  ASSERT(pos != NULL);
  ASSERT(thisPlyData != NULL);
  ASSERT(others != NULL);

  /* Not enough material to win */
  if (bitCount(pos->WPBoard) == 0 && bitCount(pos->BPBoard) == 0)
    {
      if (bitCount(pos->WBoard) <= 3 && 
	  !bitCount(pos->WQBoard) && 
	  !bitCount(pos->WRBoard))
	{
	  switch(bitCount(pos->WBBoard))
	    {
	    case 0:
	      wCantWin = true;
	      break;
	    case 1:
	      if (!bitCount(pos->WNBoard)) wCantWin = true;
	      break;
	    default:
	      break;
	    }
	}

      if (bitCount(pos->BBoard) <= 3 && 
	  !bitCount(pos->BQBoard) && 
	  !bitCount(pos->BRBoard))
	{
	  switch(bitCount(pos->BBBoard))
	    {
	    case 0:
	      bCantWin = true;
	      break;
	    case 1:
	      if (!bitCount(pos->BNBoard)) bCantWin = true;
	      break;
	    default:
	      break;
	    }
	}

      if (wCantWin && bCantWin) return 13;
    }

  if (thisPlyData->count50Moves > 99) return 50;
  else
    {
      /* According to the rule, a game is drawn if a position
	 happens 3 times, not repeated 3 times (happens 4 times).
      */
      stillGoing = (int) thisPlyData->count50Moves;
      ASSERT(others->repHead < maxMove);
      for (i = others->repHead+ply-3; 
	   i >= 0 && stillGoing >= 0; 
	   i -= 2, stillGoing -= 2)
	if (others->repKey[i] == thisPlyData->key) happenings++;

      if (happenings > 1) return happenings;

      return 0;
    }
}


/* Always return white's score */
int eval(ui ply, sPos *pos, sPlyData *plyData, sOthers *others, int lScore,
	 int wPieceScore, int bPieceScore, int alpha, int doubleBonus)
{
  ui i, wSleepy, bSleepy, openness = 0, bothSidePawns = 0, ppCount;
  int minPieceScore, score = 0, tempScore = 0, dbReduction;

  ASSERT(ply < maxPly);
  ASSERT(pos != NULL);
  ASSERT(plyData != NULL);
  ASSERT(others != NULL);

  score = lScore;

  if ((wPieceScore + bPieceScore) >= 0) minPieceScore = -bPieceScore;
  else minPieceScore = wPieceScore;

  if (minPieceScore <= endGameValue)
    {
      if (((pos->WPBoard | pos->BPBoard) & others->FarQueenSide) &&
	  ((pos->WPBoard | pos->BPBoard) & others->FarKingSide))
	bothSidePawns = 1;

      if (plyData[ply].wToMove)
	{ 
	  /* credit */
	  ppCount = 0;
	  tempScore += bPEndBonus(wPieceScore, pos, &plyData[ply], others,
				  bothSidePawns, &ppCount);
	  if (ppCount > 0 && doubleBonus > 0)
	    {
	      dbReduction = min(ppCount*PENALTY21, doubleBonus);
	      tempScore -= dbReduction;
	    }
	  tempScore += bBEndBonus(pos, others);
	  tempScore += wREndPenalty(pos);
	  tempScore += bNEndBonus(pos, others);
	  tempScore += bKEndBonus(wPieceScore, bPieceScore, pos, others);
	  tempScore += bREndBonus(pos);

	  /* debit */
	  ppCount = 0;
	  tempScore += wPEndBonus(bPieceScore, pos, &plyData[ply], others, 
				  bothSidePawns, &ppCount);   /* 2202 */
	  if (ppCount > 0 && doubleBonus < 0)
	    {
	      dbReduction = max(-ppCount*PENALTY21, doubleBonus);
	      tempScore -= dbReduction;
	    }
	  tempScore += wBEndBonus(pos, others);   /* 78 */
	  tempScore += bREndPenalty(pos);   /* 50 */
	  tempScore += wNEndBonus(pos, others);   /* 48 */
	  tempScore += wKEndBonus(wPieceScore, bPieceScore, pos, others);   /* 40 */
	  tempScore += wREndBonus(pos);   /* 20 */
	}
      else
	{
	  /* credit */
	  ppCount = 0;
	  tempScore += wPEndBonus(bPieceScore, pos, &plyData[ply], others, 
				  bothSidePawns, &ppCount);
	  if (ppCount > 0 && doubleBonus < 0)
	    {
	      dbReduction = max(-ppCount*PENALTY21, doubleBonus);
	      tempScore -= dbReduction;
	    }
	  tempScore += wBEndBonus(pos, others);
	  tempScore += bREndPenalty(pos);
	  tempScore += wREndBonus(pos);
	  tempScore += wNEndBonus(pos, others);
	  tempScore += wKEndBonus(wPieceScore, bPieceScore, pos, others);

	  /* debit */
	  ppCount = 0;
	  tempScore += bPEndBonus(wPieceScore, pos, &plyData[ply], others, 
				  bothSidePawns, &ppCount);   /* 2202 */
	  if (ppCount > 0 && doubleBonus > 0)
	    {
	      dbReduction = min(ppCount*PENALTY21, doubleBonus);
	      tempScore -= dbReduction;
	    }
	  tempScore += bBEndBonus(pos, others);   /* 78 */
	  tempScore += wREndPenalty(pos);   /* 50 */
	  tempScore += bNEndBonus(pos, others);   /* 48 */
	  tempScore += bKEndBonus(wPieceScore, bPieceScore, pos, others); /* 40 */
	  tempScore += bREndBonus(pos);   /* 20 */
	}
      ASSERT(tempScore >= -mateScore && tempScore <= mateScore);

      if ((others->Flags & OneMask[8]) ||        /* drawish */
	  /* queens tend to make end game drawish */
	  (tempScore > 0 && pos->BQBoard) ||
	  (tempScore < 0 && pos->WQBoard))
	tempScore = tempScore/4;
      score += tempScore;
    }
  else
    {
      /* sleepy means not awake/developed */
      wSleepy = bSleepy = 0;
      wSleepy += bitCount(pos->WNBoard & others->Row[0]);
      wSleepy += bitCount(pos->WBBoard & others->Row[0]);
      wSleepy += bitCount(pos->WRBoard & (OneMask[A1] | OneMask[H1]));
      bSleepy += bitCount(pos->BNBoard & others->Row[7]);
      bSleepy += bitCount(pos->BBBoard & others->Row[7]);
      bSleepy += bitCount(pos->BRBoard & (OneMask[A8] | OneMask[H8]));
      ASSERT(wSleepy <= 8 && bSleepy <= 8);
      if (wSleepy != bSleepy)
	{
	  for (i = 0; i < 8; i++)
	    if (!pos->wPCount[i] && !pos->bPCount[i]) openness++;
	  if (bSleepy > wSleepy) score += openness * 5;
	  else if (bSleepy < wSleepy) score -= openness * 5;
	}
      
      if (plyData[ply].wToMove)
	{ 
	  /* credit */
	  score += wKPenalty(ply, pos, plyData, others);
	  score += bPBonus(bSleepy, minPieceScore, pos, others);
	  score += bNBonus(bSleepy, pos, others);
	  score += bRBonus(bSleepy, minPieceScore, pos, others);
	  score += bBBonus(pos, others);
	  score += wRPenalty(pos);
	  score += bKBonus(pos, others);
	  score += wBPenalty(pos);
	  score += bQBonus(wSleepy, minPieceScore, pos, others);

	  /* debit */
	  score += bKPenalty(ply, pos, plyData, others);   /* 600 */
	  if (score + 915 <= alpha)
	    {
	      return score;
	    }
	  score += wPBonus(wSleepy, minPieceScore, pos, others);   /* 371 */
	  if (score + 544 <= alpha)
	    {
	      return score;
	    }
	  score += wNBonus(wSleepy, pos, others);   /* 248 */
	  if (score + 296 <= alpha)
	    {
	      return score;
	    }
	  score += wRBonus(wSleepy, minPieceScore, pos, others);   /* 110 */
	  if (score + 186 <= alpha)
	    {
	      return score;
	    }
	  score += wBBonus(pos, others);   /* 78 */
	  if (score + 108 <= alpha)
	    {
	      return score;
	    }
	  score += bRPenalty(pos);   /* 50 */
	  score += bBPenalty(pos);   /* 30 */
	  score += wKBonus(pos, others);   /* 18 */
	  score += wQBonus(wSleepy, minPieceScore, pos, others);   /* 10 */
	}
      else
	{ 
	  /* credit */
	  score += bKPenalty(ply, pos, plyData, others);
	  score += wPBonus(wSleepy, minPieceScore, pos, others);
	  score += wNBonus(wSleepy, pos, others);
	  score += wRBonus(wSleepy, minPieceScore, pos, others);
	  score += wBBonus(pos, others);
	  score += bRPenalty(pos);
	  score += wKBonus(pos, others);
	  score += bBPenalty(pos);
	  score += wQBonus(wSleepy, minPieceScore, pos, others);

	  /* debit */
	  score += wKPenalty(ply, pos, plyData, others);   /*600 */
	  if (-score + 915 <= alpha)
	    {
	      return score;
	    }
	  score += bPBonus(bSleepy, minPieceScore, pos, others);   /* 371 */
	  if (-score + 544 <= alpha)
	    {
	      return score;
	    }
	  score += bNBonus(bSleepy, pos, others);   /* 248 */
	  if (-score + 296 <= alpha)
	    {
	      return score;
	    }
	  score += bRBonus(bSleepy, minPieceScore, pos, others);   /* 110 */
	  if (-score + 186 <= alpha)
	    {
	      return score;
	    }
	  score += bBBonus(pos, others);   /* 78 */
	  if (-score + 108 <= alpha)
	    {
	      return score;
	    }
	  score += wRPenalty(pos);   /* 50 */
	  score += wBPenalty(pos);   /* 30 */
	  score += bKBonus(pos, others);   /* 18 */
	  score += bQBonus(wSleepy, minPieceScore, pos, others);   /* 10 */
	}
    }

  ASSERT(score >= -mateScore && score <= mateScore);
  return score;
}


/********************** Root evaluation ***********************/


void revalWOpening(BB move, sPos *pos, sOthers *others) 
{
  ASSERT(from(move) <= H8 && to(move) <= H8 &&
	 from(move) != to(move));
  ASSERT(piece(move) < 13);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);
  ASSERT(others->rOpening < 4 && others->moveNumber < maxPly);

  /********************************************************************/
  /* If out of opening early, use Purdy's All Purpose System (Colle)  */
  /********************************************************************/

  if (others->rOpening == 1 && others->moveNumber < 7)
    {
      /* d4 */
      if (piece(move) == wP && from(move) == D2 && to(move) == D4)
	pos->wScore += 32;

      /* Nf3 */
      else if (piece(move) == wN && from(move) == G1 && to(move) == F3)
	pos->wScore += 30;

      /* e3 */
      else if (piece(move) == wP && from(move) == E2 && to(move) == E3)
	pos->wScore += 28;

      /* Bd3 */
      else if (!(pos->WPBoard & OneMask[D2]) &&        /* no pawn on D2 */
	       piece(move) == wB && from(move) == F1 && to(move) == D3)
	pos->wScore += 26;

      /* Nbd2 */
      else if (piece(move) == wN && from(move) == B1 && to(move) == D2)
	pos->wScore += 24;

      /* O-O */
      else if (piece(move) == wK && from(move) == E1 && to(move) == G1)
	pos->wScore += 22;

      /* c3 */
      else if (piece(move) == wP && from(move) == C2 && to(move) == C3)
	pos->wScore += 20;
    }


  /********************************************************************/
  /* If out of opening early, use King's Indian Attack (Pirc flipped) */
  /* 1. Nf3 2. g3 3. Bg2 4. O-O 5. d3 6. Nc3 7. e4                    */
  /********************************************************************/

  if (others->rOpening == 2 && others->moveNumber < 7)
    {
      /* Nf3 */
      if (piece(move) == wN && from(move) == G1 && to(move) == F3)
	pos->wScore += 32;

      /* g3 */
      else if (piece(move) == wP && from(move) == G2 && to(move) == G3)
	pos->wScore += 30;

      /* Bg2 */
      else if (piece(move) == wB && from(move) == F1 && to(move) == G2)
	pos->wScore += 28;

      /* O-O */
      else if (piece(move) == wK && from(move) == E1 && to(move) == G1)
	pos->wScore += 26;

      /* d3 */
      else if (piece(move) == wP && from(move) == D2 && to(move) == D3)
	pos->wScore += 24;

      /* Nc3 */
      else if (piece(move) == wN && from(move) == B1 && to(move) == C3)
	pos->wScore += 22;

      /* e4 */
      else if (piece(move) == wP && from(move) == E2 && to(move) == E4)
	pos->wScore += 20;
    }


  /********************************************************************/
  /* If out of opening early, use Richter-Verosov                     */
  /* 1. d4 2. Nc3 3. Bg5 4. Bxf6 5. e3 6. Bd3 7. Qf3                  */
  /********************************************************************/

  if (others->rOpening == 3 && others->moveNumber < 7)
    {
      /* d4 */
      if (piece(move) == wP && from(move) == D2 && to(move) == D4)
	pos->wScore += 32;

      /* Nc3 */
      else if (piece(move) == wN && from(move) == B1 && to(move) == C3)
	pos->wScore += 30;

      /* Bg5 */
      else if (piece(move) == wB && from(move) == C1 && to(move) == G5)
	pos->wScore += 28;

      /* Bxf6 */
      else if ((pos->BNBoard & OneMask[F6]) &&        /* capture bN */
	       piece(move) == wB && from(move) == G5 && to(move) == F6)
	pos->wScore += 26;

      /* e3 */
      else if (piece(move) == wP && from(move) == E2 && to(move) == E3)
	pos->wScore += 24;

      /* Bd3 */
      else if (!(pos->WPBoard & OneMask[D2]) &&        /* no pawn on D2 */
	       piece(move) == wB && from(move) == F1 && to(move) == D3)
	pos->wScore += 26;

      /* Qf3 */
      else if (piece(move) == wQ && from(move) == D1 && to(move) == F3)
	pos->wScore += 20;
    }
}


/* Root evaluation at the root */
void revalBOpening(BB move, sPos *pos, sOthers *others) 
{
  ASSERT(from(move) <= H8 && to(move) <= H8 &&
	 from(move) != to(move));
  ASSERT(piece(move) < 13);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);
  ASSERT(others->rOpening < 4 && others->moveNumber < maxPly);

  /********************************************************************/
  /* If out of opening early, use Purdy's All Purpose System          */
  /********************************************************************/

  if (others->rOpening == 1 && others->moveNumber < 7)
    {
      /* d5 */
      if (piece(move) == bP && from(move) == D7 && to(move) == D5)
	pos->wScore -= 32;

      /* e6 */
      else if (piece(move) == bP && from(move) == E7 && to(move) == E6)
	pos->wScore -= 30;

      /* Nf6 */
      else if (piece(move) == bN && from(move) == G8 && to(move) == F6)
	pos->wScore -= 28;

      /* Be7 */
      else if (piece(move) == bB && from(move) == F8 && to(move) == E7)
	pos->wScore -= 26;

      /* O-O */
      else if (piece(move) == bK && from(move) == E8 && to(move) == G8)
	pos->wScore -= 24;

      /* b6 */
      else if (piece(move) == bP && from(move) == B7 && to(move) == B6)
	pos->wScore -= 22;
      
      /* c5 */
      else if (piece(move) == bP && from(move) == C7 && to(move) == C5)
	pos->wScore -= 20;
    }


  /*********************************************************************/
  /* If out of opening early, use Richter-Veresov                      */
  /* 1. .. f5 2. .. Nf6 3. .. e6 4. .. Be7 5. .. O-O 5. .. d5 7. .. c6 */
  /*********************************************************************/

  if (others->rOpening == 2 && others->moveNumber < 7)
    {
      /* f5 */
      if (piece(move) == bP && from(move) == F7 && to(move) == F5)
	pos->wScore -= 32;

      /* Nf6 */
      else if (piece(move) == bN && from(move) == G8 && to(move) == F6)
	pos->wScore -= 30;

      /* e6 */
      else if (piece(move) == bP && from(move) == E7 && to(move) == E6)
	pos->wScore -= 28;

      /* Be7 */
      else if (piece(move) == bB && from(move) == F8 && to(move) == E7)
	pos->wScore -= 26;

      /* O-O */
      else if (piece(move) == bK && from(move) == E8 && to(move) == G8)
	pos->wScore -= 24;

      /* d5 */
      else if (piece(move) == bP && from(move) == D7 && to(move) == D5)
	pos->wScore -= 22;

      /* c6 */
      else if (piece(move) == bP && from(move) == C7 && to(move) == C6)
	pos->wScore -= 20;
    }


  /**********************************************************************/
  /* If out of opening early, use Pirc (King's Indian Attack flipped)   */
  /* 1. .. d6 2. .. Nf6 3. .. g6 4. .. Bg7 5. .. Nc6 6. .. O-O 7. .. e5 */
  /**********************************************************************/

  if (others->rOpening == 3 && others->moveNumber < 7)
    {
      /* d6 */
      if (piece(move) == bP && from(move) == D7 && to(move) == D6)
	pos->wScore -= 32;
      
      /* Nf6 */
      else if (piece(move) == bN && from(move) == G8 && to(move) == F6)
	pos->wScore -= 30;

      /* g6 */
      else if (piece(move) == bP && from(move) == G7 && to(move) == G6)
	pos->wScore -= 28;

      /* Bg7 */
      else if (piece(move) == bB && from(move) == F8 && to(move) == G7)
	pos->wScore -= 26;

      /* Nc6 */
      else if (piece(move) == bN && from(move) == B8 && to(move) == C6)
	pos->wScore -= 24;

      /* O-O */
      else if (piece(move) == bK && from(move) == E8 && to(move) == G8)
	pos->wScore -= 22;
      
      /* e5 */
      else if (piece(move) == bP && from(move) == E7 && to(move) == E5)
	pos->wScore -= 20;
    }
}


/* Root evaluation.  Calculate white's score and store it in pos->wScore. */
void reval(BB move, sPos *pos, sOthers *others)
{
  ui wSleepy, bSleepy;

  ASSERT(from(move) <= H8 && to(move) <= H8 &&
	 from(move) != to(move));
  ASSERT(piece(move) < 13);
  ASSERT(pos != NULL);
  ASSERT(others != NULL);
  
  /* sleepy means not awake/developed */
  wSleepy = bSleepy = 0;
  wSleepy += bitCount(pos->WNBoard & others->Row[0]);
  wSleepy += bitCount(pos->WBBoard & others->Row[0]);
  wSleepy += bitCount(pos->WRBoard & (OneMask[A1] | OneMask[H1]));
  bSleepy += bitCount(pos->BNBoard & others->Row[7]);
  bSleepy += bitCount(pos->BBBoard & others->Row[7]);
  bSleepy += bitCount(pos->BRBoard & (OneMask[A8] | OneMask[H8]));
  ASSERT(wSleepy <= 8 && bSleepy <= 8);
      
  /* White is not developed yet */
  if (wSleepy > 0) revalWOpening(move, pos, others);

  /* Black is not developed yet */
  if (bSleepy > 0) revalBOpening(move, pos, others);
}
/* hash.c */
/* Last modified: January 16, 2006 */


/* hash entries (2 * 64 bits), counting bits from right to left
   bits        variable        description
   0-63        key             pos.key
   64-79       score           actual score + infinity (it's always positive)
   80-87       depth
   88-93       from            hash move
   94-99       to              hash move
   100         mateThreat      dangerous situation after null move 
   101-103     hashFlag        EXACT, FAIL_LOW, or FAIL_HIGH
*/   


void hashStore(ui ply, ui depth, int score, ui from, ui to, ui mateThreat,
	       ui hashFlag, sPlyData *thisPlyData, sHash **hash, 
	       sOthers *others)
{
  struct HashTable *thisHash;
  ui index;

  ASSERT(thisPlyData != NULL);
  ASSERT(*hash != NULL);
  ASSERT(others != NULL);
  ASSERT(score >= -mateScore && score <= mateScore);

  /* locate the storage area */
  ASSERT(others->hashSize > 0);
  index = (ui)(thisPlyData->key & others->hashSize);
  ASSERT(index <= others->hashSize);
  if (thisPlyData->wToMove) 
    thisHash = &hash[white][index];
  else thisHash = &hash[black][index];

  /* deep enough?  then store new data */
  if (depth >= ((ui)(thisHash->stuffs >> 16) & 0xFF))
    {
      /* back up first, */
      if (((ui)(thisHash->stuffsSave >> 16) & 0xFF) <
	  ((ui)(thisHash->stuffs >> 16) & 0xFF))
	{
	  thisHash->keySave = thisHash->key;
	  thisHash->stuffsSave = thisHash->stuffs;
	}

      /* then replace */
      thisHash->key = thisPlyData->key;
      if (score > almostScore) score += ply;
      else if (score < -almostScore) score -= ply;
      /* *score can be out of bound here due to null move */
      thisHash->stuffs = (((ui)(score + infinity)) & 0xFFFF) | 
	(depth << 16) | (from << 24) | ((BB) to << 30) | 
	((BB) mateThreat << 36) | ((BB) hashFlag << 37);
    }
}


int hashProbe(ui ply, ui depth, int *score, ui *from1, ui *to1, 
	      ui *mateThreat, ui *hashFlag, 
	      sPlyData *thisPlyData, sHash **hash, sOthers *others)
{
  struct HashTable *thisHash;
  ui index, hashDepth;

  ASSERT(*from1 == 0 && *to1 == 0);
  ASSERT(mateThreat != NULL);
  ASSERT(hashFlag != NULL);
  ASSERT(thisPlyData != NULL);
  ASSERT(*hash != NULL);
  ASSERT(others != NULL);
  ASSERT(others->hashSize > 0);

  /* locate the storage area */
  ASSERT(others->hashSize > 0);
  index = (ui)(thisPlyData->key & others->hashSize);
  ASSERT(index <= others->hashSize);
  if (thisPlyData->wToMove) thisHash = &hash[white][index];
  else thisHash = &hash[black][index];

  /* the key matches and the stored data was from a deeper search */

  /* try the primary storage */
  if (thisHash->key == thisPlyData->key)
    {
      /* get hash move, don't have to be deep enough */
      *from1 = (ui)(thisHash->stuffs >> 24) & 63;
      *to1 = (ui)(thisHash->stuffs >> 30) & 63;
      
      *score = (thisHash->stuffs & 0xFFFF) - infinity;
      /* *score can be out of bound here due to null move */
      hashDepth = ((ui)(thisHash->stuffs >> 16) & 0xFF);
      ASSERT(hashDepth > 0 && hashDepth <= maxPly);
      *hashFlag = (ui)(thisHash->stuffs >> 37) & 7;
      ASSERT(*hashFlag == FAIL_LOW || *hashFlag == FAIL_HIGH || 
	     *hashFlag == EXACT);
      if (depth <= hashDepth)
	{
	  *mateThreat = (ui)(thisHash->stuffs >> 36) & 1;
	  if (*score > almostScore) *score -= ply;
	  else if (*score < -almostScore) *score += ply;
	  return true;
	}
    }

  /* try the back up storage */
  if (thisHash->keySave == thisPlyData->key)
    {
      /* get hash move, don't have to be deep enough */
      if (*from1 == 0 && *to1 == 0)
	{
	  *from1 = (ui)(thisHash->stuffsSave >> 24) & 63;
	  *to1 = (ui)(thisHash->stuffsSave >> 30) & 63;
	}
      
      *score = (thisHash->stuffsSave & 0xFFFF) - infinity;
      /* *score can be out of bound here due to null move */
      hashDepth = ((ui)(thisHash->stuffsSave >> 16) & 0xFF);
      ASSERT(hashDepth > 0 && hashDepth <= maxPly);
      *hashFlag = (ui)(thisHash->stuffs >> 37) & 7;
      ASSERT(*hashFlag == FAIL_LOW || *hashFlag == FAIL_HIGH || 
	     *hashFlag == EXACT);
      if (depth <= hashDepth)
	{
	  *mateThreat = (ui)(thisHash->stuffsSave >> 36) & 1;
	  if (*score > almostScore) *score -= ply;
	  else if (*score < -almostScore) *score += ply;
	  return true;
	}
    }
  
  return false;
}
/* init.c */
/* Last modified: April 19, 2006 */


/* Initialize useful bitboards */
void initBitBoards(sOthers *others)
{
  BB Temp;
  ui sq, i, j;
  int k;
  ui r, c;       /* row, column */


  for (i = 0; i <= 7; i++)
    {
      others->Row[i] = OneMask[i*8] | OneMask[i*8+1] | OneMask[i*8+2] | 
	OneMask[i*8+3] | OneMask[i*8+4] | OneMask[i*8+5] | 
	OneMask[i*8+6] | OneMask[i*8+7];
      others->Up1Of[i*8] = OneMask[i*8+1] | OneMask[i*8+2] | 
	OneMask[i*8+3] | OneMask[i*8+4] | OneMask[i*8+5] | 
	OneMask[i*8+6] | OneMask[i*8+7];
      others->Up1Of[i*8+1] = OneMask[i*8+2] | 
	OneMask[i*8+3] | OneMask[i*8+4] | OneMask[i*8+5] | 
	OneMask[i*8+6] | OneMask[i*8+7];
      others->Up1Of[i*8+2] = OneMask[i*8+3] | OneMask[i*8+4] | OneMask[i*8+5] |
	OneMask[i*8+6] | OneMask[i*8+7];
      others->Up1Of[i*8+3] = OneMask[i*8+4] | OneMask[i*8+5] | 
	OneMask[i*8+6] | OneMask[i*8+7];
      others->Up1Of[i*8+4] = OneMask[i*8+5] | OneMask[i*8+6] | OneMask[i*8+7];
      others->Up1Of[i*8+5] = OneMask[i*8+6] | OneMask[i*8+7];
      others->Up1Of[i*8+6] = OneMask[i*8+7];
      others->Up1Of[i*8+7] = (BB) 0;
      others->Down1Of[i*8] = others->Row[i] & ~others->Up1Of[i*8] & 
	NilMask[i*8];
      others->Down1Of[i*8+1] = others->Row[i] & ~others->Up1Of[i*8+1] & 
	NilMask[i*8+1];
      others->Down1Of[i*8+2] = others->Row[i] & ~others->Up1Of[i*8+2] & 
	NilMask[i*8+2];
      others->Down1Of[i*8+3] = others->Row[i] & ~others->Up1Of[i*8+3] & 
	NilMask[i*8+3];
      others->Down1Of[i*8+4] = others->Row[i] & ~others->Up1Of[i*8+4] & 
	NilMask[i*8+4];
      others->Down1Of[i*8+5] = others->Row[i] & ~others->Up1Of[i*8+5] & 
	NilMask[i*8+5];
      others->Down1Of[i*8+6] = others->Row[i] & ~others->Up1Of[i*8+6] & 
	NilMask[i*8+6];
      others->Down1Of[i*8+7] = others->Row[i] & ~others->Up1Of[i*8+7] & 
	NilMask[i*8+7];
    }

  for (i = 0; i <= 7; i++)
    others->Column[i] = OneMask[i] | OneMask[i+8] | OneMask[i+16] | 
      OneMask[i+24] | OneMask[i+32] | OneMask[i+40] | OneMask[i+48] | 
      OneMask[i+56];

  others->Edge = others->Row[0] | others->Row[7] |
    others->Column[0] | others->Column[7];
  others->InnerEdge = others->Row[1] | others->Row[6] |
    others->Column[1] | others->Column[6];

  for (sq = A1; sq <= H8; sq++)
    {
      others->Up8Of[sq] = others->Down8Of[sq] = others->SideOf[sq] = 
	others->Up8SideOf[sq] = others->Down8SideOf[sq] = (BB) 0;
      if (sq+8 <= H8) for (k = sq+8; k <= H8; k += 8) others->Up8Of[sq] |= OneMask[k];
      if (sq-8 >= A1) for (k = sq-8; k >= A1; k -= 8) others->Down8Of[sq] |= OneMask[k];
      if (column(sq) < 7)
	{
	  others->SideOf[sq] |= OneMask[sq+1];
	  if (sq+9 <= H8) for (k = sq+9; k <= H8; k += 8) others->Up8SideOf[sq] |= OneMask[k];
	  if (sq-7>= A1) for (k = sq-7; k >= A1; k -= 8) others->Down8SideOf[sq] |= OneMask[k];
	}
      if (column(sq) > 0)
	{
	  others->SideOf[sq] |= OneMask[sq-1];
	  if (sq+7 <= H8) for (k = sq+7; k <= H8; k += 8) others->Up8SideOf[sq] |= OneMask[k];
	  if (sq-9 >= A1) for (k = sq-9; k >= A1; k -= 8) others->Down8SideOf[sq] |= OneMask[k];
	}
    }

  others->QueenSide = others->Column[0] | others->Column[1] | others->Column[2] | others->Column[3];
  others->FarQueenSide = others->Column[0] | others->Column[1] | others->Column[2];
  others->KingSide = others->Column[4] | others->Column[5] | others->Column[6] | others->Column[7];
  others->FarKingSide = others->Column[5] | others->Column[6] | others->Column[7];

  others->Dark = 
    OneMask[A1] | OneMask[C1] | OneMask[E1] | OneMask[G1] | 
    OneMask[B2] | OneMask[D2] | OneMask[F2] | OneMask[H2] | 
    OneMask[A3] | OneMask[C3] | OneMask[E3] | OneMask[G3] | 
    OneMask[B4] | OneMask[D4] | OneMask[F4] | OneMask[H4] | 
    OneMask[A5] | OneMask[C5] | OneMask[E5] | OneMask[G5] | 
    OneMask[B6] | OneMask[D6] | OneMask[F6] | OneMask[H6] | 
    OneMask[A7] | OneMask[C7] | OneMask[E7] | OneMask[G7] | 
    OneMask[B8] | OneMask[D8] | OneMask[F8] | OneMask[H8];
  others->Light = 
    OneMask[B1] | OneMask[D1] | OneMask[F1] | OneMask[H1] | 
    OneMask[A2] | OneMask[C2] | OneMask[E2] | OneMask[G2] | 
    OneMask[B3] | OneMask[D3] | OneMask[F3] | OneMask[H3] | 
    OneMask[A4] | OneMask[C4] | OneMask[E4] | OneMask[G4] | 
    OneMask[B5] | OneMask[D5] | OneMask[F5] | OneMask[H5] | 
    OneMask[A6] | OneMask[C6] | OneMask[E6] | OneMask[G6] | 
    OneMask[B7] | OneMask[D7] | OneMask[F7] | OneMask[H7] | 
    OneMask[A8] | OneMask[C8] | OneMask[E8] | OneMask[G8];

  /* Rows above and below specific squares */
  for (sq = A1; sq <= H7; sq++) others->OneUp[sq] = others->Row[row(sq)+1];
  for (sq = A1; sq <= H6; sq++) others->TwoUp[sq] = others->Row[row(sq)+2];
  for (sq = H8; sq >= A2; sq--) others->OneDown[sq] = others->Row[row(sq)-1];
  for (sq = H8; sq >= A3; sq--) others->TwoDown[sq] = others->Row[row(sq)-2];

  /* Columns on either sides of specific squares */
  for (sq = A1; sq <= H8; sq++)
    {
      c = column(sq);
      if (c > 0) others->OneSide[sq] = others->Column[c-1];
      if (c < 7) others->OneSide[sq] |= others->Column[c+1];
    }

  for (sq = A1; sq <= H8; sq++)
    {
      c = column(sq);
      if (c > 1) others->TwoSide[sq] = others->Column[c-2];
      if (c < 6) others->TwoSide[sq] |= others->Column[c+2];
    }

  /* White pawn captures */
  for (sq = A1; sq <= H7; sq++)
    {
      c = column(sq);
      others->WPCaptures[sq] = others->OneUp[sq] & others->OneSide[c];
    }

  /* Black pawn captures */
  for (sq = H8; sq >= A2; sq--)
    {
      c = column(sq);
      others->BPCaptures[sq] = others->OneDown[sq] & others->OneSide[c];
    }

  /* Knight moves */
  for (sq = A1; sq <= H8; sq++)
    {
      others->NMoves[sq] = 
	(others->OneUp[sq] | others->OneDown[sq]) & others->TwoSide[sq];
      others->NMoves[sq] |= 
	(others->TwoUp[sq] | others->TwoDown[sq]) & others->OneSide[sq];
    }

  /* Bishop moves */
  for (sq = A1; sq <= H8; sq++)
    {
      r = row(sq);
      c = column(sq);
      others->BMoves[sq] = (BB) 0;

      /* plus 7 dkrection */
      if (sq+7 <= H8) for (k = sq+7; k <= H8 && row(k) > r && column(k) < c; k += 7)
	others->Up7Of[sq] |= OneMask[k];

      /* plus 9 dkrection */
      if (sq+9 <= H8) for (k = sq+9; k <= H8 && row(k) > r && column(k) > c; k += 9)
	others->Up9Of[sq] |= OneMask[k];

      /* mknus 7 dkrection */
      if (sq-7 >= A1) for (k = sq-7; k >= A1 && row(k) < r && column(k) > c; k -= 7)
	others->Down7Of[sq] |= OneMask[k];

      /* mknus 9 dkrection */
      if (sq-9 >= A1) for (k = sq-9; k >= A1 && row(k) < r && column(k) < c; k -= 9)
	others->Down9Of[sq] |= OneMask[k];
      
      others->BMoves[sq] = others->Up7Of[sq] | others->Up9Of[sq] | 
	others->Down7Of[sq] | others->Down9Of[sq]; 
    }

  /* King moves and neighborhood */
  for (sq = A1; sq <= H8; sq++)
    for (i = A1; i <= H8; i++)
      {
	if (distance(sq,i) == 1) others->KMoves[sq] |= OneMask[i];
	if (distance(sq,i) < 3) others->KHood[sq] |= OneMask[i];
      }

  /* direction from i to j */
  for (i = A1; i <= H8; i++)
    for (j = A1; j <= H8; j++)
      others->direction[i][j] = 0;
  for (i = A1; i <= H8; i++)
    {
      /* up 1 direction */
      Temp = others->Up1Of[i];
      while (Temp)
	{
	  j = firstOne(Temp);
	  others->direction[i][j] = 1;
	  Temp &= NilMask[j];
	}

      /* up 7 direction */
      Temp = others->Up7Of[i];
      while (Temp)
	{
	  j = firstOne(Temp);
	  others->direction[i][j] = 7;
	  Temp &= NilMask[j];
	}

      /* up 8 direction */
      Temp = others->Up8Of[i];
      while (Temp)
	{
	  j = firstOne(Temp);
	  others->direction[i][j] = 8;
	  Temp &= NilMask[j];
	}

      /* up 9 direction */
      Temp = others->Up9Of[i];
      while (Temp)
	{
	  j = firstOne(Temp);
	  others->direction[i][j] = 9;
	  Temp &= NilMask[j];
	}
      /* down 1 direction */
      Temp = others->Down1Of[i];
      while (Temp)
	{
	  j = firstOne(Temp);
	  others->direction[i][j] = -1;
	  Temp &= NilMask[j];
	}

      /* down 7 direction */
      Temp = others->Down7Of[i];
      while (Temp)
	{
	  j = firstOne(Temp);
	  others->direction[i][j] = -7;
	  Temp &= NilMask[j];
	}

      /* down 8 direction */
      Temp = others->Down8Of[i];
      while (Temp)
	{
	  j = firstOne(Temp);
	  others->direction[i][j] = -8;
	  Temp &= NilMask[j];
	}

      /* down 9 direction */
      Temp = others->Down9Of[i];
      while (Temp)
	{
	  j = firstOne(Temp);
	  others->direction[i][j] = -9;
	  Temp &= NilMask[j];
	}
    }
}


/* Initialize hash keys */
void initKeys(sOthers *others)
{
  ui i;

  /* without initial seed, random64() produces repeated keys */
  setRandomSeed(11141957);
  for (i = A1; i <= H8; i++)
    {
      others->wPKey[i] = random64();
      others->wNKey[i] = random64();
      others->wBKey[i] = random64();
      others->wRKey[i] = random64();
      others->wQKey[i] = random64();
      others->wKKey[i] = random64();
      others->bPKey[i] = random64();
      others->bNKey[i] = random64();
      others->bBKey[i] = random64();
      others->bRKey[i] = random64();
      others->bQKey[i] = random64();
      others->bKKey[i] = random64();
      others->ePKey[i] = random64();
    }

  for (i = 0; i < 4; i++)
    {
      others->wCanCastleKey[i] = random64();
      others->bCanCastleKey[i] = random64();
    }
}


/* Initialize hash tables */
void clearHash(sHash **hash, sOthers *others)
{
  memset(hash[white], 0, sizeof(struct HashTable) * (others->hashSize+1));
  memset(hash[black], 0, sizeof(struct HashTable) * (others->hashSize+1));
}


/* Initialize the hash keys and various bitboards */
void finalizeInit(ui* piece, sPos *pos, sPlyData *thisPlyData,
		  sOthers *others)
{
  ui sq;
  double timePerMove;

  BB *pWAttacker = (BB *) thisPlyData->wAttacker;
  BB *pBAttacker = (BB *) thisPlyData->bAttacker;

  /* Attack tables */
  pWAttacker[0] =   pWAttacker[1] =   pWAttacker[2] =   pWAttacker[3] = 
    pWAttacker[4] =   pWAttacker[5] =   pWAttacker[6] =   pWAttacker[7] = 0; 
  pBAttacker[0] =   pBAttacker[1] =   pBAttacker[2] =   pBAttacker[3] = 
    pBAttacker[4] =   pBAttacker[5] =   pBAttacker[6] =   pBAttacker[7] = 0; 

  strcpy(others->opponentName, "opponent");
  others->iAm = black;
  others->sNodes = others->qNodes = 0;

  pos->WPBoard = pos->WNBoard = pos->WBBoard = 
    pos->WRBoard = pos->WQBoard = 0;
  pos->wKSq = 64;
  pos->BPBoard = pos->BNBoard = pos->BBBoard = 
    pos->BRBoard = pos->BQBoard = 0;
  pos->bKSq = 64;
  pos->RotatedBoard90 = pos->RotatedBoard45l = 
    pos->RotatedBoard45r = 0;

  thisPlyData->key = 0;

  pos->wPCount[0] = pos->wPCount[1] = pos->wPCount[2] = 
  pos->wPCount[3] = pos->wPCount[4] = pos->wPCount[5] = 
  pos->wPCount[6] = pos->wPCount[7] = 
  pos->bPCount[0] = pos->bPCount[1] = pos->bPCount[2] = 
  pos->bPCount[3] = pos->bPCount[4] = pos->bPCount[5] = 
  pos->bPCount[6] = pos->bPCount[7] = 0;
  for (sq = A1; sq <= H8; sq++)
    {
      switch (piece[sq])
	{
	case wP:
	  pos->WPBoard |= OneMask[sq];
	  thisPlyData->key ^= others->wPKey[sq];
	  pos->wPCount[column(sq)]++;
	  break;
	case wN:
	  pos->WNBoard |= OneMask[sq];
	  thisPlyData->key ^= others->wNKey[sq];
	  break;
	case wB:
	  pos->WBBoard |= OneMask[sq];
	  thisPlyData->key ^= others->wBKey[sq];
	  break;
	case wR:
	  pos->WRBoard |= OneMask[sq];
	  thisPlyData->key ^= others->wRKey[sq];
	  break;
	case wQ:
	  pos->WQBoard |= OneMask[sq];
	  thisPlyData->key ^= others->wQKey[sq];
	  break;
	case wK:
	  pos->wKSq = sq;
	  thisPlyData->key ^= others->wKKey[sq];
	  break;
	case bP:
	  pos->BPBoard |= OneMask[sq];
	  thisPlyData->key ^= others->bPKey[sq];
	  pos->bPCount[column(sq)]++;
	  break;
	case bN:
	  pos->BNBoard |= OneMask[sq];
	  thisPlyData->key ^= others->bNKey[sq];
	  break;
	case bB:
	  pos->BBBoard |= OneMask[sq];
	  thisPlyData->key ^= others->bBKey[sq];
	  break;
	case bR:
	  pos->BRBoard |= OneMask[sq];
	  thisPlyData->key ^= others->bRKey[sq];
	  break;
	case bQ:
	  pos->BQBoard |= OneMask[sq];
	  thisPlyData->key ^= others->bQKey[sq];
	  break;
	case bK:
	  pos->bKSq = sq;
	  thisPlyData->key ^= others->bKKey[sq];
	  break;
	}

      if (piece[sq] != none)
	{
	  pos->RotatedBoard90 |= OneMask[rotate90[sq]];
	  pos->RotatedBoard45l |= OneMask[rotate45l[sq]];
	  pos->RotatedBoard45r |= OneMask[rotate45r[sq]];
	}
    }

  if (pos->wKSq == 64) 
    {
      printf("ERROR: I can't play chess without White King.\n");
      exit(EXIT_FAILURE);
    }
  if (pos->bKSq == 64) 
    {
      printf("ERROR: I can't play chess without Black King.\n");
      exit(EXIT_FAILURE);
    }

  thisPlyData->key ^= others->wCanCastleKey[thisPlyData->wCanCastle];
  thisPlyData->key ^= others->bCanCastleKey[thisPlyData->bCanCastle];
 
  pos->WBoard = pos->WPBoard | pos->WNBoard | pos->WBBoard | 
                  pos->WRBoard | pos->WQBoard | OneMask[pos->wKSq];
  pos->BBoard = pos->BPBoard | pos->BNBoard | pos->BBBoard | 
                  pos->BRBoard | pos->BQBoard | OneMask[pos->bKSq];
  
  others->moveNumber = others->repHead = 0;
  others->Flags &= NilMask[1];    /* not forced */
  others->Flags &= NilMask[2];    /* game has not been listed yet */
  others->Flags &= NilMask[4];    /* search not terminated */
  others->movesPerTC = 40;
  /* default: 5 minutes per game */
  others->timeLeft0 = others->timeLeft = 30000;
  timePerMove = others->timeLeft/40;
  others->oTimeLeft = others->timeLeft - timePerMove;
  others->timeInc = 0;                        /* default: no time increment */
  others->deltaRating = 0;
}


/* Initialize the position boards */
void initPosBoard(sPos *pos, sPlyData *thisPlyData, sOthers *others)
{
  ui sq, piece[64];

  piece[A1] = wR;
  piece[B1] = wN;
  piece[C1] = wB;
  piece[D1] = wQ;
  piece[E1] = wK;
  piece[F1] = wB;
  piece[G1] = wN;
  piece[H1] = wR;
  for (sq = A2; sq <=H2; sq++) piece[sq] = wP;
  for (sq = A3; sq <=H6; sq++) piece[sq] = none;
  for (sq = A7; sq <=H7; sq++) piece[sq] = bP;
  piece[A8] = bR;
  piece[B8] = bN;
  piece[C8] = bB;
  piece[D8] = bQ;
  piece[E8] = bK;
  piece[F8] = bB;
  piece[G8] = bN;
  piece[H8] = bR;

#ifdef VERIFY
  for (sq = A1; sq <= H8; sq++) thisPlyData->piece[sq] = piece[sq];
#endif

  thisPlyData->wCanCastle = thisPlyData->bCanCastle = 3;
  thisPlyData->EP = 0;
  thisPlyData->wToMove = true;

  finalizeInit(piece, pos, thisPlyData, others);
}


#ifdef VERIFY
/* Scan the board to check position boards */
BB scanBoard(ui *piece, BB EP, int wCanCastle, int bCanCastle,
	       sPos *pos, sOthers *others)
{
  BB key = 0;
  ui sq;

  pos->WPBoard = pos->WNBoard = pos->WBBoard = 
    pos->WRBoard = pos->WQBoard = 0;
  pos->BPBoard = pos->BNBoard = pos->BBBoard = 
    pos->BRBoard = pos->BQBoard = 0;
  pos->RotatedBoard90 = pos->RotatedBoard45l = 
    pos->RotatedBoard45r = 0;

  pos->wPCount[0] = pos->wPCount[1] = pos->wPCount[2] = 
  pos->wPCount[3] = pos->wPCount[4] = pos->wPCount[5] = 
  pos->wPCount[6] = pos->wPCount[7] = 
  pos->bPCount[0] = pos->bPCount[1] = pos->bPCount[2] = 
  pos->bPCount[3] = pos->bPCount[4] = pos->bPCount[5] = 
  pos->bPCount[6] = pos->bPCount[7] = 0;
  for (sq = A1; sq <= H8; sq++)
    {
      switch (piece[sq])
	{
	case wP:
	  pos->WPBoard |= OneMask[sq];
	  key ^= others->wPKey[sq];
	  pos->wPCount[column(sq)]++;
	  break;
	case wN:
	  pos->WNBoard |= OneMask[sq];
	  key ^= others->wNKey[sq];
	  break;
	case wB:
	  pos->WBBoard |= OneMask[sq];
	  key ^= others->wBKey[sq];
	  break;
	case wR:
	  pos->WRBoard |= OneMask[sq];
	  key ^= others->wRKey[sq];
	  break;
	case wQ:
	  pos->WQBoard |= OneMask[sq];
	  key ^= others->wQKey[sq];
	  break;
	case wK:
	  pos->wKSq = sq;
	  key ^= others->wKKey[sq];
	  break;
	case bP:
	  pos->BPBoard |= OneMask[sq];
	  key ^= others->bPKey[sq];
	  pos->bPCount[column(sq)]++;
	  break;
	case bN:
	  pos->BNBoard |= OneMask[sq];
	  key ^= others->bNKey[sq];
	  break;
	case bB:
	  pos->BBBoard |= OneMask[sq];
	  key ^= others->bBKey[sq];
	  break;
	case bR:
	  pos->BRBoard |= OneMask[sq];
	  key ^= others->bRKey[sq];
	  break;
	case bQ:
	  pos->BQBoard |= OneMask[sq];
	  key ^= others->bQKey[sq];
	  break;
	case bK:
	  pos->bKSq = sq;
	  key ^= others->bKKey[sq];
	  break;
	}

      if (piece[sq] != none)
	{
	  pos->RotatedBoard90 |= OneMask[rotate90[sq]];
	  pos->RotatedBoard45l |= OneMask[rotate45l[sq]];
	  pos->RotatedBoard45r |= OneMask[rotate45r[sq]];
	}
    }

  if (EP) key ^= others->ePKey[firstOne(EP)];
 if (wCanCastle != 13)
    {
      if (wCanCastle > 0) 
	key ^= others->wCanCastleKey[wCanCastle];
      else key ^= others->wCanCastleKey[0];
      if (bCanCastle > 0) 
	key ^= others->bCanCastleKey[bCanCastle];
      else key ^= others->bCanCastleKey[0];
    }

  pos->WBoard = pos->WPBoard | pos->WNBoard | pos->WBBoard | 
    pos->WRBoard | pos->WQBoard | OneMask[pos->wKSq];
  pos->BBoard = pos->BPBoard | pos->BNBoard | pos->BBBoard | 
    pos->BRBoard | pos->BQBoard | OneMask[pos->bKSq];

  return key;
}
#endif


/* Read the piece square tables */
void readSqvalue(sOthers *others)
{
  int sq;
  char line[80];
  FILE *fdin;

  if ((fdin = fopen("sqvalue", "r")) == NULL)
    {
      printf("Playing without a square value table.\n");
      return;
    }

  /* wP */
  fgets(line, 80, fdin);
  for(sq = A8; sq <= H8; sq++) fscanf(fdin, "%d", &others->wPSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A7; sq <= H7; sq++) fscanf(fdin, "%d", &others->wPSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A6; sq <= H6; sq++) fscanf(fdin, "%d", &others->wPSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A5; sq <= H5; sq++) fscanf(fdin, "%d", &others->wPSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A4; sq <= H4; sq++) fscanf(fdin, "%d", &others->wPSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A3; sq <= H3; sq++) fscanf(fdin, "%d", &others->wPSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A2; sq <= H2; sq++) fscanf(fdin, "%d", &others->wPSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A1; sq <= H1; sq++) fscanf(fdin, "%d", &others->wPSqvalue[sq]);
  fgets(line, 80, fdin);
  fgets(line, 80, fdin);

  /* wN */
  fgets(line, 80, fdin);
  for(sq = A8; sq <= H8; sq++) fscanf(fdin, "%d", &others->wNSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A7; sq <= H7; sq++) fscanf(fdin, "%d", &others->wNSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A6; sq <= H6; sq++) fscanf(fdin, "%d", &others->wNSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A5; sq <= H5; sq++) fscanf(fdin, "%d", &others->wNSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A4; sq <= H4; sq++) fscanf(fdin, "%d", &others->wNSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A3; sq <= H3; sq++) fscanf(fdin, "%d", &others->wNSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A2; sq <= H2; sq++) fscanf(fdin, "%d", &others->wNSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A1; sq <= H1; sq++) fscanf(fdin, "%d", &others->wNSqvalue[sq]);
  fgets(line, 80, fdin);
  fgets(line, 80, fdin);

  /* wB */
  fgets(line, 80, fdin);
  for(sq = A8; sq <= H8; sq++) fscanf(fdin, "%d", &others->wBSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A7; sq <= H7; sq++) fscanf(fdin, "%d", &others->wBSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A6; sq <= H6; sq++) fscanf(fdin, "%d", &others->wBSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A5; sq <= H5; sq++) fscanf(fdin, "%d", &others->wBSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A4; sq <= H4; sq++) fscanf(fdin, "%d", &others->wBSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A3; sq <= H3; sq++) fscanf(fdin, "%d", &others->wBSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A2; sq <= H2; sq++) fscanf(fdin, "%d", &others->wBSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A1; sq <= H1; sq++) fscanf(fdin, "%d", &others->wBSqvalue[sq]);
  fgets(line, 80, fdin);
  fgets(line, 80, fdin);

  /* wR */
  fgets(line, 80, fdin);
  for(sq = A8; sq <= H8; sq++) fscanf(fdin, "%d", &others->wRSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A7; sq <= H7; sq++) fscanf(fdin, "%d", &others->wRSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A6; sq <= H6; sq++) fscanf(fdin, "%d", &others->wRSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A5; sq <= H5; sq++) fscanf(fdin, "%d", &others->wRSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A4; sq <= H4; sq++) fscanf(fdin, "%d", &others->wRSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A3; sq <= H3; sq++) fscanf(fdin, "%d", &others->wRSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A2; sq <= H2; sq++) fscanf(fdin, "%d", &others->wRSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A1; sq <= H1; sq++) fscanf(fdin, "%d", &others->wRSqvalue[sq]);
  fgets(line, 80, fdin);
  fgets(line, 80, fdin);

  /* wQ */
  fgets(line, 80, fdin);
  for(sq = A8; sq <= H8; sq++) fscanf(fdin, "%d", &others->wQSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A7; sq <= H7; sq++) fscanf(fdin, "%d", &others->wQSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A6; sq <= H6; sq++) fscanf(fdin, "%d", &others->wQSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A5; sq <= H5; sq++) fscanf(fdin, "%d", &others->wQSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A4; sq <= H4; sq++) fscanf(fdin, "%d", &others->wQSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A3; sq <= H3; sq++) fscanf(fdin, "%d", &others->wQSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A2; sq <= H2; sq++) fscanf(fdin, "%d", &others->wQSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A1; sq <= H1; sq++) fscanf(fdin, "%d", &others->wQSqvalue[sq]);
  fgets(line, 80, fdin);
  fgets(line, 80, fdin);

  /* wK */
  fgets(line, 80, fdin);
  for(sq = A8; sq <= H8; sq++) fscanf(fdin, "%d", &others->wKSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A7; sq <= H7; sq++) fscanf(fdin, "%d", &others->wKSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A6; sq <= H6; sq++) fscanf(fdin, "%d", &others->wKSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A5; sq <= H5; sq++) fscanf(fdin, "%d", &others->wKSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A4; sq <= H4; sq++) fscanf(fdin, "%d", &others->wKSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A3; sq <= H3; sq++) fscanf(fdin, "%d", &others->wKSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A2; sq <= H2; sq++) fscanf(fdin, "%d", &others->wKSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A1; sq <= H1; sq++) fscanf(fdin, "%d", &others->wKSqvalue[sq]);
  fgets(line, 80, fdin);
  fgets(line, 80, fdin);

  /* bP */
  fgets(line, 80, fdin);
  for(sq = A8; sq <= H8; sq++) fscanf(fdin, "%d", &others->bPSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A7; sq <= H7; sq++) fscanf(fdin, "%d", &others->bPSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A6; sq <= H6; sq++) fscanf(fdin, "%d", &others->bPSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A5; sq <= H5; sq++) fscanf(fdin, "%d", &others->bPSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A4; sq <= H4; sq++) fscanf(fdin, "%d", &others->bPSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A3; sq <= H3; sq++) fscanf(fdin, "%d", &others->bPSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A2; sq <= H2; sq++) fscanf(fdin, "%d", &others->bPSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A1; sq <= H1; sq++) fscanf(fdin, "%d", &others->bPSqvalue[sq]);
  fgets(line, 80, fdin);
  fgets(line, 80, fdin);

  /* bN */
  fgets(line, 80, fdin);
  for(sq = A8; sq <= H8; sq++) fscanf(fdin, "%d", &others->bNSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A7; sq <= H7; sq++) fscanf(fdin, "%d", &others->bNSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A6; sq <= H6; sq++) fscanf(fdin, "%d", &others->bNSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A5; sq <= H5; sq++) fscanf(fdin, "%d", &others->bNSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A4; sq <= H4; sq++) fscanf(fdin, "%d", &others->bNSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A3; sq <= H3; sq++) fscanf(fdin, "%d", &others->bNSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A2; sq <= H2; sq++) fscanf(fdin, "%d", &others->bNSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A1; sq <= H1; sq++) fscanf(fdin, "%d", &others->bNSqvalue[sq]);
  fgets(line, 80, fdin);
  fgets(line, 80, fdin);

  /* bB */
  fgets(line, 80, fdin);
  for(sq = A8; sq <= H8; sq++) fscanf(fdin, "%d", &others->bBSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A7; sq <= H7; sq++) fscanf(fdin, "%d", &others->bBSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A6; sq <= H6; sq++) fscanf(fdin, "%d", &others->bBSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A5; sq <= H5; sq++) fscanf(fdin, "%d", &others->bBSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A4; sq <= H4; sq++) fscanf(fdin, "%d", &others->bBSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A3; sq <= H3; sq++) fscanf(fdin, "%d", &others->bBSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A2; sq <= H2; sq++) fscanf(fdin, "%d", &others->bBSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A1; sq <= H1; sq++) fscanf(fdin, "%d", &others->bBSqvalue[sq]);
  fgets(line, 80, fdin);
  fgets(line, 80, fdin);

  /* bR */
  fgets(line, 80, fdin);
  for(sq = A8; sq <= H8; sq++) fscanf(fdin, "%d", &others->bRSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A7; sq <= H7; sq++) fscanf(fdin, "%d", &others->bRSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A6; sq <= H6; sq++) fscanf(fdin, "%d", &others->bRSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A5; sq <= H5; sq++) fscanf(fdin, "%d", &others->bRSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A4; sq <= H4; sq++) fscanf(fdin, "%d", &others->bRSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A3; sq <= H3; sq++) fscanf(fdin, "%d", &others->bRSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A2; sq <= H2; sq++) fscanf(fdin, "%d", &others->bRSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A1; sq <= H1; sq++) fscanf(fdin, "%d", &others->bRSqvalue[sq]);
  fgets(line, 80, fdin);
  fgets(line, 80, fdin);

  /* bQ */
  fgets(line, 80, fdin);
  for(sq = A8; sq <= H8; sq++) fscanf(fdin, "%d", &others->bQSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A7; sq <= H7; sq++) fscanf(fdin, "%d", &others->bQSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A6; sq <= H6; sq++) fscanf(fdin, "%d", &others->bQSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A5; sq <= H5; sq++) fscanf(fdin, "%d", &others->bQSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A4; sq <= H4; sq++) fscanf(fdin, "%d", &others->bQSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A3; sq <= H3; sq++) fscanf(fdin, "%d", &others->bQSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A2; sq <= H2; sq++) fscanf(fdin, "%d", &others->bQSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A1; sq <= H1; sq++) fscanf(fdin, "%d", &others->bQSqvalue[sq]);
  fgets(line, 80, fdin);
  fgets(line, 80, fdin);

  /* bK */
  fgets(line, 80, fdin);
  for(sq = A8; sq <= H8; sq++) fscanf(fdin, "%d", &others->bKSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A7; sq <= H7; sq++) fscanf(fdin, "%d", &others->bKSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A6; sq <= H6; sq++) fscanf(fdin, "%d", &others->bKSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A5; sq <= H5; sq++) fscanf(fdin, "%d", &others->bKSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A4; sq <= H4; sq++) fscanf(fdin, "%d", &others->bKSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A3; sq <= H3; sq++) fscanf(fdin, "%d", &others->bKSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A2; sq <= H2; sq++) fscanf(fdin, "%d", &others->bKSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A1; sq <= H1; sq++) fscanf(fdin, "%d", &others->bKSqvalue[sq]);
  fgets(line, 80, fdin);
  fgets(line, 80, fdin);

  /* wPassedP */
  fgets(line, 80, fdin);
  for(sq = A8; sq <= H8; sq++) fscanf(fdin, "%d", &others->wPassedPSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A7; sq <= H7; sq++) fscanf(fdin, "%d", &others->wPassedPSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A6; sq <= H6; sq++) fscanf(fdin, "%d", &others->wPassedPSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A5; sq <= H5; sq++) fscanf(fdin, "%d", &others->wPassedPSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A4; sq <= H4; sq++) fscanf(fdin, "%d", &others->wPassedPSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A3; sq <= H3; sq++) fscanf(fdin, "%d", &others->wPassedPSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A2; sq <= H2; sq++) fscanf(fdin, "%d", &others->wPassedPSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A1; sq <= H1; sq++) fscanf(fdin, "%d", &others->wPassedPSqvalue[sq]);
  fgets(line, 80, fdin);
  fgets(line, 80, fdin);

  /* bPassedP */
  fgets(line, 80, fdin);
  for(sq = A8; sq <= H8; sq++) fscanf(fdin, "%d", &others->bPassedPSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A7; sq <= H7; sq++) fscanf(fdin, "%d", &others->bPassedPSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A6; sq <= H6; sq++) fscanf(fdin, "%d", &others->bPassedPSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A5; sq <= H5; sq++) fscanf(fdin, "%d", &others->bPassedPSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A4; sq <= H4; sq++) fscanf(fdin, "%d", &others->bPassedPSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A3; sq <= H3; sq++) fscanf(fdin, "%d", &others->bPassedPSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A2; sq <= H2; sq++) fscanf(fdin, "%d", &others->bPassedPSqvalue[sq]);
  fgets(line, 80, fdin);
  for(sq = A1; sq <= H1; sq++) fscanf(fdin, "%d", &others->bPassedPSqvalue[sq]);
  fgets(line, 80, fdin);
  fgets(line, 80, fdin);

  fclose(fdin);
}


/* Initialize */
void init(sPos *pos, sPlyData *plyData, sHash **hash, sOthers *others)
{
  time_t when;
  struct tm *timeStr;

  others->failBook = others->openingNo = 0;
  others->posID[0] = others->solution[0] = others->myMove[0] = 
    '\0';
  others->gameID[0] = '\0';
  /* today's date */
  time(&when);
  timeStr = localtime(&when);
  strftime(others->fullDate, 16, "%m%d%y_%H:%M:%S", timeStr);
  others->searchDepth = maxPly;

  initFLOne();
  initBitBoards(others);
  initRankAttacks();
  initKeys(others);
  clearHash(hash, others);
  initPosBoard(pos, &plyData[0], others);
  readSqvalue(others);
  setRandomSeed((unsigned int) time ((time_t *) 0));
  others->Flags |= OneMask[3];    /* randomize by default */
  others->rOpening = 1 + random32() % 3;  /* choose an all purpose opening */
}
/* move.c */
/* Last modified: June 4, 2006 */


static ui pieceAtSq(ui sq, sPos *pos)
{
  BB Sq;

  ASSERT(sq <= H8);
  ASSERT(pos != NULL);
  ASSERT(!(pos->WBoard & pos->BBoard));

  Sq = OneMask[sq];
  if (Sq & pos->WPBoard) return wP;
  if (Sq & pos->BPBoard) return bP;
  if (Sq & pos->WNBoard) return wN;
  if (Sq & pos->BNBoard) return bN;
  if (Sq & pos->WBBoard) return wB;
  if (Sq & pos->BBBoard) return bB;
  if (Sq & pos->WRBoard) return wR;
  if (Sq & pos->BRBoard) return bR;
  if (Sq & pos->WQBoard) return wQ;
  if (Sq & pos->BQBoard) return bQ;

  if (sq == pos->wKSq) return wK;
  if (sq == pos->bKSq) return bK;

  return none;
}


ui testMove(ui ply, BB move, sPos *pos, sPlyData *plyData, sOthers *others)
{
  ui notOK;

  ASSERT(ply <= maxPly);
  ASSERT(from(move) <= H8 && to(move) <= H8 &&
	 from(move) != to(move));
  ASSERT(piece(move) < 13);
  ASSERT(pos != NULL);
  ASSERT(plyData != NULL);
  ASSERT(others != NULL);
  ASSERT(!(pos->WBoard & pos->BBoard));

  makeMove(ply, move, pos, plyData, others);
  if (plyData[ply-1].wToMove) 
    notOK = (getBAttacker(pos->wKSq, pos, &plyData[ply], others) & 2);
  else
    notOK = (getWAttacker(pos->bKSq, pos, &plyData[ply], others) & 2);
  unmakeMove(ply, move, pos, plyData);

  return !notOK;
}


/* Use see() (Static Exchange Evaluation) */
ui captures(ui ply, BB *moveList, int quiescent, sPos *pos, sPlyData *plyData,
	    sOthers *others, ui pseudolegal)
{
  BB None, From, To, Them, move;
  ui sq, t;
  ui c;
  int tValue, mVVLVAValue;
  ui count = 0;

  ASSERT(ply <= maxPly);
  ASSERT(moveList != NULL);
  ASSERT(pos != NULL);
  ASSERT(plyData != NULL);
  ASSERT(others != NULL);
  ASSERT(!(pos->WBoard & pos->BBoard));

  None = ~(pos->WBoard | pos->BBoard);

  *moveList = 0;

  if (plyData[ply].wToMove == true)
    {
      Them = pos->BBoard;

      /* pawn */
      if (pos->WPBoard)
	{
	  From = pos->WPBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      if (OneMask[sq] & others->Row[6])   /* promotions */
		{
		  if (OneMask[sq+8] & None)
		    {
		      move = sq | ((sq+8) << 6) | (wP << 12);
		      if (pseudolegal || testMove(ply+1, move, pos, plyData, 
						  others))
			{
			  mVVLVAValue = qValue - pValue;
			  count++;
			  *moveList++ = move | (wQ << 24) | 
			    ((BB)(mVVLVAValue+mateScore) << 30);

			  if (!quiescent)
			    {
			      mVVLVAValue = rValue - pValue;
			      count++;
			      *moveList++ = move | (wR << 24) |
				((BB)(mVVLVAValue+mateScore) << 30);
			      
			      mVVLVAValue = bValue - pValue;
			      count++;
			      *moveList++ = move | (wB << 24) |
				((BB)(mVVLVAValue+mateScore) << 30);

			      mVVLVAValue = nValue - pValue;
			      count++;
			      *moveList++ = move | (wN << 24) |
				((BB)(mVVLVAValue+mateScore) << 30);
			    }
			}
		    }
		}
	      To = others->WPCaptures[sq] & Them;
	      while (To)
		{
		  t = lastOne(To);
		  c = pieceAtSq(t, pos);
		  ASSERT(c < 13);
		  tValue = see(1, sq, t, pos, others);
#ifdef VERIFY
		  if (c == bK) 
		    {
		      printf("bK is captured by wP in captures().\n");
		      exit(EXIT_FAILURE);
		    }
#endif
		  move = sq | (t << 6) | (wP << 12) | (c << 18);
		  if (pseudolegal || testMove(ply+1, move, pos, plyData, 
					      others))
		    {
		      if (OneMask[sq] & others->Row[6])   /* promotions */
			{
			  mVVLVAValue = qValue - pValue + tValue;
			  count++;
			  *moveList++ = move | (wQ << 24) |
			    ((BB)(mVVLVAValue+mateScore) << 30);

			  if (!quiescent)
			    {
			      mVVLVAValue = rValue - pValue + tValue;
			      count++;
			      *moveList++ = move | (wR << 24) |
				((BB)(mVVLVAValue+mateScore) << 30);

			      mVVLVAValue = bValue - pValue + tValue;
			      count++;
			      *moveList++ = move | (wB << 24) |
				((BB)(mVVLVAValue+mateScore) << 30);

			      mVVLVAValue = nValue - pValue + tValue;
			      count++;
			      *moveList++ = move | (wN << 24) |
				((BB)(mVVLVAValue+mateScore) << 30);
			    }
			}
		      else
			{
			  count++;
			  *moveList++ = move |
			    ((BB)(tValue+mateScore) << 30);
			}
		    }
		  To &= NilMask[t];
		}
	      /* en-passant */
	      if (plyData[ply].EP & others->WPCaptures[sq])
		{
		  move = sq | (firstOne(plyData[ply].EP) << 6)| (wP << 12);
		  if (pseudolegal || 
		      testMove(ply+1, move, pos, plyData, others)) 
		    {
		      mVVLVAValue = pValue;
		      count++;
		      *moveList++ = move |
			((BB)(mVVLVAValue+mateScore) << 30);
		    }
		}
	      From &= NilMask[sq];
	    }
	}

      /* knight */
      if (pos->WNBoard)
	{
	  From = pos->WNBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      To = others->NMoves[sq] & Them;
	      while (To)
		{
		  t = lastOne(To);
		  c = pieceAtSq(t, pos);
		  ASSERT(c < 13);
		  tValue = see(1, sq, t, pos, others);
#ifdef VERIFY
		  if (c == bK) 
		    {
		      printf("bK is captured by wN in captures().\n");
		      exit(EXIT_FAILURE);
		    }
#endif
		  move = sq | (t << 6) | (wN << 12) | (c << 18);
		  if (pseudolegal || testMove(ply+1, move, pos, plyData, 
					      others))
		    {
		      count++;
		      *moveList++ = move |
			((BB)(tValue+mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* bishop */
      if (pos->WBBoard)
	{
	  From = pos->WBBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      To = bishopAttacks(sq, pos) & Them;
	      while (To)
		{
		  t = lastOne(To);
		  c = pieceAtSq(t, pos);
		  ASSERT(c < 13);
		  tValue = see(1, sq, t, pos, others);
#ifdef VERIFY
		  if (c == bK) 
		    {
		      printf("bK is captured by wB in captures().\n");
		      exit(EXIT_FAILURE);
		    }
#endif
		  move = sq | (t << 6) | (wB << 12) | (c << 18);
		  if (pseudolegal || testMove(ply+1, move, pos, plyData, 
					      others))
		    {
		      count++;
		      *moveList++ = move |
			((BB)(tValue+mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* rook */
      if (pos->WRBoard)
	{
	  From = pos->WRBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      To = rookAttacks(sq, pos) & Them;
	      while (To)
		{
		  t = lastOne(To);
		  c = pieceAtSq(t, pos);
		  ASSERT(c < 13);
		  tValue = see(1, sq, t, pos, others);
#ifdef VERIFY
		  if (c == bK) 
		    {
		      printf("bK is captured by wR in captures().\n");
		      exit(EXIT_FAILURE);
		    }
#endif
		  move = sq | (t << 6) | (wR << 12) | (c << 18);
		  if (pseudolegal || testMove(ply+1, move, pos, plyData, 
					      others))
		    {
		      count++;
		      *moveList++ = move |
			((BB)(tValue+mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* queen */
      if (pos->WQBoard)
	{
	  From = pos->WQBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      To = (rookAttacks(sq, pos) | 
		    bishopAttacks(sq, pos)) 
		& Them;
	      while (To)
		{
		  t = lastOne(To);
		  c = pieceAtSq(t, pos);
		  ASSERT(c < 13);
		  tValue = see(1, sq, t, pos, others);
#ifdef VERIFY
		  if (c == bK) 
		    {
		      printf("bK is captured by wQ in captures().\n");
		      exit(EXIT_FAILURE);
		    }
#endif
		  move = sq | (t << 6) | (wQ << 12) | (c << 18);
		  if (pseudolegal || testMove(ply+1, move, pos, plyData, 
					      others))
		    {
		      count++;
		      *moveList++ = move |
			((BB)(tValue+mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* king */
      /* There is one king, no need to if or loop it */
      sq = pos->wKSq;
      To = others->KMoves[sq] & Them;
      while (To)
	{
	  t = lastOne(To);
	  c = pieceAtSq(t, pos);
	  ASSERT(c < 13);
	  tValue = value[c];
#ifdef VERIFY
	  if (c == bK) 
	    {
	      printf("bK is captured by wK in captures().\n");
	      exit(EXIT_FAILURE);
	    }
#endif
	  move = sq | (t << 6) | (wK << 12) | (c << 18);
	  if (pseudolegal || testMove(ply+1, move, pos, plyData, others))
	    {
	      count++;
	      *moveList++ = move | 
		((BB)(tValue+mateScore) << 30);
	    }
	  To &= NilMask[t];
	}
    }
  else
    {
      Them = pos->WBoard;

      /* pawn */
      if (pos->BPBoard)
	{
	  From = pos->BPBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      if (OneMask[sq] & others->Row[1])   /* promotions */
		{
		  if (OneMask[sq-8] & None)
		    {
		      move = sq | ((sq-8) << 6) | (bP << 12);
		      if (pseudolegal || testMove(ply+1, move, pos, plyData, 
						  others))
			{
			  mVVLVAValue = qValue - pValue;
			  count++;
			  *moveList++ = move | (bQ << 24) |
			    ((BB)(mVVLVAValue+mateScore) << 30);

			  if (!quiescent)
			    {
			      mVVLVAValue = rValue - pValue;
			      count++;
			      *moveList++ = move | (bR << 24) |
				((BB)(mVVLVAValue+mateScore) << 30);
			      
			      mVVLVAValue = bValue - pValue;
			      count++;
			      *moveList++ = move | (bB << 24) |
				((BB)(mVVLVAValue+mateScore) << 30);

			      mVVLVAValue = nValue - pValue;
			      count++;
			      *moveList++ = move | (bN << 24) |
				((BB)(mVVLVAValue+mateScore) << 30);
			    }
			}
		    }
		}
	      To = others->BPCaptures[sq] & Them;
	      while (To)
		{
		  t = firstOne(To);
		  c = pieceAtSq(t, pos);
		  ASSERT(c < 13);
		  tValue = see(0, sq, t, pos, others);
#ifdef VERIFY
		  if (c == wK) 
		    {
		      printf("wK is captured by bP in captures().\n");
		      exit(EXIT_FAILURE);
		    }
#endif
		  move = sq | (t << 6) | (bP << 12) | (c << 18);
		  if (pseudolegal || testMove(ply+1, move, pos, plyData, 
					      others)) 
		    {
		      if (OneMask[sq] & others->Row[1])   /* promotions */
			{
			  mVVLVAValue = qValue - pValue + tValue;
			  count++;
			  *moveList++ = move | (bQ << 24) |
			    ((BB)(mVVLVAValue+mateScore) << 30);

			  if (!quiescent)
			    {
			      mVVLVAValue = rValue - pValue + tValue;
			      count++;
			      *moveList++ = move | (bR << 24) |
				((BB)(mVVLVAValue+mateScore) << 30);
			      
			      mVVLVAValue = bValue - pValue + tValue;
			      count++;
			      *moveList++ = move | (bB << 24) |
				((BB)(mVVLVAValue+mateScore) << 30);

			      mVVLVAValue = nValue - pValue + tValue;
			      count++;
			      *moveList++ = move | (bN << 24) |
				((BB)(mVVLVAValue+mateScore) << 30);
			    }
			}
		      else
			{
			  count++;
			  *moveList++ = move |
			    ((BB)(tValue+mateScore) << 30);
			}
		    }
		  To &= NilMask[t];
		}
	      /* en-passant */
	      if (plyData[ply].EP & others->BPCaptures[sq])
		{
		  move = sq | (firstOne(plyData[ply].EP) << 6) | (bP << 12);
		  if (pseudolegal || 
		      testMove(ply+1, move, pos, plyData, others)) 
		    {
		      mVVLVAValue = pValue;
		      count++;
		      *moveList++ = move |
			((BB)(mVVLVAValue+mateScore) << 30);
		    }
		}
	      From &= NilMask[sq];
	    }
	}

      /* knight */
      if (pos->BNBoard)
	{
	  From = pos->BNBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      To = others->NMoves[sq] & Them;
	      while (To)
		{
		  t = firstOne(To);
		  c = pieceAtSq(t, pos);
		  ASSERT(c < 13);
		  tValue = see(0, sq, t, pos, others);
#ifdef VERIFY
		  if (c == wK) 
		    {
		      printf("wK is captured by bN in captures().\n");
		      exit(EXIT_FAILURE);
		    }
#endif
		  move = sq | (t << 6) | (bN << 12) | (c << 18);
		  if (pseudolegal || testMove(ply+1, move, pos, plyData, 
					      others)) 
		    {
		      count++;
		      *moveList++ = move |
			((BB)(tValue+mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* bishop */
      if (pos->BBBoard)
	{
	  From = pos->BBBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      To = bishopAttacks(sq, pos) & Them;
	      while (To)
		{
		  t = firstOne(To);
		  c = pieceAtSq(t, pos);
		  ASSERT(c < 13);
		  tValue = see(0, sq, t, pos, others);
#ifdef VERIFY
		  if (c == wK) 
		    {
		      printf("wK is captured by bB in captures().\n");
		      exit(EXIT_FAILURE);
		    }
#endif
		  move = sq | (t << 6) | (bB << 12) | (c << 18);
		  if (pseudolegal || testMove(ply+1, move, pos, plyData, 
					      others)) 
		    {
		      count++;
		      *moveList++ = move |
			((BB)(tValue+mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* rook */
      if (pos->BRBoard)
	{
	  From = pos->BRBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      To = rookAttacks(sq, pos) & Them;
	      while (To)
		{
		  t = firstOne(To);
		  c = pieceAtSq(t, pos);
		  ASSERT(c < 13);
		  tValue = see(0, sq, t, pos, others);
#ifdef VERIFY
		  if (c == wK) 
		    {
		      printf("wK is captured by bR in captures().\n");
		      exit(EXIT_FAILURE);
		    }
#endif
		  move = sq | (t << 6) | (bR << 12) | (c << 18);
		  if (pseudolegal || testMove(ply+1, move, pos, plyData, 
					      others)) 
		    {
		      count++;
		      *moveList++ = move |
			((BB)(tValue+mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* queen */
      if (pos->BQBoard)
	{
	  From = pos->BQBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      To = (rookAttacks(sq, pos) | 
		    bishopAttacks(sq, pos)) 
		& Them;
	      while (To)
		{
		  t = firstOne(To);
		  c = pieceAtSq(t, pos);
		  ASSERT(c < 13);
		  tValue = see(0, sq, t, pos, others);
#ifdef VERIFY
		  if (c == wK) 
		    {
		      printf("wK is captured by bQ in captures().\n");
		      exit(EXIT_FAILURE);
		    }
#endif
		  move = sq | (t << 6) | (bQ << 12) | (c << 18);
		  if (pseudolegal || testMove(ply+1, move, pos, plyData, 
					      others))
		    {
		      count++;
		      *moveList++ = move |
			((BB)(tValue+mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* king */
      /* There is one king, no need to if or loop it */
      sq = pos->bKSq;
      To = others->KMoves[sq] & Them;
      while (To)
	{
	  t = lastOne(To);
	  c = pieceAtSq(t, pos);
	  ASSERT(c < 13);
	  tValue = value[c];
#ifdef VERIFY
	  if (c == wK)
	    {
	      printf("wK is captured by bK in captures().\n");
	      exit(EXIT_FAILURE);
	    }
#endif
	  move = sq | (t << 6) | (bK << 12) | (c << 18);
	  if (pseudolegal || testMove(ply+1, move, pos, plyData, others))
	    {
	      count++;
	      *moveList++ = move |
		((BB)(tValue+mateScore) << 30);
	    }
	  To &= NilMask[t];
	}
    }

  return count;
}


ui nonCaptures(ui ply, BB *moveList, sPos *pos, sPlyData *plyData, 
	       sOthers *others, ui pseudolegal)
{
  BB None, From, To, Them, move;
  ui sq, t;
  ui count = 0;

  ASSERT(ply <= maxPly);
  ASSERT(moveList != NULL);
  ASSERT(pos != NULL);
  ASSERT(plyData != NULL);
  ASSERT(others != NULL);
  ASSERT(!(pos->WBoard & pos->BBoard));

  None = ~(pos->WBoard | pos->BBoard);

  *moveList = 0;

  if (plyData[ply].wToMove == true)
    {
      Them = ~(pos->WBoard | pos->BBoard);

      /* pawn */
      if (pos->WPBoard)
	{
	  From = pos->WPBoard & ~others->Row[6];        /* not a promotion */
	  while (From)
	    {
	      sq = firstOne(From);
	      if (OneMask[sq+8] & Them)
		{
		  t = sq+8;
		  move = sq | (t << 6) | (wP << 12);
		  if (pseudolegal || testMove(ply+1, move, pos, plyData, 
					      others)) 
		    {
		      count++;
		      *moveList++ = move |
			((BB)(mateScore) << 30);
		    }
		  if ((OneMask[sq] & others->Row[1]) && (OneMask[sq+16] & Them))
		    {
		      t = sq+16;
		      move = sq | (t << 6) | (wP << 12);
		      if (pseudolegal || testMove(ply+1, move, pos, plyData, 
						  others))
			{
			  count++;
			  *moveList++ = move |
			    ((BB)(mateScore) << 30);
			}
		    }
		}
	      From &= NilMask[sq];
	    }
	}

      /* knight */
      if (pos->WNBoard)
	{
	  From = pos->WNBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      To = others->NMoves[sq] & Them;
	      while (To)
		{
		  t = lastOne(To);
		  move = sq | (t << 6) | (wN << 12);
		  if (pseudolegal || testMove(ply+1, move, pos, plyData, 
					      others))
		    {
		      count++;
		      *moveList++ = move |
			((BB)(mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* rook */
      if (pos->WRBoard)
	{
	  From = pos->WRBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      To = rookAttacks(sq, pos) & Them;
	      while (To)
		{
		  t = lastOne(To);
		  move = sq | (t << 6) | (wR << 12);
		  if (pseudolegal || testMove(ply+1, move, pos, plyData, 
					      others))
		    {
		      count++;
		      *moveList++ = move |
			((BB)(mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* bishop */
      if (pos->WBBoard)
	{
	  From = pos->WBBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      To = bishopAttacks(sq, pos) & Them;
	      while (To)
		{
		  t = lastOne(To);
		  move = sq | (t << 6) | (wB << 12);
		  if (pseudolegal || testMove(ply+1, move, pos, plyData, 
					      others))
		    {
		      count++;
		      *moveList++ = move |
			((BB)(mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* queen */
      if (pos->WQBoard)
	{
	  From = pos->WQBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      To = (rookAttacks(sq, pos) | 
		    bishopAttacks(sq, pos)) 
		& Them;
	      while (To)
		{
		  t = lastOne(To);
		  move = sq | (t << 6) | (wQ << 12);
		  if (pseudolegal || testMove(ply+1, move, pos, plyData, 
					      others))
		    {
		      count++;
		      *moveList++ = move |
			((BB)(mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* king */
      /* There is one king, no need to if or loop it */
      sq = pos->wKSq;

      if (plyData[ply].wCanCastle > 0)
	{
	  /* long castling */
	  if ((plyData[ply].wCanCastle & 2) &&
	      OneMask[A1] & pos->WRBoard &&
	      OneMask[B1] & None &&
	      OneMask[C1] & None &&
	      OneMask[D1] & None &&
	      !(getBAttacker(C1, pos, &plyData[ply], others) & 2) &&
	      !(getBAttacker(D1, pos, &plyData[ply], others) & 2) &&
	      !(getBAttacker(E1, pos, &plyData[ply], others) & 2))
	    {
	      count++;
	      *moveList++ = (BB)(E1 | (C1 << 6) | (wK << 12)) |
		((BB)(mateScore) << 30);
	    }
	  /* short castling */
	  if ((plyData[ply].wCanCastle & 1) &&
	      OneMask[H1] & pos->WRBoard &&
	      OneMask[G1] & None &&
	      OneMask[F1] & None &&
	      !(getBAttacker(G1, pos, &plyData[ply], others) & 2) &&
	      !(getBAttacker(F1, pos, &plyData[ply], others) & 2) &&
	      !(getBAttacker(E1, pos, &plyData[ply], others) & 2))
	    {
	      count++;
	      *moveList++ = (BB)(E1 | (G1 << 6) | (wK << 12)) |
		((BB)(mateScore) << 30);
	    }
	}
      /* Normal move */
      To = others->KMoves[sq] & Them;
      while (To)
	{
	  t = lastOne(To);
	  if (pseudolegal || 
	      !(getBAttacker(t, pos, &plyData[ply], others) & 2))
	    {
	      move = sq | (t << 6) | (wK << 12);
	      if (pseudolegal || testMove(ply+1, move, pos, plyData, others))
		{
		  count++;
		  *moveList++ = move |
		    ((BB)(mateScore) << 30);
		}
	    }
	  To &= NilMask[t];
	}
    }
  else
    {
      Them = ~(pos->WBoard | pos->BBoard);

      /* pawn */
      if (pos->BPBoard)
	{
	  From = pos->BPBoard & ~others->Row[1];        /* not a promotion */
	  while (From)
	    {
	      sq = lastOne(From);
	      if (OneMask[sq-8] & Them)
		{
		  t = sq-8;
		  move = sq | (t << 6) | (bP << 12);
		  if (pseudolegal || testMove(ply+1, move, pos, plyData, 
					      others))
		    {
		      count++;
		      *moveList++ = move |
			((BB)(mateScore) << 30);
		    }
		  if ((OneMask[sq] & others->Row[6]) && 
		      (OneMask[sq-16] & Them))
		    {
		      t = sq-16;
		      move = sq | (t << 6) | (bP << 12);
		      if (pseudolegal || testMove(ply+1, move, pos, plyData, 
						  others))
			{
			  count++;
			  *moveList++ = move |
			    ((BB)(mateScore) << 30);
			}
		    }
		}
	      From &= NilMask[sq];
	    }
	}

      /* knight */
      if (pos->BNBoard)
	{
	  From = pos->BNBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      To = others->NMoves[sq] & Them;
	      while (To)
		{
		  t = firstOne(To);
		  move = sq | (t << 6) | (bN << 12);
		  if (pseudolegal || testMove(ply+1, move, pos, plyData, 
					      others))
		    {
		      count++;
		      *moveList++ = move |
			((BB)(mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* rook */
      if (pos->BRBoard)
	{
	  From = pos->BRBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      To = rookAttacks(sq, pos) & Them;
	      while (To)
		{
		  t = firstOne(To);
		  move = sq | (t << 6) | (bR << 12);
		  if (pseudolegal || testMove(ply+1, move, pos, plyData, 
					      others))
		    {
		      count++;
		      *moveList++ = move |
			((BB)(mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* bishop */
      if (pos->BBBoard)
	{
	  From = pos->BBBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      To = bishopAttacks(sq, pos) & Them;
	      while (To)
		{
		  t = firstOne(To);
		  move = sq | (t << 6) | (bB << 12);
		  if (pseudolegal || testMove(ply+1, move, pos, plyData, 
					      others))
		    {
		      count++;
		      *moveList++ = move |
			((BB)(mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* queen */
      if (pos->BQBoard)
	{
	  From = pos->BQBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      To = (rookAttacks(sq, pos) | 
		    bishopAttacks(sq, pos)) 
		& Them;
	      while (To)
		{
		  t = firstOne(To);
		  move = sq | (t << 6) | (bQ << 12);
		  if (pseudolegal || testMove(ply+1, move, pos, plyData, 
					      others))
		    {
		      count++;
		      *moveList++ = move |
			((BB)(mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* king */
      /* There is one king, no need to if or loop it */
      sq = pos->bKSq;

      if (plyData[ply].bCanCastle > 0)
	{
	  /* long castling */
	  if ((plyData[ply].bCanCastle & 2) &&
	      OneMask[A8] & pos->BRBoard &&
	      OneMask[B8] & None &&
	      OneMask[C8] & None &&
	      OneMask[D8] & None &&
	      !(getWAttacker(C8, pos, &plyData[ply], others) & 2) &&
	      !(getWAttacker(D8, pos, &plyData[ply], others) & 2) &&
	      !(getWAttacker(E8, pos, &plyData[ply], others) & 2))
	    {
	      count++;
	      *moveList++ = (BB)(E8 | (C8 << 6) | (bK << 12)) |
		((BB)(mateScore) << 30);
	    }
	  /* short castling */
	  if ((plyData[ply].bCanCastle & 1) &&
	      OneMask[H8] & pos->BRBoard &&
	      OneMask[G8] & None &&
	      OneMask[F8] & None &&
	      !(getWAttacker(G8, pos, &plyData[ply], others) & 2) &&
	      !(getWAttacker(F8, pos, &plyData[ply], others) & 2) &&
	      !(getWAttacker(E8, pos, &plyData[ply], others) & 2))
	    {
	      count++;
	      *moveList++ = (BB)(E8 | (G8 << 6) | (bK << 12)) |
		((BB)(mateScore) << 30);
	    }
	}
      /* Normal move */
      To = others->KMoves[sq] & Them;
      while (To)
	{
	  t = lastOne(To);
	  if (pseudolegal || 
	      !(getWAttacker(t, pos, &plyData[ply], others) & 2))
	    {
	      move = sq | (t << 6) | (bK << 12);
	      if (pseudolegal || testMove(ply+1, move, pos, plyData, others))
		{
		  count++;
		  *moveList++ = move |
		    ((BB)(mateScore) << 30);
		}
	    }
	  To &= NilMask[t];
	}
    }

  return count;
}


void makeMove(ui ply, BB move, sPos *pos, sPlyData *plyData, sOthers *others)
{
#ifdef VERIFY
  sPos thisPos;
  BB key = 0;
#endif
  ui p, f, t, c;
  char castleFlag = '-';

  BB *pWAttacker = (BB *) plyData[ply].wAttacker;
  BB *pBAttacker = (BB *) plyData[ply].bAttacker;

  ASSERT(ply <= maxPly);
  ASSERT(from(move) <= H8 && to(move) <= H8 &&
	 from(move) != to(move));
  ASSERT(piece(move) < 13);
  ASSERT(pos != NULL);
  ASSERT(plyData != NULL);
  ASSERT(others != NULL);
  ASSERT(!(pos->WBoard & pos->BBoard));

  plyData[ply] = plyData[ply-1];
  plyData[ply].wToMove ^= 1;
  plyData[ply].count50Moves++;
  if (plyData[ply].EP) plyData[ply].key ^= 
			 others->ePKey[firstOne(plyData[ply].EP)];
  plyData[ply].EP = 0;

  /* Attack tables */
  pWAttacker[0] =   pWAttacker[1] =   pWAttacker[2] =   pWAttacker[3] = 
    pWAttacker[4] =   pWAttacker[5] =   pWAttacker[6] =   pWAttacker[7] = 0; 
  pBAttacker[0] =   pBAttacker[1] =   pBAttacker[2] =   pBAttacker[3] = 
    pBAttacker[4] =   pBAttacker[5] =   pBAttacker[6] =   pBAttacker[7] = 0; 

  p = piece(move);
  f = from(move);
  t = to(move);
  c = captured(move);

#ifdef VERIFY
  plyData[ply].piece[f] = none;
  plyData[ply].piece[t] = plyData[ply-1].piece[f];
#endif
  if (plyData[ply-1].wToMove)
    {
      pos->WBoard &= NilMask[f];
      pos->WBoard |= OneMask[t];
    }
  else
    {
      pos->BBoard &= NilMask[f];
      pos->BBoard |= OneMask[t];
    }

  /* moved piece */
  switch(p)
    {
    case wP:
      plyData[ply].count50Moves = 0;
      pos->WPBoard &= NilMask[f];
      plyData[ply].key ^= others->wPKey[f];
      pos->WPBoard |= OneMask[t];
      plyData[ply].key ^= others->wPKey[t];
      if (t-f == 16) 
	{
	  plyData[ply].EP = OneMask[t-8];
	  plyData[ply].key ^= others->ePKey[t-8];
	}
      else if (OneMask[t] & plyData[ply-1].EP)
	{
#ifdef VERIFY
	  plyData[ply].piece[t-8] = none;
#endif
	  pos->BPBoard &= NilMask[t-8];
	  plyData[ply].key ^= others->bPKey[t-8];
	  pos->BBoard &= NilMask[t-8];
	  pos->wPCount[column(f)]--;
	  pos->wPCount[column(t)]++;
	  pos->bPCount[column(t)]--;
	}
      else if (c)
	{
	  pos->wPCount[column(f)]--;
	  pos->wPCount[column(t)]++;
	}
      if (promotedTo(move)) pos->wPCount[column(t)]--;
      break;
    case wN:
      pos->WNBoard &= NilMask[f];
      plyData[ply].key ^= others->wNKey[f];
      pos->WNBoard |= OneMask[t];
      plyData[ply].key ^= others->wNKey[t];
      break;
    case wB:
      pos->WBBoard &= NilMask[f];
      plyData[ply].key ^= others->wBKey[f];
      pos->WBBoard |= OneMask[t];
      plyData[ply].key ^= others->wBKey[t];
      break;
    case wR:
      if (plyData[ply].wCanCastle > 0)
	{
	  if (f == A1) plyData[ply].wCanCastle &= 1;
	  else if (f == H1) plyData[ply].wCanCastle &= 2;
	  plyData[ply].key ^= others->wCanCastleKey[plyData[ply-1].wCanCastle];
	  plyData[ply].key ^= others->wCanCastleKey[plyData[ply].wCanCastle];
	}
      pos->WRBoard &= NilMask[f];
      plyData[ply].key ^= others->wRKey[f];
      pos->WRBoard |= OneMask[t];
      plyData[ply].key ^= others->wRKey[t];
      break;
    case wQ:
      pos->WQBoard &= NilMask[f];
      plyData[ply].key ^= others->wQKey[f];
      pos->WQBoard |= OneMask[t];
      plyData[ply].key ^= others->wQKey[t];
      break;
    case wK:
      plyData[ply].key ^= others->wKKey[f];
      pos->wKSq = t;
      plyData[ply].key ^= others->wKKey[t];
      if (plyData[ply].wCanCastle > 0)
	{
	  /* long castling */
	  if (f == E1 && t == C1)
	    {
#ifdef VERIFY
	      plyData[ply].piece[A1] = none;
	      plyData[ply].piece[D1] = wR;
#endif
	      pos->WBoard &= NilMask[A1];
	      pos->WBoard |= OneMask[D1];
	      pos->WRBoard &= NilMask[A1];
	      plyData[ply].key ^= others->wRKey[A1];
	      pos->WRBoard |= OneMask[D1];
	      plyData[ply].key ^= others->wRKey[D1];
	      plyData[ply].wCanCastle = -(int) ply;
	      castleFlag = 'Q';
	    }
	  /* short castling */
	  else if (f == E1 && t == G1)
	    {
#ifdef VERIFY
	      plyData[ply].piece[H1] = none;
	      plyData[ply].piece[F1] = wR;
#endif
	      pos->WBoard &= NilMask[H1];
	      pos->WBoard |= OneMask[F1];
	      pos->WRBoard &= NilMask[H1];
	      plyData[ply].key ^= others->wRKey[H1];
	      pos->WRBoard |= OneMask[F1];
	      plyData[ply].key ^= others->wRKey[F1];
	      plyData[ply].wCanCastle = -(int) ply;
	      castleFlag = 'K';
	    }
	  else plyData[ply].wCanCastle = 0;
	  plyData[ply].key ^= others->wCanCastleKey[plyData[ply-1].wCanCastle];
	  plyData[ply].key ^= others->wCanCastleKey[0];
	}
      break;
    case bP:
      plyData[ply].count50Moves = 0;
      pos->BPBoard &= NilMask[f];
      plyData[ply].key ^= others->bPKey[f];
      pos->BPBoard |= OneMask[t];
      plyData[ply].key ^= others->bPKey[t];
      if (f-t == 16) 
	{
	  plyData[ply].EP = OneMask[t+8];
	  plyData[ply].key ^= others->ePKey[t+8];
	}
      else if (OneMask[t] & plyData[ply-1].EP)
	{
#ifdef VERIFY
	  plyData[ply].piece[t+8] = none;
#endif
	  pos->WPBoard &= NilMask[t+8];
	  plyData[ply].key ^= others->wPKey[t+8];
	  pos->WBoard &= NilMask[t+8];
	  pos->bPCount[column(f)]--;
	  pos->bPCount[column(t)]++;
	  pos->wPCount[column(t)]--;
	}
      else if (c)
	{
	  pos->bPCount[column(f)]--;
	  pos->bPCount[column(t)]++;
	}
      if (promotedTo(move)) pos->bPCount[column(t)]--;
      break;
    case bN:
      pos->BNBoard &= NilMask[f];
      plyData[ply].key ^= others->bNKey[f];
      pos->BNBoard |= OneMask[t];
      plyData[ply].key ^= others->bNKey[t];
      break;
    case bB:
      pos->BBBoard &= NilMask[f];
      plyData[ply].key ^= others->bBKey[f];
      pos->BBBoard |= OneMask[t];
      plyData[ply].key ^= others->bBKey[t];
      break;
    case bR:
      if (plyData[ply].bCanCastle > 0)
	{
	  if (f == A8) plyData[ply].bCanCastle &= 1;
	  else if (f == H8) plyData[ply].bCanCastle &= 2;
	  plyData[ply].key ^= others->bCanCastleKey[plyData[ply-1].bCanCastle];
	  plyData[ply].key ^= others->bCanCastleKey[plyData[ply].bCanCastle];
	}
      pos->BRBoard &= NilMask[f];
      plyData[ply].key ^= others->bRKey[f];
      pos->BRBoard |= OneMask[t];
      plyData[ply].key ^= others->bRKey[t];
      break;
    case bQ:
      pos->BQBoard &= NilMask[f];
      plyData[ply].key ^= others->bQKey[f];
      pos->BQBoard |= OneMask[t];
      plyData[ply].key ^= others->bQKey[t];
      break;
    case bK:
      plyData[ply].key ^= others->bKKey[f];
      pos->bKSq = t;
      plyData[ply].key ^= others->bKKey[t];
      if (plyData[ply].bCanCastle > 0)
	{
	  /* long castling */
	  if (f == E8 && t == C8)
	    {
#ifdef VERIFY
	      plyData[ply].piece[A8] = none;
	      plyData[ply].piece[D8] = bR;
#endif
	      pos->BBoard &= NilMask[A8];
	      pos->BBoard |= OneMask[D8];
	      pos->BRBoard &= NilMask[A8];
	      plyData[ply].key ^= others->bRKey[A8];
	      pos->BRBoard |= OneMask[D8];
	      plyData[ply].key ^= others->bRKey[D8];
	      plyData[ply].bCanCastle = -(int) ply;
	      castleFlag = 'q';
	    }
	  /* short castling */
	  else if (f == E8 && t == G8)
	    {
#ifdef VERIFY
	      plyData[ply].piece[H8] = none;
	      plyData[ply].piece[F8] = bR;
#endif
	      pos->BBoard &= NilMask[H8];
	      pos->BBoard |= OneMask[F8];
	      pos->BRBoard &= NilMask[H8];
	      plyData[ply].key ^= others->bRKey[H8];
	      pos->BRBoard |= OneMask[F8];
	      plyData[ply].key ^= others->bRKey[F8];
	      plyData[ply].bCanCastle = -(int) ply;
	      castleFlag = 'k';
	    }
	  else plyData[ply].bCanCastle = 0;
	  plyData[ply].key ^= others->bCanCastleKey[plyData[ply-1].bCanCastle];
	  plyData[ply].key ^= others->bCanCastleKey[0];
	}
      break;
    default:
      break;
    }

  /* captured piece */
  if (c)
    {
      ASSERT(c != wK && c != bK);
      plyData[ply].count50Moves = 0;
      switch(c)
	{
	case wP:
	  pos->WBoard &= NilMask[t];
	  pos->WPBoard &= NilMask[t];
	  plyData[ply].key ^= others->wPKey[t];
	  pos->wPCount[column(t)]--;
	  break;
	case wN:
	  pos->WBoard &= NilMask[t];
	  pos->WNBoard &= NilMask[t];
	  plyData[ply].key ^= others->wNKey[t];
	  break;
	case wB:
	  pos->WBoard &= NilMask[t];
	  pos->WBBoard &= NilMask[t];
	  plyData[ply].key ^= others->wBKey[t];
	  break;
	case wR:
	  pos->WBoard &= NilMask[t];
	  pos->WRBoard &= NilMask[t];
	  plyData[ply].key ^= others->wRKey[t];
	  break;
	case wQ:
	  pos->WBoard &= NilMask[t];
	  pos->WQBoard &= NilMask[t];
	  plyData[ply].key ^= others->wQKey[t];
	  break;
	case wK:
	  printf("makeMove: What the heck?  %u-%u\n", f, t);
	  display(*pos);
	  printf("makeMove: What the heck?  wK at %u is captured.\n", t);
	  break;
	case bP:
	  pos->BBoard &= NilMask[t];
	  pos->BPBoard &= NilMask[t];
	  plyData[ply].key ^= others->bPKey[t];
	  pos->bPCount[column(t)]--;
	  break;
	case bN:
	  pos->BBoard &= NilMask[t];
	  pos->BNBoard &= NilMask[t];
	  plyData[ply].key ^= others->bNKey[t];
	  break;
	case bB:
	  pos->BBoard &= NilMask[t];
	  pos->BBBoard &= NilMask[t];
	  plyData[ply].key ^= others->bBKey[t];
	  break;
	case bR:
	  pos->BBoard &= NilMask[t];
	  pos->BRBoard &= NilMask[t];
	  plyData[ply].key ^= others->bRKey[t];
	  break;
	case bQ:
	  pos->BBoard &= NilMask[t];
	  pos->BQBoard &= NilMask[t];
	  plyData[ply].key ^= others->bQKey[t];
	  break;
	case bK:
	  printf("makeMove: What the heck?  %u-%u\n", f, t);
	  display(*pos);
	  printf("makeMove: What the heck?  bK at %u is captured.\n", t);
	  break;
	default:
	  break;
	}
    }

  /* promoted piece */
  switch(promotedTo(move))
    {
    case none:
      break;
    case wN:
#ifdef VERIFY
      plyData[ply].piece[t] = wN;
#endif
      pos->WPBoard &= NilMask[t];
      plyData[ply].key ^= others->wPKey[t];
      pos->WNBoard |= OneMask[t];
      plyData[ply].key ^= others->wNKey[t]; 
     break;
    case wB:
#ifdef VERIFY
      plyData[ply].piece[t] = wB;
#endif
      pos->WPBoard &= NilMask[t];
      plyData[ply].key ^= others->wPKey[t];
      pos->WBBoard |= OneMask[t];
      plyData[ply].key ^= others->wBKey[t];
      break;
    case wR:
#ifdef VERIFY
      plyData[ply].piece[t] = wR;
#endif
      pos->WPBoard &= NilMask[t];
      plyData[ply].key ^= others->wPKey[t];
      pos->WRBoard |= OneMask[t];
      plyData[ply].key ^= others->wRKey[t];
      break;
    case wQ:
#ifdef VERIFY
      plyData[ply].piece[t] = wQ;
#endif
      pos->WPBoard &= NilMask[t];
      plyData[ply].key ^= others->wPKey[t];
      pos->WQBoard |= OneMask[t];
      plyData[ply].key ^= others->wQKey[t];
      break;
    case bN:
#ifdef VERIFY
      plyData[ply].piece[t] = bN;
#endif
      pos->BPBoard &= NilMask[t];
      plyData[ply].key ^= others->bPKey[t];
      pos->BNBoard |= OneMask[t];
      plyData[ply].key ^= others->bNKey[t];
      break;
    case bB:
#ifdef VERIFY
      plyData[ply].piece[t] = bB;
#endif
      pos->BPBoard &= NilMask[t];
      plyData[ply].key ^= others->bPKey[t];
      pos->BBBoard |= OneMask[t];
      plyData[ply].key ^= others->bBKey[t];
      break;
    case bR:
#ifdef VERIFY
      plyData[ply].piece[t] = bR;
#endif
      pos->BPBoard &= NilMask[t];
      plyData[ply].key ^= others->bPKey[t];
      pos->BRBoard |= OneMask[t];
      plyData[ply].key ^= others->bRKey[t];
      break;
    case bQ:
#ifdef VERIFY
      plyData[ply].piece[t] = bQ;
#endif
      pos->BPBoard &= NilMask[t];
      plyData[ply].key ^= others->bPKey[t];
      pos->BQBoard |= OneMask[t];
      plyData[ply].key ^= others->bQKey[t];
      break;
    default:
      break;
    }

  /* update rotated bitboards */
  pos->RotatedBoard90 &= NilMask[rotate90[f]];
  pos->RotatedBoard45l &= NilMask[rotate45l[f]];
  pos->RotatedBoard45r &= NilMask[rotate45r[f]];
  if (!c)
    {
      pos->RotatedBoard90 |= OneMask[rotate90[t]];
      pos->RotatedBoard45l |= OneMask[rotate45l[t]];
      pos->RotatedBoard45r |= OneMask[rotate45r[t]];
    }

  /* update rotated bitboards due to rook movement in castling */
  switch (castleFlag)
    {
    case '-':
      break;
    case 'K':
      pos->RotatedBoard90 &= NilMask[rotate90[H1]];
      pos->RotatedBoard45l &= NilMask[rotate45l[H1]];
      pos->RotatedBoard45r &= NilMask[rotate45r[H1]];
      pos->RotatedBoard90 |= OneMask[rotate90[F1]];
      pos->RotatedBoard45l |= OneMask[rotate45l[F1]];
      pos->RotatedBoard45r |= OneMask[rotate45r[F1]];
      break;
    case 'Q':
      pos->RotatedBoard90 &= NilMask[rotate90[A1]];
      pos->RotatedBoard45l &= NilMask[rotate45l[A1]];
      pos->RotatedBoard45r &= NilMask[rotate45r[A1]];
      pos->RotatedBoard90 |= OneMask[rotate90[D1]];
      pos->RotatedBoard45l |= OneMask[rotate45l[D1]];
      pos->RotatedBoard45r |= OneMask[rotate45r[D1]];
      break;
    case 'k':
      pos->RotatedBoard90 &= NilMask[rotate90[H8]];
      pos->RotatedBoard45l &= NilMask[rotate45l[H8]];
      pos->RotatedBoard45r &= NilMask[rotate45r[H8]];
      pos->RotatedBoard90 |= OneMask[rotate90[F8]];
      pos->RotatedBoard45l |= OneMask[rotate45l[F8]];
      pos->RotatedBoard45r |= OneMask[rotate45r[F8]];
      break;
    case 'q':
      pos->RotatedBoard90 &= NilMask[rotate90[A8]];
      pos->RotatedBoard45l &= NilMask[rotate45l[A8]];
      pos->RotatedBoard45r &= NilMask[rotate45r[A8]];
      pos->RotatedBoard90 |= OneMask[rotate90[D8]];
      pos->RotatedBoard45l |= OneMask[rotate45l[D8]];
      pos->RotatedBoard45r |= OneMask[rotate45r[D8]];
      break;
    default:
      break;
    }

  /* update rotated bitboards due to disappearing pawn in en-passant */
  if (OneMask[t] & plyData[ply-1].EP)
    {
      if (p == wP)
	{
	  pos->RotatedBoard90 &= NilMask[rotate90[t-8]];
	  pos->RotatedBoard45l &= NilMask[rotate45l[t-8]];
	  pos->RotatedBoard45r &= NilMask[rotate45r[t-8]];
	}
      else if (p == bP)
	{
	  pos->RotatedBoard90 &= NilMask[rotate90[t+8]];
	  pos->RotatedBoard45l &= NilMask[rotate45l[t+8]];
	  pos->RotatedBoard45r &= NilMask[rotate45r[t+8]];
	}
    }

  if (others->repHead + ply < 101)
    others->repKey[others->repHead + ply - 1] = plyData[ply].key;

#ifdef VERIFY
  key = scanBoard(plyData[ply].piece, plyData[ply].EP,
		  plyData[ply].wCanCastle, plyData[ply].bCanCastle, &thisPos, 
		  others);

  if (pos->WBoard != thisPos.WBoard)
    {
      displayBitBoard(thisPos.WBoard);
      printf("ERROR %s: WBoard don't match in makeMove().\n", others->posID);
      displayBitBoard(pos->WBoard);
      exit(EXIT_FAILURE);
    }
  if (pos->BBoard != thisPos.BBoard)
    {
      displayBitBoard(thisPos.BBoard);
      printf("ERROR %d-%d: BBoard don't match in makeMove().\n", f, t);
      displayBitBoard(pos->BBoard);
      exit(EXIT_FAILURE);
    }
  if (pos->WPBoard != thisPos.WPBoard)
    {
      displayBitBoard(thisPos.WPBoard);
      printf("ERROR %s: WPBoard don't match in makeMove().\n", others->posID);
      displayBitBoard(pos->WPBoard);
      exit(EXIT_FAILURE);
    }
  if (pos->WNBoard != thisPos.WNBoard)
    {
      displayBitBoard(thisPos.WNBoard);
      printf("ERROR %s: WNBoard don't match in makeMove().\n", others->posID);
      displayBitBoard(pos->WNBoard);
      exit(EXIT_FAILURE);
    }
  if (pos->WBBoard != thisPos.WBBoard)
    {
      displayBitBoard(thisPos.WBBoard);
      printf("ERROR %s: WBBoard don't match in makeMove().\n", others->posID);
      displayBitBoard(pos->WBBoard);
      exit(EXIT_FAILURE);
    }
  if (pos->WRBoard != thisPos.WRBoard)
    {
      displayBitBoard(thisPos.WRBoard);
      printf("ERROR %s: WRBoard don't match in makeMove().\n", others->posID);
      displayBitBoard(pos->WRBoard);
      exit(EXIT_FAILURE);
    }
  if (pos->WQBoard != thisPos.WQBoard)
    {
      displayBitBoard(thisPos.WQBoard);
      printf("ERROR %s: WQBoard don't match in makeMove().\n", others->posID);
      displayBitBoard(pos->WQBoard);
      exit(EXIT_FAILURE);
    }
  if (pos->BPBoard != thisPos.BPBoard)
    {
      displayBitBoard(thisPos.BPBoard);
      printf("ERROR %s: BPBoard don't match in makeMove().\n", others->posID);
      displayBitBoard(pos->BPBoard);
      exit(EXIT_FAILURE);
    }
  if (pos->BNBoard != thisPos.BNBoard)
    {
      displayBitBoard(thisPos.BNBoard);
      printf("ERROR %s: BNBoard don't match in makeMove().\n", others->posID);
      displayBitBoard(pos->BNBoard);
      exit(EXIT_FAILURE);
    }
  if (pos->BBBoard != thisPos.BBBoard)
    {
      displayBitBoard(thisPos.BBBoard);
      printf("ERROR %s: BBBoard don't match in makeMove().\n", others->posID);
      displayBitBoard(pos->BBBoard);
      exit(EXIT_FAILURE);
    }
  if (pos->BRBoard != thisPos.BRBoard)
    {
      displayBitBoard(thisPos.BRBoard);
      printf("ERROR %s: BRBoard don't match in makeMove().\n", others->posID);
      displayBitBoard(pos->BRBoard);
      exit(EXIT_FAILURE);
    }
  if (pos->BQBoard != thisPos.BQBoard)
    {
      displayBitBoard(thisPos.BQBoard);
      printf("ERROR %s: BQBoard don't match in makeMove().\n", others->posID);
      displayBitBoard(pos->BQBoard);
      exit(EXIT_FAILURE);
    }
  if (plyData[ply].key != key)
    {
      display(*pos);
      printf("ERROR %s: key don't match in makeMove().\n", others->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->wPCount[0] != thisPos.wPCount[0])
    {
      display(*pos);
      printf("ERROR %s: wPCount[0] don't match in makeMove().\n", others->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->wPCount[1] != thisPos.wPCount[1])
    {
      display(*pos);
      printf("ERROR %s: wPCount[1] don't match in makeMove().\n", others->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->wPCount[2] != thisPos.wPCount[2])
    {
      display(*pos);
      printf("ERROR %s: wPCount[2] don't match in makeMove().\n", others->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->wPCount[3] != thisPos.wPCount[3])
    {
      display(*pos);
      printf("ERROR %s: wPCount[3] don't match in makeMove().\n", others->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->wPCount[4] != thisPos.wPCount[4])
    {
      display(*pos);
      printf("ERROR %s: wPCount[4] don't match in makeMove().\n", others->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->wPCount[5] != thisPos.wPCount[5])
    {
      display(*pos);
      printf("ERROR %s: wPCount[5] don't match in makeMove().\n", others->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->wPCount[6] != thisPos.wPCount[6])
    {
      display(*pos);
      printf("ERROR %s: wPCount[6] don't match in makeMove().\n", others->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->wPCount[7] != thisPos.wPCount[7])
    {
      display(*pos);
      printf("ERROR %s: wPCount[7] don't match in makeMove().\n", others->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->bPCount[0] != thisPos.bPCount[0])
    {
      display(*pos);
      printf("ERROR %s: bPCount[0] don't match in makeMove().\n", others->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->bPCount[1] != thisPos.bPCount[1])
    {
      display(*pos);
      printf("ERROR %s: bPCount[1] don't match in makeMove().\n", others->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->bPCount[2] != thisPos.bPCount[2])
    {
      display(*pos);
      printf("ERROR %s: bPCount[2] don't match in makeMove().\n", others->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->bPCount[3] != thisPos.bPCount[3])
    {
      display(*pos);
      printf("ERROR %s: bPCount[3] don't match in makeMove().\n", others->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->bPCount[4] != thisPos.bPCount[4])
    {
      display(*pos);
      printf("ERROR %s: bPCount[4] don't match in makeMove().\n", others->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->bPCount[5] != thisPos.bPCount[5])
    {
      display(*pos);
      printf("ERROR %s: bPCount[5] don't match in makeMove().\n", others->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->bPCount[6] != thisPos.bPCount[6])
    {
      display(*pos);
      printf("ERROR %s: bPCount[6] don't match in makeMove().\n", others->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->bPCount[7] != thisPos.bPCount[7])
    {
      display(*pos);
      printf("ERROR %s: bPCount[7] don't match in makeMove().\n", others->posID);
      exit(EXIT_FAILURE);
    }
  if (pos->RotatedBoard90 != thisPos.RotatedBoard90)
    {
      displayBitBoard(thisPos.RotatedBoard90);
      printf("ERROR %s: RotatedBoard90 don't match in makeMove().\n", others->posID);
      displayBitBoard(pos->RotatedBoard90);
      exit(EXIT_FAILURE);
    }
  if (pos->RotatedBoard45l != thisPos.RotatedBoard45l)
    {
      displayBitBoard(thisPos.RotatedBoard45l);
      printf("ERROR %s: RotatedBoard45l don't match in makeMove().\n", others->posID);
      displayBitBoard(pos->RotatedBoard45l);
      exit(EXIT_FAILURE);
    }
  if (pos->RotatedBoard45r != thisPos.RotatedBoard45r)
    {
      displayBitBoard(thisPos.RotatedBoard45r);
      printf("ERROR %s: RotatedBoard45r don't match in makeMove().\n", others->posID);
      displayBitBoard(pos->RotatedBoard45r);
      exit(EXIT_FAILURE);
    }
#endif
}


void unmakeMove(ui ply, BB move, sPos *pos, sPlyData *plyData)
{
  ui f, t, p, c;
  char castleFlag = '-';

  ASSERT(ply <= maxPly);
  ASSERT(from(move) <= H8 && to(move) <= H8 &&
	 from(move) != to(move));
  ASSERT(piece(move) < 13);
  ASSERT(pos != NULL);
  ASSERT(plyData != NULL);
  ASSERT(!(pos->WBoard & pos->BBoard));

  f = from(move);
  t = to(move);
  p = piece(move);
  c = captured(move);

  if (c != none)
    {
      if (plyData[ply-1].wToMove) pos->BBoard |= OneMask[t];
      else pos->WBoard |= OneMask[t];
      
      switch (c)
	{
	case wP: pos->WPBoard |= OneMask[t];  break;
	case wN: pos->WNBoard |= OneMask[t];  break;
	case wB: pos->WBBoard |= OneMask[t];  break;
	case wR: pos->WRBoard |= OneMask[t];  break;
	case wQ: pos->WQBoard |= OneMask[t];  break;
	case bP: pos->BPBoard |= OneMask[t];  break;
	case bN: pos->BNBoard |= OneMask[t];  break;
	case bB: pos->BBBoard |= OneMask[t];  break;
	case bR: pos->BRBoard |= OneMask[t];  break;
	case bQ: pos->BQBoard |= OneMask[t];  break;
	default: break;
	}
    }

  if (plyData[ply-1].wToMove)
    {
      pos->WBoard |= OneMask[f];
      pos->WBoard &= NilMask[t];
    }
  else
    {
      pos->BBoard |= OneMask[f];
      pos->BBoard &= NilMask[t];
    }

  /* promoted piece */
  switch(promotedTo(move))
    {
    case none:
      break;
    case wN:
      pos->WPBoard |= OneMask[t];
      pos->WNBoard &= NilMask[t];
     break;
    case wB:
      pos->WPBoard |= OneMask[t];
      pos->WBBoard &= NilMask[t];
      break;
    case wR:
      pos->WPBoard |= OneMask[t];
      pos->WRBoard &= NilMask[t];
      break;
    case wQ:
      pos->WPBoard |= OneMask[t];
      pos->WQBoard &= NilMask[t];
      break;
    case bN:
      pos->BPBoard |= OneMask[t];
      pos->BNBoard &= NilMask[t];
      break;
    case bB:
      pos->BPBoard |= OneMask[t];
      pos->BBBoard &= NilMask[t];
      break;
    case bR:
      pos->BPBoard |= OneMask[t];
      pos->BRBoard &= NilMask[t];
      break;
    case bQ:
      pos->BPBoard |= OneMask[t];
      pos->BQBoard &= NilMask[t];
      break;
    default:
      break;
    }

  /* moved piece */
  switch(p)
    {
    case wP:
      pos->WPBoard |= OneMask[f];
      pos->WPBoard &= NilMask[t];
      if (OneMask[t] & plyData[ply-1].EP)
	{
	  pos->BPBoard |= OneMask[t-8];
	  pos->BBoard |= OneMask[t-8];
	  pos->wPCount[column(f)]++;
	  pos->wPCount[column(t)]--;
	  pos->bPCount[column(t)]++;
	}
      else if (c)
	{
	  pos->wPCount[column(f)]++;
	  pos->wPCount[column(t)]--;
	}
      if (promotedTo(move)) pos->wPCount[column(t)]++;
      break;
    case wN:
      pos->WNBoard |= OneMask[f];
      pos->WNBoard &= NilMask[t];
      break;
    case wB:
      pos->WBBoard |= OneMask[f];
      pos->WBBoard &= NilMask[t];
      break;
    case wR:
      pos->WRBoard |= OneMask[f];
      pos->WRBoard &= NilMask[t];
      break;
    case wQ:
      pos->WQBoard |= OneMask[f];
      pos->WQBoard &= NilMask[t];
      break;
    case wK:
      pos->wKSq = f;
      if (plyData[ply-1].wCanCastle > 0)
	{
	  /* long castling */
	  if (f == E1 && t == C1)
	    {
	      pos->WBoard |= OneMask[A1];
	      pos->WBoard &= NilMask[D1];
	      pos->WRBoard |= OneMask[A1];
	      pos->WRBoard &= NilMask[D1];
	      castleFlag = 'Q';
	    }
	  /* short castling */
	  else if (f == E1 && t == G1)
	    {
	      pos->WBoard |= OneMask[H1];
	      pos->WBoard &= NilMask[F1];
	      pos->WRBoard |= OneMask[H1];
	      pos->WRBoard &= NilMask[F1];
	      castleFlag = 'K';
	    }
	}
      break;
    case bP:
      pos->BPBoard |= OneMask[f];
      pos->BPBoard &= NilMask[t];
      if (OneMask[t] & plyData[ply-1].EP)
	{
	  pos->WPBoard |= OneMask[t+8];
	  pos->WBoard |= OneMask[t+8];
	  pos->bPCount[column(f)]++;
	  pos->bPCount[column(t)]--;
	  pos->wPCount[column(t)]++;
	}
      else if (c)
	{
	  pos->bPCount[column(f)]++;
	  pos->bPCount[column(t)]--;
	}
      if (promotedTo(move)) pos->bPCount[column(t)]++;
      break;
    case bN:
      pos->BNBoard |= OneMask[f];
      pos->BNBoard &= NilMask[t];
      break;
    case bB:
      pos->BBBoard |= OneMask[f];
      pos->BBBoard &= NilMask[t];
      break;
    case bR:
      pos->BRBoard |= OneMask[f];
      pos->BRBoard &= NilMask[t];
      break;
    case bQ:
      pos->BQBoard |= OneMask[f];
      pos->BQBoard &= NilMask[t];
      break;
    case bK:
      pos->bKSq = f;
      if (plyData[ply-1].bCanCastle > 0)
	{
	  /* long castling */
	  if (f == E8 && t == C8)
	    {
	      pos->BBoard |= OneMask[A8];
	      pos->BBoard &= NilMask[D8];
	      pos->BRBoard |= OneMask[A8];
	      pos->BRBoard &= NilMask[D8];
	      castleFlag = 'q';
	    }
	  /* short castling */
	  else if (f == E8 && t == G8)
	    {
	      pos->BBoard |= OneMask[H8];
	      pos->BBoard &= NilMask[F8];
	      pos->BRBoard |= OneMask[H8];
	      pos->BRBoard &= NilMask[F8];
	      castleFlag = 'k';
	    }
	}
      break;
    default:
      break;
    }

  /* captured piece */
  ASSERT(c != wK && c != bK);
  switch(c)
    {
    case none:
      break;
    case wP:
      pos->WBoard |= OneMask[t];
      pos->WPBoard |= OneMask[t];
      pos->wPCount[column(t)]++;
      break;
    case wN:
      pos->WBoard |= OneMask[t];
      pos->WNBoard |= OneMask[t];
      break;
    case wB:
      pos->WBoard |= OneMask[t];
      pos->WBBoard |= OneMask[t];
      break;
    case wR:
      pos->WBoard |= OneMask[t];
      pos->WRBoard |= OneMask[t];
      break;
    case wQ:
      pos->WBoard |= OneMask[t];
      pos->WQBoard |= OneMask[t];
      break;
    case wK:
      printf("unmakeMove: What the heck?  %u-%u\n", f, t);
      display(*pos);
      printf("unmakeMove: What the heck?  wK at %u is captured.\n", t);
      break;
    case bP:
      pos->BBoard |= OneMask[t];
      pos->BPBoard |= OneMask[t];
      pos->bPCount[column(t)]++;
      break;
    case bN:
      pos->BBoard |= OneMask[t];
      pos->BNBoard |= OneMask[t];
      break;
    case bB:
      pos->BBoard |= OneMask[t];
      pos->BBBoard |= OneMask[t];
      break;
    case bR:
      pos->BBoard |= OneMask[t];
      pos->BRBoard |= OneMask[t];
      break;
    case bQ:
      pos->BBoard |= OneMask[t];
      pos->BQBoard |= OneMask[t];
      break;
    case bK:
      printf("unmakeMove: What the heck?  %u-%u\n", f, t);
      display(*pos);
      printf("unmakeMove: What the heck?  bK at %u is captured.\n", t);
      break;
    default:
      break;
    }

  /* update rotated bitboards */
  pos->RotatedBoard90 |= OneMask[rotate90[f]];
  pos->RotatedBoard45l |= OneMask[rotate45l[f]];
  pos->RotatedBoard45r |= OneMask[rotate45r[f]];
  if (!c)
    {
      pos->RotatedBoard90 &= NilMask[rotate90[t]];
      pos->RotatedBoard45l &= NilMask[rotate45l[t]];
      pos->RotatedBoard45r &= NilMask[rotate45r[t]];
    }

  /* update rotated bitboards due to rook movement in castling */
  switch (castleFlag)
    {
    case '-':
      break;
    case 'K':
      pos->RotatedBoard90 |= OneMask[rotate90[H1]];
      pos->RotatedBoard45l |= OneMask[rotate45l[H1]];
      pos->RotatedBoard45r |= OneMask[rotate45r[H1]];
      pos->RotatedBoard90 &= NilMask[rotate90[F1]];
      pos->RotatedBoard45l &= NilMask[rotate45l[F1]];
      pos->RotatedBoard45r &= NilMask[rotate45r[F1]];
      break;
    case 'Q':
      pos->RotatedBoard90 |= OneMask[rotate90[A1]];
      pos->RotatedBoard45l |= OneMask[rotate45l[A1]];
      pos->RotatedBoard45r |= OneMask[rotate45r[A1]];
      pos->RotatedBoard90 &= NilMask[rotate90[D1]];
      pos->RotatedBoard45l &= NilMask[rotate45l[D1]];
      pos->RotatedBoard45r &= NilMask[rotate45r[D1]];
      break;
    case 'k':
      pos->RotatedBoard90 |= OneMask[rotate90[H8]];
      pos->RotatedBoard45l |= OneMask[rotate45l[H8]];
      pos->RotatedBoard45r |= OneMask[rotate45r[H8]];
      pos->RotatedBoard90 &= NilMask[rotate90[F8]];
      pos->RotatedBoard45l &= NilMask[rotate45l[F8]];
      pos->RotatedBoard45r &= NilMask[rotate45r[F8]];
      break;
    case 'q':
      pos->RotatedBoard90 |= OneMask[rotate90[A8]];
      pos->RotatedBoard45l |= OneMask[rotate45l[A8]];
      pos->RotatedBoard45r |= OneMask[rotate45r[A8]];
      pos->RotatedBoard90 &= NilMask[rotate90[D8]];
      pos->RotatedBoard45l &= NilMask[rotate45l[D8]];
      pos->RotatedBoard45r &= NilMask[rotate45r[D8]];
      break;
    default:
      break;
    }

  /* update rotated bitboards due to disappearing pawn in en-passant */
  if (OneMask[t] & plyData[ply-1].EP)
    {
      if (p == wP)
	{
	  pos->RotatedBoard90 |= OneMask[rotate90[t-8]];
	  pos->RotatedBoard45l |= OneMask[rotate45l[t-8]];
	  pos->RotatedBoard45r |= OneMask[rotate45r[t-8]];
	}
      else if (p == bP)
	{
	  pos->RotatedBoard90 |= OneMask[rotate90[t+8]];
	  pos->RotatedBoard45l |= OneMask[rotate45l[t+8]];
	  pos->RotatedBoard45r |= OneMask[rotate45r[t+8]];
	}
    }
}


/* Black is checking me, I want to save my king. */
ui outOfBCheck(ui ply, ui kSq, BB Checkers, BB *moveList, sPos *pos, 
	       sPlyData *plyData, sOthers *others)
{
  BB Rank = (BB) 0, Fyle = (BB) 0, Diagonal = (BB) 0;
  BB From = (BB) 0, To = (BB) 0, Them = (BB) 0, move = (BB) 0;
  ui attacker1 = 64, attacker2 = 64;

  ui capture = 1;
  ui block = 1;
  ui sq, t, c, ok;
  ui kRank, kFyle;
  int tValue, mVVLVAValue;
  ui count = 0;

  ASSERT(ply <= maxPly);
  ASSERT(kSq <= H8);
  ASSERT(moveList != NULL);
  ASSERT(pos != NULL);
  ASSERT(plyData != NULL);
  ASSERT(others != NULL);
  ASSERT(!(pos->WBoard & pos->BBoard));

  kRank = row(kSq);
  kFyle = column(kSq);

  /* 
   * count how many pieces are attacking the king
   * if there are more than one then move king
   * else if attakers are non slider then try
   * 1. capture the attacker1
   * 2. move the king
   * else try     
   * 1. capture the attacker1
   * 2. move the king (including captures)
   * 3. block
  */

  if (bitCount(Checkers) == 2)
    {
      capture = block = 0;

      attacker1 = firstOne(Checkers);
      switch (pieceAtSq(attacker1, pos))
	{
	case bP:
	case bN:
	  break;
	case bB:
	  Diagonal = others->BMoves[attacker1] & others->BMoves[kSq];
	  break;
	case bR:
	  if (row(attacker1) == kRank) Rank = others->Row[kRank];
	  else if (column(attacker1) == kFyle) Fyle = others->Column[kFyle];
	  break;
	case bQ:
	  if (row(attacker1) == kRank) Rank = others->Row[kRank];
	  else if (column(attacker1) == kFyle) Fyle = others->Column[kFyle];
	  else Diagonal = others->BMoves[attacker1] & others->BMoves[kSq];
	  break;
	default:
	  break;
	}
      
      attacker2 = lastOne(Checkers);
      switch (pieceAtSq(attacker2, pos))
	{
	case bP:
	case bN:
	  break;
	case bB:
	  Diagonal = others->BMoves[attacker2] & others->BMoves[kSq];
	  break;
	case bR:
	  if (row(attacker2) == kRank) Rank = others->Row[kRank];
	  else if (column(attacker2) == kFyle) Fyle = others->Column[kFyle];
	  break;
	case bQ:
	  if (row(attacker2) == kRank) Rank = others->Row[kRank];
	  else if (column(attacker2) == kFyle) Fyle = others->Column[kFyle];
	  else Diagonal = others->BMoves[attacker2] & others->BMoves[kSq];
	  break;
	default:
	  break;
	}
    }
  else 
    {
      attacker1 = firstOne(Checkers);
      c = pieceAtSq(attacker1, pos);
      ASSERT(c < 13);
      switch (c)
	{
	case bP:
	case bN:
	  block = 0;
	  break;
	case bB:
	  Diagonal = others->BMoves[attacker1] & others->BMoves[kSq];
	  break;
	case bR:
	  if (row(attacker1) == kRank) Rank = others->Row[kRank];
	  else if (column(attacker1) == kFyle) Fyle = others->Column[kFyle];
	  break;
	case bQ:
	  if (row(attacker1) == kRank) Rank = others->Row[kRank];
	  else if (column(attacker1) == kFyle) Fyle = others->Column[kFyle];
	  else Diagonal = others->BMoves[attacker1] & others->BMoves[kSq];
	  break;
	default:
	  break;
	}
    }

  /* capture the attacker1 */
  if (capture)
    {
      /* pawn */
      if (pos->WPBoard)
	{
	  t = attacker1;
	  From = pos->WPBoard & others->BPCaptures[t];
	  while (From)
	    {
	      sq = firstOne(From);
	      tValue = see(1, sq, t, pos, others);
#ifdef VERIFY
	      if (t == pos->bKSq) 
		{
		  printf("bK is captured by wP in outOfBCheck().\n");
		  exit(EXIT_FAILURE);
		}
#endif
	      move = sq | (t << 6) | (wP << 12) | (c << 18);
	      if (!pinner(white, sq, pos->wKSq, pos, others))
		{
		  if (OneMask[sq] & others->Row[6])   /* promotions */
		    {
		      mVVLVAValue = qValue + tValue - pValue;
		      count++;
		      *moveList++ = move | (wQ << 24) |
			((BB)(mVVLVAValue+mateScore) << 30);
		      
		      mVVLVAValue = rValue + tValue - pValue;
		      count++;
		      *moveList++ = move | (wR << 24) |
			((BB)(mVVLVAValue+mateScore) << 30);
		      
		      mVVLVAValue = bValue + tValue - pValue;
		      count++;
		      *moveList++ = move | (wB << 24) |
			((BB)(mVVLVAValue+mateScore) << 30);
		      
		      mVVLVAValue = nValue + tValue - pValue;
		      count++;
		      *moveList++ = move | (wN << 24) |
			((BB)(mVVLVAValue+mateScore) << 30);
		    }
		  else
		    {
		      count++;
		      *moveList++ = move | ((BB)(tValue+mateScore) << 30);
		    }
		}
	      From &= NilMask[sq];
	    }

	  /* en-passant */
	  if (plyData[ply].EP)
	    {
	      t = firstOne(plyData[ply].EP);
	      if (t-attacker1 == 8)
		{
		  From = pos->WPBoard & others->BPCaptures[t];
		  while (From)
		    {
		      sq = firstOne(From);
		      move = sq | (t << 6) | (wP << 12);
                      if (!pinner(white, sq, pos->wKSq, pos, others))
			{
			  mVVLVAValue = pValue;
			  count++;
			  *moveList++ = move |
			    ((BB)(mVVLVAValue+mateScore) << 30);
			}		      
		      From &= NilMask[sq];
		    }
		}
	    }
	}

      /* knight */
      if (pos->WNBoard)
	{
	  t = attacker1;
	  From = pos->WNBoard & others->NMoves[t];
	  while (From)
	    {
	      sq = firstOne(From);
	      tValue = see(1, sq, t, pos, others);
#ifdef VERIFY
	      if (t == pos->bKSq) 
		{
		  printf("bK is captured by wN in outOfBCheck().\n");
		  exit(EXIT_FAILURE);
		}
#endif
	      move = sq | (t << 6) | (wN << 12) | (c << 18);
	      if (!pinner(white, sq, pos->wKSq, pos, others))
		{
		  count++;
		  *moveList++ = move | ((BB)(tValue+mateScore) << 30);
		}
	      From &= NilMask[sq];
	    }
	}


      /* bishop */
      if (pos->WBBoard)
	{
	  t = attacker1;
	  From = pos->WBBoard & bishopAttacks(t, pos);
	  while (From)
	    {
	      sq = firstOne(From);
	      tValue = see(1, sq, t, pos, others);
#ifdef VERIFY
	      if (t == pos->bKSq) 
		{
		  printf("bK is captured by wB in outOfBCheck().\n");
		  exit(EXIT_FAILURE);
		}
#endif
	      move = sq | (t << 6) | (wB << 12) | (c << 18);
	      if (!pinner(white, sq, pos->wKSq, pos, others))
		{
		  count++;
		  *moveList++ = move | ((BB)(tValue+mateScore) << 30);
		}
	      From &= NilMask[sq];
	    }
	}

      /* rook */
      if (pos->WRBoard)
	{
	  t = attacker1;
	  From = pos->WRBoard & rookAttacks(t, pos);
	  while (From)
	    {
	      sq = firstOne(From);
	      tValue = see(1, sq, t, pos, others);
#ifdef VERIFY
	      if (t == pos->bKSq) 
		{
		  printf("bK is captured by wR in outOfBCheck().\n");
		  exit(EXIT_FAILURE);
		}
#endif
	      move = sq | (t << 6) | (wR << 12) | (c << 18);
	      if (!pinner(white, sq, pos->wKSq, pos, others))
		{
		  count++;
		  *moveList++ = move | ((BB)(tValue+mateScore) << 30);
		}
	      From &= NilMask[sq];
	    }
	}

      /* queen */
      if (pos->WQBoard)
	{
	  t = attacker1;
	  From = pos->WQBoard & 
	    (bishopAttacks(t, pos) | rookAttacks(t, pos));
	  while (From)
	    {
	      sq = firstOne(From);
	      tValue = see(1, sq, t, pos, others);
#ifdef VERIFY
	      if (t == pos->bKSq) 
		{
		  printf("bK is captured by wQ in outOfBCheck().\n");
		  exit(EXIT_FAILURE);
		}
#endif
	      move = sq | (t << 6) | (wQ << 12) | (c << 18);
	      if (!pinner(white, sq, pos->wKSq, pos, others))
		{
		  count++;
		  *moveList++ = move | ((BB)(tValue+mateScore) << 30);
		}
	      From &= NilMask[sq];
	    }
	} 
   }


  /* block */
  if (block)
    {
      switch (c)
	{
	/* the beauty of rotated bitboard */
	case bB: 
	  Them = bishopAttacks(kSq, pos) &  
	    bishopAttacks(attacker1, pos);
	  break;
	case bR: 
	  Them = rookAttacks(kSq, pos) &  
	    rookAttacks(attacker1, pos);
	  break;
	case bQ: 
	  /* be careful here */
	  if (row(attacker1) == row(kSq) || column(attacker1) == column(kSq))
	    Them = rookAttacks(kSq, pos) &  
	      rookAttacks(attacker1, pos);
	  else
	  Them = bishopAttacks(kSq, pos) &  
	    bishopAttacks(attacker1, pos);
	  break;
	default:
	  break;
	}

      /* pawn */
      if (pos->WPBoard)
	{
	  From = pos->WPBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      if (OneMask[sq+8] & Them)
		{
		  t = sq+8;
		  move = sq | (t << 6) | (wP << 12);
	          if (!pinner(white, sq, pos->wKSq, pos, others))
		    {
		      if (OneMask[sq] & others->Row[6])
			{
			  mVVLVAValue = qValue - pValue;
			  count++;
			  *moveList++ = move | (wQ << 24) | 
			    ((BB)(mVVLVAValue+mateScore) << 30);

			  mVVLVAValue = rValue - pValue;
			  count++;
			  *moveList++ = move | (wR << 24) |
			    ((BB)(mVVLVAValue+mateScore) << 30);
			      
			  mVVLVAValue = bValue - pValue;
			  count++;
			  *moveList++ = move | (wB << 24) |
			    ((BB)(mVVLVAValue+mateScore) << 30);

			  mVVLVAValue = nValue - pValue;
			  count++;
			  *moveList++ = move | (wN << 24) |
			    ((BB)(mVVLVAValue+mateScore) << 30);
			}
		      else
			{
			  count++;
			  *moveList++ = move | ((BB)(mateScore) << 30);
			}
		    }
		}

		  if ((OneMask[sq] & others->Row[1]) && 
		      (OneMask[sq+16] & Them) &&
		      (!(OneMask[sq+8] & (pos->WBoard | pos->BBoard))))
		    {
		      t = sq+16;
		      move = sq | (t << 6) | (wP << 12);
	              if (!pinner(white, sq, pos->wKSq, pos, others))
			{
			  count++;
			  *moveList++ = move | ((BB)(mateScore) << 30);
			}
		    }


	      From &= NilMask[sq];
	    }
	}

      /* knight */
      if (pos->WNBoard)
	{
	  From = pos->WNBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      To = others->NMoves[sq] & Them;
	      while (To)
		{
		  t = lastOne(To);
		  move = sq | (t << 6) | (wN << 12);
	          if (!pinner(white, sq, pos->wKSq, pos, others))
		    {
		      count++;
		      *moveList++ = move | ((BB)(mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* bishop */
      if (pos->WBBoard)
	{
	  From = pos->WBBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      To = bishopAttacks(sq, pos) & Them;
	      while (To)
		{
		  t = lastOne(To);
		  move = sq | (t << 6) | (wB << 12);
	          if (!pinner(white, sq, pos->wKSq, pos, others))
		    {
		      count++;
		      *moveList++ = move | ((BB)(mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* rook */
      if (pos->WRBoard)
	{
	  From = pos->WRBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      To = rookAttacks(sq, pos) & Them;
	      while (To)
		{
		  t = lastOne(To);
		  move = sq | (t << 6) | (wR << 12);
	          if (!pinner(white, sq, pos->wKSq, pos, others))
		    {
		      count++;
		      *moveList++ = move | ((BB)(mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* queen */
      if (pos->WQBoard)
	{
	  From = pos->WQBoard;
	  while (From)
	    {
	      sq = firstOne(From);
	      To = (rookAttacks(sq, pos) | 
		    bishopAttacks(sq, pos)) 
		& Them;
	      while (To)
		{
		  t = lastOne(To);
		  move = sq | (t << 6) | (wQ << 12);
	          if (!pinner(white, sq, pos->wKSq, pos, others))
		    {
		      count++;
		      *moveList++ = move | ((BB)(mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}
    }


  /* move the king, including captures, should be tried in all cases */
  sq = kSq;

  To = others->KMoves[sq] & ~pos->WBoard;
  while (To)
    {
      t = lastOne(To);
      c = pieceAtSq(t, pos);
      ASSERT(c < 13);
      move = sq | (t << 6) | (wK << 12) | (c << 18);
      ok = true;
      if (t != attacker1 && t != attacker2)
	{
	  if ((Rank | Fyle | Diagonal) & OneMask[t]) ok = false; 
	}
      if (ok && !(getBAttacker(t, pos, &plyData[ply], others) & 2))
	{
	  count++;
	  *moveList++ = move | ((BB)(value[c]+mateScore) << 30);
	}
      To &= NilMask[t];
    }

  return count;
}


/* White is checking me, I bant to save my king. */
ui outOfWCheck(ui ply, ui kSq, BB Checkers, BB *moveList, sPos *pos, 
	       sPlyData *plyData, sOthers *others)
{
  BB Rank = (BB) 0, Fyle = (BB) 0, Diagonal = (BB) 0;
  BB From = (BB) 0, To = (BB) 0, Them = (BB) 0, move = (BB) 0;
  ui attacker1 = 64, attacker2 = 64;

  ui capture = 1;
  ui block = 1;
  ui sq, t, c, ok;
  ui kRank, kFyle;
  int tValue, mVVLVAValue;
  ui count = 0;

  ASSERT(ply <= maxPly);
  ASSERT(kSq <= H8);
  ASSERT(moveList != NULL);
  ASSERT(pos != NULL);
  ASSERT(plyData != NULL);
  ASSERT(others != NULL);
  ASSERT(!(pos->WBoard & pos->BBoard));

  kRank = row(kSq);
  kFyle = column(kSq);

  /* 
   * count how many pieces are attacking the king
   * if there are more than one then move king
   * else if attakers are non slider then try
   * 1. capture the attacker1
   * 2. move the king
   * else try     
   * 1. capture the attacker1
   * 2. move the king (including captures)
   * 3. block
  */

  if (bitCount(Checkers) == 2)
    {
      capture = block = 0;

      attacker1 = firstOne(Checkers);
      switch (pieceAtSq(attacker1, pos))
	{
	case wP:
	case wN:
	  break;
	case wB:
	  Diagonal = others->BMoves[attacker1] & others->BMoves[kSq];
	  break;
	case wR:
	  if (row(attacker1) == kRank) Rank = others->Row[kRank];
	  else if (column(attacker1) == kFyle) Fyle = others->Column[kFyle];
	  break;
	case wQ:
	  if (row(attacker1) == kRank) Rank = others->Row[kRank];
	  else if (column(attacker1) == kFyle) Fyle = others->Column[kFyle];
	  else Diagonal = others->BMoves[attacker1] & others->BMoves[kSq];
	  break;
	default:
	  break;
	}
      
      attacker2 = lastOne(Checkers);
      switch (pieceAtSq(attacker2, pos))
	{
	case wP:
	case wN:
	  break;
	case wB:
	  Diagonal = others->BMoves[attacker2] & others->BMoves[kSq];
	  break;
	case wR:
	  if (row(attacker2) == kRank) Rank = others->Row[kRank];
	  else if (column(attacker2) == kFyle) Fyle = others->Column[kFyle];
	  break;
	case wQ:
	  if (row(attacker2) == kRank) Rank = others->Row[kRank];
	  else if (column(attacker2) == kFyle) Fyle = others->Column[kFyle];
	  else Diagonal = others->BMoves[attacker2] & others->BMoves[kSq];
	  break;
	default:
	  break;
	}
    }
  else 
    {
      attacker1 = lastOne(Checkers);
      c = pieceAtSq(attacker1, pos);
      ASSERT(c < 13);
      switch (c)
	{
	case wP:
	case wN:
	  block = 0;
	  break;
	case wB:
	  Diagonal = others->BMoves[attacker1] & others->BMoves[kSq];
	  break;
	case wR:
	  if (row(attacker1) == kRank) Rank = others->Row[kRank];
	  else if (column(attacker1) == kFyle) Fyle = others->Column[kFyle];
	  break;
	case wQ:
	  if (row(attacker1) == kRank) Rank = others->Row[kRank];
	  else if (column(attacker1) == kFyle) Fyle = others->Column[kFyle];
	  else Diagonal = others->BMoves[attacker1] & others->BMoves[kSq];
	  break;
	default:
	  break;
	}
    }

  /* capture the attacker1 */
  if (capture)
    {
      /* pawn */
      if (pos->BPBoard)
	{
	  t = attacker1;
	  From = pos->BPBoard & others->WPCaptures[t];
	  while (From)
	    {
	      sq = lastOne(From);
	      tValue = see(0, sq, t, pos, others);
#ifdef VERIFY
	      if (t == pos->wKSq) 
		{
		  printf("wK is captured wy bP in outOfWCheck().\n");
		  exit(EXIT_FAILURE);
		}
#endif
	      move = sq | (t << 6) | (bP << 12) | (c << 18);
	      if (!pinner(black, sq, pos->bKSq, pos, others))
		{
		  if (OneMask[sq] & others->Row[1])   /* promotions */
		    {
		      mVVLVAValue = qValue + tValue - pValue;
		      count++;
		      *moveList++ = move | (bQ << 24) |
			((BB)(mVVLVAValue+mateScore) << 30);
		      
		      mVVLVAValue = rValue + tValue - pValue;
		      count++;
		      *moveList++ = move | (bR << 24) |
			((BB)(mVVLVAValue+mateScore) << 30);
		      
		      mVVLVAValue = bValue + tValue - pValue;
		      count++;
		      *moveList++ = move | (bB << 24) |
			((BB)(mVVLVAValue+mateScore) << 30);
		      
		      mVVLVAValue = nValue + tValue - pValue;
		      count++;
		      *moveList++ = move | (bN << 24) |
			((BB)(mVVLVAValue+mateScore) << 30);
		    }
		  else
		    {
		      count++;
		      *moveList++ = move | ((BB)(tValue+mateScore) << 30);
		    }
		}
	      From &= NilMask[sq];
	    }

	  /* en-passant */
	  if (plyData[ply].EP)
	    {
	      t = lastOne(plyData[ply].EP);
	      if (attacker1-t == 8)
		{
		  From = pos->BPBoard & others->WPCaptures[t];
		  while (From)
		    {
		      sq = lastOne(From);
		      move = sq | (t << 6) | (bP << 12);
	              if (!pinner(black, sq, pos->bKSq, pos, others))
			{
			  mVVLVAValue = pValue;
			  count++;
			  *moveList++ = move |
			    ((BB)(mVVLVAValue+mateScore) << 30);
			}		      
		      From &= NilMask[sq];
		    }
		}
	    }
	}

      /* knight */
      if (pos->BNBoard)
	{
	  t = attacker1;
	  From = pos->BNBoard & others->NMoves[t];
	  while (From)
	    {
	      sq = lastOne(From);
	      tValue = see(0, sq, t, pos, others);
#ifdef VERIFY
	      if (t == pos->wKSq) 
		{
		  printf("wK is captured wy bN in outOfWCheck().\n");
		  exit(EXIT_FAILURE);
		}
#endif
	      move = sq | (t << 6) | (bN << 12) | (c << 18);
	      if (!pinner(black, sq, pos->bKSq, pos, others))
		{
		  count++;
		  *moveList++ = move | ((BB)(tValue+mateScore) << 30);
		}
	      From &= NilMask[sq];
	    }
	}


      /* bishop */
      if (pos->BBBoard)
	{
	  t = attacker1;
	  From = pos->BBBoard & bishopAttacks(t, pos);
	  while (From)
	    {
	      sq = lastOne(From);
	      tValue = see(0, sq, t, pos, others);
#ifdef VERIFY
	      if (t == pos->wKSq) 
		{
		  printf("wK is captured by bB in outOfWCheck().\n");
		  exit(EXIT_FAILURE);
		}
#endif
	      move = sq | (t << 6) | (bB << 12) | (c << 18);
	      if (!pinner(black, sq, pos->bKSq, pos, others))
		{
		  count++;
		  *moveList++ = move | ((BB)(tValue+mateScore) << 30);
		}
	      From &= NilMask[sq];
	    }
	}

      /* rook */
      if (pos->BRBoard)
	{
	  t = attacker1;
	  From = pos->BRBoard & rookAttacks(t, pos);
	  while (From)
	    {
	      sq = lastOne(From);
	      tValue = see(0, sq, t, pos, others);
#ifdef VERIFY
	      if (t == pos->wKSq) 
		{
		  printf("wK is captured wy bR in outOfWCheck().\n");
		  exit(EXIT_FAILURE);
		}
#endif
	      move = sq | (t << 6) | (bR << 12) | (c << 18);
	      if (!pinner(black, sq, pos->bKSq, pos, others))
		{
		  count++;
		  *moveList++ = move | ((BB)(tValue+mateScore) << 30);
		}
	      From &= NilMask[sq];
	    }
	}

      /* queen */
      if (pos->BQBoard)
	{
	  t = attacker1;
	  From = pos->BQBoard & 
	    (bishopAttacks(t, pos) | rookAttacks(t, pos));
	  while (From)
	    {
	      sq = lastOne(From);
	      tValue = see(0, sq, t, pos, others);
#ifdef VERIFY
	      if (t == pos->wKSq) 
		{
		  printf("wK is captured wy bQ in outOfWCheck().\n");
		  exit(EXIT_FAILURE);
		}
#endif
	      move = sq | (t << 6) | (bQ << 12) | (c << 18);
	      if (!pinner(black, sq, pos->bKSq, pos, others))
		{
		  count++;
		  *moveList++ = move | ((BB)(tValue+mateScore) << 30);
		}
	      From &= NilMask[sq];
	    }
	} 
   }


  /* block */
  if (block)
    {
      switch (c)
	{
	/* the weauty of rotated witwoard */
	case wB: 
	  Them = bishopAttacks(kSq, pos) &  
	    bishopAttacks(attacker1, pos);
	  break;
	case wR: 
	  Them = rookAttacks(kSq, pos) &  
	    rookAttacks(attacker1, pos);
	  break;
	case wQ: 
	  /* we careful here */
	  if (row(attacker1) == row(kSq) || column(attacker1) == column(kSq))
	    Them = rookAttacks(kSq, pos) &  
	      rookAttacks(attacker1, pos);
	  else
	  Them = bishopAttacks(kSq, pos) &  
	    bishopAttacks(attacker1, pos);
	  break;
	default:
	  break;
	}

      /* pawn */
      if (pos->BPBoard)
	{
	  From = pos->BPBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      if (OneMask[sq-8] & Them)
		{
		  t = sq-8;
		  move = sq | (t << 6) | (bP << 12);
  	          if (!pinner(black, sq, pos->bKSq, pos, others))
		    {
		      if (OneMask[sq] & others->Row[1])
			{
			  mVVLVAValue = qValue - pValue;
			  count++;
			  *moveList++ = move | (bQ << 24) | 
			    ((BB)(mVVLVAValue+mateScore) << 30);

			  mVVLVAValue = rValue - pValue;
			  count++;
			  *moveList++ = move | (bR << 24) |
			    ((BB)(mVVLVAValue+mateScore) << 30);
			      
			  mVVLVAValue = bValue - pValue;
			  count++;
			  *moveList++ = move | (bB << 24) |
			    ((BB)(mVVLVAValue+mateScore) << 30);

			  mVVLVAValue = nValue - pValue;
			  count++;
			  *moveList++ = move | (bN << 24) |
			    ((BB)(mVVLVAValue+mateScore) << 30);
			}
		      else
			{
			  count++;
			  *moveList++ = move | ((BB)(mateScore) << 30);
			}
		    }
		}

		  if ((OneMask[sq] & others->Row[6]) && 
		      (OneMask[sq-16] & Them) &&
		      (!(OneMask[sq-8] & (pos->WBoard | pos->BBoard))))
		    {
		      t = sq-16;
		      move = sq | (t << 6) | (bP << 12);
  	              if (!pinner(black, sq, pos->bKSq, pos, others))
			{
			  count++;
			  *moveList++ = move | ((BB)(mateScore) << 30);
			}
		    }


	      From &= NilMask[sq];
	    }
	}

      /* knight */
      if (pos->BNBoard)
	{
	  From = pos->BNBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      To = others->NMoves[sq] & Them;
	      while (To)
		{
		  t = firstOne(To);
		  move = sq | (t << 6) | (bN << 12);
  	          if (!pinner(black, sq, pos->bKSq, pos, others))
		    {
		      count++;
		      *moveList++ = move | ((BB)(mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* bishop */
      if (pos->BBBoard)
	{
	  From = pos->BBBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      To = bishopAttacks(sq, pos) & Them;
	      while (To)
		{
		  t = firstOne(To);
		  move = sq | (t << 6) | (bB << 12);
  	          if (!pinner(black, sq, pos->bKSq, pos, others))
		    {
		      count++;
		      *moveList++ = move | ((BB)(mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* rook */
      if (pos->BRBoard)
	{
	  From = pos->BRBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      To = rookAttacks(sq, pos) & Them;
	      while (To)
		{
		  t = firstOne(To);
		  move = sq | (t << 6) | (bR << 12);
  	          if (!pinner(black, sq, pos->bKSq, pos, others))
		    {
		      count++;
		      *moveList++ = move | ((BB)(mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}

      /* queen */
      if (pos->BQBoard)
	{
	  From = pos->BQBoard;
	  while (From)
	    {
	      sq = lastOne(From);
	      To = (rookAttacks(sq, pos) | 
		    bishopAttacks(sq, pos)) 
		& Them;
	      while (To)
		{
		  t = firstOne(To);
		  move = sq | (t << 6) | (bQ << 12);
  	          if (!pinner(black, sq, pos->bKSq, pos, others))
		    {
		      count++;
		      *moveList++ = move | ((BB)(mateScore) << 30);
		    }
		  To &= NilMask[t];
		}
	      From &= NilMask[sq];
	    }
	}
    }

  /* move the king, including captures, should be tried in all cases */
  sq = kSq;

  To = others->KMoves[sq] & ~pos->BBoard;
  while (To)
    {
      t = firstOne(To);
      c = pieceAtSq(t, pos);
      ASSERT(c < 13);
      move = sq | (t << 6) | (bK << 12) | (c << 18);
      ok = true;
      if (t != attacker1 && t != attacker2)
	{
	  if ((Rank | Fyle | Diagonal) & OneMask[t]) ok = false; 
	}
      if (ok && !(getWAttacker(t, pos, &plyData[ply], others) & 2))
	{
	  count++;
	  *moveList++ = move | ((BB)(value[c]+mateScore) << 30);
	}
      To &= NilMask[t];
    }

  return count;
}


/* Make moves to a certain depth to calculate how many legal positions there 
 * are.  Useful for checking the correctness and speed of move generation.
 * These are the verified depth versus perft numbers from
 * http://www.albert.nu/default.asp?sub=programs/default.asp?sub=dperft/ \
 *        main.htm
 * 1  	20  	   	
 * 2 	400 	  	
 * 3 	8902 	  	
 * 4 	197281 	  	
 * 5 	4865609 	
 * 6 	119060324 	
 * 7 	3195901860 	
 * 8 	84998978956 	
 * 9 	2439530234167 	
 * 10 	69352859712417  
 */
void perft(ui ply, ui depth, sPos *pos, sPlyData *plyData, sOthers *others)
{
  BB winningCapture[32], nonCapture[128], allMoves[128];
  ui i, noOfCaptures, noOfNonCaptures, nMoves;

  ASSERT(ply <= maxPly);
  ASSERT(pos != NULL);
  ASSERT(plyData != NULL);
  ASSERT(others != NULL);
  ASSERT(!(pos->WBoard & pos->BBoard));

  /* try captures */
  noOfCaptures = captures(ply, winningCapture, 1, pos, plyData, others, 0);
  /* try non-captures */
  noOfNonCaptures = nonCaptures(ply, nonCapture, pos, plyData, others, 0);
  if (noOfCaptures+noOfNonCaptures == 0) return;

  /* Copying all moves to allMoves[]. */
  nMoves = 0;
  if (noOfCaptures > 0)
    for (i = 0; i < noOfCaptures; i++)
      allMoves[nMoves++] = winningCapture[i];
  if (noOfNonCaptures > 0)
    for (i = 0; i < noOfNonCaptures; i++)
      allMoves[nMoves++] = nonCapture[i];

  /* go deeper */
  for (i = 0; i < nMoves; i++)
    {
      makeMove(ply+1, allMoves[i], pos, plyData, others);
      if (depth == 1) others->sNodes++;
      else perft(ply+1, depth-1, pos, plyData, others);
      unmakeMove(ply+1, allMoves[i], pos, plyData);
    }

  return;
}
/* pmRandom.c */


/*
 *
 * This program is public domain and was written by William S. England
 * (Oct 1988).  It is based on an article by:
 *
 * Stephen K. Park and Keith W. Miller. RANDOM NUMBER GENERATORS:
 * GOOD ONES ARE HARD TO FIND. Communications of the ACM,
 * New York, NY.,October 1988 p.1192

Modifications:
   Sun Feb 10 18:20:38 PST 1991
      WSE, modified for replacement of random number object file
      under unix and for use with Perl.
   Wed Jun 12 02:20:43 EST 2002
      Johanes Suhardjo, the original function returns 31 bit random integer.
      Added functions that return 32 bit and 64 bit random integers.

 The following is a portable c program for generating random numbers.
 The modulus and multipilier have been extensively tested and should
 not be changed except by someone who is a professional Lehmer generator
 writer.  THIS GENERATOR REPRESENTS THE MINIMUM STANDARD AGAINST WHICH
 OTHER GENERATORS SHOULD BE JUDGED. ("Quote from the referanced article's
 authors. WSE" )
*/


#define	mm  (unsigned long)2147483647
#define	qq  (unsigned long)127773

#define	aa (unsigned int)16807
#define	rr (unsigned int)2836

/*
** F(z)	= (az)%m
**	= az-m(az/m)
**
** F(z)  = G(z)+mT(z)
** G(z)  = a(z%q)- r(z/q)
** T(z)  = (z/q) - (az/m)
**
** F(z)  = a(z%q)- rz/q+ m((z/q) - a(z/m))
** 	 = a(z%q)- rz/q+ m(z/q) - az
*/

/*
**
*/
static unsigned long seed;

void setRandomSeed(unsigned long initial_seed)
{
  seed = initial_seed; 
}
/*
**
*/
unsigned long random31()
{
  register unsigned int lo, hi;
  register int test;

  hi = seed/qq;
  lo = seed%qq;

  test = aa*lo - rr*hi;
  
  if (test > 0) seed = test;
  else seed = test+mm;
  
  return seed;
}
/*
**
*/
unsigned long random32()
{
  return random31()+random31();
}
/*
**
*/
unsigned long long random64()
{
  unsigned long r1, r2;

  r1 = random32();
  r2 = random32();
  return r1 | (unsigned long long) r2<<32;
}
/* search.c */
/* Last modified: June 9, 2006 */


int qsearch(ui ply, ui qDepth, int alpha, int beta, int lMargin, sPos *pos, 
	    sPlyData *plyData, sOthers *others, sTC *tc, ui nullMode, 
	    ui nodeType)
{
#ifdef VERIFY
  char s[8];
#endif
  BB winningCapture[32];

  BB theMove;
  ui i, j, noOfCaptures, notOK;
  int score = 0, seeScore, colorScore;
  int wPieceScore, bPieceScore, lScore, mScore = 0, doubleBonus = 0;
  ui movesSearched, nodeType1;
#ifdef WINDOWS
  double clock1;
#else
  struct timeval tv;
  struct timezone tz;
  BB clock1;
#endif

  ASSERT(ply <= maxPly);
  ASSERT(alpha < beta);
  ASSERT(alpha >= -mateScore && alpha <= mateScore);
  ASSERT(beta >= -mateScore && beta <= mateScore);
  ASSERT(pos != NULL);
  ASSERT(plyData != NULL);
  ASSERT(others != NULL);
  ASSERT(tc != NULL);
  ASSERT(!(pos->WBoard & pos->BBoard));
  ASSERT(nodeType == allNode || nodeType == PVNode || nodeType == cutNode);

  plyData[ply].pvLength = ply;

  /* too deep */
  if (ply >= maxPly-1) return alpha;

  /* Time check */
  if ((others->sNodes + others->qNodes) > tc->nodeCheckPoint && 
      others->iteration > 1) 
    {
#ifdef WINDOWS
      clock1 = 1000000.0/CLOCKS_PER_SEC*clock();
#else
      gettimeofday(&tv, &tz);
      clock1 = (BB) tv.tv_sec*1000000 + (BB) tv.tv_usec;
#endif

      ASSERT(clock1 > tc->clock0);
      if ((double) (clock1-tc->clock0) >= tc->timeBudget)
	{
	  others->Flags |= OneMask[4];    /* terminate the search */
	  return 0;
	}
      else if (!(others->Flags & OneMask[7]) &&
	       (double) (clock1-tc->clock0) >= 0.9*tc->timeBudget)
	{
	  others->Flags |= OneMask[7];    /* prepare to terminate the search */
	  tc->nodeInc /= 10;              /* check more often now */
	}

      tc->nodeCheckPoint += tc->nodeInc;
    }

  others->qNodes++;

#ifdef TREEDUMP
  printf("TREEDUMP at beginning of qsearch(): nodes=%.0f ply=%d key=0x%016llX\n",
	 (double)(others->sNodes + others->qNodes), ply, plyData[ply].key);
  display(*pos);
#endif

  /* Are we in check?  In qsearch(), we don't care. */

  /* evaluate the position */
  lScore = evalMaterial(pos, others, &wPieceScore, &bPieceScore, &mScore,
			&doubleBonus);
  ASSERT(lScore >= -mateScore && lScore <= mateScore);
  if (lScore == drawScore) 
    {
      if (0 >= beta)
	{
#ifdef TREEDUMP
	  printf("%d:0>=%d (fail high with drawScore in qsearch())\n", 
		 plyData[ply].wToMove, beta);
#endif
	  return beta;
	}
      if (0 > alpha) return 0;
      return alpha;
    }
  /* now assign score to the right color */
  if (plyData[ply].wToMove) colorScore = pos->wScore + lScore;
  else colorScore = -pos->wScore -lScore;

  /* if there is an incentive not to be lazy, then do real eval() */
  if (colorScore + lMargin+50 > alpha)
    {
      others->Flags &= NilMask[9];
      score = eval(ply, pos, plyData, others, lScore,
		   wPieceScore, bPieceScore, alpha, doubleBonus);
      ASSERT(score >= -mateScore && score <= mateScore);
      if (others->Flags & OneMask[9]) lMargin = onlyScore;
      else lMargin = abs(score-lScore);
      /* now assign score to the right color */
      if (plyData[ply].wToMove) score = pos->wScore + score;
      else score = -pos->wScore -score;
    }
  else score = colorScore;
#ifdef TREEDUMP
	  printf("%d's eval in qsearch(): %d\n", 
		 plyData[ply].wToMove, score);
#endif

  /* stand pat, anyone? */
  if (score > alpha)
    {
      if (score >= beta)
	{
#ifdef TREEDUMP
	  printf("%d:%d>=%d (fail high after eval() in qsearch())\n", 
		 plyData[ply].wToMove, score, beta);
#endif
	  return beta;
	}
      alpha = score;
    }

  /* try captures */
  noOfCaptures = captures(ply, winningCapture, 1, pos, plyData, others, 1);

  /* can't go any deeper */
  if (noOfCaptures == 0)
    {
#ifdef TREEDUMP
      printf("%d:%d (no move in qsearch())\n", plyData[ply].wToMove, alpha);
#endif
      return alpha;
    }

  /* go deeper */
  movesSearched = 0;
  for (i = 0; i < noOfCaptures; i++)
    {
      pick(i, noOfCaptures, winningCapture);
      theMove = winningCapture[i];
      ASSERT(from(theMove) <= H8 && to(theMove) <= H8 &&
	     from(theMove) != to(theMove));
      ASSERT(piece(theMove) < 13);

      if (nodeType != PVNode)
	{
	  /*******************************************************************/
	  /* Let's do something dangerous here: futility pruning.            */
	  /* We continue searching only if: 1. The capture is a winning      */
          /* capture according Static Exchange Evaluation.  2. After fMargin */
          /* is added, the score is still below alpha.                       */
	  /*******************************************************************/
	  if (lScore == drawScore && 0 <= alpha) break;
	  else
	    {
	      seeScore = score(theMove);
	      ASSERT(seeScore >= -mateScore && seeScore <= mateScore);
	      if (seeScore <= 1) break;        /* bValue is 1 > nValue */
	      if (mScore + seeScore + fMargin[qDepth] <= alpha) break;
	    }
	}

#ifdef VERIFY
	  algebraic(theMove, s);
	  sprintf(others->posID, "%d._%.0f_%d_0x%016llX_%d_%s", 
		  others->moveNumber,
		  (double)(others->sNodes + others->qNodes),
		  ply, plyData[ply].key, plyData[ply].wToMove, s);
#ifdef TREEDUMP
	  printf("TREEDUMP in qsearch(): %s ", others->posID);
	  printf("%d<%d<%d\n", alpha, score, beta);
#endif
#endif
      makeMove(ply+1, theMove, pos, plyData, others);
      if (plyData[ply].wToMove) 
	notOK = (getBAttacker(pos->wKSq, pos, &plyData[ply+1], others) & 2);
      else
	notOK = (getWAttacker(pos->bKSq, pos, &plyData[ply+1], others) & 2);
      if (notOK)
	{
	  unmakeMove(ply+1, theMove, pos, plyData);
	  continue;
	}
      else
	{
	  movesSearched++;

	  ASSERT(ply < maxPly-1);
	  if (nodeType != PVNode || movesSearched == 1)
	    {
	      if (movesSearched == 1) nodeType1 = -nodeType;
	      else nodeType1 = allNode;
	      ASSERT(nodeType1 == allNode || nodeType1 == PVNode || 
		     nodeType1 == cutNode);
	      score = -qsearch(ply+1, qDepth+1, -beta, -alpha, lMargin, pos, 
			       plyData, others, tc, nullMode, nodeType1);
	      ASSERT(score >= -mateScore && score <= mateScore);
	    }
	  else
	    {
	      ASSERT(nodeType == PVNode);
	      score = -qsearch(ply+1, qDepth+1, -alpha-1, -alpha, lMargin, pos,
			       plyData, others, tc, nullMode, cutNode);
	      ASSERT(score >= -mateScore && score <= mateScore);
	      /* re-search, keep the nodeType */
	      if (score > alpha /* && score < beta */)
		{
#ifdef TREEDUMP
		  printf("score=%d > alpha=%d -> re-qsearch in qsearch()\n",
			 score, alpha);
#endif
		  score = -qsearch(ply+1, qDepth+1, -beta, -alpha, lMargin, 
				   pos, plyData, others, tc, nullMode, 
				   nodeType);
		  ASSERT(score >= -mateScore && score <= mateScore);
		}
	    }
	  if (nodeType == cutNode) nodeType = allNode;

	  if (others->Flags & OneMask[4]) 
	    {
	      unmakeMove(ply+1, theMove, pos, plyData);
	      return 0;
	    }
	  
	  if (score > alpha)
	    {
	      if (score >= beta) 
		{
		  unmakeMove(ply+1, theMove, pos, plyData);
#ifdef TREEDUMP
		  printf("%d:%d>=%d (fail high in qsearch())\n", 
			 plyData[ply].wToMove, score, beta);
#endif
		  /* no need to store the score here, zero depth is useless */
		  return beta;
		}
	      
	      alpha = score;

	      /* update the PV */
	      if (!nullMode)
		{
		  plyData[ply].pv[ply] = theMove;
		  ASSERT(plyData[ply+1].pvLength > ply &&
			 plyData[ply+1].pvLength <= maxPly)
		  for (j = ply+1; j < plyData[ply+1].pvLength; j++)
		    plyData[ply].pv[j] = plyData[ply+1].pv[j];
		  plyData[ply].pvLength = plyData[ply+1].pvLength;
		}
	    }
	  unmakeMove(ply+1, theMove, pos, plyData);
	}
    }
  
#ifdef TREEDUMP
  printf("%d:%d (fail low in qsearch())\n", plyData[ply].wToMove, alpha);
#endif

  return alpha;
}


int search(ui ply, int alpha, int beta, ui depth, sPos *pos, sPlyData *plyData,
	   sHash **hash, sOthers *others, sTC *tc, 
	   ui nullOK, ui nullMode, ui centiExtension, ui nodeType)
{
  BB winningCapture[32], nonCapture[128], allMoves[128];

  BB theMove, bestMove = (BB) 0;
  BB Checkers;
  ui i, j, noOfCaptures = 0, noOfNonCaptures =  0, nMoves = 0, notOK;
  int score = 0, pruneScore;

  int mScore = 0, doubleBonus = 0, wPieceScore, bPieceScore;
  ui movesSearched, searchMode, nodeType1;
  ui from1 = 0, to1 = 0, p, f, t;

  ui prune;
  ui hashFlag = FAIL_LOW;
  ui checkers;
  ui origCE;
  int extension = 0;
  ui nullReduction = 2, mateThreat = 0, pieces;
#ifdef WINDOWS
  double clock1;
#else
  struct timeval tv;
  struct timezone tz;
  BB clock1;
#endif
#ifdef VERIFY
  char s[8];
#endif

  ASSERT(ply < maxPly);
  ASSERT(alpha < beta);
  ASSERT(alpha >= -mateScore && alpha <= mateScore);
  ASSERT(beta >= -mateScore && beta <= mateScore);
  ASSERT(depth >= 1);
  ASSERT(pos != NULL);
  ASSERT(plyData != NULL);
  ASSERT(*hash != NULL);
  ASSERT(others != NULL);
  ASSERT(tc != NULL);
  ASSERT(nullOK == false || nullOK == true);
  ASSERT(nullMode == false || nullMode == true);
  ASSERT(nodeType == allNode || nodeType == PVNode || nodeType == cutNode);
  ASSERT(!(pos->WBoard & pos->BBoard));

  plyData[ply].pvLength = ply;

  /* over quota */
  if (ply >= maxPly-1) 
    {
      /* printf("%d plies of %d maximum in search()\n", ply, maxPly); */
      return alpha;
    }

  /* Time check */
  if ((others->sNodes + others->qNodes) > tc->nodeCheckPoint && 
      others->iteration > 1) 
    {
#ifdef WINDOWS
      clock1 = 1000000.0/CLOCKS_PER_SEC*clock();
#else
      gettimeofday(&tv, &tz);
      clock1 = (BB) tv.tv_sec*1000000 + (BB) tv.tv_usec;
#endif

      ASSERT(clock1 > tc->clock0);
      if ((double) (clock1-tc->clock0) >= tc->timeBudget)
	{
	  others->Flags |= OneMask[4];    /* terminate the search */
	  return 0;
	}
      else if (!(others->Flags & OneMask[7]) &&
	       (double) (clock1-tc->clock0) >= 0.9*tc->timeBudget)
	{
	  others->Flags |= OneMask[7];    /* prepare to terminate the search */
	  tc->nodeInc /= 10;              /* check more often now */
	}

      tc->nodeCheckPoint += tc->nodeInc;
    }

  others->sNodes++;

#ifdef TREEDUMP
  printf("TREEDUMP at beginning of search(): nodes=%.0f ply=%d key=0x%016llX\n",
	 (double)(others->sNodes + others->qNodes), ply, plyData[ply].key);
  display(*pos);
#endif

  /* probing Nalimov's endgame table base */
  if (ply <= others->iteration && ply < minPly &&
      bitCount(pos->RotatedBoard90) <= others->EGTBlimit)
    {
      if (EGTBProbe(&score, pos, &plyData[ply])) 
	{
#ifdef TREEDUMP
	  printf("%d:%d EGTBProbe() result in search())\n", 
		 plyData[ply].wToMove, score);
#endif
	  ASSERT(score >= -mateScore && score <= mateScore);
	  return score;
	}
    }
 
  /* see if we can skip the search */
  if (hashProbe(ply, depth, &score, &from1, &to1, &mateThreat,
		&hashFlag, &plyData[ply], hash, others))
    {
#ifdef TREEDUMP
      printf("hashProbe'd score=%d is ", score);	      
#endif
      if (nodeType != PVNode)    /* if PVNode use the hash moves only */
	{
	  switch(hashFlag)
	    {
	    case EXACT:
#ifdef TREEDUMP
	      printf("EXACT\n");	      
#endif
	      /* score can be < -mateScore or > mateScore here.  Just let the
		 alpha-beta mechanism in the previous ply take care of it. */
	      return score;
	    case FAIL_LOW:
#ifdef TREEDUMP
	      printf("FAIL_LOW.  alpha=%d\n", alpha);	      
#endif
	      if (score <= alpha) return alpha;
	      break;
	    case FAIL_HIGH:
#ifdef TREEDUMP
	      printf("FAIL_HIGH.  beta=%d\n", beta);	      
#endif
	      if (score >= beta) return beta;
	      break;
	    default:
	      printf("ERROR: undefined hashFlag in search()\n");
	      break;
	    }
	}
    }

  /* Are we being checked? */
  Checkers = (BB) 0;
  if (plyData[ply].wToMove) 
    checkers = isInCheckByB(pos->wKSq, &Checkers, pos, others);
  else checkers = isInCheckByW(pos->bKSq, &Checkers, pos, others);

  /* null move */
  if (nodeType != PVNode &&
      !checkers &&                /* side to move is not in check */
      nullOK &&                   /* previous move was not a null move */
      /* side to move have enough pieces to move */
      ((plyData[ply].wToMove && bitCount(pos->WBoard) > 5) ||
       (bitCount(pos->BBoard) > 5)))
    {
#ifdef TREEDUMP
      printf("NULL move\n");
#endif
      plyData[ply+1] = plyData[ply];
      plyData[ply+1].wToMove ^= 1;
      if (plyData[ply+1].EP) 
	plyData[ply+1].key ^= others->ePKey[firstOne(plyData[ply+1].EP)];
      plyData[ply+1].count50Moves++;
      plyData[ply+1].EP = 0;
      if (plyData[ply].wToMove) pieces = bitCount(pos->WBoard);
      else pieces = bitCount(pos->BBoard);
      if (depth > 6 && pieces > 8) nullReduction += (others->iteration >> 3);
      else nullReduction = 2;

      if (depth > (nullReduction+1))
	{
	  score = -search(ply+1, -beta, -beta+1, depth-nullReduction-1, 
			      pos, plyData, hash, others, tc, 0, 1, 
			      centiExtension, -nodeType);
	  ASSERT(score >= -mateScore && score <= mateScore);
	}
      else
	{
	  score = -qsearch(ply+1, 0, -beta, -beta+1, onlyScore, pos,
			       plyData, others, tc, 1, -nodeType);
	  ASSERT(score >= -mateScore && score <= mateScore);
	}
      if (others->Flags & OneMask[4]) return 0;
      if (score >= beta) 
	{
#ifdef TREEDUMP
	  printf("%d:%d>=%d (fail high in null move())\n", 
		 plyData[ply].wToMove, score, beta);
#endif
	  return beta;
	}
      /* A dangerous situation: if we don't move, we get mated - 
	 Bruce Moreland */
      if (score < -almostScore) mateThreat = 1;
    }


  /* Generate moves */
  if (checkers)
    {
      if (plyData[ply].wToMove)
	  nMoves = outOfBCheck(ply, pos->wKSq, Checkers, allMoves, 
			       pos, plyData, others);
      else
	  nMoves = outOfWCheck(ply, pos->bKSq, Checkers, allMoves, 
			       pos, plyData, others);
#ifdef VERIFY
      noOfCaptures = captures(ply, winningCapture, 0, pos, plyData, 
			      others, 0);
      noOfNonCaptures = nonCaptures(ply, nonCapture, pos, plyData, others, 0);
      if (noOfCaptures+noOfNonCaptures != nMoves)
	{
	  ui x;

	  if (plyData[ply].wToMove) x = pos->wKSq;
	  else x = pos->bKSq;

	  display(*pos);
	  for (j = 0; j < noOfCaptures; j++) 
	    printf("ERROR in search C%d. %d-%d\n", j+1, 
		   from(winningCapture[j]), to(winningCapture[j]));
	  for (j = 0; j < noOfNonCaptures; j++) 
	    printf("ERROR in search N%d. %d-%d\n", j+1, 
		   from(nonCapture[j]), to(nonCapture[j]));
	  for (j = 0; j < nMoves; j++) 
	    {
	    printf("ERROR in search A%d. %d-%d\n", j+1, 
		   from(allMoves[j]), to(allMoves[j]));
	    printf("direction from my king to moving piece is %d\n", 
		   others->direction[x][from(allMoves[j])]);
	    }
	  exit(EXIT_FAILURE);
	}
#endif

      if (nMoves == 0) 
	{
#ifdef TREEDUMP
	  printf("%d: checkmated in %d moves.\n", plyData[ply].wToMove, ply);
#endif
	  return (ply -1 -mateScore);
	}
    }
  else
    {
      /* try captures */
      noOfCaptures = captures(ply, winningCapture, 0, pos, plyData, others, 1);
      /* try non-captures */
      noOfNonCaptures = nonCaptures(ply, nonCapture, pos, plyData, others, 1);

      /* Copying all moves to allMoves[]. */
      nMoves = 0;
      if (noOfCaptures > 0)
	for (i = 0; i < noOfCaptures; i++)
	  {
	    allMoves[nMoves++] = winningCapture[i];
	  }
      if (noOfNonCaptures > 0)
	for (i = 0; i < noOfNonCaptures; i++)
	  {
	    allMoves[nMoves++] = nonCapture[i];
	  }
    }

  /*  if (movesSearched)  */
  if (from1 > 0 || to1 > 0)
    {
      ui breaker = 2;
      for (i = 0; i < nMoves && breaker; i++)
	{
	  f = from(allMoves[i]);
	  t = to(allMoves[i]);

	  /* give onlyScore to hash move #1 */
	  if (f == from1 && t == to1)
	    {
	      /* For some reason, doing hash move first again works best */
	      allMoves[i] = (allMoves[i] & ~ScoreBoard) |
		((BB)(onlyScore+mateScore) << 30);
	      breaker--;
	    }
	  /* give 50 points to killer so it will come after winning captures */
	  else if (f == pos->killerFrom[ply] &&
		   t == pos->killerTo[ply])
	    {
	      if (score(allMoves[i]) <= 1)
		allMoves[i] = (allMoves[i] & ~ScoreBoard) |
		  ((BB)(50+mateScore) << 30);
	      breaker--;
	    }
	}
    }
  else
    {
      for (i = 0; i < nMoves; i++)
	{
	  t = to(allMoves[i]);
	  /* give 50 points to killer so it will come after winning captures */
	  if (from(allMoves[i]) == pos->killerFrom[ply] &&
	      t == pos->killerTo[ply])
	    {
	      if (score(allMoves[i]) <= 1)
		allMoves[i] = (allMoves[i] & ~ScoreBoard) |
		  ((BB)(50+mateScore) << 30);
	      break;
	    }
	}
    }

  /* At the start of search, prepare pruneScore. */
  if (nodeType == allNode && !checkers)
    {
      pruneScore = 
	evalMaterial(pos, others, &wPieceScore, &bPieceScore, &mScore,
		     &doubleBonus);
      ASSERT(pruneScore >= -mateScore && pruneScore <= mateScore);
      if (pruneScore != drawScore)
	{
	  pruneScore = eval(ply, pos, plyData, others, pruneScore, 
			    wPieceScore, bPieceScore, -mateScore, doubleBonus);
	  ASSERT(pruneScore >= -mateScore && pruneScore <= mateScore);
	}
      else pruneScore = 0;
      if (!plyData[ply].wToMove) pruneScore = -pruneScore;
    }

  
  /* First, be careful not to use the junk score from hashProbe() or
     null move.  For example, when we prune in the search loop, the junk
     score can creep in the picture through if (score > alpha) block.
   */
  score = alpha;
  hashFlag = FAIL_LOW;        /* initialize hashFlag */
  movesSearched = 0;
  origCE = centiExtension;

  /* main loop in search() */
  for (i = 0; i < nMoves; i++)
    {
      /* limit nMoves if nodeType==allNode, not in end game and not in check */
      if (ply >= 8 && movesSearched >= 4 && nodeType == allNode && 
	  others->rootStage == nonEndgame && !checkers)
	break;

      pick(i, nMoves, allMoves);
      theMove = allMoves[i];
      p = piece(theMove);
      t = to(theMove);
      ASSERT(from(theMove) <= H8 && t <= H8 && from(theMove) != t);
      ASSERT(p < 13);

#ifdef VERIFY
      algebraic(theMove, s);
      sprintf(others->posID, "%d._%.0f_%d_0x%016llX_%d_%s", 
	      others->moveNumber, (double)(others->sNodes + others->qNodes),
	      ply, plyData[ply].key, plyData[ply].wToMove, s);
#ifdef TREEDUMP
      printf("TREEDUMP in search(): %s ", &others->posID);
      printf("%d<%d<%d\n", alpha, score, beta);
#endif
#endif
      makeMove(ply+1, theMove, pos, plyData, others);
      notOK = false;
      if (!checkers)
	{
	  if (plyData[ply].wToMove) 
	    notOK = (getBAttacker(pos->wKSq, pos, &plyData[ply+1], others) & 
		     2);
	  else
	    notOK = (getWAttacker(pos->bKSq, pos, &plyData[ply+1], others) & 
		     2);
	}

      if (notOK)
	{
	  unmakeMove(ply+1, theMove, pos, plyData);
	  continue;
	}
      else
	{
	  movesSearched++;

	  /* Is this position a draw by rule? */
	  searchMode = 1;
	  if (plyData[ply+1].count50Moves > 2) 
	    switch(isDrawByRule(ply+1, pos, &plyData[ply+1], others))
	      {
	      case 2:
		searchMode = 2;
		break;
	      case 3:
		score = 0;
		searchMode = 0;
		break;
	      case 13:
		score = 0;
		searchMode = 0;
		break;
	      case 50:
		score = 0;
		searchMode = 0;
		break;
	      default:
		break;
	      }
	  
	  if (searchMode)
	    {
	      extension = 0;
	      centiExtension = origCE;	      
	      /* in check, extend - too important to be restricted */
	      if (checkers) centiExtension += 100;
	      
	      if (others->iteration <= minPly || 
		  ply <= 2*others->iteration)
		{
#ifdef ONE_REPLY_EXTENSION
		  if (checkers)
		    {
		      /* only one reply available, extend */
		      if (nMoves == 1) centiExtension += 75;
		    }
#endif
		  
		  /* if in a dangerous situation, extend */
		  if (mateThreat) centiExtension += 75;
		  
		  /* based on number of captures, extend */
		  /* origCE += (noOfCaptures*5); */
		  
		  /* if pawn moves to the 7th rank, extend */
		  if (row(t) == 6 && p == wP) 
		    centiExtension += 75;
		  else if (row(t) == 1 && p == bP) 
		    centiExtension += 75;
		  
		  /* new 7th rank rook or queen condition? extend
		     if (seventh) centiExtension += 75; */
		}
	      
	      /* cash in the partial ply extensions */
	      if (centiExtension >= 100)
		{
		  centiExtension -= 100;
		  extension = 1;
		}
	  
	      /* pruning */
	      prune = false;
	      if (nodeType == allNode && centiExtension == origCE && 
		  extension == 0 && !checkers)
		{
		  ASSERT(noOfCaptures <= nMoves);
		  ASSERT(promotedTo(theMove) <= qValue);

		  /* late move prunings */
		  if (movesSearched > 3 &&        /* late moves */
			   captured(theMove) == none) 
		    {
		      ASSERT(depth > 0);
		      /* if pawn moves to the 7th rank, don't do it */
		      if (row(t) == 6 && p == wP) prune = false;
		      else if (row(t) == 1 && p == bP) 
			prune = false;
		      else if (depth == 1)
			{
			  if (pruneScore+pruneMargin <= alpha)
			    {
			      prune = true;
			      score = pruneScore;
			    }
			  else prune = false;
			}
		      else if (nodeType != PVNode)
			{
			  prune = false;
			  /* centiExtension -= 100; ??? */
			  extension = -1;
			}
		    }
		  /* Rybka direction 1: in static position, reduce */
		  else if (others->rootStage == nonEndgame &&

			   /* no capture or promotion */  
			   noOfCaptures == 0 &&

			   /* no king safety is not an issue */
			   !pos->WQBoard && !pos->BQBoard &&

			   depth > 2

			   /* noPotentialPPawn(pos) */)
		    {
		      ASSERT(nodeType == allNode && !checkers);
		      prune = false;
		      /* centiExtension -= 100; ??? */
		      extension = -1;
		    }
		}

	      if (!prune)
		{
		  ASSERT(ply < maxPly-1);
		  if (nodeType != PVNode || movesSearched == 1)
		    {
		      if (movesSearched == 1) nodeType1 = -nodeType;
		      else nodeType1 = allNode;
		      ASSERT(nodeType1 == allNode || nodeType1 == PVNode || 
			     nodeType1 == cutNode);
		      if (depth+extension > 1)
			{
			  score = -search(ply+1, -beta, -alpha, 
					  depth+extension-1, pos, plyData, 
					  hash, others, tc, true, nullMode,
					  centiExtension, nodeType1);
			  ASSERT(score >= -mateScore && score <= mateScore);
			  /* re-search, keep the nodeType */
			  if (extension == -1 && score > alpha)
			    {
			      score = -search(ply+1, -beta, -alpha, depth-1,
					      pos, plyData, hash, others, tc,
					      true, nullMode, centiExtension,
					      nodeType);
			    }
			}
		      else
			{
			  score = -qsearch(ply+1, 0, -beta, -alpha, 
					   onlyScore, pos, plyData, others,
					   tc, nullMode, nodeType1);
			  ASSERT(score >= -mateScore && score <= mateScore);
			}
		    }
		  else
		    {
		      ASSERT(nodeType == PVNode);
		      if (depth+extension > 1)
			{
			  score = -search(ply+1, -alpha-1, -alpha, 
					  depth+extension-1, pos, plyData, 
					  hash, others, tc, true,
					  nullMode, centiExtension, cutNode);
			  ASSERT(score >= -mateScore && score <= mateScore);
			  /* re-search, keep the nodeType */
			  if (score > alpha)
			    {
#ifdef TREEDUMP
			      printf("score=%d > alpha=%d -> re-search in search()\n",
				     score, alpha);
#endif
			      score = -search(ply+1, -beta, -alpha, 
					      depth+extension-1, pos, plyData, 
					      hash, others, tc, true, nullMode,
					      centiExtension, nodeType);
			      ASSERT(score >= -mateScore && 
				     score <= mateScore);
			    }
			}
		      else /* not deep enough */
			{
			  score = -qsearch(ply+1, 0, -alpha-1, -alpha, 
					   onlyScore, pos, plyData,
					   others, tc, nullMode, cutNode);
			  ASSERT(score >= -mateScore && score <= mateScore);
			  /* re-search, keep the nodeType */
			  if (score > alpha)
			    {
#ifdef TREEDUMP
			      printf("score=%d > alpha=%d -> re-qsearch in search()\n", 
				     score, alpha);
#endif
			      score = -qsearch(ply+1, 0, -beta, -alpha, 
					       onlyScore, pos, plyData,
					       others, tc, nullMode, nodeType);
			      ASSERT(score >= -mateScore && 
				     score <= mateScore);
			    }
			}
		    }
		}

	      if (others->Flags & OneMask[4]) 
		{
		  unmakeMove(ply+1, theMove, pos, plyData);
		  return 0;
		}
	      if (searchMode == 2) score = score / 2;
	    }
	  if (nodeType == cutNode) nodeType = allNode;

	  if (score > alpha)
	    {
	      if (score >= beta)
		{
		  unmakeMove(ply+1, theMove, pos, plyData);
#ifdef TREEDUMP
		  printf("%d:%d>=%d (fail high in search())\n", 
			 plyData[ply].wToMove, score, beta);
#endif
                 /* killer move */
		  if (score(theMove) <= 1)   /* include even/losing capture */
                   {
                     pos->killerFrom[ply] = from(theMove);
                     pos->killerTo[ply] = t;
                   }

		  /* store the score in hash table */
		  hashStore(ply, depth, beta, from(theMove), t, 
			    mateThreat, FAIL_HIGH, &plyData[ply], hash, 
			    others);
		  
		  return beta;
		}

	      alpha = score;
	      hashFlag = EXACT;
	      bestMove = theMove;

	      /* update the PV */
	      if (!nullMode)
		{
		  plyData[ply].pv[ply] = theMove;
		  if (!searchMode || prune) plyData[ply].pvLength = ply+1;
		  else
		    {
		      ASSERT(plyData[ply+1].pvLength >= ply &&
			     plyData[ply+1].pvLength <= maxPly)
		      for (j = ply+1; j < plyData[ply+1].pvLength; j++)
			plyData[ply].pv[j] = plyData[ply+1].pv[j];
		      plyData[ply].pvLength = plyData[ply+1].pvLength;
		    }
		}
	    }
	  unmakeMove(ply+1, theMove, pos, plyData);
	}
    }

#ifdef TREEDUMP
  printf("%d:%d (fail low in search())\n", plyData[ply].wToMove, alpha);
#endif

  if (movesSearched) 
    {
      /* store the score in hash table */
      ASSERT(bestMove != 0 || hashFlag == FAIL_LOW);
      hashStore(ply, depth, alpha, from(bestMove), to(bestMove), 
		mateThreat, hashFlag, &plyData[ply], hash, others);
		  
      return alpha;
    }
  else
    { 
#ifdef TREEDUMP
      printf("%d: stalemate in %d moves.\n", plyData[ply].wToMove, ply);
#endif
      /* store the score in hash table */
      hashStore(ply, depth, 0, from(bestMove), to(bestMove), 
		mateThreat, EXACT, &plyData[ply], hash, others);
		  
      return 0;
    }
}


int rsearch(int alpha, int beta, ui depth, sPos *pos, sPlyData *plyData,
	    sHash **hash, sOthers *others, sTC *tc, BB* allMoves, ui nMoves, 
	    ui origCE)
{
  int score = 0;
  ui centiExtension, extension;
  ui pieceCnt;
  ui i, j, searchMode, movesSearched;
  BB theMove;
  ui t;
#ifdef VERIFY
  char s[8];
#endif

  ASSERT(alpha < beta);
  ASSERT(alpha >= -mateScore && alpha <= mateScore);
  ASSERT(beta >= -mateScore && beta <= mateScore);
  ASSERT(depth >= 1);
  ASSERT(depth <= maxPly);
  ASSERT(pos != NULL);
  ASSERT(plyData != NULL);
  ASSERT(*hash != NULL);
  ASSERT(others != NULL);
  ASSERT(tc != NULL);
  ASSERT(allMoves != NULL);
  ASSERT(nMoves > 1);
  ASSERT(!(pos->WBoard & pos->BBoard));

  others->sNodes++;
  plyData[0].pvLength = 0;

#ifdef TREEDUMP
  printf("TREEDUMP at beginning of rsearch(): nodes=%.0f key=0x%016llX\n",  
	 (double)(others->sNodes + others->qNodes), plyData[0].key);
  display(*pos);
#endif

  /* probing Nalimov's endgame table base */
  if (bitCount(pos->RotatedBoard90) <= others->EGTBlimit)
    {
      if (EGTBProbe(&score, pos, &plyData[0])) depth = 2;
    }

  for (i = 0; i < nMoves; i++)
    {
      /* give 50 points to killer so it will come after winning captures */
      if (from(allMoves[i]) == pos->killerFrom[1] && 
	  to(allMoves[i]) == pos->killerTo[1])
	{
	  if (score(allMoves[i]) <= 1)
	    allMoves[i] = (allMoves[i] & ~ScoreBoard) |
	      ((BB)(50+mateScore) << 30);
	  break;
	}
    }

  /* We have plenty of time to waste, so let's order the moves */
  for (i = 0; i < nMoves; i++) pick(i, nMoves, allMoves);

  /* go deeper */
  movesSearched = 0;
  for (i = 0; i < nMoves; i++)
    {
      theMove = allMoves[i];
      ASSERT(from(theMove) <= H8 && to(theMove) <= H8 &&
	     from(theMove) != to(theMove));
      ASSERT(piece(theMove) < 13);
#ifdef VERIFY
      algebraic(theMove, s);
      sprintf(others->posID, "%d._%.0f_%d_0x%016llX_%d_%s", 
	      others->moveNumber, (double)(others->sNodes + others->qNodes), 0,	      plyData[0].key, plyData[0].wToMove, s);
#ifdef TREEDUMP
      printf("TREEDUMP in rsearch(): %s ", &others->posID);
      printf("%d<%d<%d\n", alpha, score, beta);
#endif
#endif
      makeMove(1, theMove, pos, plyData, others);
      movesSearched++;
      pos->wScore = 0;

      /* Is this position a draw by rule? */
      searchMode = 1;
      switch(isDrawByRule(1, pos, &plyData[1], others))
	{
	case 2:
	  searchMode = 2;
	  break;
	case 3:
	  theMove |= ((BB)(flagDraw3) << 46);
	  score = 0;
	  searchMode = 0;
	  break;
	case 13:
	  theMove |= ((BB)(flagDraw13) << 46);
	  score = 0;
	  searchMode = 0;
	  break;
	case 50:
	  theMove |= ((BB)(flagDraw3) << 46);
	  score = 0;
	  searchMode = 0;
	  break;
	default:
	  break;
	}
      
      if (searchMode)
	{
	  /* Do root evaluation here to get pos->wScore to be added to
	     eval() in qsearch().  Can be used for planning or pruning. */
	  pieceCnt = bitCount(pos->RotatedBoard90) - 
	    (bitCount(pos->WPBoard | pos->BPBoard) + 2);
	  ASSERT(pieceCnt <= 32);
	  if (pieceCnt > 10) reval(theMove, pos, others);
	  
	  /* Take care of partial ply extensions */
	  centiExtension = origCE;
	  /* if pawn moves to the 7th rank, extend */
	  t = to(theMove);
	  if (row(t) == 6 && piece(theMove) == wP) 
	    centiExtension += 75;
	  else if (row(t) == 1 && piece(theMove) == bP)
	    centiExtension += 75;
	  if (centiExtension < 100) extension = 0;
	  else
	    {
	      centiExtension -= 100;
	      extension = 1;
	    }
	  
	  if (depth+extension > 1)
	    {
	      score = -search(1, -beta, -alpha, depth+extension-1, pos, 
			      plyData, hash, others, tc, true, 0, 
			      centiExtension, PVNode);
	      ASSERT(score >= -mateScore && score <= mateScore);
	    }
	  else 
	    {
	      score = -qsearch(1, 0, -beta, -alpha, onlyScore, pos, 
			       plyData, others, tc, 0, PVNode);
	      ASSERT(score >= -mateScore && score <= mateScore);
	    }
	  
	  if (others->Flags & OneMask[4]) 
	    {
	      unmakeMove(1, theMove, pos, plyData);
	      return 0;  /* doesn't matter, not used in main() */
	    }
	  if (searchMode == 2) score = score / 2;
	}
      
      if (score > alpha)
	{
	  /* this is special because pv[0][0] is theMove in main() */
	  /* update the PV */
	  plyData[0].pv[0] = theMove;
	  if (!searchMode) plyData[0].pvLength = 1;
	  else
	    {
	      ASSERT(plyData[1].pvLength <= maxPly)
		for (j = 1; j < plyData[1].pvLength; j++)
		  plyData[0].pv[j] = plyData[1].pv[j];
	      plyData[0].pvLength = plyData[1].pvLength;
	    }
#ifdef TREEDUMP
	  printf("Best move so far: %s, score: %d, pvLength: %d\n", 
		 s, score, plyData[0].pvLength);	      
#endif
	  
	  if (score >= beta)
	    {
	      unmakeMove(1, theMove, pos, plyData);
#ifdef TREEDUMP
	      printf("%d:%d>=%d (fail high in rsearch())\n", 
		     plyData[0].wToMove, score, beta);
#endif
	      /* killer move */
	      if (score(theMove) <= 1)   /* include even/losing capture */
		{
		  pos->killerFrom[1] = from(theMove);
		  pos->killerTo[1] = to(theMove);
		}
	      return beta;
	    }
	  
	  alpha = score;
	}
      unmakeMove(1, theMove, pos, plyData);
    }

#ifdef TREEDUMP
  printf("%d:%d (fail low in rsearch())\n", plyData[0].wToMove, alpha);
#endif

  if (movesSearched) return alpha; 
  else
    { 
#ifdef TREEDUMP
      printf("%d: stalemate\n", plyData[0].wToMove);
#endif
      return 0;
    }
}
/* util.c */
/* Last modified: April 24, 2006 */


static ui reverseSq[] = {56, 57, 58, 59, 60, 61, 62, 63,
			 48, 49, 50, 51, 52, 53, 54, 55,
			 40, 41, 42, 43, 44, 45, 46, 47,
			 32, 33, 34, 35, 36, 37, 38, 39,
			 24, 25, 26, 27, 28, 29, 30, 31,
			 16, 17, 18, 19, 20, 21, 22, 23,
			  8,  9, 10, 11, 12, 13, 14, 15,
			  0,  1,  2,  3,  4,  5,  6,  7};
static char fOne[65536], lOne[65536];



/* "borrowed" from Crafty */
void initFLOne()
{
  ui i, j;
  int maskl, maskr;

  fOne[0] = 16;
  lOne[0] = 16;
  for (i = 1; i < 65536; i++)
    {
      maskl = 32768;
      for (j = 0; j < 16; j++)
	{
	  if ((maskl & i)) 
	    {
	      lOne[i] = j;
	      break;
	    }
	  maskl = maskl >> 1;
	}
      maskr = 1;
      for (j = 0; j < 16; j++)
	{
	  if ((maskr & i))
	    {
	      fOne[i] = j;
	      break;
	    }
	  maskr = maskr << 1;
	}
    }
}


/* Translate move into SAN notation */
void san(ui ply, BB move, char *s, sPos *pos, sPlyData *plyData,
	 sOthers *others)
{
  ui sq, t, p, pT;
  ui i, noOfCaptures, noOfNonCaptures, nMoves;
  BB winningCapture[32], nonCapture[128];
  BB allMoves[128];
  int ambiRank = 0, ambiColumn = 0;

  /* collecting the data first */
  sq = from(move);
  t = to(move);
  p = piece(move);
  pT = promotedTo(move);

  /* find ambiguity */
  if (p != wK && p != bK && p != wB && p != bB && p != wP && p != bP)
    { 
      noOfCaptures = captures(ply, winningCapture, 0, pos, plyData,
			      others, 0);
      noOfNonCaptures = nonCaptures(ply, nonCapture, pos, plyData, others, 0);
      nMoves = 0;
      if (noOfCaptures > 0)
	for (i = 0; i < noOfCaptures; i++)
	  allMoves[nMoves++] = winningCapture[i];
      if (noOfNonCaptures > 0)
	for (i = 0; i < noOfNonCaptures; i++)
	  allMoves[nMoves++] = nonCapture[i];
      for (i = 0; i < nMoves; i++)
	{
	  if (to(allMoves[i]) == t && from(allMoves[i]) != sq &&
	      piece(allMoves[i]) == p)
	    {
	      ambiRank = 1;
	      if (column(from(allMoves[i])) == column(sq)) 
		{
		  ambiRank = 0;
		  ambiColumn = 1;
		}
	    }
	}
    }

  /* the moment of truth */
  /* castling */
  if ((p == wK && sq == E1 && (t == G1 || t == C1)) || 
      (p == bK && sq == E8 && (t == G8 || t == C8)))
    {
      if ((t == G1) || (t == G8)) s += sprintf(s, "O-O");
      else if ((t == C1) || (t == C8)) s += sprintf(s, "O-O-O");
    }
  else
    {
      /* pawn move, just the column */
      if (p == wP || p == bP) *s++ = column(sq)+'a';
      else
	{
	  /* "N", or "Nb", or "N1" */
	  switch (p)
	    {
	    case wN:
	      *s++ = 'N';
	      if (ambiRank) *s++ = column(sq)+'a';
	      if (ambiColumn) *s++ = row(sq)+'1';
	      break;
	    case wB:
	      *s++ = 'B';
	      break;
	    case wR:
	      *s++ = 'R';
	      if (ambiRank) *s++ = column(sq)+'a';
	      if (ambiColumn) *s++ = row(sq)+'1';
	      break;
	    case wQ:
	      *s++ = 'Q';
	      if (ambiRank) *s++ = column(sq)+'a';
	      if (ambiColumn) *s++ = row(sq)+'1';
	      break;
	    case wK:
	      *s++ = 'K';
	      break;
	    case bN:
	      *s++ = 'N';
	      if (ambiRank) *s++ = column(sq)+'a';
	      if (ambiColumn) *s++ = row(sq)+'1';
	      break;
	    case bB:
	      *s++ = 'B';
	      break;
	    case bR:
	      *s++ = 'R';
	      if (ambiRank) *s++ = column(sq)+'a';
	      if (ambiColumn) *s++ = row(sq)+'1';
	      break;
	    case bQ:
	      *s++ = 'Q';
	      if (ambiRank) *s++ = column(sq)+'a';
	      if (ambiColumn) *s++ = row(sq)+'1';
	      break;
	    case bK:
	      *s++ = 'K';
	      break;
	    default:
	      printf("ERROR: undefined piece in san()\n");
	      break;
	    }
	}

      /* capture */
      if ((OneMask[t] & (pos->WBoard | pos->BBoard)) ||
	  /* en-passant */
	  ((p == wP || p == bP) && (OneMask[t] & plyData[ply].EP)))
	{
	  *s++ = 'x';
	  s += sprintf(s, "%c%c", column(t)+'a', row(t)+'1'); 
	}
      else /* non capture */
	{
	  /* pawn move, need only the rank */
	  if (p == wP || p == bP) *s++ = row(t)+'1';
	  /* need both column and rank */
	  else s += sprintf(s, "%c%c", column(t)+'a', row(t)+'1'); 
	}

      /* promotion */
      switch(pT)
	{
	case wN:
	  *s++ = '=';
	  *s++ = 'N';
	  break;
	case wB:
	  *s++ = '=';
	  *s++ = 'B';
	  break;
	case wR:
	  *s++ = '=';
	  *s++ = 'R';
	  break;
	case wQ:
	  *s++ = '=';
	  *s++ = 'Q';
	  break;
	case bN:
	  *s++ = '=';
	  *s++ = 'N';
	  break;
	case bB:
	  *s++ = '=';
	  *s++ = 'B';
	  break;
	case bR:
	  *s++ = '=';
	  *s++ = 'R';
	  break;
	case bQ:
	  *s++ = '=';
	  *s++ = 'Q';
	  break;
	default:
	  break;
	}
    }
  
  /* Do we need a '#' or '+'? */
  makeMove(ply+1, move, pos, plyData, others);
  if ((plyData[ply].wToMove && 
       (getWAttacker(pos->bKSq, pos, &plyData[ply+1], others) & 2)) ||
      (!plyData[ply].wToMove && 
       (getBAttacker(pos->wKSq, pos, &plyData[ply+1], others) & 2)))
    {
      noOfCaptures = captures(ply+1, winningCapture, 0, pos, plyData,
			      others, 0);
      noOfNonCaptures = nonCaptures(ply+1, nonCapture, pos, plyData,
				    others, 0);
      if (noOfCaptures+noOfNonCaptures == 0) *s++ = '#';
      else *s++ = '+';
    }
  unmakeMove(ply+1, move, pos, plyData);
  *s = '\0';
}


/* Translate move into file_number_file_number notation */
void algebraic(BB move, char *s)
{
  ui sq, t, pT;

  sq = from(move);
  t = to(move);
  pT = promotedTo(move);

  s += sprintf(s, "%c%c%c%c", column(sq)+'a', row(sq)+'1', 
	       column(t)+'a', row(t)+'1');

  switch(pT)
    {
    case wN:
      *s++ = 'n';
      break;
    case wB:
      *s++ = 'b';
      break;
    case wR:
      *s++ = 'r';
      break;
    case wQ:
      *s++ = 'q';
      break;
    case bN:
      *s++ = 'n';
      break;
    case bB:
      *s++ = 'b';
      break;
    case bR:
      *s++ = 'r';
      break;
    case bQ:
      *s++ = 'q';
      break;
    default:
      break;
    }
  *s = '\0';
}


void display(sPos pos)
{
  ui sq, piece[64];
  BB temp;

  for (sq = A1; sq <= H8; sq++) piece[sq] = 0;

  temp = pos.WPBoard;
  while (temp)
    {
      sq = firstOne(temp);
      piece[sq] = wP;
      temp &= NilMask[sq];
    }

  temp = pos.WNBoard;
  while (temp)
    {
      sq = firstOne(temp);
      piece[sq] = wN;
      temp &= NilMask[sq];
    }

  temp = pos.WBBoard;
  while (temp)
    {
      sq = firstOne(temp);
      piece[sq] = wB;
      temp &= NilMask[sq];
    }

  temp = pos.WRBoard;
  while (temp)
    {
      sq = firstOne(temp);
      piece[sq] = wR;
      temp &= NilMask[sq];
    }

  temp = pos.WQBoard;
  while (temp)
    {
      sq = firstOne(temp);
      piece[sq] = wQ;
      temp &= NilMask[sq];
    }

  piece[pos.wKSq] = wK;

  temp = pos.BPBoard;
  while (temp)
    {
      sq = firstOne(temp);
      piece[sq] = bP;
      temp &= NilMask[sq];
    }

  temp = pos.BNBoard;
  while (temp)
    {
      sq = firstOne(temp);
      piece[sq] = bN;
      temp &= NilMask[sq];
    }

  temp = pos.BBBoard;
  while (temp)
    {
      sq = firstOne(temp);
      piece[sq] = bB;
      temp &= NilMask[sq];
    }

  temp = pos.BRBoard;
  while (temp)
    {
      sq = firstOne(temp);
      piece[sq] = bR;
      temp &= NilMask[sq];
    }

  temp = pos.BQBoard;
  while (temp)
    {
      sq = firstOne(temp);
      piece[sq] = bQ;
      temp &= NilMask[sq];
    }

  piece[pos.bKSq] = bK;

  printf("\n  A B C D E F G H");
  printf("\n8 ");  
  for(sq = A8; sq <= H8; sq++) printf("%c ", boardChars[piece[sq]]);
  printf("\n7 ");  
  for(sq = A7; sq <= H7; sq++) printf("%c ", boardChars[piece[sq]]);
  printf("\n6 ");  
  for(sq = A6; sq <= H6; sq++) printf("%c ", boardChars[piece[sq]]);
  printf("\n5 ");  
  for(sq = A5; sq <= H5; sq++) printf("%c ", boardChars[piece[sq]]);
  printf("\n4 ");  
  for(sq = A4; sq <= H4; sq++) printf("%c ", boardChars[piece[sq]]);
  printf("\n3 ");  
  for(sq = A3; sq <= H3; sq++) printf("%c ", boardChars[piece[sq]]);
  printf("\n2 ");  
  for(sq = A2; sq <= H2; sq++) printf("%c ", boardChars[piece[sq]]);
  printf("\n1 ");  
  for(sq = A1; sq <= H1; sq++) printf("%c ", boardChars[piece[sq]]);
  printf("\n\n");
}


#ifdef VERIFY
/* From crafty */
void displayBitBoard(BB Board)
{
  union doub 
  {
    char i[8];
    BB d;
  };
  union doub x;
  int i, j;
#if !defined(LINUX)
  ui subs[8] = {7, 6, 5, 4, 3, 2, 1, 0};
#endif

  x.d = Board;
#if !defined(LINUX)
  for(i=7; i>=0; i--) 
    {
      printf("  %2d ", i*8);
      for(j=1; j<=128; j=j<<1)
	{
	  if(x.i[subs[i]] & j) printf("X ");
	  else printf("- ");
	}
      printf("\n");
    }
#else
  for(i=7; i>=0; i--) 
    {
      printf("  %2d ",i*8);
      for(j=1; j<=128; j=j<<1)
	{
	  if(x.i[i] & j) printf("X ");
	  else printf("- ");
	}
      printf("\n");
    }
#endif /* LINUX */
      printf("\n");
}


void displayBitRank(ui bitRank)
{
  ui i;

  for(i=1; i<=128; i=i<<1)
    {
      if(bitRank & i) printf("X ");
      else printf("- ");
    }
  printf("\n");
}
#endif /* VERIFY */


/*
 * Set up a board position based on FEN notation.  For example for command:
 *         setboard 3r2k1/p4ppp/1q6/8/8/2R1P3/P3QPPP/6K1 b - - 0 1
 */
void setBoard(char s1[80], char s2, char* s3, char* s4, char* s5, char* s6,
	      sPos *pos, sPlyData *thisPlyData, sOthers *others)
{
  ui i, c, r, ep;
  ui sqstart, sq, piece[64];

  /* Empty board */
  for (sq = A1; sq <= H8; sq++) piece[sq] = none;

  /* Piece positions */
  sq = sqstart = A8;
  i = 0;
  while (s1[i++])
    {
      switch (s1[i-1])
	{
	case 'p':
	  piece[sq] = bP;
	  sq++;
	  break;
	case 'n':
	  piece[sq] = bN;
	  sq++;
	  break;
	case 'b':
	  piece[sq] = bB;
	  sq++;
	  break;
	case 'r':
	  piece[sq] = bR;
	  sq++;
	  break;
	case 'q':
	  piece[sq] = bQ;
	  sq++;
	  break;
	case 'k':
	  piece[sq] = bK;
	  sq++;
	  break;
	case 'P':
	  piece[sq] = wP;
	  sq++;
	  break;
	case 'N':
	  piece[sq] = wN;
	  sq++;
	  break;
	case 'B':
	  piece[sq] = wB;
	  sq++;
	  break;
	case 'R':
	  piece[sq] = wR;
	  sq++;
	  break;
	case 'Q':
	  piece[sq] = wQ;
	  sq++;
	  break;
	case 'K':
	  piece[sq] = wK;
	  sq++;
	  break;
	case '1':
	  sq += 1;
	  break;
	case '2':
	  sq += 2;
	  break;
	case '3':
	  sq += 3;
	  break;
	case '4':
	  sq += 4;
	  break;
	case '5':
	  sq += 5;
	  break;
	case '6':
	  sq += 6;
	  break;
	case '7':
	  sq += 7;
	  break;
	case '8':
	  sq += 8;
	  break;
	case '/':
	  sqstart -= 8;
	  sq = sqstart;
	  break;
	default:
	  fprintf (stderr, "Strange character in FEN\n");
	  return;
	}
    }

#ifdef VERIFY
  for (sq = A1; sq <= H8; sq++) thisPlyData->piece[sq] = piece[sq];
#endif

  /* castling information */
  thisPlyData->wCanCastle = thisPlyData->bCanCastle = 0;
  for (i = 0; i < (ui) strlen(s3); i++)
    {
      if (s3[i] == 'K') thisPlyData->wCanCastle |= 1;
      if (s3[i] == 'Q') thisPlyData->wCanCastle |= 2;
      if (s3[i] == 'k') thisPlyData->bCanCastle |= 1;
      if (s3[i] == 'q') thisPlyData->bCanCastle |= 2;
    }

  /* en-passant square */
  if (s4[0] == '-' || s4[0] == '0')
    {
      thisPlyData->EP = 0;      
    }
  else
    {
      c = (ui) s4[0] - (ui) 'a';
      r = (ui) s4[1] - (ui) '1';
      ep = r*8 + c;
      thisPlyData->EP = OneMask[ep];
    }

  /* This has to be done before settings side to move, etc. */
  finalizeInit(piece, pos, thisPlyData, others);
  display(*pos);

  /* side to move.  "go" switch others->iAm */
  if (s2 == 'w') 
    {
      thisPlyData->wToMove = true;
      others->iAm = black;
    }
  else if (s2 == 'b') 
    {
      thisPlyData->wToMove = false;
      others->iAm = white;
    }  

  /* 50 move count */
  thisPlyData->count50Moves = (ui) atoi(s5);

  /* move number */
  others->moveNumber = (ui) atoi(s6);
}


/*
 * Return the FEN string of a position.
 */
void getFEN(sPos *pos, sPlyData *thisPlyData, sOthers *others, char *fen)
{
  int i, j, blanks;
  ui sq, piece[64];
  BB Temp;
  ui ep;

  for (i = A1; i <= H8; i++) piece[i] = 0;
  
  /* white pawns */
  Temp = pos->WPBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      piece[sq] = wP;
      Temp &= NilMask[sq];
    }      
  
  /* white knights */
  Temp = pos->WNBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      piece[sq] = wN;
      Temp &= NilMask[sq];
    }      
  
  /* white bishops */
  Temp = pos->WBBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      piece[sq] = wB;
      Temp &= NilMask[sq];
    }      
  
  /* white rooks */
  Temp = pos->WRBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      piece[sq] = wR;
      Temp &= NilMask[sq];
    }      
  
  /* white queens */
  Temp = pos->WQBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      piece[sq] = wQ;
      Temp &= NilMask[sq];
    }      
  
  /* white king */
  piece[pos->wKSq] = wK;
  
  /* black pawns */
  Temp = pos->BPBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      piece[sq] = bP;
      Temp &= NilMask[sq];
    }      
  
  /* black knights */
  Temp = pos->BNBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      piece[sq] = bN;
      Temp &= NilMask[sq];
    }      
  
  /* black bishops */
  Temp = pos->BBBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      piece[sq] = bB;
      Temp &= NilMask[sq];
    }      
  
  /* black rooks */
  Temp = pos->BRBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      piece[sq] = bR;
      Temp &= NilMask[sq];
    }      
  
  /* black queens */
  Temp = pos->BQBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      piece[sq] = bQ;
      Temp &= NilMask[sq];
    }      
  
  /* black king */
  piece[pos->bKSq] = bK;

  /* the pieces */
  i = 0;
  for (j = A8; j >= A1; j -= 8)
    {
      blanks = 0;
      for (sq = (ui) j; sq <= (ui) j+7; sq++)
	{
	  switch(piece[sq])
	    {
	    case wP:
	      if (blanks != 0)
		{
		  fen[i++] = blanks+'0';
		  blanks = 0;
		}
	      fen[i++] = 'P';
	      break;
	    case wN:
	      if (blanks != 0)
		{
		  fen[i++] = blanks+'0';
		  blanks = 0;
		}
	      fen[i++] = 'N';
	      break;
	    case wB:
	      if (blanks != 0)
		{
		  fen[i++] = blanks+'0';
		  blanks = 0;
		}
	      fen[i++] = 'B';
	      break;
	    case wR:
	      if (blanks != 0)
		{
		  fen[i++] = blanks+'0';
		  blanks = 0;
		}
	      fen[i++] = 'R';
	      break;
	    case wQ:
	      if (blanks != 0)
		{
		  fen[i++] = blanks+'0';
		  blanks = 0;
		}
	      fen[i++] = 'Q';
	      break;
	    case wK:
	      if (blanks != 0)
		{
		  fen[i++] = blanks+'0';
		  blanks = 0;
		}
	      fen[i++] = 'K';
	      break;
	    case bP:
	      if (blanks != 0)
		{
		  fen[i++] = blanks+'0';
		  blanks = 0;
		}
	      fen[i++] = 'p';
	      break;
	    case bN:
	      if (blanks != 0)
		{
		  fen[i++] = blanks+'0';
		  blanks = 0;
		}
	      fen[i++] = 'n';
	      break;
	    case bB:
	      if (blanks != 0)
		{
		  fen[i++] = blanks+'0';
		  blanks = 0;
		}
	      fen[i++] = 'b';
	      break;
	    case bR:
	      if (blanks != 0)
		{
		  fen[i++] = blanks+'0';
		  blanks = 0;
		}
	      fen[i++] = 'r';
	      break;
	    case bQ:
	      if (blanks != 0)
		{
		  fen[i++] = blanks+'0';
		  blanks = 0;
		}
	      fen[i++] = 'q';
	      break;
	    case bK:
	      if (blanks != 0)
		{
		  fen[i++] = blanks+'0';
		  blanks = 0;
		}
	      fen[i++] = 'k';
	      break;
	    default:
	      blanks++;
	      break;
	    }
	}
      if (blanks != 0)
	{
	  fen[i++] = blanks+'0';
	  blanks = 0;
	}
      if (j != A1) fen[i++] = '/';
      else fen[i++] = ' ';
    }

  /* side to move */
  if (thisPlyData->wToMove) fen[i++] = 'w';
  else fen[i++] = 'b';
  fen[i++] = ' ';

  /* castling information */
  if (thisPlyData->wCanCastle <= 0 && thisPlyData->bCanCastle <= 0)
    fen[i++] = '-';
  else
    {
      if (thisPlyData->wCanCastle > 0)
	{
	  if (thisPlyData->wCanCastle & 1) fen[i++] = 'K';
	  if (thisPlyData->wCanCastle & 2) fen[i++] = 'Q';
	}
      if (thisPlyData->bCanCastle > 0)
	{
	  if (thisPlyData->bCanCastle & 1) fen[i++] = 'k';
	  if (thisPlyData->bCanCastle & 2) fen[i++] = 'q';
	}
    }
  fen[i++] = ' ';

  /* en-passant square */
  if (thisPlyData->EP)
    {
      ep = firstOne(thisPlyData->EP);
      fen[i++] = column(ep) + 'a';
      fen[i++] = row(ep) + '1';
    }
  else fen[i++] = '-';
  fen[i++] = ' ';

  /* 50 move count */
  if (thisPlyData->count50Moves > 9) 
    fen[i++] = (int)(thisPlyData->count50Moves/10) + '0';
  fen[i++] = thisPlyData->count50Moves%10 + '0';
  fen[i++] = ' ';

  /* move number */
  if (others->moveNumber > 9) 
    fen[i++] = (int)(others->moveNumber/10) + '0';
  fen[i++] = others->moveNumber%10 + '0';

  /* end the string */
  fen[i] = '\0';
}


/*
 * Return the reverse FEN string of a position.
 */
void reverse(sPos *pos, sPlyData *thisPlyData, sOthers *others, char *fen)
{
  int i, j, blanks;
  ui sq, piece[64];
  BB Temp;
  ui ep;
  BB *pWAttacker = (BB *) thisPlyData->wAttacker;
  BB *pBAttacker = (BB *) thisPlyData->bAttacker;

  for (i = A1; i <= H8; i++) piece[i] = 0;

  /* Attack tables */
  pWAttacker[0] =   pWAttacker[1] =   pWAttacker[2] =   pWAttacker[3] = 
    pWAttacker[4] =   pWAttacker[5] =   pWAttacker[6] =   pWAttacker[7] = 0; 
  pBAttacker[0] =   pBAttacker[1] =   pBAttacker[2] =   pBAttacker[3] = 
    pBAttacker[4] =   pBAttacker[5] =   pBAttacker[6] =   pBAttacker[7] = 0; 

  /* white pawns */
  Temp = pos->WPBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      piece[reverseSq[sq]] = bP;
      Temp &= NilMask[sq];
    }      
  
  /* white knights */
  Temp = pos->WNBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      piece[reverseSq[sq]] = bN;
      Temp &= NilMask[sq];
    }      
  
  /* white bishops */
  Temp = pos->WBBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      piece[reverseSq[sq]] = bB;
      Temp &= NilMask[sq];
    }      
  
  /* white rooks */
  Temp = pos->WRBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      piece[reverseSq[sq]] = bR;
      Temp &= NilMask[sq];
    }      
  
  /* white queens */
  Temp = pos->WQBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      piece[reverseSq[sq]] = bQ;
      Temp &= NilMask[sq];
    }      
  
  /* white king */
  piece[reverseSq[pos->wKSq]] = bK;
  
  /* black pawns */
  Temp = pos->BPBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      piece[reverseSq[sq]] = wP;
      Temp &= NilMask[sq];
    }      
  
  /* black knights */
  Temp = pos->BNBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      piece[reverseSq[sq]] = wN;
      Temp &= NilMask[sq];
    }      
  
  /* black bishops */
  Temp = pos->BBBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      piece[reverseSq[sq]] = wB;
      Temp &= NilMask[sq];
    }      
  
  /* black rooks */
  Temp = pos->BRBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      piece[reverseSq[sq]] = wR;
      Temp &= NilMask[sq];
    }      
  
  /* black queens */
  Temp = pos->BQBoard;
  while (Temp)
    {
      sq = firstOne(Temp);
      piece[reverseSq[sq]] = wQ;
      Temp &= NilMask[sq];
    }      
  
  /* black king */
  piece[reverseSq[pos->bKSq]] = wK;

  /* the pieces */
  i = 0;
  for (j = A8; j >= A1; j -= 8)
    {
      blanks = 0;
      for (sq = (ui) j; sq <= (ui) j+7; sq++)
	{
	  switch(piece[sq])
	    {
	    case wP:
	      if (blanks != 0)
		{
		  fen[i++] = blanks+'0';
		  blanks = 0;
		}
	      fen[i++] = 'P';
	      break;
	    case wN:
	      if (blanks != 0)
		{
		  fen[i++] = blanks+'0';
		  blanks = 0;
		}
	      fen[i++] = 'N';
	      break;
	    case wB:
	      if (blanks != 0)
		{
		  fen[i++] = blanks+'0';
		  blanks = 0;
		}
	      fen[i++] = 'B';
	      break;
	    case wR:
	      if (blanks != 0)
		{
		  fen[i++] = blanks+'0';
		  blanks = 0;
		}
	      fen[i++] = 'R';
	      break;
	    case wQ:
	      if (blanks != 0)
		{
		  fen[i++] = blanks+'0';
		  blanks = 0;
		}
	      fen[i++] = 'Q';
	      break;
	    case wK:
	      if (blanks != 0)
		{
		  fen[i++] = blanks+'0';
		  blanks = 0;
		}
	      fen[i++] = 'K';
	      break;
	    case bP:
	      if (blanks != 0)
		{
		  fen[i++] = blanks+'0';
		  blanks = 0;
		}
	      fen[i++] = 'p';
	      break;
	    case bN:
	      if (blanks != 0)
		{
		  fen[i++] = blanks+'0';
		  blanks = 0;
		}
	      fen[i++] = 'n';
	      break;
	    case bB:
	      if (blanks != 0)
		{
		  fen[i++] = blanks+'0';
		  blanks = 0;
		}
	      fen[i++] = 'b';
	      break;
	    case bR:
	      if (blanks != 0)
		{
		  fen[i++] = blanks+'0';
		  blanks = 0;
		}
	      fen[i++] = 'r';
	      break;
	    case bQ:
	      if (blanks != 0)
		{
		  fen[i++] = blanks+'0';
		  blanks = 0;
		}
	      fen[i++] = 'q';
	      break;
	    case bK:
	      if (blanks != 0)
		{
		  fen[i++] = blanks+'0';
		  blanks = 0;
		}
	      fen[i++] = 'k';
	      break;
	    default:
	      blanks++;
	      break;
	    }
	}
      if (blanks != 0)
	{
	  fen[i++] = blanks+'0';
	  blanks = 0;
	}
      if (j != A1) fen[i++] = '/';
      else fen[i++] = ' ';
    }

  /* side to move */
  if (thisPlyData->wToMove) fen[i++] = 'b';
  else fen[i++] = 'w';
  thisPlyData->wToMove ^= 1;
  fen[i++] = ' ';

  /* castling information */
  if (thisPlyData->wCanCastle <= 0 && thisPlyData->bCanCastle <= 0)
    fen[i++] = '-';
  else
    {
      if (thisPlyData->bCanCastle > 0)
	{
	  if (thisPlyData->bCanCastle & 1) fen[i++] = 'K';
	  if (thisPlyData->bCanCastle & 2) fen[i++] = 'Q';
	}
      if (thisPlyData->wCanCastle > 0)
	{
	  if (thisPlyData->wCanCastle & 1) fen[i++] = 'k';
	  if (thisPlyData->wCanCastle & 2) fen[i++] = 'q';
	}
    }
  fen[i++] = ' ';

  /* en-passant square */
  if (thisPlyData->EP)
    {
      ep = firstOne(thisPlyData->EP);
      fen[i++] = column(reverseSq[ep]) + 'a';
      fen[i++] = row(reverseSq[ep]) + '1';
    }
  else fen[i++] = '-';
  fen[i++] = ' ';

  /* 50 move count */
  if (thisPlyData->count50Moves > 9) 
    fen[i++] = (int)(thisPlyData->count50Moves/10) + '0';
  fen[i++] = thisPlyData->count50Moves%10 + '0';
  fen[i++] = ' ';

  /* move number */
  if (others->moveNumber > 9) 
    fen[i++] = (int)(others->moveNumber/10) + '0';
  fen[i++] = others->moveNumber%10 + '0';

  /* end the string */
  fen[i] = '\0';

  /* This has to be at the end, after fen[] is written correctly. */
  finalizeInit(piece, pos, thisPlyData, others);
  display(*pos);
}


/*
 * Find the best move in pos between indeces 
 * head and tail.  Swap the move with highest score to the head.
 */
void pick(ui head, ui tail, BB *moveList)
{
  ui i, maxIndex;
  int max;
  BB temp;

  maxIndex = head;
  max = score(moveList[head]);

  for (i = head+1; i < tail; i++)
    {
      if (max < score(moveList[i]))
	{
	  maxIndex = i;
	  max = score(moveList[i]);
	}
    }

  if (maxIndex != head)
    {
      temp = moveList[head];
      moveList[head] = moveList[maxIndex];
      moveList[maxIndex] = temp;
    }
}


/* "borrowed" from Crafty */
ui firstOne(BB arg1)
{
  if (arg1 & 65535)
    return (fOne[arg1 & 65535]);
  if ((arg1 >> 16) & 65535)
    return (fOne[(arg1 >> 16) & 65535] + 16);
  if ((arg1 >> 32) & 65535)
    return (fOne[(arg1 >> 32) & 65535] + 32);
  return (fOne[arg1 >> 48] + 48);
}


/* "borrowed" from Crafty */
ui lastOne(BB arg1)
{
  if (!arg1) return 64;
  if (arg1 >> 48)
    return (63 - lOne[arg1 >> 48]);
  if ((arg1 >> 32) & 65535)
    return (47 - lOne[(arg1 >> 32) & 65535]);
  if ((arg1 >> 16) & 65535)
    return (31 - lOne[(arg1 >> 16) & 65535]);
  return (15 - lOne[arg1 & 65535]);
}


ui bitCount(register BB a)
/* Count the number of one bits in an unsigned long long variable */
{
  register int c=0;

#ifdef HYATT
  while(a) {
    c++;
    a &= a - 1;
  }
  return(c);
#endif

  /* Bruce Moreland */
  if (!a) return (0);
  else if (!(a &= a - 1)) return (1);
  else if (!(a &= a - 1)) return (2);
  else
    for (c = 3; ; c++)
      if (!(a &= a - 1)) return (c);
}


void closing(FILE *fd, sOthers *others, char *result, char *comment)
{
  if (others->iAm == white)
    {
      if (strstr(result, "1-0"))
	fprintf(fd, "Result:  win w %5u %3u moves %s\n", 
		others->openingNo, (others->moveNumber+1), comment);
      else if (strstr(result, "0-1"))
	fprintf(fd, "Result: lost w %5u %3u moves %s\n", 
		others->openingNo, (others->moveNumber+1), comment);
      else
	fprintf(fd, "Result: draw w %5u %3u moves %s\n", 
		others->openingNo, (others->moveNumber+1), comment);
    }
  else
    {
      if (strstr(result, "1-0"))
	fprintf(fd, "Result: lost b %5u %3u moves %s\n", 
		others->openingNo, (others->moveNumber+1), comment);
      else if (strstr(result, "0-1"))
	fprintf(fd, "Result:  win b %5u %3u moves %s\n", 
		others->openingNo, (others->moveNumber+1), comment);
      else
	fprintf(fd, "Result: draw b %5u %3u moves %s\n", 
		others->openingNo, (others->moveNumber+1), comment);
    }
  fflush(fd);
}


void exiting(FILE *fd, sHash **hash, sOthers *others)
{
  free(hash[white]);
  free(hash[black]);
  free(others);
  fclose(fd);
  exit(0);
}
